import { Moves } from "#enums/moves";
import { Species } from "#enums/species";

export type LevelMoves = ([integer, Moves])[];

interface PokemonSpeciesLevelMoves {
  [key: integer]: LevelMoves
}

interface PokemonFormLevelMoves {
  [key: integer]: LevelMoves
}

interface PokemonSpeciesFormLevelMoves {
  [key: integer]: PokemonFormLevelMoves
}

/** Moves that can only be learned with a memory-mushroom */
const RELEARN_MOVE = -1;
/** Moves that can only be learned with an evolve */
const EVOLVE_MOVE = 0;

export const pokemonSpeciesLevelMoves: PokemonSpeciesLevelMoves = {
  [Species.BULBASAUR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.VINE_WHIP ],
    [ 6, Moves.GROWTH ],
    [ 9, Moves.LEECH_SEED ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 15, Moves.POISON_POWDER ],
    [ 15, Moves.SLEEP_POWDER ],
    [ 18, Moves.SEED_BOMB ],
    [ 21, Moves.TAKE_DOWN ],
    [ 24, Moves.SWEET_SCENT ],
    [ 27, Moves.SYNTHESIS ],
    [ 30, Moves.WORRY_SEED ],
    [ 33, Moves.POWER_WHIP ],
    [ 36, Moves.SOLAR_BEAM ],
  ],
  [Species.IVYSAUR]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.GROWTH ],
    [ 9, Moves.LEECH_SEED ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 15, Moves.POISON_POWDER ],
    [ 15, Moves.SLEEP_POWDER ],
    [ 20, Moves.SEED_BOMB ],
    [ 25, Moves.TAKE_DOWN ],
    [ 30, Moves.SWEET_SCENT ],
    [ 35, Moves.SYNTHESIS ],
    [ 40, Moves.WORRY_SEED ],
    [ 45, Moves.POWER_WHIP ],
    [ 50, Moves.SOLAR_BEAM ],
  ],
  [Species.VENUSAUR]: [
    [ EVOLVE_MOVE, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.PETAL_DANCE ],
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 9, Moves.LEECH_SEED ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 15, Moves.POISON_POWDER ],
    [ 15, Moves.SLEEP_POWDER ],
    [ 20, Moves.SEED_BOMB ],
    [ 25, Moves.TAKE_DOWN ],
    [ 30, Moves.SWEET_SCENT ],
    [ 37, Moves.SYNTHESIS ],
    [ 44, Moves.WORRY_SEED ],
    [ 51, Moves.POWER_WHIP ],
    [ 58, Moves.SOLAR_BEAM ],
  ],
  [Species.CHARMANDER]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.EMBER ],
    [ 8, Moves.SMOKESCREEN ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 17, Moves.FIRE_FANG ],
    [ 20, Moves.SLASH ],
    [ 24, Moves.FLAMETHROWER ],
    [ 28, Moves.SCARY_FACE ],
    [ 32, Moves.FIRE_SPIN ],
    [ 36, Moves.INFERNO ],
    [ 40, Moves.FLARE_BLITZ ],
  ],
  [Species.CHARMELEON]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 19, Moves.FIRE_FANG ],
    [ 24, Moves.SLASH ],
    [ 30, Moves.FLAMETHROWER ],
    [ 37, Moves.SCARY_FACE ],
    [ 48, Moves.INFERNO ],
    [ 54, Moves.FLARE_BLITZ ],
  ],
  [Species.CHARIZARD]: [
    [ EVOLVE_MOVE, Moves.AIR_SLASH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.HEAT_WAVE ],
    [ 1, Moves.DRAGON_CLAW ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 19, Moves.FIRE_FANG ],
    [ 24, Moves.SLASH ],
    [ 30, Moves.FLAMETHROWER ],
    [ 39, Moves.SCARY_FACE ],
    [ 46, Moves.FIRE_SPIN ],
    [ 54, Moves.INFERNO ],
    [ 62, Moves.FLARE_BLITZ ],
  ],
  [Species.SQUIRTLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 3, Moves.WATER_GUN ],
    [ 6, Moves.WITHDRAW ],
    [ 9, Moves.RAPID_SPIN ],
    [ 12, Moves.BITE ],
    [ 15, Moves.WATER_PULSE ],
    [ 18, Moves.PROTECT ],
    [ 21, Moves.RAIN_DANCE ],
    [ 24, Moves.AQUA_TAIL ],
    [ 27, Moves.SHELL_SMASH ],
    [ 30, Moves.IRON_DEFENSE ],
    [ 33, Moves.HYDRO_PUMP ],
    [ 36, Moves.WAVE_CRASH ],
  ],
  [Species.WARTORTLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WITHDRAW ],
    [ 9, Moves.RAPID_SPIN ],
    [ 12, Moves.BITE ],
    [ 15, Moves.WATER_PULSE ],
    [ 20, Moves.PROTECT ],
    [ 25, Moves.RAIN_DANCE ],
    [ 30, Moves.AQUA_TAIL ],
    [ 35, Moves.SHELL_SMASH ],
    [ 40, Moves.IRON_DEFENSE ],
    [ 45, Moves.HYDRO_PUMP ],
    [ 50, Moves.WAVE_CRASH ],
  ],
  [Species.BLASTOISE]: [
    [ EVOLVE_MOVE, Moves.FLASH_CANNON ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WITHDRAW ],
    [ 9, Moves.RAPID_SPIN ],
    [ 12, Moves.BITE ],
    [ 15, Moves.WATER_PULSE ],
    [ 20, Moves.PROTECT ],
    [ 25, Moves.RAIN_DANCE ],
    [ 30, Moves.AQUA_TAIL ],
    [ 35, Moves.SHELL_SMASH ],
    [ 42, Moves.IRON_DEFENSE ],
    [ 49, Moves.HYDRO_PUMP ],
    [ 56, Moves.WAVE_CRASH ],
  ],
  [Species.CATERPIE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 9, Moves.BUG_BITE ],
  ],
  [Species.METAPOD]: [
    [ EVOLVE_MOVE, Moves.HARDEN ],
    [ 1, Moves.HARDEN ],
  ],
  [Species.BUTTERFREE]: [
    [ EVOLVE_MOVE, Moves.GUST ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.BUG_BITE ],
    [ 4, Moves.SUPERSONIC ],
    [ 8, Moves.CONFUSION ],
    [ 12, Moves.POISON_POWDER ],
    [ 12, Moves.STUN_SPORE ],
    [ 12, Moves.SLEEP_POWDER ],
    [ 16, Moves.PSYBEAM ],
    [ 20, Moves.WHIRLWIND ],
    [ 24, Moves.AIR_SLASH ],
    [ 28, Moves.SAFEGUARD ],
    [ 32, Moves.BUG_BUZZ ],
    [ 36, Moves.TAILWIND ],
    [ 40, Moves.RAGE_POWDER ],
    [ 44, Moves.QUIVER_DANCE ],
  ],
  [Species.WEEDLE]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.STRING_SHOT ],
    [ 9, Moves.BUG_BITE ],
  ],
  [Species.KAKUNA]: [
    [ EVOLVE_MOVE, Moves.HARDEN ],
    [ 1, Moves.HARDEN ],
  ],
  [Species.BEEDRILL]: [
    [ EVOLVE_MOVE, Moves.TWINEEDLE ],
    [ 1, Moves.FURY_ATTACK ],
    [ 11, Moves.FURY_CUTTER ],
    [ 14, Moves.RAGE ],
    [ 17, Moves.PURSUIT ],
    [ 20, Moves.FOCUS_ENERGY ],
    [ 23, Moves.VENOSHOCK ],
    [ 26, Moves.ASSURANCE ],
    [ 29, Moves.TOXIC_SPIKES ],
    [ 32, Moves.PIN_MISSILE ],
    [ 35, Moves.POISON_JAB ],
    [ 38, Moves.AGILITY ],
    [ 41, Moves.ENDEAVOR ],
    [ 44, Moves.FELL_STINGER ],
  ],
  [Species.PIDGEY]: [
    [ 1, Moves.TACKLE ],
    [ 5, Moves.SAND_ATTACK ],
    [ 9, Moves.GUST ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 17, Moves.WHIRLWIND ],
    [ 21, Moves.TWISTER ],
    [ 25, Moves.FEATHER_DANCE ],
    [ 29, Moves.AGILITY ],
    [ 33, Moves.WING_ATTACK ],
    [ 37, Moves.ROOST ],
    [ 41, Moves.TAILWIND ],
    [ 45, Moves.AERIAL_ACE ],
    [ 49, Moves.AIR_SLASH ],
    [ 53, Moves.HURRICANE ],
  ],
  [Species.PIDGEOTTO]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 5, Moves.SAND_ATTACK ],
    [ 9, Moves.GUST ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 17, Moves.WHIRLWIND ],
    [ 22, Moves.TWISTER ],
    [ 27, Moves.FEATHER_DANCE ],
    [ 32, Moves.AGILITY ],
    [ 37, Moves.WING_ATTACK ],
    [ 42, Moves.ROOST ],
    [ 47, Moves.TAILWIND ],
    [ 52, Moves.AERIAL_ACE ],
    [ 57, Moves.AIR_SLASH ],
    [ 62, Moves.HURRICANE ],
  ],
  [Species.PIDGEOT]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.HURRICANE ],
    [ 5, Moves.SAND_ATTACK ],
    [ 9, Moves.GUST ],
    [ 17, Moves.WHIRLWIND ],
    [ 22, Moves.TWISTER ],
    [ 27, Moves.FEATHER_DANCE ],
    [ 32, Moves.AGILITY ],
    [ 38, Moves.WING_ATTACK ],
    [ 44, Moves.ROOST ],
    [ 50, Moves.TAILWIND ],
    [ 56, Moves.AERIAL_ACE ],
    [ 62, Moves.AIR_SLASH ],
    [ 68, Moves.HURRICANE ],
  ],
  [Species.RATTATA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 4, Moves.QUICK_ATTACK ],
    [ 7, Moves.FOCUS_ENERGY ],
    [ 10, Moves.BITE ],
    [ 13, Moves.LASER_FOCUS ],
    [ 16, Moves.TAKE_DOWN ],
    [ 19, Moves.ASSURANCE ],
    [ 22, Moves.CRUNCH ],
    [ 25, Moves.SUCKER_PUNCH ],
    [ 28, Moves.SUPER_FANG ],
    [ 31, Moves.DOUBLE_EDGE ],
    [ 34, Moves.ENDEAVOR ],
  ],
  [Species.RATICATE]: [
    [ EVOLVE_MOVE, Moves.SCARY_FACE ],
    [ 1, Moves.SWORDS_DANCE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 10, Moves.BITE ],
    [ 13, Moves.LASER_FOCUS ],
    [ 16, Moves.TAKE_DOWN ],
    [ 19, Moves.ASSURANCE ],
    [ 24, Moves.CRUNCH ],
    [ 29, Moves.SUCKER_PUNCH ],
    [ 34, Moves.SUPER_FANG ],
    [ 39, Moves.DOUBLE_EDGE ],
    [ 44, Moves.ENDEAVOR ],
  ],
  [Species.SPEAROW]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 4, Moves.LEER ],
    [ 8, Moves.ASSURANCE ],
    [ 11, Moves.FURY_ATTACK ],
    [ 15, Moves.AERIAL_ACE ],
    [ 18, Moves.WING_ATTACK ],
    [ 22, Moves.TAKE_DOWN ],
    [ 25, Moves.AGILITY ],
    [ 29, Moves.FOCUS_ENERGY ],
    [ 32, Moves.ROOST ],
    [ 36, Moves.DRILL_PECK ],
  ],
  [Species.FEAROW]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 1, Moves.ASSURANCE ],
    [ 1, Moves.PLUCK ],
    [ 1, Moves.DRILL_RUN ],
    [ 1, Moves.PURSUIT ],
    [ 4, Moves.LEER ],
    [ 8, Moves.ASSURANCE ],
    [ 11, Moves.FURY_ATTACK ],
    [ 15, Moves.AERIAL_ACE ],
    [ 18, Moves.WING_ATTACK ],
    [ 23, Moves.TAKE_DOWN ],
    [ 27, Moves.AGILITY ],
    [ 32, Moves.FOCUS_ENERGY ],
    [ 36, Moves.ROOST ],
    [ 41, Moves.DRILL_PECK ],
  ],
  [Species.EKANS]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.LEER ],
    [ 4, Moves.POISON_STING ],
    [ 9, Moves.BITE ],
    [ 12, Moves.GLARE ],
    [ 17, Moves.SCREECH ],
    [ 20, Moves.ACID ],
    [ 25, Moves.SWALLOW ],
    [ 25, Moves.STOCKPILE ],
    [ 25, Moves.SPIT_UP ],
    [ 28, Moves.ACID_SPRAY ],
    [ 33, Moves.SLUDGE_BOMB ],
    [ 36, Moves.GASTRO_ACID ],
    [ 38, Moves.BELCH ],
    [ 41, Moves.HAZE ],
    [ 44, Moves.COIL ],
    [ 49, Moves.GUNK_SHOT ],
  ],
  [Species.ARBOK]: [
    [ EVOLVE_MOVE, Moves.CRUNCH ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 12, Moves.GLARE ],
    [ 17, Moves.SCREECH ],
    [ 20, Moves.ACID ],
    [ 27, Moves.STOCKPILE ],
    [ 27, Moves.SPIT_UP ],
    [ 27, Moves.SWALLOW ],
    [ 32, Moves.ACID_SPRAY ],
    [ 39, Moves.SLUDGE_BOMB ],
    [ 44, Moves.GASTRO_ACID ],
    [ 48, Moves.BELCH ],
    [ 51, Moves.HAZE ],
    [ 56, Moves.COIL ],
    [ 63, Moves.GUNK_SHOT ],
  ],
  [Species.PIKACHU]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.NUZZLE ],
    [ 4, Moves.THUNDER_WAVE ],
    [ 8, Moves.DOUBLE_TEAM ],
    [ 12, Moves.ELECTRO_BALL ],
    [ 16, Moves.FEINT ],
    [ 20, Moves.SPARK ],
    [ 24, Moves.AGILITY ],
    [ 28, Moves.IRON_TAIL ],
    [ 32, Moves.DISCHARGE ],
    [ 36, Moves.THUNDERBOLT ],
    [ 40, Moves.LIGHT_SCREEN ],
    [ 44, Moves.THUNDER ],
    [ 48, Moves.PIKA_PAPOW ],
  ],
  [Species.RAICHU]: [
    [ EVOLVE_MOVE, Moves.ZIPPY_ZAP ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.THUNDER ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.SPARK ],
    [ 1, Moves.IRON_TAIL ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.DISCHARGE ],
    [ 1, Moves.ELECTRO_BALL ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.NUZZLE ],
    [ 1, Moves.THUNDER_PUNCH ],
    [ 5, Moves.THUNDERBOLT ],
    [ 50, Moves.PIKA_PAPOW ],
  ],
  [Species.SANDSHREW]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 3, Moves.POISON_STING ],
    [ 6, Moves.SAND_ATTACK ],
    [ 9, Moves.ROLLOUT ],
    [ 12, Moves.FURY_CUTTER ],
    [ 15, Moves.RAPID_SPIN ],
    [ 18, Moves.BULLDOZE ],
    [ 21, Moves.SWIFT ],
    [ 24, Moves.FURY_SWIPES ],
    [ 27, Moves.AGILITY ],
    [ 30, Moves.SLASH ],
    [ 33, Moves.DIG ],
    [ 36, Moves.GYRO_BALL ],
    [ 39, Moves.SWORDS_DANCE ],
    [ 42, Moves.SANDSTORM ],
    [ 45, Moves.EARTHQUAKE ],
  ],
  [Species.SANDSLASH]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.CRUSH_CLAW ],
    [ 9, Moves.ROLLOUT ],
    [ 12, Moves.FURY_CUTTER ],
    [ 15, Moves.RAPID_SPIN ],
    [ 18, Moves.BULLDOZE ],
    [ 21, Moves.SWIFT ],
    [ 26, Moves.FURY_SWIPES ],
    [ 31, Moves.SAND_TOMB ],
    [ 36, Moves.SLASH ],
    [ 41, Moves.DIG ],
    [ 46, Moves.GYRO_BALL ],
    [ 51, Moves.SWORDS_DANCE ],
    [ 56, Moves.SANDSTORM ],
    [ 61, Moves.EARTHQUAKE ],
  ],
  [Species.NIDORAN_F]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.SCRATCH ],
    [ 10, Moves.TAIL_WHIP ],
    [ 15, Moves.FURY_SWIPES ],
    [ 20, Moves.TOXIC_SPIKES ],
    [ 25, Moves.DOUBLE_KICK ],
    [ 30, Moves.BITE ],
    [ 35, Moves.HELPING_HAND ],
    [ 40, Moves.TOXIC ],
    [ 45, Moves.FLATTER ],
    [ 50, Moves.CRUNCH ],
    [ 55, Moves.EARTH_POWER ],
  ],
  [Species.NIDORINA]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.GROWL ],
    [ 15, Moves.FURY_SWIPES ],
    [ 22, Moves.TOXIC_SPIKES ],
    [ 29, Moves.DOUBLE_KICK ],
    [ 36, Moves.BITE ],
    [ 43, Moves.HELPING_HAND ],
    [ 50, Moves.TOXIC ],
    [ 57, Moves.FLATTER ],
    [ 64, Moves.CRUNCH ],
    [ 71, Moves.EARTH_POWER ],
  ],
  [Species.NIDOQUEEN]: [
    [ EVOLVE_MOVE, Moves.SUPERPOWER ],
    [ 1, Moves.SLUDGE_WAVE ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.DOUBLE_KICK ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.FURY_SWIPES ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.FLATTER ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.TOXIC_SPIKES ],
    [ 1, Moves.EARTH_POWER ],
  ],
  [Species.NIDORAN_M]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 5, Moves.PECK ],
    [ 10, Moves.FOCUS_ENERGY ],
    [ 15, Moves.FURY_ATTACK ],
    [ 20, Moves.TOXIC_SPIKES ],
    [ 25, Moves.DOUBLE_KICK ],
    [ 30, Moves.HORN_ATTACK ],
    [ 35, Moves.HELPING_HAND ],
    [ 40, Moves.TOXIC ],
    [ 45, Moves.FLATTER ],
    [ 50, Moves.POISON_JAB ],
    [ 55, Moves.EARTH_POWER ],
  ],
  [Species.NIDORINO]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 15, Moves.FURY_ATTACK ],
    [ 22, Moves.TOXIC_SPIKES ],
    [ 29, Moves.DOUBLE_KICK ],
    [ 36, Moves.HORN_ATTACK ],
    [ 43, Moves.HELPING_HAND ],
    [ 50, Moves.TOXIC ],
    [ 57, Moves.FLATTER ],
    [ 64, Moves.POISON_JAB ],
    [ 71, Moves.EARTH_POWER ],
  ],
  [Species.NIDOKING]: [
    [ EVOLVE_MOVE, Moves.MEGAHORN ],
    [ 1, Moves.SLUDGE_WAVE ],
    [ 1, Moves.DOUBLE_KICK ],
    [ 1, Moves.HORN_ATTACK ],
    [ 1, Moves.FURY_ATTACK ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FLATTER ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.TOXIC_SPIKES ],
    [ 1, Moves.POISON_JAB ],
    [ 1, Moves.EARTH_POWER ],
  ],
  [Species.CLEFAIRY]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SING ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.SPOTLIGHT ],
    [ 4, Moves.STORED_POWER ],
    [ 8, Moves.ENCORE ],
    [ 12, Moves.AFTER_YOU ],
    [ 16, Moves.LIFE_DEW ],
    [ 20, Moves.METRONOME ],
    [ 24, Moves.MOONLIGHT ],
    [ 28, Moves.GRAVITY ],
    [ 32, Moves.METEOR_MASH ],
    [ 36, Moves.FOLLOW_ME ],
    [ 40, Moves.COSMIC_POWER ],
    [ 44, Moves.MOONBLAST ],
    [ 48, Moves.HEALING_WISH ],
  ],
  [Species.CLEFABLE]: [
    [ RELEARN_MOVE, Moves.POUND ],
    [ RELEARN_MOVE, Moves.GROWL ],
    [ RELEARN_MOVE, Moves.SING ],
    [ RELEARN_MOVE, Moves.DEFENSE_CURL ],
    [ RELEARN_MOVE, Moves.SPLASH ],
    [ RELEARN_MOVE, Moves.SWEET_KISS ],
    [ RELEARN_MOVE, Moves.CHARM ],
    [ RELEARN_MOVE, Moves.ENCORE ],
    [ RELEARN_MOVE, Moves.MOONLIGHT ],
    [ RELEARN_MOVE, Moves.FOLLOW_ME ],
    [ RELEARN_MOVE, Moves.COSMIC_POWER ],
    [ RELEARN_MOVE, Moves.GRAVITY ],
    [ RELEARN_MOVE, Moves.HEALING_WISH ],
    [ RELEARN_MOVE, Moves.COPYCAT ],
    [ RELEARN_MOVE, Moves.AFTER_YOU ],
    [ RELEARN_MOVE, Moves.STORED_POWER ],
    [ RELEARN_MOVE, Moves.DISARMING_VOICE ],
    [ 1, Moves.METRONOME ],
    [ 1, Moves.METEOR_MASH ],
    [ 1, Moves.MOONBLAST ],
    [ 1, Moves.LIFE_DEW ],
    [ 1, Moves.SPOTLIGHT ],
  ],
  [Species.VULPIX]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.EMBER ],
    [ 4, Moves.DISABLE ],
    [ 8, Moves.QUICK_ATTACK ],
    [ 12, Moves.SPITE ],
    [ 16, Moves.INCINERATE ],
    [ 20, Moves.CONFUSE_RAY ],
    [ 24, Moves.WILL_O_WISP ],
    [ 28, Moves.EXTRASENSORY ],
    [ 32, Moves.FLAMETHROWER ],
    [ 36, Moves.IMPRISON ],
    [ 40, Moves.FIRE_SPIN ],
    [ 44, Moves.SAFEGUARD ],
    [ 48, Moves.INFERNO ],
    [ 52, Moves.FIRE_BLAST ],
  ],
  [Species.NINETALES]: [
    [ RELEARN_MOVE, Moves.DISABLE ],
    [ RELEARN_MOVE, Moves.EMBER ],
    [ RELEARN_MOVE, Moves.FIRE_SPIN ],
    [ RELEARN_MOVE, Moves.CONFUSE_RAY ],
    [ RELEARN_MOVE, Moves.FIRE_BLAST ],
    [ RELEARN_MOVE, Moves.SPITE ],
    [ RELEARN_MOVE, Moves.SAFEGUARD ],
    [ RELEARN_MOVE, Moves.WILL_O_WISP ],
    [ RELEARN_MOVE, Moves.IMPRISON ],
    [ RELEARN_MOVE, Moves.EXTRASENSORY ],
    [ RELEARN_MOVE, Moves.NASTY_PLOT ],
    [ RELEARN_MOVE, Moves.INCINERATE ],
    [ RELEARN_MOVE, Moves.INFERNO ],
    [ 1, Moves.FLAMETHROWER ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.QUICK_ATTACK ],
  ],
  [Species.JIGGLYPUFF]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.SING ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 4, Moves.ECHOED_VOICE ],
    [ 8, Moves.COVET ],
    [ 12, Moves.STOCKPILE ],
    [ 12, Moves.SPIT_UP ],
    [ 12, Moves.SWALLOW ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.REST ],
    [ 24, Moves.BODY_SLAM ],
    [ 28, Moves.MIMIC ],
    [ 32, Moves.GYRO_BALL ],
    [ 36, Moves.HYPER_VOICE ],
    [ 44, Moves.DOUBLE_EDGE ],
  ],
  [Species.WIGGLYTUFF]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.BODY_SLAM ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.SING ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.MIMIC ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.REST ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.STOCKPILE ],
    [ 1, Moves.SPIT_UP ],
    [ 1, Moves.SWALLOW ],
    [ 1, Moves.HYPER_VOICE ],
    [ 1, Moves.COVET ],
    [ 1, Moves.GYRO_BALL ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.ROUND ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 5, Moves.PLAY_ROUGH ],
  ],
  [Species.ZUBAT]: [
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.ABSORB ],
    [ 5, Moves.ASTONISH ],
    [ 10, Moves.MEAN_LOOK ],
    [ 15, Moves.POISON_FANG ],
    [ 20, Moves.QUICK_GUARD ],
    [ 25, Moves.AIR_CUTTER ],
    [ 30, Moves.BITE ],
    [ 35, Moves.HAZE ],
    [ 40, Moves.VENOSHOCK ],
    [ 45, Moves.CONFUSE_RAY ],
    [ 50, Moves.AIR_SLASH ],
    [ 55, Moves.LEECH_LIFE ],
  ],
  [Species.GOLBAT]: [
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.SCREECH ],
    [ 1, Moves.MEAN_LOOK ],
    [ 1, Moves.ASTONISH ],
    [ 15, Moves.POISON_FANG ],
    [ 20, Moves.QUICK_GUARD ],
    [ 27, Moves.AIR_CUTTER ],
    [ 34, Moves.BITE ],
    [ 41, Moves.HAZE ],
    [ 48, Moves.VENOSHOCK ],
    [ 55, Moves.CONFUSE_RAY ],
    [ 62, Moves.AIR_SLASH ],
    [ 69, Moves.LEECH_LIFE ],
  ],
  [Species.ODDISH]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 4, Moves.ACID ],
    [ 8, Moves.SWEET_SCENT ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 14, Moves.POISON_POWDER ],
    [ 16, Moves.STUN_SPORE ],
    [ 18, Moves.SLEEP_POWDER ],
    [ 20, Moves.GIGA_DRAIN ],
    [ 24, Moves.TOXIC ],
    [ 28, Moves.MOONBLAST ],
    [ 32, Moves.GRASSY_TERRAIN ],
    [ 36, Moves.MOONLIGHT ],
    [ 40, Moves.PETAL_DANCE ],
  ],
  [Species.GLOOM]: [
    [ 1, Moves.ACID ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.SWEET_SCENT ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 14, Moves.POISON_POWDER ],
    [ 16, Moves.STUN_SPORE ],
    [ 18, Moves.SLEEP_POWDER ],
    [ 20, Moves.GIGA_DRAIN ],
    [ 26, Moves.TOXIC ],
    [ 32, Moves.MOONBLAST ],
    [ 38, Moves.GRASSY_TERRAIN ],
    [ 44, Moves.MOONLIGHT ],
    [ 50, Moves.PETAL_DANCE ],
  ],
  [Species.VILEPLUME]: [
    [ EVOLVE_MOVE, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.ACID ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.SLEEP_POWDER ],
    [ 1, Moves.PETAL_DANCE ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.MOONLIGHT ],
    [ 1, Moves.AROMATHERAPY ],
    [ 1, Moves.GRASSY_TERRAIN ],
    [ 1, Moves.MOONBLAST ],
  ],
  [Species.PARAS]: [
    [ 1, Moves.SCRATCH ],
    [ 6, Moves.POISON_POWDER ],
    [ 6, Moves.STUN_SPORE ],
    [ 11, Moves.ABSORB ],
    [ 17, Moves.FURY_CUTTER ],
    [ 22, Moves.SPORE ],
    [ 27, Moves.SLASH ],
    [ 33, Moves.GROWTH ],
    [ 38, Moves.GIGA_DRAIN ],
    [ 43, Moves.AROMATHERAPY ],
    [ 49, Moves.RAGE_POWDER ],
    [ 54, Moves.X_SCISSOR ],
  ],
  [Species.PARASECT]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.CROSS_POISON ],
    [ 6, Moves.POISON_POWDER ],
    [ 6, Moves.STUN_SPORE ],
    [ 11, Moves.ABSORB ],
    [ 17, Moves.FURY_CUTTER ],
    [ 22, Moves.SPORE ],
    [ 29, Moves.SLASH ],
    [ 37, Moves.GROWTH ],
    [ 44, Moves.GIGA_DRAIN ],
    [ 51, Moves.AROMATHERAPY ],
    [ 59, Moves.RAGE_POWDER ],
    [ 66, Moves.X_SCISSOR ],
  ],
  [Species.VENONAT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DISABLE ],
    [ 5, Moves.SUPERSONIC ],
    [ 11, Moves.CONFUSION ],
    [ 13, Moves.POISON_POWDER ],
    [ 17, Moves.PSYBEAM ],
    [ 23, Moves.STUN_SPORE ],
    [ 25, Moves.BUG_BUZZ ],
    [ 29, Moves.SLEEP_POWDER ],
    [ 35, Moves.LEECH_LIFE ],
    [ 37, Moves.ZEN_HEADBUTT ],
    [ 41, Moves.POISON_FANG ],
    [ 47, Moves.PSYCHIC ],
  ],
  [Species.VENOMOTH]: [
    [ EVOLVE_MOVE, Moves.AIR_SLASH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.QUIVER_DANCE ],
    [ 1, Moves.SILVER_WIND ],
    [ 11, Moves.CONFUSION ],
    [ 13, Moves.POISON_POWDER ],
    [ 17, Moves.PSYBEAM ],
    [ 23, Moves.STUN_SPORE ],
    [ 25, Moves.BUG_BUZZ ],
    [ 29, Moves.SLEEP_POWDER ],
    [ 37, Moves.LEECH_LIFE ],
    [ 41, Moves.ZEN_HEADBUTT ],
    [ 47, Moves.POISON_FANG ],
    [ 55, Moves.PSYCHIC ],
  ],
  [Species.DIGLETT]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 4, Moves.GROWL ],
    [ 8, Moves.ASTONISH ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.BULLDOZE ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.SANDSTORM ],
    [ 32, Moves.DIG ],
    [ 36, Moves.EARTH_POWER ],
    [ 40, Moves.EARTHQUAKE ],
    [ 44, Moves.FISSURE ],
  ],
  [Species.DUGTRIO]: [
    [ EVOLVE_MOVE, Moves.SAND_TOMB ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.ROTOTILLER ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.BULLDOZE ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 24, Moves.SLASH ],
    [ 30, Moves.SANDSTORM ],
    [ 36, Moves.DIG ],
    [ 42, Moves.EARTH_POWER ],
    [ 48, Moves.EARTHQUAKE ],
    [ 54, Moves.FISSURE ],
  ],
  [Species.MEOWTH]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 4, Moves.FEINT ],
    [ 8, Moves.SCRATCH ],
    [ 12, Moves.PAY_DAY ],
    [ 16, Moves.BITE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.ASSURANCE ],
    [ 29, Moves.FURY_SWIPES ],
    [ 32, Moves.SCREECH ],
    [ 36, Moves.SLASH ],
    [ 40, Moves.NASTY_PLOT ],
    [ 44, Moves.PLAY_ROUGH ],
  ],
  [Species.PERSIAN]: [
    [ EVOLVE_MOVE, Moves.POWER_GEM ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.SWITCHEROO ],
    [ 12, Moves.PAY_DAY ],
    [ 16, Moves.BITE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.ASSURANCE ],
    [ 31, Moves.FURY_SWIPES ],
    [ 36, Moves.SCREECH ],
    [ 42, Moves.SLASH ],
    [ 48, Moves.NASTY_PLOT ],
    [ 54, Moves.PLAY_ROUGH ],
  ],
  [Species.PSYDUCK]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_SPORT ],
    [ 3, Moves.WATER_GUN ],
    [ 6, Moves.CONFUSION ],
    [ 9, Moves.FURY_SWIPES ],
    [ 12, Moves.WATER_PULSE ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.ZEN_HEADBUTT ],
    [ 21, Moves.SCREECH ],
    [ 24, Moves.AQUA_TAIL ],
    [ 27, Moves.SOAK ],
    [ 30, Moves.PSYCH_UP ],
    [ 34, Moves.AMNESIA ],
    [ 39, Moves.WONDER_ROOM ],
  ],
  [Species.GOLDUCK]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.WATER_SPORT ],
    [ 1, Moves.ME_FIRST ],
    [ 9, Moves.FURY_SWIPES ],
    [ 12, Moves.WATER_PULSE ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.ZEN_HEADBUTT ],
    [ 21, Moves.SCREECH ],
    [ 24, Moves.AQUA_TAIL ],
    [ 27, Moves.SOAK ],
    [ 30, Moves.PSYCH_UP ],
    [ 36, Moves.AMNESIA ],
    [ 40, Moves.HYDRO_PUMP ],
    [ 45, Moves.WONDER_ROOM ],
  ],
  [Species.MANKEY]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.COVET ],
    [ 5, Moves.FURY_SWIPES ],
    [ 8, Moves.LOW_KICK ],
    [ 12, Moves.SEISMIC_TOSS ],
    [ 17, Moves.SWAGGER ],
    [ 22, Moves.CROSS_CHOP ],
    [ 26, Moves.ASSURANCE ],
    [ 29, Moves.THRASH ],
    [ 33, Moves.CLOSE_COMBAT ],
    [ 36, Moves.SCREECH ],
    [ 40, Moves.STOMPING_TANTRUM ],
    [ 44, Moves.OUTRAGE ],
    [ 48, Moves.FINAL_GAMBIT ],
  ],
  [Species.PRIMEAPE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FLING ],
    [ 5, Moves.FURY_SWIPES ],
    [ 8, Moves.LOW_KICK ],
    [ 15, Moves.SEISMIC_TOSS ],
    [ 17, Moves.SWAGGER ],
    [ 22, Moves.CROSS_CHOP ],
    [ 26, Moves.ASSURANCE ],
    [ 30, Moves.THRASH ],
    [ 35, Moves.RAGE_FIST ],
    [ 39, Moves.CLOSE_COMBAT ],
    [ 44, Moves.SCREECH ],
    [ 48, Moves.STOMPING_TANTRUM ],
    [ 53, Moves.OUTRAGE ],
    [ 57, Moves.FINAL_GAMBIT ],
  ],
  [Species.GROWLITHE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 4, Moves.HOWL ],
    [ 8, Moves.BITE ],
    [ 12, Moves.FLAME_WHEEL ],
    [ 16, Moves.HELPING_HAND ],
    [ 20, Moves.AGILITY ],
    [ 24, Moves.FIRE_FANG ],
    [ 28, Moves.RETALIATE ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.FLAMETHROWER ],
    [ 44, Moves.ROAR ],
    [ 48, Moves.PLAY_ROUGH ],
    [ 52, Moves.REVERSAL ],
    [ 56, Moves.FLARE_BLITZ ],
  ],
  [Species.ARCANINE]: [
    [ EVOLVE_MOVE, Moves.EXTREME_SPEED ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.ROAR ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.FLAME_WHEEL ],
    [ 1, Moves.REVERSAL ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.ODOR_SLEUTH ],
    [ 1, Moves.HOWL ],
    [ 1, Moves.FLARE_BLITZ ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.RETALIATE ],
    [ 1, Moves.PLAY_ROUGH ],
    [ 5, Moves.FLAMETHROWER ],
  ],
  [Species.POLIWAG]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.WATER_SPORT ],
    [ 6, Moves.POUND ],
    [ 12, Moves.MUD_SHOT ],
    [ 18, Moves.BUBBLE_BEAM ],
    [ 24, Moves.RAIN_DANCE ],
    [ 30, Moves.BODY_SLAM ],
    [ 36, Moves.EARTH_POWER ],
    [ 42, Moves.HYDRO_PUMP ],
    [ 48, Moves.BELLY_DRUM ],
    [ 54, Moves.DOUBLE_EDGE ],
  ],
  [Species.POLIWHIRL]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.WATER_SPORT ],
    [ 1, Moves.MUD_SHOT ],
    [ 18, Moves.BUBBLE_BEAM ],
    [ 24, Moves.RAIN_DANCE ],
    [ 32, Moves.BODY_SLAM ],
    [ 40, Moves.EARTH_POWER ],
    [ 48, Moves.HYDRO_PUMP ],
    [ 56, Moves.BELLY_DRUM ],
    [ 66, Moves.DOUBLE_EDGE ],
  ],
  [Species.POLIWRATH]: [
    [ EVOLVE_MOVE, Moves.DYNAMIC_PUNCH ],
    [ 1, Moves.BUBBLE_BEAM ],
    [ 1, Moves.BODY_SLAM ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.WATER_SPORT ],
    [ RELEARN_MOVE, Moves.POUND ],
    [ RELEARN_MOVE, Moves.DOUBLE_EDGE ],
    [ RELEARN_MOVE, Moves.WATER_GUN ],
    [ RELEARN_MOVE, Moves.HYDRO_PUMP ],
    [ RELEARN_MOVE, Moves.BELLY_DRUM ],
    [ RELEARN_MOVE, Moves.RAIN_DANCE ],
    [ RELEARN_MOVE, Moves.MUD_SHOT ],
    [ RELEARN_MOVE, Moves.EARTH_POWER ],
    [ RELEARN_MOVE, Moves.CIRCLE_THROW ],
  ],
  [Species.ABRA]: [
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.CONFUSION ], //Custom
  ],
  [Species.KADABRA]: [
    [ EVOLVE_MOVE, Moves.PSYBEAM ], //LGPE
    [ 1, Moves.DISABLE ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.KINESIS ],
    [ 10, Moves.REFLECT ],
    [ 15, Moves.ALLY_SWITCH ],
    [ 20, Moves.PSYCHO_CUT ],
    [ 25, Moves.RECOVER ],
    [ 30, Moves.PSYSHOCK ],
    [ 35, Moves.PSYCHIC ],
    [ 40, Moves.ROLE_PLAY ],
    [ 45, Moves.FUTURE_SIGHT ],
    [ 50, Moves.CALM_MIND ],
  ],
  [Species.ALAKAZAM]: [
    [ 1, Moves.DISABLE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.KINESIS ],
    [ 5, Moves.PSYBEAM ],
    [ 10, Moves.REFLECT ],
    [ 15, Moves.ALLY_SWITCH ],
    [ 20, Moves.PSYCHO_CUT ],
    [ 25, Moves.RECOVER ],
    [ 30, Moves.PSYSHOCK ],
    [ 35, Moves.PSYCHIC ],
    [ 40, Moves.ROLE_PLAY ],
    [ 45, Moves.FUTURE_SIGHT ],
    [ 50, Moves.CALM_MIND ],
  ],
  [Species.MACHOP]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 4, Moves.FOCUS_ENERGY ],
    [ 8, Moves.REVENGE ],
    [ 12, Moves.LOW_SWEEP ],
    [ 16, Moves.KNOCK_OFF ],
    [ 20, Moves.SCARY_FACE ],
    [ 24, Moves.VITAL_THROW ],
    [ 29, Moves.STRENGTH ],
    [ 32, Moves.DUAL_CHOP ],
    [ 36, Moves.BULK_UP ],
    [ 40, Moves.SEISMIC_TOSS ],
    [ 44, Moves.DYNAMIC_PUNCH ],
    [ 48, Moves.CROSS_CHOP ],
    [ 52, Moves.DOUBLE_EDGE ],
  ],
  [Species.MACHOKE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.REVENGE ],
    [ 1, Moves.KARATE_CHOP ],
    [ 12, Moves.LOW_SWEEP ],
    [ 16, Moves.KNOCK_OFF ],
    [ 20, Moves.SCARY_FACE ],
    [ 24, Moves.VITAL_THROW ],
    [ 31, Moves.STRENGTH ],
    [ 36, Moves.DUAL_CHOP ],
    [ 42, Moves.BULK_UP ],
    [ 48, Moves.SEISMIC_TOSS ],
    [ 54, Moves.DYNAMIC_PUNCH ],
    [ 60, Moves.CROSS_CHOP ],
    [ 66, Moves.DOUBLE_EDGE ],
  ],
  [Species.MACHAMP]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.REVENGE ],
    [ 1, Moves.WIDE_GUARD ],
    [ 1, Moves.KARATE_CHOP ],
    [ 12, Moves.LOW_SWEEP ],
    [ 16, Moves.KNOCK_OFF ],
    [ 20, Moves.SCARY_FACE ],
    [ 24, Moves.VITAL_THROW ],
    [ 31, Moves.STRENGTH ],
    [ 36, Moves.DUAL_CHOP ],
    [ 42, Moves.BULK_UP ],
    [ 48, Moves.SEISMIC_TOSS ],
    [ 54, Moves.DYNAMIC_PUNCH ],
    [ 60, Moves.CROSS_CHOP ],
    [ 66, Moves.DOUBLE_EDGE ],
  ],
  [Species.BELLSPROUT]: [
    [ 1, Moves.VINE_WHIP ],
    [ 7, Moves.GROWTH ],
    [ 11, Moves.WRAP ],
    [ 13, Moves.SLEEP_POWDER ],
    [ 15, Moves.POISON_POWDER ],
    [ 17, Moves.STUN_SPORE ],
    [ 23, Moves.ACID ],
    [ 27, Moves.KNOCK_OFF ],
    [ 29, Moves.SWEET_SCENT ],
    [ 35, Moves.GASTRO_ACID ],
    [ 39, Moves.RAZOR_LEAF ],
    [ 41, Moves.POISON_JAB ],
    [ 47, Moves.SLAM ],
    [ 52, Moves.POWER_WHIP ],
  ],
  [Species.WEEPINBELL]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.GROWTH ],
    [ 13, Moves.SLEEP_POWDER ],
    [ 15, Moves.POISON_POWDER ],
    [ 17, Moves.STUN_SPORE ],
    [ 24, Moves.ACID ],
    [ 29, Moves.KNOCK_OFF ],
    [ 32, Moves.SWEET_SCENT ],
    [ 39, Moves.GASTRO_ACID ],
    [ 44, Moves.RAZOR_LEAF ],
    [ 47, Moves.POISON_JAB ],
    [ 54, Moves.SLAM ],
    [ 58, Moves.POWER_WHIP ],
  ],
  [Species.VICTREEBEL]: [
    [ EVOLVE_MOVE, Moves.LEAF_STORM ],
    [ RELEARN_MOVE, Moves.STOCKPILE ],
    [ RELEARN_MOVE, Moves.SWALLOW ],
    [ RELEARN_MOVE, Moves.SPIT_UP ],
    [ RELEARN_MOVE, Moves.GASTRO_ACID ],
    [ RELEARN_MOVE, Moves.POWER_WHIP ],
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.SLEEP_POWDER ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 44, Moves.LEAF_BLADE ],
  ],
  [Species.TENTACOOL]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.ACID ],
    [ 8, Moves.WRAP ],
    [ 12, Moves.SUPERSONIC ],
    [ 16, Moves.WATER_PULSE ],
    [ 20, Moves.SCREECH ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 28, Moves.HEX ],
    [ 32, Moves.ACID_ARMOR ],
    [ 36, Moves.POISON_JAB ],
    [ 40, Moves.SURF ],
    [ 44, Moves.SLUDGE_WAVE ],
    [ 48, Moves.HYDRO_PUMP ],
  ],
  [Species.TENTACRUEL]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.ACID ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.REFLECT_TYPE ],
    [ 1, Moves.WRING_OUT ],
    [ 12, Moves.SUPERSONIC ],
    [ 16, Moves.WATER_PULSE ],
    [ 20, Moves.SCREECH ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 28, Moves.HEX ],
    [ 34, Moves.ACID_ARMOR ],
    [ 40, Moves.POISON_JAB ],
    [ 46, Moves.SURF ],
    [ 52, Moves.SLUDGE_WAVE ],
    [ 58, Moves.HYDRO_PUMP ],
  ],
  [Species.GEODUDE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 6, Moves.ROCK_POLISH ],
    [ 10, Moves.ROLLOUT ],
    [ 12, Moves.BULLDOZE ],
    [ 16, Moves.ROCK_THROW ],
    [ 18, Moves.SMACK_DOWN ],
    [ 24, Moves.SELF_DESTRUCT ],
    [ 28, Moves.STEALTH_ROCK ],
    [ 30, Moves.ROCK_BLAST ],
    [ 34, Moves.EARTHQUAKE ],
    [ 36, Moves.EXPLOSION ],
    [ 40, Moves.DOUBLE_EDGE ],
    [ 42, Moves.STONE_EDGE ],
  ],
  [Species.GRAVELER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROCK_POLISH ],
    [ 10, Moves.ROLLOUT ],
    [ 12, Moves.BULLDOZE ],
    [ 16, Moves.ROCK_THROW ],
    [ 18, Moves.SMACK_DOWN ],
    [ 24, Moves.SELF_DESTRUCT ],
    [ 30, Moves.STEALTH_ROCK ],
    [ 34, Moves.ROCK_BLAST ],
    [ 40, Moves.EARTHQUAKE ],
    [ 44, Moves.EXPLOSION ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 54, Moves.STONE_EDGE ],
  ],
  [Species.GOLEM]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROCK_POLISH ],
    [ 1, Moves.HEAVY_SLAM ],
    [ 16, Moves.ROCK_THROW ],
    [ 18, Moves.SMACK_DOWN ],
    [ 22, Moves.BULLDOZE ],
    [ 24, Moves.SELF_DESTRUCT ],
    [ 30, Moves.STEALTH_ROCK ],
    [ 34, Moves.ROCK_BLAST ],
    [ 40, Moves.EARTHQUAKE ],
    [ 44, Moves.EXPLOSION ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 54, Moves.STONE_EDGE ],
  ],
  [Species.PONYTA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.TAIL_WHIP ],
    [ 10, Moves.EMBER ],
    [ 15, Moves.FLAME_CHARGE ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.FLAME_WHEEL ],
    [ 30, Moves.STOMP ],
    [ 35, Moves.FIRE_SPIN ],
    [ 41, Moves.TAKE_DOWN ],
    [ 45, Moves.INFERNO ],
    [ 50, Moves.FIRE_BLAST ],
    [ 55, Moves.FLARE_BLITZ ],
  ],
  [Species.RAPIDASH]: [
    [ EVOLVE_MOVE, Moves.SMART_STRIKE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.MEGAHORN ],
    [ 1, Moves.POISON_JAB ],
    [ 15, Moves.FLAME_CHARGE ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.FLAME_WHEEL ],
    [ 30, Moves.STOMP ],
    [ 35, Moves.FIRE_SPIN ],
    [ 43, Moves.TAKE_DOWN ],
    [ 49, Moves.INFERNO ],
    [ 56, Moves.FIRE_BLAST ],
    [ 63, Moves.FLARE_BLITZ ],
  ],
  [Species.SLOWPOKE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CURSE ],
    [ 3, Moves.GROWL ],
    [ 6, Moves.WATER_GUN ],
    [ 9, Moves.YAWN ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SLACK_OFF ],
    [ 36, Moves.PSYCHIC ],
    [ 39, Moves.PSYCH_UP ],
    [ 42, Moves.RAIN_DANCE ],
    [ 45, Moves.HEAL_PULSE ],
  ],
  [Species.SLOWBRO]: [
    [ RELEARN_MOVE, Moves.FUTURE_SIGHT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.CURSE ],
    [ 9, Moves.YAWN ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SLACK_OFF ],
    [ 36, Moves.PSYCHIC ],
    [ 41, Moves.PSYCH_UP ],
    [ 46, Moves.RAIN_DANCE ],
    [ 51, Moves.HEAL_PULSE ],
  ],
  [Species.MAGNEMITE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 4, Moves.SUPERSONIC ],
    [ 8, Moves.THUNDER_WAVE ],
    [ 12, Moves.ELECTRO_BALL ],
    [ 16, Moves.GYRO_BALL ],
    [ 20, Moves.SPARK ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.MAGNET_RISE ],
    [ 32, Moves.FLASH_CANNON ],
    [ 36, Moves.DISCHARGE ],
    [ 40, Moves.METAL_SOUND ],
    [ 44, Moves.LIGHT_SCREEN ],
    [ 48, Moves.LOCK_ON ],
    [ 52, Moves.ZAP_CANNON ],
  ],
  [Species.MAGNETON]: [
    [ EVOLVE_MOVE, Moves.TRI_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 12, Moves.ELECTRO_BALL ],
    [ 16, Moves.GYRO_BALL ],
    [ 20, Moves.SPARK ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.MAGNET_RISE ],
    [ 34, Moves.FLASH_CANNON ],
    [ 40, Moves.DISCHARGE ],
    [ 46, Moves.METAL_SOUND ],
    [ 52, Moves.LIGHT_SCREEN ],
    [ 58, Moves.LOCK_ON ],
    [ 64, Moves.ZAP_CANNON ],
  ],
  [Species.FARFETCHD]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.PECK ],
    [ 5, Moves.LEER ],
    [ 10, Moves.FURY_CUTTER ],
    [ 15, Moves.CUT ],
    [ 20, Moves.AERIAL_ACE ],
    [ 25, Moves.AIR_CUTTER ],
    [ 30, Moves.KNOCK_OFF ],
    [ 35, Moves.FALSE_SWIPE ],
    [ 40, Moves.SLASH ],
    [ 45, Moves.SWORDS_DANCE ],
    [ 50, Moves.AIR_SLASH ],
    [ 55, Moves.LEAF_BLADE ],
    [ 60, Moves.AGILITY ],
    [ 65, Moves.BRAVE_BIRD ],
  ],
  [Species.DODUO]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 9, Moves.FURY_ATTACK ],
    [ 14, Moves.PLUCK ],
    [ 19, Moves.DOUBLE_HIT ],
    [ 23, Moves.AGILITY ],
    [ 27, Moves.UPROAR ],
    [ 30, Moves.ACUPRESSURE ],
    [ 33, Moves.SWORDS_DANCE ],
    [ 36, Moves.DRILL_PECK ],
    [ 39, Moves.ENDEAVOR ],
    [ 43, Moves.THRASH ],
  ],
  [Species.DODRIO]: [
    [ EVOLVE_MOVE, Moves.TRI_ATTACK ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 12, Moves.FURY_ATTACK ],
    [ 15, Moves.PLUCK ],
    [ 19, Moves.DOUBLE_HIT ],
    [ 23, Moves.AGILITY ],
    [ 26, Moves.UPROAR ],
    [ 30, Moves.ACUPRESSURE ],
    [ 34, Moves.SWORDS_DANCE ],
    [ 38, Moves.DRILL_PECK ],
    [ 43, Moves.ENDEAVOR ],
    [ 50, Moves.THRASH ],
  ],
  [Species.SEEL]: [
    [ 1, Moves.HEADBUTT ],
    [ 3, Moves.GROWL ],
    [ 7, Moves.CHARM ],
    [ 11, Moves.ICY_WIND ],
    [ 13, Moves.ENCORE ],
    [ 17, Moves.ICE_SHARD ],
    [ 21, Moves.REST ],
    [ 23, Moves.AQUA_RING ],
    [ 27, Moves.AURORA_BEAM ],
    [ 31, Moves.AQUA_JET ],
    [ 33, Moves.BRINE ],
    [ 37, Moves.TAKE_DOWN ],
    [ 41, Moves.DIVE ],
    [ 43, Moves.AQUA_TAIL ],
    [ 47, Moves.ICE_BEAM ],
    [ 51, Moves.SAFEGUARD ],
    [ 53, Moves.SNOWSCAPE ],
  ],
  [Species.DEWGONG]: [
    [ EVOLVE_MOVE, Moves.SHEER_COLD ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ICY_WIND ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.SIGNAL_BEAM ],
    [ 13, Moves.ENCORE ],
    [ 17, Moves.ICE_SHARD ],
    [ 21, Moves.REST ],
    [ 23, Moves.AQUA_RING ],
    [ 27, Moves.AURORA_BEAM ],
    [ 31, Moves.AQUA_JET ],
    [ 33, Moves.BRINE ],
    [ 39, Moves.TAKE_DOWN ],
    [ 45, Moves.DIVE ],
    [ 49, Moves.AQUA_TAIL ],
    [ 55, Moves.ICE_BEAM ],
    [ 61, Moves.SAFEGUARD ],
    [ 65, Moves.SNOWSCAPE ],
  ],
  [Species.GRIMER]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.POISON_GAS ],
    [ 4, Moves.HARDEN ],
    [ 7, Moves.MUD_SLAP ],
    [ 12, Moves.DISABLE ],
    [ 15, Moves.SLUDGE ],
    [ 18, Moves.MUD_SHOT ],
    [ 21, Moves.MINIMIZE ],
    [ 26, Moves.TOXIC ],
    [ 29, Moves.SLUDGE_BOMB ],
    [ 32, Moves.SLUDGE_WAVE ],
    [ 37, Moves.SCREECH ],
    [ 40, Moves.GUNK_SHOT ],
    [ 43, Moves.ACID_ARMOR ],
    [ 46, Moves.BELCH ],
    [ 48, Moves.MEMENTO ],
  ],
  [Species.MUK]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.MUD_SLAP ],
    [ 12, Moves.DISABLE ],
    [ 15, Moves.SLUDGE ],
    [ 18, Moves.MUD_SHOT ],
    [ 21, Moves.MINIMIZE ],
    [ 26, Moves.TOXIC ],
    [ 29, Moves.SLUDGE_BOMB ],
    [ 32, Moves.SLUDGE_WAVE ],
    [ 37, Moves.SCREECH ],
    [ 40, Moves.GUNK_SHOT ],
    [ 46, Moves.ACID_ARMOR ],
    [ 52, Moves.BELCH ],
    [ 57, Moves.MEMENTO ],
  ],
  [Species.SHELLDER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.WITHDRAW ],
    [ 8, Moves.ICE_SHARD ],
    [ 12, Moves.LEER ],
    [ 16, Moves.WHIRLPOOL ],
    [ 20, Moves.SUPERSONIC ],
    [ 24, Moves.AURORA_BEAM ],
    [ 28, Moves.PROTECT ],
    [ 32, Moves.RAZOR_SHELL ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 40, Moves.ICE_BEAM ],
    [ 44, Moves.SHELL_SMASH ],
    [ 48, Moves.HYDRO_PUMP ],
  ],
  [Species.CLOYSTER]: [
    [ EVOLVE_MOVE, Moves.ICICLE_SPEAR ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HYDRO_PUMP ],
    [ 1, Moves.ICE_BEAM ],
    [ 1, Moves.AURORA_BEAM ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SPIKES ],
    [ 1, Moves.WHIRLPOOL ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.TOXIC_SPIKES ],
    [ 1, Moves.ICE_SHARD ],
    [ 1, Moves.SHELL_SMASH ],
    [ 1, Moves.ICICLE_CRASH ],
    [ 5, Moves.RAZOR_SHELL ],
  ],
  [Species.GASTLY]: [
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.LICK ],
    [ 1, Moves.ACID ], //Custom
    [ 4, Moves.HYPNOSIS ],
    [ 8, Moves.MEAN_LOOK ],
    [ 12, Moves.PAYBACK ],
    [ 16, Moves.SPITE ],
    [ 20, Moves.CURSE ],
    [ 24, Moves.HEX ],
    [ 28, Moves.NIGHT_SHADE ],
    [ 32, Moves.SUCKER_PUNCH ],
    [ 36, Moves.DARK_PULSE ],
    [ 40, Moves.SHADOW_BALL ],
    [ 44, Moves.DESTINY_BOND ],
    [ 48, Moves.DREAM_EATER ],
  ],
  [Species.HAUNTER]: [
    [ EVOLVE_MOVE, Moves.SHADOW_PUNCH ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.LICK ],
    [ 1, Moves.MEAN_LOOK ],
    [ 12, Moves.PAYBACK ],
    [ 16, Moves.SPITE ],
    [ 20, Moves.CURSE ],
    [ 24, Moves.HEX ],
    [ 30, Moves.NIGHT_SHADE ],
    [ 36, Moves.SUCKER_PUNCH ],
    [ 42, Moves.DARK_PULSE ],
    [ 48, Moves.SHADOW_BALL ],
    [ 54, Moves.DESTINY_BOND ],
    [ 60, Moves.DREAM_EATER ],
  ],
  [Species.GENGAR]: [
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.LICK ],
    [ 1, Moves.PERISH_SONG ],
    [ 1, Moves.MEAN_LOOK ],
    [ 1, Moves.SHADOW_PUNCH ],
    [ 1, Moves.REFLECT_TYPE ],
    [ 12, Moves.PAYBACK ],
    [ 16, Moves.SPITE ],
    [ 20, Moves.CURSE ],
    [ 24, Moves.HEX ],
    [ 30, Moves.NIGHT_SHADE ],
    [ 36, Moves.SUCKER_PUNCH ],
    [ 42, Moves.DARK_PULSE ],
    [ 48, Moves.SHADOW_BALL ],
    [ 54, Moves.DESTINY_BOND ],
    [ 60, Moves.DREAM_EATER ],
  ],
  [Species.ONIX]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.MUD_SPORT ],
    [ 4, Moves.SMACK_DOWN ],
    [ 8, Moves.ROCK_POLISH ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 16, Moves.CURSE ],
    [ 20, Moves.ROCK_SLIDE ],
    [ 22, Moves.GYRO_BALL ], //Custom, from USUM
    [ 24, Moves.SCREECH ],
    [ 28, Moves.SAND_TOMB ],
    [ 32, Moves.STEALTH_ROCK ],
    [ 36, Moves.SLAM ],
    [ 40, Moves.SANDSTORM ],
    [ 44, Moves.DIG ],
    [ 48, Moves.IRON_TAIL ],
    [ 52, Moves.STONE_EDGE ],
    [ 56, Moves.DOUBLE_EDGE ],
  ],
  [Species.DROWZEE]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.HYPNOSIS ],
    [ 5, Moves.DISABLE ],
    [ 9, Moves.CONFUSION ],
    [ 13, Moves.HEADBUTT ],
    [ 17, Moves.POISON_GAS ],
    [ 21, Moves.PSYBEAM ],
    [ 25, Moves.PSYCH_UP ],
    [ 29, Moves.ZEN_HEADBUTT ],
    [ 33, Moves.SWAGGER ],
    [ 37, Moves.PSYCHIC ],
    [ 41, Moves.NASTY_PLOT ],
    [ 45, Moves.PSYSHOCK ],
    [ 49, Moves.FUTURE_SIGHT ],
  ],
  [Species.HYPNO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.SWITCHEROO ],
    [ 1, Moves.NIGHTMARE ],
    [ 13, Moves.HEADBUTT ],
    [ 17, Moves.POISON_GAS ],
    [ 21, Moves.PSYBEAM ],
    [ 25, Moves.PSYCH_UP ],
    [ 32, Moves.ZEN_HEADBUTT ],
    [ 37, Moves.SWAGGER ],
    [ 42, Moves.PSYCHIC ],
    [ 47, Moves.NASTY_PLOT ],
    [ 51, Moves.PSYSHOCK ],
    [ 56, Moves.FUTURE_SIGHT ],
  ],
  [Species.KRABBY]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.MUD_SPORT ],
    [ 4, Moves.HARDEN ],
    [ 8, Moves.METAL_CLAW ],
    [ 12, Moves.MUD_SHOT ],
    [ 16, Moves.PROTECT ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.STOMP ],
    [ 29, Moves.FLAIL ],
    [ 32, Moves.RAZOR_SHELL ],
    [ 36, Moves.SLAM ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 44, Moves.CRABHAMMER ],
    [ 48, Moves.GUILLOTINE ],
  ],
  [Species.KINGLER]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.HAMMER_ARM ],
    [ 1, Moves.WIDE_GUARD ],
    [ 1, Moves.MUD_SPORT ],
    [ 12, Moves.MUD_SHOT ],
    [ 16, Moves.PROTECT ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.STOMP ],
    [ 31, Moves.FLAIL ],
    [ 36, Moves.RAZOR_SHELL ],
    [ 42, Moves.SLAM ],
    [ 48, Moves.SWORDS_DANCE ],
    [ 54, Moves.CRABHAMMER ],
    [ 60, Moves.GUILLOTINE ],
  ],
  [Species.VOLTORB]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CHARGE ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 6, Moves.EERIE_IMPULSE ],
    [ 9, Moves.SPARK ],
    [ 11, Moves.ROLLOUT ],
    [ 13, Moves.SCREECH ],
    [ 16, Moves.CHARGE_BEAM ],
    [ 20, Moves.SWIFT ],
    [ 22, Moves.ELECTRO_BALL ],
    [ 26, Moves.SELF_DESTRUCT ],
    [ 29, Moves.LIGHT_SCREEN ],
    [ 34, Moves.MAGNET_RISE ],
    [ 37, Moves.DISCHARGE ],
    [ 41, Moves.EXPLOSION ],
    [ 46, Moves.GYRO_BALL ],
    [ 50, Moves.MIRROR_COAT ],
  ],
  [Species.ELECTRODE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.EERIE_IMPULSE ],
    [ 1, Moves.MAGNETIC_FLUX ],
    [ 9, Moves.SPARK ],
    [ 11, Moves.ROLLOUT ],
    [ 13, Moves.SCREECH ],
    [ 16, Moves.CHARGE_BEAM ],
    [ 20, Moves.SWIFT ],
    [ 22, Moves.ELECTRO_BALL ],
    [ 26, Moves.SELF_DESTRUCT ],
    [ 29, Moves.LIGHT_SCREEN ],
    [ 36, Moves.MAGNET_RISE ],
    [ 41, Moves.DISCHARGE ],
    [ 47, Moves.EXPLOSION ],
    [ 54, Moves.GYRO_BALL ],
    [ 58, Moves.MIRROR_COAT ],
  ],
  [Species.EXEGGCUTE]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.BARRAGE ],
    [ 5, Moves.REFLECT ],
    [ 10, Moves.LEECH_SEED ],
    [ 15, Moves.MEGA_DRAIN ],
    [ 20, Moves.CONFUSION ],
    [ 25, Moves.SYNTHESIS ],
    [ 30, Moves.BULLET_SEED ],
    [ 35, Moves.GIGA_DRAIN ],
    [ 40, Moves.EXTRASENSORY ],
    [ 45, Moves.UPROAR ],
    [ 50, Moves.WORRY_SEED ],
    [ 55, Moves.SOLAR_BEAM ],
  ],
  [Species.EXEGGUTOR]: [
    [ EVOLVE_MOVE, Moves.STOMP ],
    [ RELEARN_MOVE, Moves.GROWTH ],
    [ 1, Moves.BARRAGE ],
    [ 1, Moves.SEED_BOMB ],
    [ 1, Moves.PSYSHOCK ],
    [ 1, Moves.WOOD_HAMMER ],
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.SYNTHESIS ],
    [ 1, Moves.BULLET_SEED ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.EXTRASENSORY ],
    [ 1, Moves.UPROAR ],
    [ 1, Moves.WORRY_SEED ],
    [ 1, Moves.SOLAR_BEAM ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.REFLECT ],
    [ 1, Moves.LEECH_SEED ],
  ],
  [Species.CUBONE]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.MUD_SLAP ],
    [ 4, Moves.TAIL_WHIP ],
    [ 8, Moves.FALSE_SWIPE ],
    [ 12, Moves.HEADBUTT ],
    [ 16, Moves.RETALIATE ],
    [ 20, Moves.FLING ],
    [ 24, Moves.STOMPING_TANTRUM ],
    [ 29, Moves.BONE_RUSH ],
    [ 32, Moves.FOCUS_ENERGY ],
    [ 36, Moves.ENDEAVOR ],
    [ 40, Moves.BONEMERANG ],
    [ 44, Moves.THRASH ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.MAROWAK]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 1, Moves.BONE_CLUB ],
    [ 12, Moves.HEADBUTT ],
    [ 16, Moves.RETALIATE ],
    [ 20, Moves.FLING ],
    [ 24, Moves.STOMPING_TANTRUM ],
    [ 31, Moves.BONE_RUSH ],
    [ 36, Moves.FOCUS_ENERGY ],
    [ 42, Moves.ENDEAVOR ],
    [ 48, Moves.BONEMERANG ],
    [ 54, Moves.THRASH ],
    [ 60, Moves.DOUBLE_EDGE ],
  ],
  [Species.HITMONLEE]: [
    [ EVOLVE_MOVE, Moves.BRICK_BREAK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.LOW_SWEEP ],
    [ 1, Moves.JUMP_KICK ],
    [ 1, Moves.ROLLING_KICK ],
    [ 4, Moves.DOUBLE_KICK ],
    [ 8, Moves.LOW_KICK ],
    [ 12, Moves.ENDURE ],
    [ 16, Moves.SUCKER_PUNCH ],
    [ 21, Moves.WIDE_GUARD ],
    [ 24, Moves.BLAZE_KICK ],
    [ 28, Moves.FEINT ],
    [ 32, Moves.MEGA_KICK ],
    [ 36, Moves.CLOSE_COMBAT ],
    [ 40, Moves.REVERSAL ],
    [ 44, Moves.HIGH_JUMP_KICK ],
    [ 50, Moves.AXE_KICK ],
  ],
  [Species.HITMONCHAN]: [
    [ EVOLVE_MOVE, Moves.DRAIN_PUNCH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.PURSUIT ],
    [ 1, Moves.COMET_PUNCH ],
    [ 4, Moves.MACH_PUNCH ],
    [ 8, Moves.VACUUM_WAVE ],
    [ 12, Moves.DETECT ],
    [ 16, Moves.BULLET_PUNCH ],
    [ 21, Moves.QUICK_GUARD ],
    [ 24, Moves.THUNDER_PUNCH ],
    [ 24, Moves.ICE_PUNCH ],
    [ 24, Moves.FIRE_PUNCH ],
    [ 28, Moves.AGILITY ],
    [ 32, Moves.MEGA_PUNCH ],
    [ 36, Moves.CLOSE_COMBAT ],
    [ 40, Moves.COUNTER ],
    [ 44, Moves.FOCUS_PUNCH ],
  ],
  [Species.LICKITUNG]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.LICK ],
    [ 1, Moves.TACKLE ], //Custom
    [ 6, Moves.REST ],
    [ 12, Moves.SUPERSONIC ],
    [ 18, Moves.WRAP ],
    [ 24, Moves.DISABLE ],
    [ 30, Moves.STOMP ],
    [ 32, Moves.ROLLOUT ],
    [ 36, Moves.KNOCK_OFF ],
    [ 42, Moves.SCREECH ],
    [ 48, Moves.SLAM ],
    [ 54, Moves.POWER_WHIP ],
    [ 60, Moves.BELLY_DRUM ],
  ],
  [Species.KOFFING]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POISON_GAS ],
    [ 4, Moves.SMOG ],
    [ 8, Moves.SMOKESCREEN ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.SLUDGE ],
    [ 24, Moves.HAZE ],
    [ 28, Moves.SELF_DESTRUCT ],
    [ 32, Moves.SLUDGE_BOMB ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.BELCH ],
    [ 44, Moves.EXPLOSION ],
    [ 48, Moves.MEMENTO ],
    [ 52, Moves.DESTINY_BOND ],
  ],
  [Species.WEEZING]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_HIT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.HEAT_WAVE ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.SLUDGE ],
    [ 24, Moves.HAZE ],
    [ 28, Moves.SELF_DESTRUCT ],
    [ 32, Moves.SLUDGE_BOMB ],
    [ 38, Moves.TOXIC ],
    [ 44, Moves.BELCH ],
    [ 50, Moves.EXPLOSION ],
    [ 56, Moves.MEMENTO ],
    [ 62, Moves.DESTINY_BOND ],
  ],
  [Species.RHYHORN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.SMACK_DOWN ],
    [ 10, Moves.BULLDOZE ],
    [ 15, Moves.HORN_ATTACK ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.STOMP ],
    [ 30, Moves.ROCK_BLAST ],
    [ 35, Moves.DRILL_RUN ],
    [ 40, Moves.TAKE_DOWN ],
    [ 45, Moves.EARTHQUAKE ],
    [ 50, Moves.STONE_EDGE ],
    [ 55, Moves.MEGAHORN ],
    [ 60, Moves.HORN_DRILL ],
  ],
  [Species.RHYDON]: [
    [ EVOLVE_MOVE, Moves.HAMMER_ARM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.SMACK_DOWN ],
    [ 1, Moves.BULLDOZE ],
    [ 15, Moves.HORN_ATTACK ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.STOMP ],
    [ 30, Moves.ROCK_BLAST ],
    [ 35, Moves.DRILL_RUN ],
    [ 40, Moves.TAKE_DOWN ],
    [ 47, Moves.EARTHQUAKE ],
    [ 54, Moves.STONE_EDGE ],
    [ 61, Moves.MEGAHORN ],
    [ 68, Moves.HORN_DRILL ],
  ],
  [Species.CHANSEY]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 4, Moves.TAIL_WHIP ],
    [ 8, Moves.ECHOED_VOICE ],
    [ 12, Moves.LIFE_DEW ],
    [ 16, Moves.SING ],
    [ 20, Moves.FLING ],
    [ 24, Moves.TAKE_DOWN ],
    [ 28, Moves.HEAL_PULSE ],
    [ 32, Moves.HELPING_HAND ],
    [ 36, Moves.LIGHT_SCREEN ],
    [ 40, Moves.DOUBLE_EDGE ],
    [ 44, Moves.SOFT_BOILED ],
    [ 48, Moves.LAST_RESORT ],
    [ 52, Moves.HEALING_WISH ],
  ],
  [Species.TANGELA]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.CONSTRICT ],
    [ 4, Moves.STUN_SPORE ],
    [ 8, Moves.GROWTH ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.VINE_WHIP ],
    [ 20, Moves.POISON_POWDER ],
    [ 24, Moves.DOUBLE_HIT ],
    [ 28, Moves.KNOCK_OFF ],
    [ 32, Moves.GIGA_DRAIN ],
    [ 34, Moves.ANCIENT_POWER ],
    [ 36, Moves.SLEEP_POWDER ],
    [ 40, Moves.SLAM ],
    [ 44, Moves.TICKLE ],
    [ 48, Moves.POWER_WHIP ],
    [ 52, Moves.INGRAIN ],
    [ 56, Moves.GRASSY_TERRAIN ],
  ],
  [Species.KANGASKHAN]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.COMET_PUNCH ],
    [ 4, Moves.GROWL ],
    [ 8, Moves.FAKE_OUT ],
    [ 12, Moves.BITE ],
    [ 16, Moves.STOMP ],
    [ 20, Moves.FOCUS_ENERGY ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.DOUBLE_HIT ],
    [ 36, Moves.CRUNCH ],
    [ 40, Moves.ENDURE ],
    [ 44, Moves.REVERSAL ],
    [ 48, Moves.OUTRAGE ],
    [ 52, Moves.LAST_RESORT ],
  ],
  [Species.HORSEA]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 5, Moves.SMOKESCREEN ],
    [ 10, Moves.TWISTER ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 20, Moves.DRAGON_BREATH ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.AGILITY ],
    [ 35, Moves.LASER_FOCUS ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 45, Moves.HYDRO_PUMP ],
    [ 50, Moves.DRAGON_DANCE ],
    [ 55, Moves.RAIN_DANCE ],
  ],
  [Species.SEADRA]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.TWISTER ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 20, Moves.DRAGON_BREATH ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.AGILITY ],
    [ 37, Moves.LASER_FOCUS ],
    [ 44, Moves.DRAGON_PULSE ],
    [ 51, Moves.HYDRO_PUMP ],
    [ 58, Moves.DRAGON_DANCE ],
    [ 65, Moves.RAIN_DANCE ],
  ],
  [Species.GOLDEEN]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.PECK ],
    [ 1, Moves.WATER_SPORT ],
    [ 5, Moves.SUPERSONIC ],
    [ 10, Moves.WATER_PULSE ],
    [ 15, Moves.HORN_ATTACK ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.AQUA_RING ],
    [ 30, Moves.FLAIL ],
    [ 35, Moves.WATERFALL ],
    [ 40, Moves.SOAK ],
    [ 45, Moves.MEGAHORN ],
    [ 50, Moves.HORN_DRILL ],
  ],
  [Species.SEAKING]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_SPORT ],
    [ 1, Moves.PECK ],
    [ 1, Moves.WATER_PULSE ],
    [ 15, Moves.HORN_ATTACK ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.AQUA_RING ],
    [ 30, Moves.FLAIL ],
    [ 37, Moves.WATERFALL ],
    [ 44, Moves.SOAK ],
    [ 51, Moves.MEGAHORN ],
    [ 58, Moves.HORN_DRILL ],
  ],
  [Species.STARYU]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 4, Moves.WATER_GUN ],
    [ 8, Moves.CONFUSE_RAY ],
    [ 12, Moves.RAPID_SPIN ],
    [ 16, Moves.MINIMIZE ],
    [ 20, Moves.SWIFT ],
    [ 24, Moves.PSYBEAM ],
    [ 28, Moves.BRINE ],
    [ 32, Moves.LIGHT_SCREEN ],
    [ 36, Moves.POWER_GEM ],
    [ 40, Moves.PSYCHIC ],
    [ 44, Moves.SURF ],
    [ 48, Moves.RECOVER ],
    [ 52, Moves.COSMIC_POWER ],
    [ 56, Moves.HYDRO_PUMP ],
  ],
  [Species.STARMIE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HYDRO_PUMP ],
    [ 1, Moves.SURF ],
    [ 1, Moves.PSYBEAM ],
    [ 1, Moves.PSYCHIC ],
    [ 1, Moves.RECOVER ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.MINIMIZE ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.SPOTLIGHT ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.COSMIC_POWER ],
    [ 1, Moves.BRINE ],
    [ 1, Moves.POWER_GEM ],
  ],
  [Species.MR_MIME]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.POWER_SWAP ],
    [ 1, Moves.GUARD_SWAP ],
    [ 1, Moves.WIDE_GUARD ],
    [ 1, Moves.QUICK_GUARD ],
    [ 1, Moves.BARRIER ],
    [ 12, Moves.CONFUSION ],
    [ 16, Moves.ROLE_PLAY ],
    [ 20, Moves.PROTECT ],
    [ 24, Moves.RECYCLE ],
    [ 28, Moves.PSYBEAM ],
    [ 32, Moves.MIMIC ],
    [ 36, Moves.LIGHT_SCREEN ],
    [ 36, Moves.REFLECT ],
    [ 36, Moves.SAFEGUARD ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 44, Moves.DAZZLING_GLEAM ],
    [ 48, Moves.PSYCHIC ],
    [ 52, Moves.TEETER_DANCE ],
  ],
  [Species.SCYTHER]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.FURY_CUTTER ],
    [ 8, Moves.FALSE_SWIPE ],
    [ 12, Moves.WING_ATTACK ],
    [ 16, Moves.DOUBLE_TEAM ],
    [ 20, Moves.DOUBLE_HIT ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.FOCUS_ENERGY ],
    [ 30, Moves.STEEL_WING ], //Custom
    [ 32, Moves.AGILITY ],
    [ 36, Moves.AIR_SLASH ],
    [ 40, Moves.X_SCISSOR ],
    [ 44, Moves.SWORDS_DANCE ],
  ],
  [Species.JYNX]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.LICK ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.COPYCAT ],
    [ 12, Moves.CONFUSION ],
    [ 16, Moves.COVET ],
    [ 20, Moves.SING ],
    [ 24, Moves.FAKE_TEARS ],
    [ 28, Moves.ICE_PUNCH ],
    [ 34, Moves.PSYCHIC ],
    [ 40, Moves.LOVELY_KISS ],
    [ 46, Moves.MEAN_LOOK ],
    [ 52, Moves.PERISH_SONG ],
    [ 58, Moves.BLIZZARD ],
  ],
  [Species.ELECTABUZZ]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.CHARGE ],
    [ 12, Moves.SWIFT ],
    [ 16, Moves.SHOCK_WAVE ],
    [ 20, Moves.THUNDER_WAVE ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.THUNDER_PUNCH ],
    [ 34, Moves.DISCHARGE ],
    [ 40, Moves.LOW_KICK ],
    [ 46, Moves.THUNDERBOLT ],
    [ 52, Moves.LIGHT_SCREEN ],
    [ 58, Moves.THUNDER ],
    [ 64, Moves.GIGA_IMPACT ],
  ],
  [Species.MAGMAR]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.SMOG ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 16, Moves.FLAME_WHEEL ],
    [ 20, Moves.CONFUSE_RAY ],
    [ 24, Moves.SCARY_FACE ],
    [ 28, Moves.FIRE_PUNCH ],
    [ 34, Moves.LAVA_PLUME ],
    [ 40, Moves.LOW_KICK ],
    [ 46, Moves.FLAMETHROWER ],
    [ 52, Moves.SUNNY_DAY ],
    [ 58, Moves.FIRE_BLAST ],
    [ 64, Moves.HYPER_BEAM ],
  ],
  [Species.PINSIR]: [
    [ 1, Moves.VISE_GRIP ],
    [ 1, Moves.HARDEN ],
    [ 4, Moves.FOCUS_ENERGY ],
    [ 8, Moves.BIND ],
    [ 12, Moves.SEISMIC_TOSS ],
    [ 16, Moves.BUG_BITE ],
    [ 20, Moves.STORM_THROW ],
    [ 24, Moves.DOUBLE_HIT ],
    [ 28, Moves.VITAL_THROW ],
    [ 32, Moves.X_SCISSOR ],
    [ 36, Moves.STRENGTH ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 44, Moves.SUBMISSION ],
    [ 48, Moves.GUILLOTINE ],
    [ 52, Moves.SUPERPOWER ],
  ],
  [Species.TAUROS]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.WORK_UP ],
    [ 10, Moves.PAYBACK ],
    [ 15, Moves.ASSURANCE ],
    [ 20, Moves.HORN_ATTACK ],
    [ 25, Moves.SCARY_FACE ],
    [ 30, Moves.ZEN_HEADBUTT ],
    [ 35, Moves.RAGING_BULL ],
    [ 40, Moves.REST ],
    [ 45, Moves.SWAGGER ],
    [ 50, Moves.THRASH ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 60, Moves.GIGA_IMPACT ],
  ],
  [Species.MAGIKARP]: [
    [ 1, Moves.SPLASH ],
    [ 15, Moves.TACKLE ],
    [ 25, Moves.FLAIL ],
  ],
  [Species.GYARADOS]: [
    [ EVOLVE_MOVE, Moves.BITE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.TWISTER ],
    [ 4, Moves.WHIRLPOOL ],
    [ 8, Moves.ICE_FANG ],
    [ 12, Moves.BRINE ],
    [ 16, Moves.SCARY_FACE ],
    [ 21, Moves.WATERFALL ],
    [ 24, Moves.CRUNCH ],
    [ 28, Moves.RAIN_DANCE ],
    [ 32, Moves.AQUA_TAIL ],
    [ 36, Moves.DRAGON_DANCE ],
    [ 40, Moves.HYDRO_PUMP ],
    [ 44, Moves.HURRICANE ],
    [ 48, Moves.THRASH ],
    [ 52, Moves.HYPER_BEAM ],
  ],
  [Species.LAPRAS]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 5, Moves.SING ],
    [ 10, Moves.MIST ],
    [ 15, Moves.LIFE_DEW ],
    [ 20, Moves.ICE_SHARD ],
    [ 25, Moves.CONFUSE_RAY ],
    [ 30, Moves.WATER_PULSE ],
    [ 35, Moves.BRINE ],
    [ 40, Moves.BODY_SLAM ],
    [ 45, Moves.ICE_BEAM ],
    [ 50, Moves.RAIN_DANCE ],
    [ 55, Moves.HYDRO_PUMP ],
    [ 60, Moves.PERISH_SONG ],
    [ 65, Moves.SHEER_COLD ],
  ],
  [Species.DITTO]: [
    [ 1, Moves.TRANSFORM ],
  ],
  [Species.EEVEE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.QUICK_ATTACK ],
    [ 15, Moves.BABY_DOLL_EYES ],
    [ 20, Moves.SWIFT ],
    [ 25, Moves.BITE ],
    [ 30, Moves.COPYCAT ],
    [ 35, Moves.BATON_PASS ],
    [ 40, Moves.TAKE_DOWN ],
    [ 45, Moves.CHARM ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 55, Moves.LAST_RESORT ],
    [ 60, Moves.VEEVEE_VOLLEY ],
  ],
  [Species.VAPOREON]: [
    [ EVOLVE_MOVE, Moves.BOUNCY_BUBBLE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.WATER_GUN ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.HAZE ],
    [ 30, Moves.WATER_PULSE ],
    [ 35, Moves.AURORA_BEAM ],
    [ 40, Moves.AQUA_RING ],
    [ 45, Moves.MUDDY_WATER ],
    [ 50, Moves.ACID_ARMOR ],
    [ 55, Moves.HYDRO_PUMP ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.JOLTEON]: [
    [ EVOLVE_MOVE, Moves.BUZZY_BUZZ ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.THUNDER_SHOCK ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.THUNDER_WAVE ],
    [ 30, Moves.DOUBLE_KICK ],
    [ 35, Moves.THUNDER_FANG ],
    [ 40, Moves.PIN_MISSILE ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.AGILITY ],
    [ 55, Moves.THUNDER ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.FLAREON]: [
    [ EVOLVE_MOVE, Moves.SIZZLY_SLIDE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.EMBER ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.SMOG ],
    [ 30, Moves.BITE ],
    [ 35, Moves.FIRE_FANG ],
    [ 40, Moves.FIRE_SPIN ],
    [ 45, Moves.LAVA_PLUME ],
    [ 50, Moves.SCARY_FACE ],
    [ 55, Moves.FLARE_BLITZ ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.PORYGON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CONVERSION ],
    [ 5, Moves.RECYCLE ],
    [ 10, Moves.MAGNET_RISE ],
    [ 15, Moves.THUNDER_SHOCK ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.CONVERSION_2 ],
    [ 30, Moves.AGILITY ],
    [ 35, Moves.RECOVER ],
    [ 40, Moves.DISCHARGE ],
    [ 45, Moves.TRI_ATTACK ],
    [ 50, Moves.MAGIC_COAT ],
    [ 55, Moves.LOCK_ON ],
    [ 60, Moves.ZAP_CANNON ],
  ],
  [Species.OMANYTE]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.CONSTRICT ],
    [ 5, Moves.ROLLOUT ],
    [ 10, Moves.SAND_ATTACK ],
    [ 15, Moves.WATER_GUN ],
    [ 20, Moves.LEER ],
    [ 25, Moves.MUD_SHOT ],
    [ 30, Moves.ANCIENT_POWER ],
    [ 35, Moves.BRINE ],
    [ 41, Moves.PROTECT ],
    [ 45, Moves.ROCK_BLAST ],
    [ 50, Moves.SURF ],
    [ 55, Moves.SHELL_SMASH ],
    [ 60, Moves.HYDRO_PUMP ],
  ],
  [Species.OMASTAR]: [
    [ EVOLVE_MOVE, Moves.CRUNCH ],
    [ 1, Moves.BIND ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.CONSTRICT ],
    [ 1, Moves.SPIKE_CANNON ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.ROLLOUT ],
    [ 15, Moves.WATER_GUN ],
    [ 20, Moves.LEER ],
    [ 25, Moves.MUD_SHOT ],
    [ 30, Moves.ANCIENT_POWER ],
    [ 35, Moves.BRINE ],
    [ 43, Moves.PROTECT ],
    [ 49, Moves.ROCK_BLAST ],
    [ 56, Moves.SURF ],
    [ 63, Moves.SHELL_SMASH ],
    [ 70, Moves.HYDRO_PUMP ],
  ],
  [Species.KABUTO]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.SCRATCH ],
    [ 10, Moves.SAND_ATTACK ],
    [ 15, Moves.AQUA_JET ],
    [ 20, Moves.LEER ],
    [ 25, Moves.MUD_SHOT ],
    [ 30, Moves.ANCIENT_POWER ],
    [ 35, Moves.BRINE ],
    [ 41, Moves.PROTECT ],
    [ 45, Moves.LEECH_LIFE ],
    [ 50, Moves.LIQUIDATION ],
    [ 55, Moves.METAL_SOUND ],
    [ 60, Moves.STONE_EDGE ],
  ],
  [Species.KABUTOPS]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 15, Moves.AQUA_JET ],
    [ 20, Moves.LEER ],
    [ 25, Moves.MUD_SHOT ],
    [ 30, Moves.ANCIENT_POWER ],
    [ 35, Moves.BRINE ],
    [ 43, Moves.PROTECT ],
    [ 49, Moves.LEECH_LIFE ],
    [ 56, Moves.LIQUIDATION ],
    [ 63, Moves.METAL_SOUND ],
    [ 70, Moves.STONE_EDGE ],
  ],
  [Species.AERODACTYL]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 5, Moves.SUPERSONIC ],
    [ 10, Moves.WING_ATTACK ],
    [ 15, Moves.SCARY_FACE ],
    [ 20, Moves.ROCK_SLIDE ],
    [ 25, Moves.ROAR ],
    [ 30, Moves.CRUNCH ],
    [ 35, Moves.IRON_HEAD ],
    [ 40, Moves.TAKE_DOWN ],
    [ 45, Moves.STONE_EDGE ],
    [ 50, Moves.AGILITY ],
    [ 55, Moves.HYPER_BEAM ],
    [ 60, Moves.GIGA_IMPACT ],
  ],
  [Species.SNORLAX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SCREECH ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.METRONOME ],
    [ 1, Moves.LICK ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.STOCKPILE ],
    [ 1, Moves.SWALLOW ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.BLOCK ],
    [ 1, Moves.COVET ],
    [ 1, Moves.FLING ],
    [ 1, Moves.LAST_RESORT ],
    [ 12, Moves.YAWN ],
    [ 16, Moves.BITE ],
    [ 20, Moves.REST ],
    [ 20, Moves.SNORE ],
    [ 20, Moves.SLEEP_TALK ],
    [ 24, Moves.CRUNCH ],
    [ 28, Moves.BODY_SLAM ],
    [ 32, Moves.HEAVY_SLAM ],
    [ 36, Moves.AMNESIA ],
    [ 40, Moves.HIGH_HORSEPOWER ],
    [ 44, Moves.HAMMER_ARM ],
    [ 48, Moves.BELLY_DRUM ],
    [ 52, Moves.BELCH ],
    [ 56, Moves.GIGA_IMPACT ],
  ],
  [Species.ARTICUNO]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.MIST ],
    [ 5, Moves.POWDER_SNOW ],
    [ 10, Moves.REFLECT ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.ANCIENT_POWER ],
    [ 30, Moves.TAILWIND ],
    [ 35, Moves.FREEZE_DRY ],
    [ 40, Moves.ROOST ],
    [ 45, Moves.ICE_BEAM ],
    [ 50, Moves.SNOWSCAPE ],
    [ 55, Moves.HURRICANE ],
    [ 60, Moves.HAZE ],
    [ 65, Moves.BLIZZARD ],
    [ 70, Moves.SHEER_COLD ],
  ],
  [Species.ZAPDOS]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 5, Moves.THUNDER_SHOCK ],
    [ 10, Moves.LIGHT_SCREEN ],
    [ 15, Moves.PLUCK ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.ANCIENT_POWER ],
    [ 30, Moves.CHARGE ],
    [ 35, Moves.DRILL_PECK ],
    [ 40, Moves.ROOST ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.RAIN_DANCE ],
    [ 55, Moves.THUNDER ],
    [ 60, Moves.DETECT ],
    [ 65, Moves.MAGNETIC_FLUX ],
    [ 70, Moves.ZAP_CANNON ],
  ],
  [Species.MOLTRES]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 5, Moves.EMBER ],
    [ 10, Moves.SAFEGUARD ],
    [ 15, Moves.WING_ATTACK ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.ANCIENT_POWER ],
    [ 30, Moves.INCINERATE ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.ROOST ],
    [ 45, Moves.HEAT_WAVE ],
    [ 50, Moves.SUNNY_DAY ],
    [ 55, Moves.HURRICANE ],
    [ 60, Moves.ENDURE ],
    [ 65, Moves.OVERHEAT ],
    [ 70, Moves.SKY_ATTACK ],
  ],
  [Species.DRATINI]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.LEER ],
    [ 5, Moves.TWISTER ],
    [ 10, Moves.THUNDER_WAVE ],
    [ 15, Moves.DRAGON_TAIL ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.SLAM ],
    [ 31, Moves.AQUA_TAIL ],
    [ 35, Moves.DRAGON_RUSH ],
    [ 40, Moves.SAFEGUARD ],
    [ 45, Moves.RAIN_DANCE ],
    [ 50, Moves.DRAGON_DANCE ],
    [ 55, Moves.OUTRAGE ],
    [ 60, Moves.HYPER_BEAM ],
  ],
  [Species.DRAGONAIR]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.TWISTER ],
    [ 15, Moves.DRAGON_TAIL ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.SLAM ],
    [ 33, Moves.AQUA_TAIL ],
    [ 39, Moves.DRAGON_RUSH ],
    [ 46, Moves.SAFEGUARD ],
    [ 53, Moves.RAIN_DANCE ],
    [ 60, Moves.DRAGON_DANCE ],
    [ 67, Moves.OUTRAGE ],
    [ 74, Moves.HYPER_BEAM ],
  ],
  [Species.DRAGONITE]: [
    [ EVOLVE_MOVE, Moves.HURRICANE ],
    [ 1, Moves.FIRE_PUNCH ],
    [ 1, Moves.THUNDER_PUNCH ],
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.TWISTER ],
    [ 1, Moves.EXTREME_SPEED ],
    [ 1, Moves.ROOST ],
    [ 15, Moves.DRAGON_TAIL ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.SLAM ],
    [ 33, Moves.AQUA_TAIL ],
    [ 39, Moves.DRAGON_RUSH ],
    [ 41, Moves.OUTRAGE ],
    [ 46, Moves.SAFEGUARD ],
    [ 53, Moves.RAIN_DANCE ],
    [ 62, Moves.DRAGON_DANCE ],
    [ 80, Moves.HYPER_BEAM ],
  ],
  [Species.MEWTWO]: [
    [ 1, Moves.DISABLE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.LIFE_DEW ],
    [ 8, Moves.ANCIENT_POWER ],
    [ 16, Moves.PSYCHO_CUT ],
    [ 24, Moves.SAFEGUARD ],
    [ 32, Moves.AMNESIA ],
    [ 40, Moves.AURA_SPHERE ],
    [ 48, Moves.PSYCHIC ],
    [ 56, Moves.POWER_SWAP ],
    [ 56, Moves.GUARD_SWAP ],
    [ 64, Moves.MIST ],
    [ 72, Moves.PSYSTRIKE ],
    [ 80, Moves.RECOVER ],
    [ 88, Moves.FUTURE_SIGHT ],
  ],
  [Species.MEW]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.REFLECT_TYPE ],
    [ 10, Moves.AMNESIA ],
    [ 20, Moves.BATON_PASS ],
    [ 30, Moves.ANCIENT_POWER ],
    [ 40, Moves.LIFE_DEW ],
    [ 50, Moves.NASTY_PLOT ],
    [ 60, Moves.METRONOME ],
    [ 70, Moves.IMPRISON ],
    [ 80, Moves.TRANSFORM ],
    [ 90, Moves.AURA_SPHERE ],
    [ 100, Moves.PSYCHIC ],
  ],
  [Species.CHIKORITA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.RAZOR_LEAF ], //Custom, moved from 6 to 5
    [ 9, Moves.POISON_POWDER ],
    [ 12, Moves.SYNTHESIS ],
    [ 17, Moves.REFLECT ],
    [ 20, Moves.MAGICAL_LEAF ],
    [ 23, Moves.LEECH_SEED ],
    [ 28, Moves.SWEET_SCENT ],
    [ 31, Moves.LIGHT_SCREEN ],
    [ 34, Moves.BODY_SLAM ],
    [ 39, Moves.SAFEGUARD ],
    [ 42, Moves.GIGA_DRAIN ],
    [ 45, Moves.SOLAR_BEAM ],
  ],
  [Species.BAYLEEF]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.POISON_POWDER ],
    [ 12, Moves.SYNTHESIS ],
    [ 18, Moves.REFLECT ],
    [ 22, Moves.MAGICAL_LEAF ],
    [ 26, Moves.LEECH_SEED ],
    [ 32, Moves.SWEET_SCENT ],
    [ 36, Moves.LIGHT_SCREEN ],
    [ 40, Moves.BODY_SLAM ],
    [ 46, Moves.SAFEGUARD ],
    [ 50, Moves.GIGA_DRAIN ],
    [ 54, Moves.SOLAR_BEAM ],
  ],
  [Species.MEGANIUM]: [
    [ EVOLVE_MOVE, Moves.PETAL_DANCE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.PETAL_BLIZZARD ],
    [ 12, Moves.SYNTHESIS ],
    [ 18, Moves.REFLECT ],
    [ 22, Moves.MAGICAL_LEAF ],
    [ 26, Moves.LEECH_SEED ],
    [ 34, Moves.SWEET_SCENT ],
    [ 40, Moves.LIGHT_SCREEN ],
    [ 46, Moves.BODY_SLAM ],
    [ 54, Moves.SAFEGUARD ],
    [ 60, Moves.GIGA_DRAIN ],
    [ 65, Moves.SOLAR_BEAM ],
  ],
  [Species.CYNDAQUIL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 5, Moves.EMBER ], //Custom, moved from 10 to 5
    [ 10, Moves.SMOKESCREEN ], //Custom, moved from 6 to 10
    [ 13, Moves.QUICK_ATTACK ],
    [ 19, Moves.FLAME_WHEEL ],
    [ 22, Moves.DEFENSE_CURL ],
    [ 28, Moves.FLAME_CHARGE ],
    [ 31, Moves.SWIFT ],
    [ 37, Moves.LAVA_PLUME ],
    [ 40, Moves.FLAMETHROWER ],
    [ 46, Moves.INFERNO ],
    [ 49, Moves.ROLLOUT ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 58, Moves.OVERHEAT ],
    [ 64, Moves.ERUPTION ],
  ],
  [Species.QUILAVA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 10, Moves.EMBER ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 20, Moves.FLAME_WHEEL ],
    [ 24, Moves.DEFENSE_CURL ],
    [ 31, Moves.SWIFT ],
    [ 35, Moves.FLAME_CHARGE ],
    [ 42, Moves.LAVA_PLUME ],
    [ 46, Moves.FLAMETHROWER ],
    [ 53, Moves.INFERNO ],
    [ 57, Moves.ROLLOUT ],
    [ 64, Moves.DOUBLE_EDGE ],
    [ 68, Moves.OVERHEAT ],
    [ 75, Moves.ERUPTION ],
  ],
  [Species.TYPHLOSION]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.ERUPTION ],
    [ 1, Moves.GYRO_BALL ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 20, Moves.FLAME_WHEEL ],
    [ 24, Moves.DEFENSE_CURL ],
    [ 31, Moves.SWIFT ],
    [ 35, Moves.FLAME_CHARGE ],
    [ 43, Moves.LAVA_PLUME ],
    [ 48, Moves.FLAMETHROWER ],
    [ 56, Moves.INFERNO ],
    [ 61, Moves.ROLLOUT ],
    [ 74, Moves.OVERHEAT ],
  ],
  [Species.TOTODILE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.WATER_GUN ], //Custom, moved from 6 to 5
    [ 9, Moves.BITE ],
    [ 13, Moves.SCARY_FACE ],
    [ 19, Moves.ICE_FANG ],
    [ 22, Moves.FLAIL ],
    [ 27, Moves.CRUNCH ],
    [ 30, Moves.SLASH ],
    [ 33, Moves.SCREECH ],
    [ 37, Moves.THRASH ],
    [ 41, Moves.AQUA_TAIL ],
    [ 45, Moves.SUPERPOWER ],
    [ 50, Moves.HYDRO_PUMP ],
  ],
  [Species.CROCONAW]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 13, Moves.BITE ],
    [ 15, Moves.SCARY_FACE ],
    [ 21, Moves.ICE_FANG ],
    [ 24, Moves.FLAIL ],
    [ 30, Moves.CRUNCH ],
    [ 34, Moves.SLASH ],
    [ 37, Moves.SCREECH ],
    [ 42, Moves.THRASH ],
    [ 47, Moves.AQUA_TAIL ],
    [ 50, Moves.SUPERPOWER ],
    [ 55, Moves.HYDRO_PUMP ],
  ],
  [Species.FERALIGATR]: [
    [ 1, Moves.AGILITY ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 13, Moves.BITE ],
    [ 15, Moves.SCARY_FACE ],
    [ 21, Moves.ICE_FANG ],
    [ 24, Moves.FLAIL ],
    [ 32, Moves.CRUNCH ],
    [ 37, Moves.SLASH ],
    [ 44, Moves.SCREECH ],
    [ 51, Moves.THRASH ],
    [ 59, Moves.AQUA_TAIL ],
    [ 65, Moves.SUPERPOWER ],
    [ 70, Moves.HYDRO_PUMP ],
  ],
  [Species.SENTRET]: [
    [ 1, Moves.SCRATCH ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 7, Moves.QUICK_ATTACK ],
    [ 13, Moves.FURY_SWIPES ],
    [ 16, Moves.HELPING_HAND ],
    [ 19, Moves.FOLLOW_ME ],
    [ 25, Moves.SLAM ],
    [ 28, Moves.REST ],
    [ 31, Moves.SUCKER_PUNCH ],
    [ 36, Moves.AMNESIA ],
    [ 39, Moves.BATON_PASS ],
    [ 42, Moves.DOUBLE_EDGE ],
    [ 47, Moves.HYPER_VOICE ],
  ],
  [Species.FURRET]: [
    [ EVOLVE_MOVE, Moves.AGILITY ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.COIL ],
    [ 13, Moves.FURY_SWIPES ],
    [ 17, Moves.HELPING_HAND ],
    [ 21, Moves.FOLLOW_ME ],
    [ 28, Moves.SLAM ],
    [ 32, Moves.REST ],
    [ 36, Moves.SUCKER_PUNCH ],
    [ 42, Moves.AMNESIA ],
    [ 46, Moves.BATON_PASS ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 56, Moves.HYPER_VOICE ],
  ],
  [Species.HOOTHOOT]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 3, Moves.TACKLE ],
    [ 6, Moves.ECHOED_VOICE ],
    [ 9, Moves.CONFUSION ],
    [ 12, Moves.REFLECT ],
    [ 15, Moves.DEFOG ],
    [ 18, Moves.AIR_SLASH ],
    [ 21, Moves.EXTRASENSORY ],
    [ 24, Moves.TAKE_DOWN ],
    [ 27, Moves.UPROAR ],
    [ 30, Moves.ROOST ],
    [ 33, Moves.MOONBLAST ],
    [ 36, Moves.HYPNOSIS ],
    [ 39, Moves.DREAM_EATER ],
  ],
  [Species.NOCTOWL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 1, Moves.SKY_ATTACK ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 9, Moves.CONFUSION ],
    [ 12, Moves.REFLECT ],
    [ 15, Moves.DEFOG ],
    [ 18, Moves.AIR_SLASH ],
    [ 23, Moves.EXTRASENSORY ],
    [ 28, Moves.TAKE_DOWN ],
    [ 33, Moves.UPROAR ],
    [ 38, Moves.ROOST ],
    [ 43, Moves.MOONBLAST ],
    [ 48, Moves.HYPNOSIS ],
    [ 53, Moves.DREAM_EATER ],
  ],
  [Species.LEDYBA]: [
    [ 1, Moves.TACKLE ],
    [ 5, Moves.SUPERSONIC ],
    [ 8, Moves.SWIFT ],
    [ 12, Moves.LIGHT_SCREEN ],
    [ 12, Moves.REFLECT ],
    [ 12, Moves.SAFEGUARD ],
    [ 15, Moves.MACH_PUNCH ],
    [ 19, Moves.ROOST ],
    [ 22, Moves.STRUGGLE_BUG ],
    [ 26, Moves.BATON_PASS ],
    [ 29, Moves.AGILITY ],
    [ 33, Moves.BUG_BUZZ ],
    [ 36, Moves.AIR_SLASH ],
    [ 40, Moves.DOUBLE_EDGE ],
  ],
  [Species.LEDIAN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.SWIFT ],
    [ 5, Moves.SUPERSONIC ],
    [ 12, Moves.LIGHT_SCREEN ],
    [ 12, Moves.REFLECT ],
    [ 12, Moves.SAFEGUARD ],
    [ 15, Moves.MACH_PUNCH ],
    [ 20, Moves.ROOST ],
    [ 24, Moves.STRUGGLE_BUG ],
    [ 29, Moves.BATON_PASS ],
    [ 33, Moves.AGILITY ],
    [ 38, Moves.BUG_BUZZ ],
    [ 42, Moves.AIR_SLASH ],
    [ 47, Moves.DOUBLE_EDGE ],
  ],
  [Species.SPINARAK]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.CONSTRICT ],
    [ 5, Moves.ABSORB ],
    [ 8, Moves.INFESTATION ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.NIGHT_SHADE ],
    [ 19, Moves.SHADOW_SNEAK ],
    [ 22, Moves.FURY_SWIPES ],
    [ 26, Moves.SUCKER_PUNCH ],
    [ 29, Moves.AGILITY ],
    [ 33, Moves.PIN_MISSILE ],
    [ 36, Moves.PSYCHIC ],
    [ 40, Moves.POISON_JAB ],
    [ 44, Moves.CROSS_POISON ],
    [ 47, Moves.STICKY_WEB ],
    [ 51, Moves.TOXIC_THREAD ],
  ],
  [Species.ARIADOS]: [
    [ EVOLVE_MOVE, Moves.SWORDS_DANCE ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.CONSTRICT ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.FELL_STINGER ],
    [ 8, Moves.INFESTATION ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.NIGHT_SHADE ],
    [ 19, Moves.SHADOW_SNEAK ],
    [ 23, Moves.FURY_SWIPES ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 31, Moves.AGILITY ],
    [ 35, Moves.PIN_MISSILE ],
    [ 41, Moves.PSYCHIC ],
    [ 46, Moves.POISON_JAB ],
    [ 50, Moves.CROSS_POISON ],
    [ 54, Moves.STICKY_WEB ],
    [ 59, Moves.TOXIC_THREAD ],
  ],
  [Species.CROBAT]: [
    [ EVOLVE_MOVE, Moves.CROSS_POISON ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.SCREECH ],
    [ 1, Moves.MEAN_LOOK ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.TAILWIND ],
    [ 15, Moves.POISON_FANG ],
    [ 20, Moves.QUICK_GUARD ],
    [ 27, Moves.AIR_CUTTER ],
    [ 34, Moves.BITE ],
    [ 41, Moves.HAZE ],
    [ 48, Moves.VENOSHOCK ],
    [ 55, Moves.CONFUSE_RAY ],
    [ 62, Moves.AIR_SLASH ],
    [ 69, Moves.LEECH_LIFE ],
  ],
  [Species.CHINCHOU]: [
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.ELECTRO_BALL ],
    [ 8, Moves.THUNDER_WAVE ],
    [ 12, Moves.BUBBLE_BEAM ],
    [ 16, Moves.CONFUSE_RAY ],
    [ 20, Moves.SPARK ],
    [ 24, Moves.CHARGE ],
    [ 28, Moves.DISCHARGE ],
    [ 32, Moves.AQUA_RING ],
    [ 36, Moves.FLAIL ],
    [ 40, Moves.TAKE_DOWN ],
    [ 44, Moves.HYDRO_PUMP ],
  ],
  [Species.LANTURN]: [
    [ EVOLVE_MOVE, Moves.STOCKPILE ],
    [ EVOLVE_MOVE, Moves.SPIT_UP ],
    [ EVOLVE_MOVE, Moves.SWALLOW ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SPOTLIGHT ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.ELECTRO_BALL ],
    [ 1, Moves.EERIE_IMPULSE ],
    [ 12, Moves.BUBBLE_BEAM ],
    [ 16, Moves.CONFUSE_RAY ],
    [ 20, Moves.SPARK ],
    [ 24, Moves.CHARGE ],
    [ 30, Moves.DISCHARGE ],
    [ 36, Moves.AQUA_RING ],
    [ 42, Moves.FLAIL ],
    [ 48, Moves.TAKE_DOWN ],
    [ 54, Moves.HYDRO_PUMP ],
  ],
  [Species.PICHU]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 4, Moves.PLAY_NICE ],
    [ 8, Moves.SWEET_KISS ],
    [ 12, Moves.NUZZLE ],
    [ 16, Moves.NASTY_PLOT ],
    [ 20, Moves.CHARM ],
  ],
  [Species.CLEFFA]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.COPYCAT ],
    [ 4, Moves.SING ],
    [ 8, Moves.SWEET_KISS ],
    [ 12, Moves.DISARMING_VOICE ],
    [ 16, Moves.ENCORE ],
    [ 20, Moves.CHARM ],
  ],
  [Species.IGGLYBUFF]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.SING ],
    [ 1, Moves.COPYCAT ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 8, Moves.SWEET_KISS ],
    [ 12, Moves.DISARMING_VOICE ],
    [ 16, Moves.DISABLE ],
    [ 20, Moves.CHARM ],
  ],
  [Species.TOGEPI]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.SWEET_KISS ],
    [ 8, Moves.LIFE_DEW ],
    [ 12, Moves.CHARM ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 20, Moves.YAWN ],
    [ 24, Moves.METRONOME ],
    [ 28, Moves.AFTER_YOU ],
    [ 32, Moves.DOUBLE_EDGE ],
    [ 36, Moves.SAFEGUARD ],
    [ 40, Moves.FOLLOW_ME ],
    [ 44, Moves.BATON_PASS ],
    [ 48, Moves.LAST_RESORT ],
    [ 52, Moves.WISH ],
  ],
  [Species.TOGETIC]: [
    [ EVOLVE_MOVE, Moves.FAIRY_WIND ],
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.LIFE_DEW ],
    [ 12, Moves.CHARM ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 20, Moves.YAWN ],
    [ 24, Moves.METRONOME ],
    [ 28, Moves.AFTER_YOU ],
    [ 32, Moves.DOUBLE_EDGE ],
    [ 36, Moves.SAFEGUARD ],
    [ 40, Moves.FOLLOW_ME ],
    [ 44, Moves.BATON_PASS ],
    [ 48, Moves.LAST_RESORT ],
    [ 52, Moves.WISH ],
  ],
  [Species.NATU]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 5, Moves.STORED_POWER ],
    [ 10, Moves.TELEPORT ],
    [ 15, Moves.CONFUSE_RAY ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 26, Moves.PSYCHO_SHIFT ],
    [ 30, Moves.POWER_SWAP ],
    [ 35, Moves.PSYCHIC ],
    [ 35, Moves.GUARD_SWAP ],
    [ 40, Moves.WISH ],
    [ 45, Moves.FUTURE_SIGHT ],
  ],
  [Species.XATU]: [
    [ EVOLVE_MOVE, Moves.AIR_SLASH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.TAILWIND ],
    [ 1, Moves.STORED_POWER ],
    [ 15, Moves.CONFUSE_RAY ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 28, Moves.PSYCHO_SHIFT ],
    [ 34, Moves.POWER_SWAP ],
    [ 34, Moves.GUARD_SWAP ],
    [ 41, Moves.PSYCHIC ],
    [ 48, Moves.WISH ],
    [ 55, Moves.FUTURE_SIGHT ],
  ],
  [Species.MAREEP]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.THUNDER_WAVE ],
    [ 8, Moves.THUNDER_SHOCK ],
    [ 11, Moves.COTTON_SPORE ],
    [ 15, Moves.CHARGE ],
    [ 18, Moves.TAKE_DOWN ],
    [ 22, Moves.ELECTRO_BALL ],
    [ 25, Moves.CONFUSE_RAY ],
    [ 29, Moves.POWER_GEM ],
    [ 32, Moves.DISCHARGE ],
    [ 36, Moves.COTTON_GUARD ],
    [ 39, Moves.DAZZLING_GLEAM ],
    [ 43, Moves.LIGHT_SCREEN ],
    [ 46, Moves.THUNDER ],
  ],
  [Species.FLAAFFY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 6, Moves.THUNDER_SHOCK ],
    [ 9, Moves.THUNDER_WAVE ],
    [ 11, Moves.COTTON_SPORE ],
    [ 16, Moves.CHARGE ],
    [ 20, Moves.TAKE_DOWN ],
    [ 25, Moves.ELECTRO_BALL ],
    [ 29, Moves.CONFUSE_RAY ],
    [ 34, Moves.POWER_GEM ],
    [ 38, Moves.DISCHARGE ],
    [ 43, Moves.COTTON_GUARD ],
    [ 47, Moves.DAZZLING_GLEAM ],
    [ 52, Moves.LIGHT_SCREEN ],
    [ 56, Moves.THUNDER ],
  ],
  [Species.AMPHAROS]: [
    [ EVOLVE_MOVE, Moves.THUNDER_PUNCH ],
    [ 1, Moves.FIRE_PUNCH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.ZAP_CANNON ],
    [ 1, Moves.DRAGON_PULSE ],
    [ 1, Moves.MAGNETIC_FLUX ],
    [ 1, Moves.ION_DELUGE ],
    [ 11, Moves.COTTON_SPORE ],
    [ 16, Moves.CHARGE ],
    [ 20, Moves.TAKE_DOWN ],
    [ 25, Moves.ELECTRO_BALL ],
    [ 29, Moves.CONFUSE_RAY ],
    [ 35, Moves.POWER_GEM ],
    [ 40, Moves.DISCHARGE ],
    [ 46, Moves.COTTON_GUARD ],
    [ 51, Moves.DAZZLING_GLEAM ],
    [ 57, Moves.LIGHT_SCREEN ],
    [ 62, Moves.THUNDER ],
  ],
  [Species.BELLOSSOM]: [
    [ EVOLVE_MOVE, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.ACID ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.SLEEP_POWDER ],
    [ 1, Moves.PETAL_DANCE ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.MOONLIGHT ],
    [ 1, Moves.QUIVER_DANCE ],
    [ 1, Moves.GRASSY_TERRAIN ],
    [ 1, Moves.MOONBLAST ],
  ],
  [Species.MARILL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.HELPING_HAND ],
    [ 6, Moves.BUBBLE_BEAM ],
    [ 9, Moves.CHARM ],
    [ 12, Moves.SLAM ],
    [ 15, Moves.BOUNCE ],
    [ 19, Moves.AQUA_TAIL ],
    [ 21, Moves.PLAY_ROUGH ],
    [ 24, Moves.AQUA_RING ],
    [ 27, Moves.RAIN_DANCE ],
    [ 30, Moves.HYDRO_PUMP ],
    [ 33, Moves.DOUBLE_EDGE ],
    [ 36, Moves.SUPERPOWER ],
  ],
  [Species.AZUMARILL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.HELPING_HAND ],
    [ 6, Moves.BUBBLE_BEAM ],
    [ 9, Moves.CHARM ],
    [ 12, Moves.SLAM ],
    [ 15, Moves.BOUNCE ],
    [ 21, Moves.AQUA_TAIL ],
    [ 25, Moves.PLAY_ROUGH ],
    [ 30, Moves.AQUA_RING ],
    [ 35, Moves.RAIN_DANCE ],
    [ 40, Moves.HYDRO_PUMP ],
    [ 45, Moves.DOUBLE_EDGE ],
    [ 50, Moves.SUPERPOWER ],
  ],
  [Species.SUDOWOODO]: [
    [ EVOLVE_MOVE, Moves.SLAM ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.FAKE_TEARS ],
    [ 1, Moves.HAMMER_ARM ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.STONE_EDGE ],
    [ 1, Moves.WOOD_HAMMER ],
    [ 12, Moves.BLOCK ],
    [ 16, Moves.MIMIC ],
    [ 20, Moves.ROCK_TOMB ],
    [ 24, Moves.TEARFUL_LOOK ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.ROCK_SLIDE ],
    [ 36, Moves.LOW_KICK ],
    [ 40, Moves.COUNTER ],
    [ 44, Moves.DOUBLE_EDGE ],
    [ 48, Moves.HEAD_SMASH ],
  ],
  [Species.POLITOED]: [
    [ EVOLVE_MOVE, Moves.BOUNCE ],
    [ RELEARN_MOVE, Moves.BODY_SLAM ],
    [ RELEARN_MOVE, Moves.DOUBLE_EDGE ],
    [ RELEARN_MOVE, Moves.WATER_GUN ],
    [ RELEARN_MOVE, Moves.BUBBLE_BEAM ],
    [ RELEARN_MOVE, Moves.HYPNOSIS ],
    [ RELEARN_MOVE, Moves.PERISH_SONG ],
    [ RELEARN_MOVE, Moves.SWAGGER ],
    [ RELEARN_MOVE, Moves.HYPER_VOICE ],
    [ RELEARN_MOVE, Moves.MUD_SHOT ],
    [ RELEARN_MOVE, Moves.EARTH_POWER ],
    [ 1, Moves.RAIN_DANCE ],
    [ 1, Moves.HYDRO_PUMP ],
    [ 1, Moves.BELLY_DRUM ],
    [ 1, Moves.POUND ],
  ],
  [Species.HOPPIP]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SPLASH ],
    [ 4, Moves.TAIL_WHIP ],
    [ 6, Moves.ABSORB ],
    [ 8, Moves.FAIRY_WIND ],
    [ 10, Moves.POISON_POWDER ],
    [ 10, Moves.STUN_SPORE ],
    [ 10, Moves.SLEEP_POWDER ],
    [ 12, Moves.BULLET_SEED ],
    [ 15, Moves.SYNTHESIS ],
    [ 19, Moves.LEECH_SEED ],
    [ 22, Moves.MEGA_DRAIN ],
    [ 24, Moves.ACROBATICS ],
    [ 27, Moves.COTTON_SPORE ],
    [ 29, Moves.U_TURN ],
    [ 32, Moves.GIGA_DRAIN ],
    [ 35, Moves.BOUNCE ],
    [ 38, Moves.MEMENTO ],
  ],
  [Species.SKIPLOOM]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.SYNTHESIS ],
    [ 8, Moves.TACKLE ],
    [ 10, Moves.FAIRY_WIND ],
    [ 12, Moves.POISON_POWDER ],
    [ 12, Moves.STUN_SPORE ],
    [ 12, Moves.SLEEP_POWDER ],
    [ 15, Moves.BULLET_SEED ],
    [ 20, Moves.LEECH_SEED ],
    [ 24, Moves.MEGA_DRAIN ],
    [ 28, Moves.ACROBATICS ],
    [ 31, Moves.COTTON_SPORE ],
    [ 34, Moves.U_TURN ],
    [ 37, Moves.GIGA_DRAIN ],
    [ 41, Moves.BOUNCE ],
    [ 44, Moves.MEMENTO ],
  ],
  [Species.JUMPLUFF]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.SYNTHESIS ],
    [ 8, Moves.TACKLE ],
    [ 10, Moves.FAIRY_WIND ],
    [ 12, Moves.POISON_POWDER ],
    [ 12, Moves.STUN_SPORE ],
    [ 12, Moves.SLEEP_POWDER ],
    [ 15, Moves.BULLET_SEED ],
    [ 20, Moves.LEECH_SEED ],
    [ 24, Moves.MEGA_DRAIN ],
    [ 30, Moves.ACROBATICS ],
    [ 35, Moves.COTTON_SPORE ],
    [ 39, Moves.U_TURN ],
    [ 43, Moves.GIGA_DRAIN ],
    [ 49, Moves.BOUNCE ],
    [ 55, Moves.MEMENTO ],
  ],
  [Species.AIPOM]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 4, Moves.SAND_ATTACK ],
    [ 8, Moves.ASTONISH ],
    [ 11, Moves.BATON_PASS ],
    [ 15, Moves.TICKLE ],
    [ 18, Moves.FURY_SWIPES ],
    [ 22, Moves.SWIFT ],
    [ 25, Moves.SCREECH ],
    [ 29, Moves.AGILITY ],
    [ 32, Moves.DOUBLE_HIT ],
    [ 36, Moves.FLING ],
    [ 39, Moves.NASTY_PLOT ],
    [ 43, Moves.LAST_RESORT ],
  ],
  [Species.SUNKERN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWTH ],
    [ 7, Moves.ABSORB ],
    [ 10, Moves.MEGA_DRAIN ],
    [ 16, Moves.RAZOR_LEAF ],
    [ 19, Moves.WORRY_SEED ],
    [ 22, Moves.GIGA_DRAIN ],
    [ 25, Moves.ENDEAVOR ],
    [ 28, Moves.SYNTHESIS ],
    [ 31, Moves.SOLAR_BEAM ],
    [ 34, Moves.DOUBLE_EDGE ],
    [ 36, Moves.SUNNY_DAY ],
    [ 39, Moves.SEED_BOMB ],
  ],
  [Species.SUNFLORA]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWTH ],
    [ 4, Moves.INGRAIN ],
    [ 7, Moves.ABSORB ],
    [ 10, Moves.MEGA_DRAIN ],
    [ 13, Moves.LEECH_SEED ],
    [ 16, Moves.RAZOR_LEAF ],
    [ 19, Moves.WORRY_SEED ],
    [ 22, Moves.GIGA_DRAIN ],
    [ 25, Moves.BULLET_SEED ],
    [ 28, Moves.PETAL_DANCE ],
    [ 31, Moves.SOLAR_BEAM ],
    [ 34, Moves.DOUBLE_EDGE ],
    [ 39, Moves.SUNNY_DAY ],
    [ 43, Moves.LEAF_STORM ],
    [ 50, Moves.PETAL_BLIZZARD ],
  ],
  [Species.YANMA]: [
    [ 1, Moves.TACKLE ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 11, Moves.DOUBLE_TEAM ],
    [ 14, Moves.AIR_CUTTER ],
    [ 17, Moves.DETECT ],
    [ 22, Moves.SUPERSONIC ],
    [ 27, Moves.UPROAR ],
    [ 30, Moves.BUG_BITE ],
    [ 33, Moves.ANCIENT_POWER ],
    [ 38, Moves.HYPNOSIS ],
    [ 43, Moves.WING_ATTACK ],
    [ 46, Moves.SCREECH ],
    [ 49, Moves.U_TURN ],
    [ 54, Moves.AIR_SLASH ],
    [ 57, Moves.BUG_BUZZ ],
  ],
  [Species.WOOPER]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.RAIN_DANCE ],
    [ 8, Moves.MUD_SHOT ],
    [ 12, Moves.MIST ],
    [ 12, Moves.HAZE ],
    [ 16, Moves.SLAM ],
    [ 21, Moves.YAWN ],
    [ 24, Moves.AQUA_TAIL ],
    [ 28, Moves.MUDDY_WATER ],
    [ 32, Moves.AMNESIA ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.EARTHQUAKE ],
  ],
  [Species.QUAGSIRE]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.RAIN_DANCE ],
    [ 1, Moves.MUD_SHOT ],
    [ 12, Moves.MIST ],
    [ 12, Moves.HAZE ],
    [ 16, Moves.SLAM ],
    [ 23, Moves.YAWN ],
    [ 28, Moves.AQUA_TAIL ],
    [ 34, Moves.MUDDY_WATER ],
    [ 40, Moves.AMNESIA ],
    [ 46, Moves.TOXIC ],
    [ 52, Moves.EARTHQUAKE ],
  ],
  [Species.ESPEON]: [
    [ EVOLVE_MOVE, Moves.GLITZY_GLOW ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.CONFUSION ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.SWIFT ],
    [ 30, Moves.PSYBEAM ],
    [ 35, Moves.MORNING_SUN ],
    [ 40, Moves.POWER_SWAP ],
    [ 45, Moves.PSYCHIC ],
    [ 50, Moves.PSYCH_UP ],
    [ 55, Moves.FUTURE_SIGHT ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.UMBREON]: [
    [ EVOLVE_MOVE, Moves.BADDY_BAD ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.PURSUIT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.SNARL ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.CONFUSE_RAY ],
    [ 30, Moves.ASSURANCE ],
    [ 35, Moves.MOONLIGHT ],
    [ 40, Moves.GUARD_SWAP ],
    [ 45, Moves.DARK_PULSE ],
    [ 50, Moves.SCREECH ],
    [ 55, Moves.MEAN_LOOK ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.MURKROW]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.GUST ],
    [ 11, Moves.HAZE ],
    [ 15, Moves.WING_ATTACK ],
    [ 21, Moves.NIGHT_SHADE ],
    [ 25, Moves.ASSURANCE ],
    [ 31, Moves.TAUNT ],
    [ 35, Moves.MEAN_LOOK ],
    [ 40, Moves.FOUL_PLAY ],
    [ 50, Moves.SUCKER_PUNCH ],
    [ 55, Moves.TORMENT ],
    [ 60, Moves.QUASH ],
  ],
  [Species.SLOWKING]: [
    [ RELEARN_MOVE, Moves.FUTURE_SIGHT ],
    [ RELEARN_MOVE, Moves.CHILLY_RECEPTION ],
    [ 1, Moves.POWER_GEM ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.SWAGGER ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CURSE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 9, Moves.YAWN ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEADBUTT ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SLACK_OFF ],
    [ 36, Moves.PSYCHIC ],
    [ 39, Moves.PSYCH_UP ],
    [ 42, Moves.RAIN_DANCE ],
    [ 45, Moves.HEAL_PULSE ],
  ],
  [Species.MISDREAVUS]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.PSYWAVE ],
    [ 10, Moves.ASTONISH ],
    [ 14, Moves.CONFUSE_RAY ],
    [ 19, Moves.MEAN_LOOK ],
    [ 23, Moves.HEX ],
    [ 28, Moves.PSYBEAM ],
    [ 32, Moves.PAIN_SPLIT ],
    [ 37, Moves.PAYBACK ],
    [ 41, Moves.SHADOW_BALL ],
    [ 46, Moves.PERISH_SONG ],
    [ 50, Moves.POWER_GEM ],
  ],
  [Species.UNOWN]: [
    [ 1, Moves.HIDDEN_POWER ],
  ],
  [Species.WOBBUFFET]: [
    [ EVOLVE_MOVE, Moves.COUNTER ],
    [ EVOLVE_MOVE, Moves.DESTINY_BOND ],
    [ EVOLVE_MOVE, Moves.SAFEGUARD ],
    [ EVOLVE_MOVE, Moves.MIRROR_COAT ],
    [ 1, Moves.COUNTER ],
    [ 1, Moves.DESTINY_BOND ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.MIRROR_COAT ],
    [ 1, Moves.AMNESIA ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.ENCORE ],
  ],
  [Species.GIRAFARIG]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.POWER_SWAP ],
    [ 1, Moves.GUARD_SWAP ],
    [ 5, Moves.CONFUSION ],
    [ 10, Moves.ASSURANCE ],
    [ 14, Moves.STOMP ],
    [ 19, Moves.PSYBEAM ],
    [ 23, Moves.AGILITY ],
    [ 28, Moves.DOUBLE_HIT ],
    [ 32, Moves.TWIN_BEAM ],
    [ 37, Moves.CRUNCH ],
    [ 41, Moves.BATON_PASS ],
    [ 46, Moves.NASTY_PLOT ],
    [ 50, Moves.PSYCHIC ],
  ],
  [Species.PINECO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PROTECT ],
    [ 6, Moves.SELF_DESTRUCT ],
    [ 9, Moves.BUG_BITE ],
    [ 12, Moves.TAKE_DOWN ],
    [ 17, Moves.RAPID_SPIN ],
    [ 20, Moves.ROLLOUT ],
    [ 23, Moves.CURSE ],
    [ 28, Moves.SPIKES ],
    [ 31, Moves.PAYBACK ],
    [ 34, Moves.EXPLOSION ],
    [ 39, Moves.IRON_DEFENSE ],
    [ 42, Moves.GYRO_BALL ],
    [ 45, Moves.DOUBLE_EDGE ],
  ],
  [Species.FORRETRESS]: [
    [ EVOLVE_MOVE, Moves.HEAVY_SLAM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SELF_DESTRUCT ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ZAP_CANNON ],
    [ 1, Moves.TOXIC_SPIKES ],
    [ 1, Moves.MAGNET_RISE ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.MIRROR_SHOT ],
    [ 12, Moves.TAKE_DOWN ],
    [ 17, Moves.RAPID_SPIN ],
    [ 20, Moves.ROLLOUT ],
    [ 23, Moves.CURSE ],
    [ 28, Moves.SPIKES ],
    [ 32, Moves.PAYBACK ],
    [ 36, Moves.EXPLOSION ],
    [ 42, Moves.IRON_DEFENSE ],
    [ 46, Moves.GYRO_BALL ],
    [ 50, Moves.DOUBLE_EDGE ],
  ],
  [Species.DUNSPARCE]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.TACKLE ], //Custom
    [ 4, Moves.MUD_SLAP ],
    [ 8, Moves.ROLLOUT ],
    [ 12, Moves.GLARE ],
    [ 16, Moves.SCREECH ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 24, Moves.DRILL_RUN ],
    [ 28, Moves.YAWN ],
    [ 32, Moves.HYPER_DRILL ],
    [ 36, Moves.ROOST ],
    [ 40, Moves.DRAGON_RUSH ],
    [ 44, Moves.COIL ],
    [ 48, Moves.DOUBLE_EDGE ],
    [ 52, Moves.ENDEAVOR ],
  ],
  [Species.GLIGAR]: [
    [ 1, Moves.POISON_STING ],
    [ 4, Moves.SAND_ATTACK ],
    [ 7, Moves.HARDEN ],
    [ 10, Moves.KNOCK_OFF ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 16, Moves.FURY_CUTTER ],
    [ 19, Moves.POISON_TAIL ],
    [ 22, Moves.ACROBATICS ],
    [ 27, Moves.SLASH ],
    [ 30, Moves.U_TURN ],
    [ 35, Moves.SCREECH ],
    [ 40, Moves.X_SCISSOR ],
    [ 45, Moves.CRABHAMMER ],
    [ 50, Moves.SWORDS_DANCE ],
  ],
  [Species.STEELIX]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.MUD_SPORT ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.ROCK_POLISH ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 4, Moves.SMACK_DOWN ],
    [ 8, Moves.AUTOTOMIZE ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 16, Moves.CURSE ],
    [ 20, Moves.ROCK_SLIDE ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.SAND_TOMB ],
    [ 32, Moves.STEALTH_ROCK ],
    [ 36, Moves.SLAM ],
    [ 40, Moves.SANDSTORM ],
    [ 44, Moves.DIG ],
    [ 48, Moves.IRON_TAIL ],
    [ 52, Moves.STONE_EDGE ],
    [ 56, Moves.DOUBLE_EDGE ],
    [ 60, Moves.MAGNET_RISE ],
  ],
  [Species.SNUBBULL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 7, Moves.BITE ],
    [ 13, Moves.LICK ],
    [ 19, Moves.HEADBUTT ],
    [ 25, Moves.ROAR ],
    [ 31, Moves.LAST_RESORT ],
    [ 37, Moves.PLAY_ROUGH ],
    [ 43, Moves.PAYBACK ],
    [ 49, Moves.CRUNCH ],
  ],
  [Species.GRANBULL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.OUTRAGE ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 7, Moves.BITE ],
    [ 13, Moves.LICK ],
    [ 19, Moves.HEADBUTT ],
    [ 27, Moves.ROAR ],
    [ 35, Moves.LAST_RESORT ],
    [ 43, Moves.PLAY_ROUGH ],
    [ 51, Moves.PAYBACK ],
    [ 59, Moves.CRUNCH ],
  ],
  [Species.QWILFISH]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POISON_STING ],
    [ 4, Moves.HARDEN ],
    [ 8, Moves.WATER_GUN ],
    [ 12, Moves.FELL_STINGER ],
    [ 16, Moves.MINIMIZE ],
    [ 20, Moves.SPIKES ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.POISON_JAB ],
    [ 32, Moves.PIN_MISSILE ],
    [ 36, Moves.TOXIC_SPIKES ],
    [ 40, Moves.STOCKPILE ],
    [ 40, Moves.SPIT_UP ],
    [ 44, Moves.TOXIC ],
    [ 48, Moves.AQUA_TAIL ],
    [ 52, Moves.ACUPRESSURE ],
    [ 56, Moves.DESTINY_BOND ],
  ],
  [Species.SCIZOR]: [
    [ EVOLVE_MOVE, Moves.BULLET_PUNCH ],
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.AIR_SLASH ],
    [ 12, Moves.METAL_CLAW ],
    [ 16, Moves.DOUBLE_TEAM ],
    [ 20, Moves.DOUBLE_HIT ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.FOCUS_ENERGY ],
    [ 32, Moves.IRON_DEFENSE ],
    [ 36, Moves.IRON_HEAD ],
    [ 40, Moves.X_SCISSOR ],
    [ 44, Moves.SWORDS_DANCE ],
  ],
  [Species.SHUCKLE]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.CONSTRICT ],
    [ 5, Moves.ROLLOUT ],
    [ 10, Moves.STRUGGLE_BUG ],
    [ 15, Moves.ROCK_THROW ],
    [ 20, Moves.SAFEGUARD ],
    [ 25, Moves.REST ],
    [ 30, Moves.BUG_BITE ],
    [ 35, Moves.GUARD_SPLIT ],
    [ 35, Moves.POWER_SPLIT ],
    [ 40, Moves.ROCK_SLIDE ],
    [ 45, Moves.GASTRO_ACID ],
    [ 50, Moves.STICKY_WEB ],
    [ 55, Moves.POWER_TRICK ],
    [ 60, Moves.STONE_EDGE ],
    [ 65, Moves.SHELL_SMASH ],
  ],
  [Species.HERACROSS]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ARM_THRUST ],
    [ 5, Moves.FURY_ATTACK ],
    [ 10, Moves.ENDURE ],
    [ 15, Moves.AERIAL_ACE ],
    [ 20, Moves.HORN_ATTACK ],
    [ 25, Moves.COUNTER ],
    [ 30, Moves.BRICK_BREAK ],
    [ 35, Moves.PIN_MISSILE ],
    [ 40, Moves.THROAT_CHOP ],
    [ 45, Moves.THRASH ],
    [ 50, Moves.SWORDS_DANCE ],
    [ 55, Moves.MEGAHORN ],
    [ 60, Moves.CLOSE_COMBAT ],
  ],
  [Species.SNEASEL]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 6, Moves.TAUNT ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 18, Moves.METAL_CLAW ],
    [ 24, Moves.ICY_WIND ],
    [ 30, Moves.FURY_SWIPES ],
    [ 36, Moves.HONE_CLAWS ],
    [ 42, Moves.BEAT_UP ],
    [ 48, Moves.AGILITY ],
    [ 54, Moves.SCREECH ],
    [ 60, Moves.SLASH ],
  ],
  [Species.TEDDIURSA]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LICK ],
    [ 1, Moves.COVET ],
    [ 1, Moves.FLING ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 8, Moves.FURY_SWIPES ],
    [ 13, Moves.PAYBACK ],
    [ 17, Moves.SWEET_SCENT ],
    [ 22, Moves.SLASH ],
    [ 25, Moves.PLAY_NICE ],
    [ 29, Moves.PLAY_ROUGH ],
    [ 33, Moves.CHARM ],
    [ 37, Moves.REST ],
    [ 37, Moves.SNORE ],
    [ 41, Moves.THRASH ],
  ],
  [Species.URSARING]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.FAKE_TEARS ],
    [ 1, Moves.COVET ],
    [ 8, Moves.FURY_SWIPES ],
    [ 13, Moves.PAYBACK ],
    [ 17, Moves.SWEET_SCENT ],
    [ 22, Moves.SLASH ],
    [ 25, Moves.PLAY_NICE ],
    [ 29, Moves.PLAY_ROUGH ],
    [ 35, Moves.SCARY_FACE ],
    [ 41, Moves.REST ],
    [ 41, Moves.SNORE ],
    [ 48, Moves.HIGH_HORSEPOWER ],
    [ 56, Moves.THRASH ],
    [ 64, Moves.HAMMER_ARM ],
  ],
  [Species.SLUGMA]: [
    [ 1, Moves.SMOG ],
    [ 1, Moves.YAWN ],
    [ 5, Moves.EMBER ], //Custom, Moved from Level 6 to 5
    [ 8, Moves.ROCK_THROW ],
    [ 13, Moves.HARDEN ],
    [ 20, Moves.CLEAR_SMOG ],
    [ 22, Moves.ANCIENT_POWER ],
    [ 27, Moves.INCINERATE ],
    [ 29, Moves.ROCK_SLIDE ],
    [ 34, Moves.LAVA_PLUME ],
    [ 36, Moves.AMNESIA ],
    [ 41, Moves.BODY_SLAM ],
    [ 43, Moves.RECOVER ],
    [ 48, Moves.FLAMETHROWER ],
    [ 50, Moves.EARTH_POWER ],
  ],
  [Species.MAGCARGO]: [
    [ EVOLVE_MOVE, Moves.SHELL_SMASH ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.EARTH_POWER ],
    [ 13, Moves.HARDEN ],
    [ 20, Moves.CLEAR_SMOG ],
    [ 22, Moves.ANCIENT_POWER ],
    [ 27, Moves.INCINERATE ],
    [ 29, Moves.ROCK_SLIDE ],
    [ 34, Moves.LAVA_PLUME ],
    [ 36, Moves.AMNESIA ],
    [ 43, Moves.BODY_SLAM ],
    [ 47, Moves.RECOVER ],
    [ 54, Moves.FLAMETHROWER ],
  ],
  [Species.SWINUB]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.ODOR_SLEUTH ],
    [ 5, Moves.POWDER_SNOW ],
    [ 10, Moves.FLAIL ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.MIST ],
    [ 25, Moves.ENDURE ],
    [ 30, Moves.ICY_WIND ],
    [ 35, Moves.AMNESIA ],
    [ 40, Moves.TAKE_DOWN ],
    [ 45, Moves.EARTHQUAKE ],
    [ 50, Moves.BLIZZARD ],
  ],
  [Species.PILOSWINE]: [
    [ EVOLVE_MOVE, Moves.ICE_FANG ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.ODOR_SLEUTH ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.MIST ],
    [ 25, Moves.ENDURE ],
    [ 30, Moves.ICY_WIND ],
    [ 37, Moves.AMNESIA ],
    [ 44, Moves.TAKE_DOWN ],
    [ 51, Moves.EARTHQUAKE ],
    [ 58, Moves.BLIZZARD ],
    [ 65, Moves.THRASH ],
  ],
  [Species.CORSOLA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.WATER_GUN ],
    [ 10, Moves.AQUA_RING ],
    [ 15, Moves.ENDURE ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.FLAIL ],
    [ 35, Moves.LIFE_DEW ],
    [ 40, Moves.POWER_GEM ],
    [ 45, Moves.EARTH_POWER ],
    [ 50, Moves.RECOVER ],
    [ 55, Moves.MIRROR_COAT ],
  ],
  [Species.REMORAID]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HELPING_HAND ],
    [ 4, Moves.WATER_PULSE ],
    [ 8, Moves.FOCUS_ENERGY ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.AURORA_BEAM ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.LOCK_ON ],
    [ 28, Moves.BULLET_SEED ],
    [ 32, Moves.ICE_BEAM ],
    [ 36, Moves.HYDRO_PUMP ],
    [ 40, Moves.SOAK ],
    [ 44, Moves.HYPER_BEAM ],
  ],
  [Species.OCTILLERY]: [
    [ EVOLVE_MOVE, Moves.OCTAZOOKA ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.CONSTRICT ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.ROCK_BLAST ],
    [ 1, Moves.WATER_PULSE ],
    [ 1, Moves.GUNK_SHOT ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.AURORA_BEAM ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.LOCK_ON ],
    [ 30, Moves.BULLET_SEED ],
    [ 36, Moves.ICE_BEAM ],
    [ 42, Moves.HYDRO_PUMP ],
    [ 48, Moves.SOAK ],
    [ 54, Moves.HYPER_BEAM ],
  ],
  [Species.DELIBIRD]: [ //Given a custom level up learnset
    [ 1, Moves.PRESENT ],
    [ 1, Moves.METRONOME ],
    [ 5, Moves.FAKE_OUT ],
    [ 5, Moves.POWDER_SNOW ],
    [ 6, Moves.MIST ],
    [ 10, Moves.ICE_SHARD ],
    [ 15, Moves.AERIAL_ACE ],
    [ 20, Moves.ICY_WIND ],
    [ 25, Moves.DRILL_PECK ],
    [ 30, Moves.ICE_PUNCH ],
    [ 35, Moves.HAZE ],
    [ 40, Moves.AIR_SLASH ],
    [ 45, Moves.TAILWIND ],
    [ 50, Moves.SNOWSCAPE ],
    [ 55, Moves.BLIZZARD ],
    [ 60, Moves.BRAVE_BIRD ],
  ],
  [Species.MANTINE]: [
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.PSYBEAM ],
    [ 1, Moves.SIGNAL_BEAM ],
    [ 1, Moves.BULLET_SEED ],
    [ 1, Moves.ROOST ],
    [ 12, Moves.WATER_PULSE ],
    [ 16, Moves.WIDE_GUARD ],
    [ 20, Moves.AGILITY ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 28, Moves.HEADBUTT ],
    [ 32, Moves.AIR_SLASH ],
    [ 36, Moves.AQUA_RING ],
    [ 40, Moves.BOUNCE ],
    [ 44, Moves.TAKE_DOWN ],
    [ 48, Moves.HYDRO_PUMP ],
  ],
  [Species.SKARMORY]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 4, Moves.SAND_ATTACK ],
    [ 8, Moves.FURY_ATTACK ],
    [ 12, Moves.METAL_CLAW ],
    [ 16, Moves.AGILITY ],
    [ 20, Moves.WING_ATTACK ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.STEEL_WING ],
    [ 32, Moves.PAYBACK ],
    [ 36, Moves.DRILL_PECK ],
    [ 40, Moves.METAL_SOUND ],
    [ 44, Moves.SPIKES ],
    [ 48, Moves.IRON_DEFENSE ],
    [ 52, Moves.BRAVE_BIRD ],
  ],
  [Species.HOUNDOUR]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 4, Moves.HOWL ],
    [ 8, Moves.SMOG ],
    [ 13, Moves.ROAR ],
    [ 16, Moves.BITE ],
    [ 20, Moves.INCINERATE ],
    [ 25, Moves.BEAT_UP ],
    [ 28, Moves.FIRE_FANG ],
    [ 32, Moves.TORMENT ],
    [ 37, Moves.COMEUPPANCE ],
    [ 40, Moves.FOUL_PLAY ],
    [ 44, Moves.FLAMETHROWER ],
    [ 49, Moves.CRUNCH ],
    [ 52, Moves.NASTY_PLOT ],
    [ 56, Moves.INFERNO ],
  ],
  [Species.HOUNDOOM]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.HOWL ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.THUNDER_FANG ],
    [ 13, Moves.ROAR ],
    [ 16, Moves.BITE ],
    [ 20, Moves.INCINERATE ],
    [ 26, Moves.BEAT_UP ],
    [ 30, Moves.FIRE_FANG ],
    [ 35, Moves.TORMENT ],
    [ 41, Moves.COMEUPPANCE ],
    [ 45, Moves.FOUL_PLAY ],
    [ 50, Moves.FLAMETHROWER ],
    [ 56, Moves.CRUNCH ],
    [ 62, Moves.INFERNO ],
  ],
  [Species.KINGDRA]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.TWISTER ],
    [ 1, Moves.WHIRLPOOL ],
    [ 1, Moves.YAWN ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 20, Moves.DRAGON_BREATH ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.AGILITY ],
    [ 37, Moves.WATER_PULSE ],
    [ 44, Moves.DRAGON_PULSE ],
    [ 51, Moves.HYDRO_PUMP ],
    [ 58, Moves.DRAGON_DANCE ],
    [ 65, Moves.RAIN_DANCE ],
    [ 72, Moves.WAVE_CRASH ],
  ],
  [Species.PHANPY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ODOR_SLEUTH ],
    [ 6, Moves.FLAIL ],
    [ 10, Moves.ROLLOUT ],
    [ 15, Moves.BULLDOZE ],
    [ 19, Moves.ENDURE ],
    [ 24, Moves.SLAM ],
    [ 28, Moves.TAKE_DOWN ],
    [ 33, Moves.CHARM ],
    [ 37, Moves.LAST_RESORT ],
    [ 42, Moves.DOUBLE_EDGE ],
  ],
  [Species.DONPHAN]: [
    [ EVOLVE_MOVE, Moves.FURY_ATTACK ],
    [ 1, Moves.HORN_ATTACK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.BULLDOZE ],
    [ 6, Moves.RAPID_SPIN ],
    [ 10, Moves.ROLLOUT ],
    [ 15, Moves.ASSURANCE ],
    [ 19, Moves.KNOCK_OFF ],
    [ 24, Moves.SLAM ],
    [ 30, Moves.STOMPING_TANTRUM ],
    [ 37, Moves.SCARY_FACE ],
    [ 43, Moves.EARTHQUAKE ],
    [ 50, Moves.GIGA_IMPACT ],
  ],
  [Species.PORYGON2]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.CONVERSION ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.MAGNET_RISE ],
    [ 15, Moves.THUNDER_SHOCK ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.CONVERSION_2 ],
    [ 30, Moves.AGILITY ],
    [ 35, Moves.RECOVER ],
    [ 40, Moves.DISCHARGE ],
    [ 45, Moves.TRI_ATTACK ],
    [ 50, Moves.LOCK_ON ],
    [ 55, Moves.ZAP_CANNON ],
    [ 60, Moves.HYPER_BEAM ],
  ],
  [Species.STANTLER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ME_FIRST ],
    [ 3, Moves.LEER ],
    [ 7, Moves.ASTONISH ],
    [ 10, Moves.HYPNOSIS ],
    [ 13, Moves.STOMP ],
    [ 16, Moves.SAND_ATTACK ],
    [ 21, Moves.TAKE_DOWN ],
    [ 23, Moves.CONFUSE_RAY ],
    [ 25, Moves.PSYSHIELD_BASH ],
    [ 27, Moves.CALM_MIND ],
    [ 32, Moves.ROLE_PLAY ],
    [ 37, Moves.ZEN_HEADBUTT ],
    [ 49, Moves.IMPRISON ],
    [ 55, Moves.DOUBLE_EDGE ],
  ],
  // Reverting Smeargle back to pre gen9 implementation, to make it less dependent on access to Memory Mushrooms
  [Species.SMEARGLE]: [
    [ 1, Moves.SKETCH ],
    [ 11, Moves.SKETCH ],
    [ 21, Moves.SKETCH ],
    [ 31, Moves.SKETCH ],
    [ 41, Moves.SKETCH ],
    [ 51, Moves.SKETCH ],
    [ 61, Moves.SKETCH ],
    [ 71, Moves.SKETCH ],
    [ 81, Moves.SKETCH ],
    [ 91, Moves.SKETCH ],
  ],
  [Species.TYROGUE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.HELPING_HAND ],
  ],
  [Species.HITMONTOP]: [
    [ EVOLVE_MOVE, Moves.TRIPLE_KICK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.PURSUIT ],
    [ 1, Moves.ROLLING_KICK ],
    [ 4, Moves.QUICK_ATTACK ],
    [ 8, Moves.GYRO_BALL ],
    [ 12, Moves.DETECT ],
    [ 16, Moves.REVENGE ],
    [ 21, Moves.WIDE_GUARD ],
    [ 21, Moves.QUICK_GUARD ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.AGILITY ],
    [ 32, Moves.DIG ],
    [ 36, Moves.CLOSE_COMBAT ],
    [ 40, Moves.COUNTER ],
    [ 44, Moves.ENDEAVOR ],
  ],
  [Species.SMOOCHUM]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.LICK ],
    [ 4, Moves.POWDER_SNOW ],
    [ 8, Moves.COPYCAT ],
    [ 12, Moves.CONFUSION ],
    [ 16, Moves.COVET ],
    [ 20, Moves.SING ],
    [ 24, Moves.FAKE_TEARS ],
    [ 28, Moves.ICE_PUNCH ],
    [ 32, Moves.PSYCHIC ],
    [ 36, Moves.SWEET_KISS ],
    [ 40, Moves.MEAN_LOOK ],
    [ 44, Moves.PERISH_SONG ],
    [ 48, Moves.BLIZZARD ],
  ],
  [Species.ELEKID]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 8, Moves.CHARGE ],
    [ 12, Moves.SWIFT ],
    [ 16, Moves.SHOCK_WAVE ],
    [ 20, Moves.THUNDER_WAVE ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.THUNDER_PUNCH ],
    [ 32, Moves.DISCHARGE ],
    [ 36, Moves.LOW_KICK ],
    [ 40, Moves.THUNDERBOLT ],
    [ 44, Moves.LIGHT_SCREEN ],
    [ 48, Moves.THUNDER ],
  ],
  [Species.MAGBY]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.SMOG ],
    [ 4, Moves.EMBER ],
    [ 8, Moves.SMOKESCREEN ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 16, Moves.FLAME_WHEEL ],
    [ 20, Moves.CONFUSE_RAY ],
    [ 24, Moves.SCARY_FACE ],
    [ 28, Moves.FIRE_PUNCH ],
    [ 32, Moves.LAVA_PLUME ],
    [ 36, Moves.LOW_KICK ],
    [ 40, Moves.FLAMETHROWER ],
    [ 44, Moves.SUNNY_DAY ],
    [ 48, Moves.FIRE_BLAST ],
  ],
  [Species.MILTANK]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.ROLLOUT ],
    [ 10, Moves.DEFENSE_CURL ],
    [ 15, Moves.STOMP ],
    [ 20, Moves.HEAL_BELL ],
    [ 25, Moves.HEADBUTT ],
    [ 30, Moves.ZEN_HEADBUTT ],
    [ 35, Moves.MILK_DRINK ],
    [ 40, Moves.BODY_SLAM ],
    [ 45, Moves.PLAY_ROUGH ],
    [ 50, Moves.CHARM ],
    [ 55, Moves.HIGH_HORSEPOWER ],
  ],
  [Species.BLISSEY]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 4, Moves.TAIL_WHIP ],
    [ 8, Moves.ECHOED_VOICE ],
    [ 12, Moves.LIFE_DEW ],
    [ 16, Moves.SING ],
    [ 20, Moves.FLING ],
    [ 24, Moves.TAKE_DOWN ],
    [ 28, Moves.HEAL_PULSE ],
    [ 32, Moves.HELPING_HAND ],
    [ 36, Moves.LIGHT_SCREEN ],
    [ 40, Moves.DOUBLE_EDGE ],
    [ 44, Moves.SOFT_BOILED ],
    [ 48, Moves.LAST_RESORT ],
    [ 52, Moves.HEALING_WISH ],
  ],
  [Species.RAIKOU]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.EXTREME_SPEED ],
    [ 1, Moves.CHARGE ],
    [ 6, Moves.SPARK ],
    [ 12, Moves.BITE ],
    [ 18, Moves.CALM_MIND ],
    [ 24, Moves.ROAR ],
    [ 30, Moves.THUNDER_FANG ],
    [ 36, Moves.HOWL ],
    [ 42, Moves.CRUNCH ],
    [ 48, Moves.EXTRASENSORY ],
    [ 54, Moves.DISCHARGE ],
    [ 60, Moves.REFLECT ],
    [ 66, Moves.RAIN_DANCE ],
    [ 72, Moves.THUNDER ],
    [ 78, Moves.ZAP_CANNON ],
  ],
  [Species.ENTEI]: [
    [ RELEARN_MOVE, Moves.SACRED_FIRE ],
    [ RELEARN_MOVE, Moves.EXTREME_SPEED ],
    [ 1, Moves.STOMP ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 6, Moves.FLAME_WHEEL ],
    [ 12, Moves.BITE ],
    [ 18, Moves.CALM_MIND ],
    [ 24, Moves.ROAR ],
    [ 30, Moves.FIRE_FANG ],
    [ 36, Moves.SCARY_FACE ],
    [ 42, Moves.CRUNCH ],
    [ 48, Moves.EXTRASENSORY ],
    [ 54, Moves.LAVA_PLUME ],
    [ 60, Moves.SWAGGER ],
    [ 66, Moves.SUNNY_DAY ],
    [ 72, Moves.FIRE_BLAST ],
    [ 78, Moves.ERUPTION ],
  ],
  [Species.SUICUNE]: [
    [ RELEARN_MOVE, Moves.EXTREME_SPEED ],
    [ RELEARN_MOVE, Moves.SHEER_COLD ],
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 1, Moves.MIST ],
    [ 1, Moves.WATER_GUN ],
    [ 6, Moves.WATER_PULSE ],
    [ 12, Moves.BITE ],
    [ 18, Moves.CALM_MIND ],
    [ 24, Moves.ROAR ],
    [ 30, Moves.ICE_FANG ],
    [ 36, Moves.TAILWIND ],
    [ 42, Moves.CRUNCH ],
    [ 48, Moves.EXTRASENSORY ],
    [ 54, Moves.SURF ],
    [ 60, Moves.MIRROR_COAT ],
    [ 66, Moves.RAIN_DANCE ],
    [ 72, Moves.HYDRO_PUMP ],
    [ 78, Moves.BLIZZARD ],
  ],
  [Species.LARVITAR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 3, Moves.ROCK_THROW ],
    [ 6, Moves.PAYBACK ],
    [ 9, Moves.BITE ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.ROCK_SLIDE ],
    [ 18, Moves.STOMPING_TANTRUM ],
    [ 21, Moves.SCREECH ],
    [ 24, Moves.SMACK_DOWN ],
    [ 27, Moves.CRUNCH ],
    [ 31, Moves.EARTHQUAKE ],
    [ 33, Moves.STONE_EDGE ],
    [ 36, Moves.THRASH ],
    [ 39, Moves.SANDSTORM ],
    [ 42, Moves.HYPER_BEAM ],
  ],
  [Species.PUPITAR]: [
    [ EVOLVE_MOVE, Moves.IRON_DEFENSE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.PAYBACK ],
    [ 9, Moves.BITE ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.ROCK_SLIDE ],
    [ 18, Moves.STOMPING_TANTRUM ],
    [ 21, Moves.SCREECH ],
    [ 24, Moves.SMACK_DOWN ],
    [ 27, Moves.CRUNCH ],
    [ 33, Moves.EARTHQUAKE ],
    [ 37, Moves.STONE_EDGE ],
    [ 42, Moves.THRASH ],
    [ 47, Moves.SANDSTORM ],
    [ 52, Moves.HYPER_BEAM ],
  ],
  [Species.TYRANITAR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.PAYBACK ],
    [ 1, Moves.DARK_PULSE ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 9, Moves.BITE ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.ROCK_SLIDE ],
    [ 18, Moves.STOMPING_TANTRUM ],
    [ 21, Moves.SCREECH ],
    [ 24, Moves.SMACK_DOWN ],
    [ 27, Moves.CRUNCH ],
    [ 33, Moves.EARTHQUAKE ],
    [ 37, Moves.STONE_EDGE ],
    [ 42, Moves.THRASH ],
    [ 47, Moves.SANDSTORM ],
    [ 52, Moves.HYPER_BEAM ],
    [ 59, Moves.GIGA_IMPACT ],
  ],
  [Species.LUGIA]: [
    [ RELEARN_MOVE, Moves.DRAGON_RUSH ],
    [ 1, Moves.GUST ],
    [ 1, Moves.WHIRLWIND ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.WEATHER_BALL ],
    [ 9, Moves.MIST ],
    [ 18, Moves.SAFEGUARD ],
    [ 27, Moves.CALM_MIND ],
    [ 36, Moves.EXTRASENSORY ],
    [ 45, Moves.RECOVER ],
    [ 54, Moves.AEROBLAST ],
    [ 63, Moves.RAIN_DANCE ],
    [ 72, Moves.HYDRO_PUMP ],
    [ 81, Moves.FUTURE_SIGHT ],
    [ 90, Moves.SKY_ATTACK ],
  ],
  [Species.HO_OH]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.WHIRLWIND ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.WEATHER_BALL ],
    [ 9, Moves.LIFE_DEW ],
    [ 18, Moves.SAFEGUARD ],
    [ 27, Moves.CALM_MIND ],
    [ 36, Moves.EXTRASENSORY ],
    [ 45, Moves.RECOVER ],
    [ 54, Moves.SACRED_FIRE ],
    [ 63, Moves.SUNNY_DAY ],
    [ 72, Moves.FIRE_BLAST ],
    [ 81, Moves.FUTURE_SIGHT ],
    [ 90, Moves.SKY_ATTACK ],
    [ 99, Moves.OVERHEAT ],
  ],
  [Species.CELEBI]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.HEAL_BELL ],
    [ 10, Moves.MAGICAL_LEAF ],
    [ 20, Moves.BATON_PASS ],
    [ 30, Moves.ANCIENT_POWER ],
    [ 40, Moves.LIFE_DEW ],
    [ 50, Moves.LEECH_SEED ],
    [ 55, Moves.HEAL_BLOCK ],
    [ 60, Moves.RECOVER ],
    [ 70, Moves.FUTURE_SIGHT ],
    [ 80, Moves.HEALING_WISH ],
    [ 90, Moves.LEAF_STORM ],
    [ 100, Moves.PERISH_SONG ],
  ],
  [Species.TREECKO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.LEER ],
    [ 3, Moves.LEAFAGE ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 9, Moves.MEGA_DRAIN ],
    [ 12, Moves.DETECT ],
    [ 15, Moves.QUICK_GUARD ],
    [ 18, Moves.ASSURANCE ],
    [ 21, Moves.GIGA_DRAIN ],
    [ 24, Moves.SLAM ],
    [ 27, Moves.DOUBLE_TEAM ],
    [ 30, Moves.ENERGY_BALL ],
    [ 33, Moves.SCREECH ],
    [ 36, Moves.ENDEAVOR ],
    [ 39, Moves.LEAF_STORM ],
  ],
  [Species.GROVYLE]: [
    [ RELEARN_MOVE, Moves.FALSE_SWIPE ],
    [ RELEARN_MOVE, Moves.FURY_CUTTER ],
    [ RELEARN_MOVE, Moves.X_SCISSOR ],
    [ RELEARN_MOVE, Moves.ENERGY_BALL ],
    [ 1, Moves.POUND ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LEAFAGE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 9, Moves.MEGA_DRAIN ],
    [ 12, Moves.DETECT ],
    [ 15, Moves.QUICK_GUARD ],
    [ 20, Moves.ASSURANCE ],
    [ 25, Moves.GIGA_DRAIN ],
    [ 30, Moves.SLAM ],
    [ 35, Moves.DOUBLE_TEAM ],
    [ 40, Moves.LEAF_BLADE ],
    [ 45, Moves.SCREECH ],
    [ 50, Moves.ENDEAVOR ],
    [ 55, Moves.LEAF_STORM ],
  ],
  [Species.SCEPTILE]: [
    [ EVOLVE_MOVE, Moves.LEAF_BLADE ],
    [ RELEARN_MOVE, Moves.FALSE_SWIPE ],
    [ RELEARN_MOVE, Moves.FURY_CUTTER ],
    [ RELEARN_MOVE, Moves.X_SCISSOR ],
    [ RELEARN_MOVE, Moves.ENERGY_BALL ],
    [ RELEARN_MOVE, Moves.SHED_TAIL ],
    [ 1, Moves.POUND ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LEAFAGE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DUAL_CHOP ],
    [ 5, Moves.MEGA_DRAIN ],
    [ 12, Moves.DETECT ],
    [ 15, Moves.QUICK_GUARD ],
    [ 20, Moves.ASSURANCE ],
    [ 25, Moves.GIGA_DRAIN ],
    [ 30, Moves.SLAM ],
    [ 35, Moves.DOUBLE_TEAM ],
    [ 42, Moves.SCREECH ],
    [ 49, Moves.ENDEAVOR ],
    [ 56, Moves.LEAF_STORM ],
  ],
  [Species.TORCHIC]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.EMBER ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 9, Moves.FLAME_CHARGE ],
    [ 12, Moves.DETECT ],
    [ 15, Moves.SAND_ATTACK ],
    [ 18, Moves.AERIAL_ACE ],
    [ 21, Moves.SLASH ],
    [ 24, Moves.BOUNCE ],
    [ 27, Moves.FOCUS_ENERGY ],
    [ 30, Moves.FLAMETHROWER ],
    [ 33, Moves.FEATHER_DANCE ],
    [ 36, Moves.REVERSAL ],
    [ 39, Moves.FLARE_BLITZ ],
  ],
  [Species.COMBUSKEN]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_KICK ],
    [ RELEARN_MOVE, Moves.FLAMETHROWER ],
    [ RELEARN_MOVE, Moves.FEATHER_DANCE ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 9, Moves.FLAME_CHARGE ],
    [ 12, Moves.DETECT ],
    [ 15, Moves.SAND_ATTACK ],
    [ 20, Moves.AERIAL_ACE ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.BOUNCE ],
    [ 35, Moves.FOCUS_ENERGY ],
    [ 40, Moves.BLAZE_KICK ],
    [ 45, Moves.BULK_UP ],
    [ 50, Moves.REVERSAL ],
    [ 55, Moves.FLARE_BLITZ ],
  ],
  [Species.BLAZIKEN]: [
    [ EVOLVE_MOVE, Moves.BLAZE_KICK ],
    [ RELEARN_MOVE, Moves.FIRE_PUNCH ],
    [ RELEARN_MOVE, Moves.EMBER ],
    [ RELEARN_MOVE, Moves.FLAMETHROWER ],
    [ RELEARN_MOVE, Moves.FEATHER_DANCE ],
    [ 1, Moves.DOUBLE_KICK ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 9, Moves.FLAME_CHARGE ],
    [ 12, Moves.DETECT ],
    [ 15, Moves.SAND_ATTACK ],
    [ 20, Moves.AERIAL_ACE ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.BOUNCE ],
    [ 35, Moves.FOCUS_ENERGY ],
    [ 42, Moves.BULK_UP ],
    [ 49, Moves.REVERSAL ],
    [ 56, Moves.FLARE_BLITZ ],
    [ 63, Moves.BRAVE_BIRD ],
  ],
  [Species.MUDKIP]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.WATER_GUN ],
    [ 6, Moves.ROCK_SMASH ],
    [ 9, Moves.ROCK_THROW ],
    [ 12, Moves.PROTECT ],
    [ 15, Moves.SUPERSONIC ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.ROCK_SLIDE ],
    [ 24, Moves.TAKE_DOWN ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SCREECH ],
    [ 36, Moves.ENDEAVOR ],
    [ 39, Moves.HYDRO_PUMP ],
  ],
  [Species.MARSHTOMP]: [
    [ EVOLVE_MOVE, Moves.MUD_SHOT ],
    [ RELEARN_MOVE, Moves.ROCK_SMASH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 9, Moves.ROCK_THROW ],
    [ 12, Moves.PROTECT ],
    [ 15, Moves.SUPERSONIC ],
    [ 20, Moves.WATER_PULSE ],
    [ 25, Moves.ROCK_SLIDE ],
    [ 30, Moves.TAKE_DOWN ],
    [ 35, Moves.AMNESIA ],
    [ 40, Moves.MUDDY_WATER ],
    [ 45, Moves.SCREECH ],
    [ 50, Moves.ENDEAVOR ],
    [ 55, Moves.HYDRO_PUMP ],
  ],
  [Species.SWAMPERT]: [
    [ RELEARN_MOVE, Moves.SURF ],
    [ RELEARN_MOVE, Moves.EARTHQUAKE ],
    [ RELEARN_MOVE, Moves.ROCK_SMASH ],
    [ RELEARN_MOVE, Moves.HAMMER_ARM ],
    [ 1, Moves.MUD_SHOT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 9, Moves.ROCK_THROW ],
    [ 12, Moves.PROTECT ],
    [ 15, Moves.SUPERSONIC ],
    [ 20, Moves.WATER_PULSE ],
    [ 25, Moves.ROCK_SLIDE ],
    [ 30, Moves.TAKE_DOWN ],
    [ 35, Moves.AMNESIA ],
    [ 42, Moves.MUDDY_WATER ],
    [ 49, Moves.SCREECH ],
    [ 56, Moves.ENDEAVOR ],
    [ 63, Moves.HYDRO_PUMP ],
  ],
  [Species.POOCHYENA]: [
    [ 1, Moves.TACKLE ],
    [ 4, Moves.HOWL ],
    [ 7, Moves.SAND_ATTACK ],
    [ 10, Moves.BITE ],
    [ 13, Moves.LEER ],
    [ 16, Moves.ROAR ],
    [ 19, Moves.SWAGGER ],
    [ 22, Moves.ASSURANCE ],
    [ 25, Moves.SCARY_FACE ],
    [ 28, Moves.TAUNT ],
    [ 31, Moves.CRUNCH ],
    [ 34, Moves.YAWN ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 44, Moves.PLAY_ROUGH ],
  ],
  [Species.MIGHTYENA]: [
    [ EVOLVE_MOVE, Moves.SNARL ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.THIEF ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 13, Moves.LEER ],
    [ 13, Moves.HOWL ],
    [ 16, Moves.ROAR ],
    [ 20, Moves.SWAGGER ],
    [ 24, Moves.ASSURANCE ],
    [ 28, Moves.SCARY_FACE ],
    [ 36, Moves.TAUNT ],
    [ 44, Moves.YAWN ],
    [ 48, Moves.TAKE_DOWN ],
    [ 52, Moves.SUCKER_PUNCH ],
    [ 56, Moves.PLAY_ROUGH ],
  ],
  [Species.ZIGZAGOON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.SAND_ATTACK ],
    [ 6, Moves.TAIL_WHIP ],
    [ 9, Moves.COVET ],
    [ 12, Moves.HEADBUTT ],
    [ 15, Moves.BABY_DOLL_EYES ],
    [ 18, Moves.PIN_MISSILE ],
    [ 21, Moves.REST ],
    [ 24, Moves.TAKE_DOWN ],
    [ 27, Moves.FLING ],
    [ 30, Moves.FLAIL ],
    [ 33, Moves.BELLY_DRUM ],
    [ 36, Moves.DOUBLE_EDGE ],
  ],
  [Species.LINOONE]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.PIN_MISSILE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWITCHEROO ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 1, Moves.ROTOTILLER ],
    [ 9, Moves.COVET ],
    [ 12, Moves.HEADBUTT ],
    [ 15, Moves.HONE_CLAWS ],
    [ 18, Moves.FURY_SWIPES ],
    [ 23, Moves.REST ],
    [ 28, Moves.TAKE_DOWN ],
    [ 33, Moves.FLING ],
    [ 38, Moves.FLAIL ],
    [ 43, Moves.BELLY_DRUM ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.WURMPLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 5, Moves.POISON_STING ],
    [ 15, Moves.BUG_BITE ],
  ],
  [Species.SILCOON]: [
    [ EVOLVE_MOVE, Moves.HARDEN ],
    [ 1, Moves.HARDEN ],
  ],
  [Species.BEAUTIFLY]: [
    [ EVOLVE_MOVE, Moves.GUST ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.GUST ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.POISON_STING ],
    [ 12, Moves.ABSORB ],
    [ 15, Moves.STUN_SPORE ],
    [ 17, Moves.MORNING_SUN ],
    [ 20, Moves.AIR_CUTTER ],
    [ 22, Moves.MEGA_DRAIN ],
    [ 25, Moves.LEECH_LIFE ],
    [ 27, Moves.ATTRACT ],
    [ 30, Moves.WHIRLWIND ],
    [ 32, Moves.GIGA_DRAIN ],
    [ 35, Moves.BUG_BUZZ ],
    [ 37, Moves.PROTECT ],
    [ 40, Moves.QUIVER_DANCE ],
  ],
  [Species.CASCOON]: [
    [ EVOLVE_MOVE, Moves.HARDEN ],
    [ 1, Moves.HARDEN ],
  ],
  [Species.DUSTOX]: [
    [ EVOLVE_MOVE, Moves.GUST ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.GUST ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.POISON_STING ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.POISON_POWDER ],
    [ 17, Moves.MOONLIGHT ],
    [ 20, Moves.VENOSHOCK ],
    [ 22, Moves.PSYBEAM ],
    [ 25, Moves.LEECH_LIFE ],
    [ 27, Moves.LIGHT_SCREEN ],
    [ 30, Moves.WHIRLWIND ],
    [ 32, Moves.TOXIC ],
    [ 35, Moves.BUG_BUZZ ],
    [ 37, Moves.PROTECT ],
    [ 40, Moves.QUIVER_DANCE ],
  ],
  [Species.LOTAD]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.ASTONISH ],
    [ 3, Moves.ABSORB ],
    [ 6, Moves.WATER_GUN ],
    [ 9, Moves.MIST ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.LEECH_SEED ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 33, Moves.RAIN_DANCE ],
    [ 38, Moves.ZEN_HEADBUTT ],
    [ 43, Moves.ENERGY_BALL ],
  ],
  [Species.LOMBRE]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.KNOCK_OFF ],
    [ 1, Moves.TEETER_DANCE ],
    [ 1, Moves.ASTONISH ],
    [ 9, Moves.MIST ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 18, Moves.FURY_SWIPES ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 30, Moves.LEECH_SEED ],
    [ 36, Moves.GIGA_DRAIN ],
    [ 50, Moves.ZEN_HEADBUTT ],
    [ 57, Moves.ENERGY_BALL ],
    [ 64, Moves.HYDRO_PUMP ],
  ],
  [Species.LUDICOLO]: [
    [ RELEARN_MOVE, Moves.GROWL ],
    [ RELEARN_MOVE, Moves.MIST ],
    [ RELEARN_MOVE, Moves.WATER_GUN ],
    [ RELEARN_MOVE, Moves.HYDRO_PUMP ],
    [ RELEARN_MOVE, Moves.ABSORB ],
    [ RELEARN_MOVE, Moves.MEGA_DRAIN ],
    [ RELEARN_MOVE, Moves.FURY_SWIPES ],
    [ RELEARN_MOVE, Moves.FLAIL ],
    [ RELEARN_MOVE, Moves.KNOCK_OFF ],
    [ RELEARN_MOVE, Moves.TEETER_DANCE ],
    [ RELEARN_MOVE, Moves.ASTONISH ],
    [ RELEARN_MOVE, Moves.ENERGY_BALL ],
    [ RELEARN_MOVE, Moves.ZEN_HEADBUTT ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.BUBBLE_BEAM ],
    [ 1, Moves.RAIN_DANCE ],
  ],
  [Species.SEEDOT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.BIDE ],
    [ 3, Moves.ABSORB ],
    [ 6, Moves.ASTONISH ],
    [ 9, Moves.GROWTH ],
    [ 12, Moves.ROLLOUT ],
    [ 15, Moves.MEGA_DRAIN ],
    [ 18, Moves.PAYBACK ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.SUNNY_DAY ],
    [ 27, Moves.SYNTHESIS ],
    [ 30, Moves.SUCKER_PUNCH ],
    [ 33, Moves.EXPLOSION ],
  ],
  [Species.NUZLEAF]: [
    [ EVOLVE_MOVE, Moves.RAZOR_LEAF ],
    [ 1, Moves.AIR_CUTTER ],
    [ 1, Moves.TORMENT ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.SWAGGER ],
    [ 1, Moves.EXPLOSION ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.ASTONISH ],
    [ 9, Moves.GROWTH ],
    [ 12, Moves.ROLLOUT ],
    [ 18, Moves.MEGA_DRAIN ],
    [ 24, Moves.PAYBACK ],
    [ 30, Moves.SYNTHESIS ],
    [ 36, Moves.SUNNY_DAY ],
    [ 43, Moves.EXTRASENSORY ],
    [ 50, Moves.SUCKER_PUNCH ],
    [ 57, Moves.LEAF_BLADE ],
  ],
  [Species.SHIFTRY]: [
    [ EVOLVE_MOVE, Moves.LEAF_BLADE ],
    [ RELEARN_MOVE, Moves.WHIRLWIND ],
    [ RELEARN_MOVE, Moves.TACKLE ],
    [ RELEARN_MOVE, Moves.ABSORB ],
    [ RELEARN_MOVE, Moves.MEGA_DRAIN ],
    [ RELEARN_MOVE, Moves.GROWTH ],
    [ RELEARN_MOVE, Moves.RAZOR_LEAF ],
    [ RELEARN_MOVE, Moves.HARDEN ],
    [ RELEARN_MOVE, Moves.EXPLOSION ],
    [ RELEARN_MOVE, Moves.ROLLOUT ],
    [ RELEARN_MOVE, Moves.SWAGGER ],
    [ RELEARN_MOVE, Moves.SYNTHESIS ],
    [ RELEARN_MOVE, Moves.BEAT_UP ],
    [ RELEARN_MOVE, Moves.FAKE_OUT ],
    [ RELEARN_MOVE, Moves.TORMENT ],
    [ RELEARN_MOVE, Moves.ASTONISH ],
    [ RELEARN_MOVE, Moves.EXTRASENSORY ],
    [ RELEARN_MOVE, Moves.SUCKER_PUNCH ],
    [ 1, Moves.AIR_CUTTER ],
    [ 1, Moves.HURRICANE ],
    [ 1, Moves.PAYBACK ],
    [ 1, Moves.SUNNY_DAY ],
  ],
  [Species.TAILLOW]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.FOCUS_ENERGY ],
    [ 9, Moves.QUICK_ATTACK ],
    [ 13, Moves.WING_ATTACK ],
    [ 17, Moves.DOUBLE_TEAM ],
    [ 21, Moves.AERIAL_ACE ],
    [ 25, Moves.QUICK_GUARD ],
    [ 29, Moves.AGILITY ],
    [ 33, Moves.AIR_SLASH ],
    [ 37, Moves.ENDEAVOR ],
    [ 41, Moves.BRAVE_BIRD ],
    [ 45, Moves.REVERSAL ],
  ],
  [Species.SWELLOW]: [
    [ 1, Moves.BRAVE_BIRD ],
    [ 1, Moves.AIR_SLASH ],
    [ 1, Moves.PLUCK ],
    [ 1, Moves.PECK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 13, Moves.WING_ATTACK ],
    [ 17, Moves.DOUBLE_TEAM ],
    [ 21, Moves.AERIAL_ACE ],
    [ 27, Moves.QUICK_GUARD ],
    [ 33, Moves.AGILITY ],
    [ 45, Moves.ENDEAVOR ],
    [ 51, Moves.BRAVE_BIRD ],
    [ 57, Moves.REVERSAL ],
  ],
  [Species.WINGULL]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 10, Moves.SUPERSONIC ],
    [ 15, Moves.WING_ATTACK ],
    [ 20, Moves.WATER_PULSE ],
    [ 26, Moves.AGILITY ],
    [ 30, Moves.AIR_SLASH ],
    [ 35, Moves.MIST ],
    [ 40, Moves.ROOST ],
    [ 45, Moves.HURRICANE ],
  ],
  [Species.PELIPPER]: [
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SOAK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.AIR_SLASH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.TAILWIND ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_SPORT ],
    [ 15, Moves.WING_ATTACK ],
    [ 20, Moves.WATER_PULSE ],
    [ 28, Moves.STOCKPILE ],
    [ 28, Moves.SPIT_UP ],
    [ 28, Moves.SWALLOW ],
    [ 34, Moves.FLING ],
    [ 41, Moves.MIST ],
    [ 48, Moves.ROOST ],
    [ 55, Moves.HURRICANE ],
    [ 62, Moves.HYDRO_PUMP ],
  ],
  [Species.RALTS]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 3, Moves.DOUBLE_TEAM ],
    [ 6, Moves.CONFUSION ],
    [ 9, Moves.HYPNOSIS ],
    [ 12, Moves.DRAINING_KISS ],
    [ 15, Moves.TELEPORT ],
    [ 18, Moves.PSYBEAM ],
    [ 21, Moves.LIFE_DEW ],
    [ 24, Moves.CHARM ],
    [ 27, Moves.CALM_MIND ],
    [ 30, Moves.PSYCHIC ],
    [ 33, Moves.HEAL_PULSE ],
    [ 36, Moves.DREAM_EATER ],
    [ 39, Moves.FUTURE_SIGHT ],
  ],
  [Species.KIRLIA]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.CONFUSION ],
    [ 9, Moves.HYPNOSIS ],
    [ 12, Moves.DRAINING_KISS ],
    [ 15, Moves.TELEPORT ],
    [ 18, Moves.PSYBEAM ],
    [ 23, Moves.LIFE_DEW ],
    [ 28, Moves.CHARM ],
    [ 33, Moves.CALM_MIND ],
    [ 38, Moves.PSYCHIC ],
    [ 43, Moves.HEAL_PULSE ],
    [ 48, Moves.DREAM_EATER ],
    [ 53, Moves.FUTURE_SIGHT ],
  ],
  [Species.GARDEVOIR]: [
    [ EVOLVE_MOVE, Moves.DAZZLING_GLEAM ],
    [ 1, Moves.MISTY_TERRAIN ],
    [ 1, Moves.HEALING_WISH ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.MYSTICAL_FIRE ],
    [ 1, Moves.HEAL_PULSE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.CONFUSION ],
    [ 9, Moves.HYPNOSIS ],
    [ 12, Moves.DRAINING_KISS ],
    [ 15, Moves.TELEPORT ],
    [ 18, Moves.PSYBEAM ],
    [ 23, Moves.LIFE_DEW ],
    [ 28, Moves.WISH ],
    [ 35, Moves.CALM_MIND ],
    [ 42, Moves.PSYCHIC ],
    [ 49, Moves.MOONBLAST ],
    [ 56, Moves.DREAM_EATER ],
    [ 63, Moves.FUTURE_SIGHT ],
  ],
  [Species.SURSKIT]: [
    [ 1, Moves.WATER_GUN ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 9, Moves.SWEET_SCENT ],
    [ 14, Moves.SOAK ],
    [ 17, Moves.BUBBLE_BEAM ],
    [ 22, Moves.AGILITY ],
    [ 25, Moves.MIST ],
    [ 25, Moves.HAZE ],
    [ 35, Moves.BATON_PASS ],
    [ 38, Moves.STICKY_WEB ],
  ],
  [Species.MASQUERAIN]: [
    [ 1, Moves.WHIRLWIND ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.SOAK ],
    [ 1, Moves.OMINOUS_WIND ],
    [ 17, Moves.GUST ],
    [ 22, Moves.SCARY_FACE ],
    [ 22, Moves.AIR_CUTTER ],
    [ 26, Moves.STUN_SPORE ],
    [ 32, Moves.AIR_SLASH ],
    [ 38, Moves.GIGA_DRAIN ],
    [ 44, Moves.BUG_BUZZ ],
    [ 52, Moves.QUIVER_DANCE ],
  ],
  [Species.SHROOMISH]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.TACKLE ],
    [ 5, Moves.STUN_SPORE ],
    [ 8, Moves.LEECH_SEED ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 15, Moves.HEADBUTT ],
    [ 19, Moves.POISON_POWDER ],
    [ 26, Moves.GIGA_DRAIN ],
    [ 29, Moves.GROWTH ],
    [ 33, Moves.TOXIC ],
    [ 36, Moves.SEED_BOMB ],
    [ 40, Moves.SPORE ],
  ],
  [Species.BRELOOM]: [
    [ EVOLVE_MOVE, Moves.MACH_PUNCH ],
    [ RELEARN_MOVE, Moves.SPORE ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.LEECH_SEED ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 15, Moves.HEADBUTT ],
    [ 19, Moves.FEINT ],
    [ 22, Moves.COUNTER ],
    [ 28, Moves.FORCE_PALM ],
    [ 33, Moves.WORRY_SEED ],
    [ 39, Moves.BRICK_BREAK ],
    [ 44, Moves.SEED_BOMB ],
    [ 50, Moves.DYNAMIC_PUNCH ],
    [ 55, Moves.FOCUS_PUNCH ],
  ],
  [Species.SLAKOTH]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.YAWN ],
    [ 6, Moves.ENCORE ],
    [ 9, Moves.SLACK_OFF ],
    [ 14, Moves.HEADBUTT ],
    [ 17, Moves.AMNESIA ],
    [ 22, Moves.COVET ],
    [ 25, Moves.THROAT_CHOP ],
    [ 30, Moves.COUNTER ],
    [ 33, Moves.FLAIL ],
    [ 38, Moves.PLAY_ROUGH ],
  ],
  [Species.VIGOROTH]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.UPROAR ],
    [ 14, Moves.FURY_SWIPES ],
    [ 17, Moves.ENDURE ],
    [ 23, Moves.SLASH ],
    [ 27, Moves.THROAT_CHOP ],
    [ 33, Moves.COUNTER ],
    [ 37, Moves.FOCUS_PUNCH ],
    [ 43, Moves.REVERSAL ],
  ],
  [Species.SLAKING]: [
    [ EVOLVE_MOVE, Moves.SWAGGER ],
    [ 1, Moves.SUCKER_PUNCH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.SLACK_OFF ],
    [ 17, Moves.AMNESIA ],
    [ 23, Moves.COVET ],
    [ 27, Moves.THROAT_CHOP ],
    [ 33, Moves.COUNTER ],
    [ 39, Moves.FLAIL ],
    [ 45, Moves.FLING ],
    [ 52, Moves.MEGA_KICK ],
    [ 63, Moves.HAMMER_ARM ],
  ],
  [Species.NINCADA]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 5, Moves.HARDEN ],
    [ 10, Moves.FALSE_SWIPE ],
    [ 15, Moves.MUD_SLAP ],
    [ 21, Moves.ABSORB ],
    [ 25, Moves.METAL_CLAW ],
    [ 30, Moves.FURY_SWIPES ],
    [ 35, Moves.MIND_READER ],
    [ 40, Moves.DIG ],
  ],
  [Species.NINJASK]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_TEAM ],
    [ EVOLVE_MOVE, Moves.SCREECH ],
    [ EVOLVE_MOVE, Moves.FURY_CUTTER ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.AERIAL_ACE ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.DIG ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 15, Moves.AGILITY ],
    [ 23, Moves.ABSORB ],
    [ 29, Moves.BUG_BITE ],
    [ 36, Moves.FURY_SWIPES ],
    [ 43, Moves.MIND_READER ],
    [ 50, Moves.SLASH ],
    [ 57, Moves.SWORDS_DANCE ],
    [ 64, Moves.X_SCISSOR ],
  ],
  [Species.SHEDINJA]: [
    [ 1, Moves.SHADOW_CLAW ],
    [ 1, Moves.GRUDGE ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.DIG ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 15, Moves.CONFUSE_RAY ],
    [ 23, Moves.ABSORB ],
    [ 29, Moves.SHADOW_SNEAK ],
    [ 36, Moves.FURY_SWIPES ],
    [ 43, Moves.MIND_READER ],
    [ 50, Moves.SHADOW_BALL ],
    [ 57, Moves.SPITE ],
    [ 64, Moves.PHANTOM_FORCE ],
  ],
  [Species.WHISMUR]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.POUND ],
    [ 5, Moves.ECHOED_VOICE ],
    [ 10, Moves.HOWL ],
    [ 15, Moves.REST ],
    [ 15, Moves.SLEEP_TALK ],
    [ 21, Moves.STOMP ],
    [ 25, Moves.ROAR ],
    [ 30, Moves.SUPERSONIC ],
    [ 35, Moves.UPROAR ],
    [ 40, Moves.SCREECH ],
    [ 45, Moves.HYPER_VOICE ],
  ],
  [Species.LOUDRED]: [
    [ EVOLVE_MOVE, Moves.BITE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.POUND ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 1, Moves.HOWL ],
    [ 15, Moves.REST ],
    [ 15, Moves.SLEEP_TALK ],
    [ 23, Moves.STOMP ],
    [ 29, Moves.ROAR ],
    [ 36, Moves.SUPERSONIC ],
    [ 43, Moves.UPROAR ],
    [ 50, Moves.SCREECH ],
    [ 57, Moves.HYPER_VOICE ],
  ],
  [Species.EXPLOUD]: [
    [ EVOLVE_MOVE, Moves.CRUNCH ],
    [ 1, Moves.BITE ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.POUND ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 1, Moves.HOWL ],
    [ 15, Moves.REST ],
    [ 15, Moves.SLEEP_TALK ],
    [ 23, Moves.STOMP ],
    [ 29, Moves.ROAR ],
    [ 36, Moves.SUPERSONIC ],
    [ 45, Moves.UPROAR ],
    [ 54, Moves.SCREECH ],
    [ 63, Moves.HYPER_VOICE ],
    [ 72, Moves.BOOMBURST ],
    [ 81, Moves.HYPER_BEAM ],
  ],
  [Species.MAKUHITA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 4, Moves.SAND_ATTACK ],
    [ 7, Moves.ARM_THRUST ],
    [ 10, Moves.FAKE_OUT ],
    [ 13, Moves.FORCE_PALM ],
    [ 16, Moves.WHIRLWIND ],
    [ 19, Moves.KNOCK_OFF ],
    [ 22, Moves.BULK_UP ],
    [ 25, Moves.BELLY_DRUM ],
    [ 28, Moves.DETECT ],
    [ 31, Moves.SEISMIC_TOSS ],
    [ 34, Moves.FOCUS_PUNCH ],
    [ 37, Moves.ENDURE ],
    [ 40, Moves.CLOSE_COMBAT ],
    [ 43, Moves.REVERSAL ],
    [ 46, Moves.HEAVY_SLAM ],
  ],
  [Species.HARIYAMA]: [
    [ 1, Moves.BRINE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.ARM_THRUST ],
    [ 10, Moves.FAKE_OUT ],
    [ 13, Moves.FORCE_PALM ],
    [ 16, Moves.WHIRLWIND ],
    [ 19, Moves.KNOCK_OFF ],
    [ 22, Moves.BULK_UP ],
    [ 26, Moves.BELLY_DRUM ],
    [ 30, Moves.DETECT ],
    [ 34, Moves.SEISMIC_TOSS ],
    [ 38, Moves.FOCUS_PUNCH ],
    [ 42, Moves.ENDURE ],
    [ 46, Moves.CLOSE_COMBAT ],
    [ 50, Moves.REVERSAL ],
    [ 54, Moves.HEAVY_SLAM ],
    [ 60, Moves.HEADLONG_RUSH ],
  ],
  [Species.AZURILL]: [
    [ 1, Moves.SPLASH ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.TAIL_WHIP ],
    [ 3, Moves.HELPING_HAND ],
    [ 6, Moves.BUBBLE_BEAM ],
    [ 9, Moves.CHARM ],
    [ 12, Moves.SLAM ],
    [ 15, Moves.BOUNCE ],
  ],
  [Species.NOSEPASS]: [
    [ 1, Moves.TACKLE ],
    [ 4, Moves.HARDEN ],
    [ 7, Moves.BLOCK ],
    [ 10, Moves.ROCK_THROW ],
    [ 13, Moves.THUNDER_WAVE ],
    [ 16, Moves.REST ],
    [ 19, Moves.SPARK ],
    [ 22, Moves.ROCK_SLIDE ],
    [ 25, Moves.POWER_GEM ],
    [ 28, Moves.ROCK_BLAST ],
    [ 31, Moves.DISCHARGE ],
    [ 34, Moves.SANDSTORM ],
    [ 37, Moves.EARTH_POWER ],
    [ 40, Moves.STONE_EDGE ],
    [ 43, Moves.LOCK_ON ],
    [ 43, Moves.ZAP_CANNON ],
  ],
  [Species.SKITTY]: [
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TAIL_WHIP ],
    [ 4, Moves.TACKLE ],
    [ 7, Moves.SING ],
    [ 10, Moves.ATTRACT ],
    [ 13, Moves.DISARMING_VOICE ],
    [ 16, Moves.FURY_SWIPES ],
    [ 19, Moves.COPYCAT ],
    [ 22, Moves.PAYBACK ],
    [ 25, Moves.CHARM ],
    [ 31, Moves.FACADE ],
    [ 34, Moves.COVET ],
    [ 37, Moves.HEAL_BELL ],
    [ 40, Moves.DOUBLE_EDGE ],
    [ 43, Moves.BABY_DOLL_EYES ],
    [ 46, Moves.PLAY_ROUGH ],
  ],
  [Species.DELCATTY]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SING ],
    [ 1, Moves.ATTRACT ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.FURY_SWIPES ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.PAYBACK ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.FACADE ],
    [ 1, Moves.COVET ],
    [ 1, Moves.HEAL_BELL ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 1, Moves.PLAY_ROUGH ],
  ],
  [Species.SABLEYE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.SCRATCH ],
    [ 3, Moves.ASTONISH ],
    [ 9, Moves.SHADOW_SNEAK ],
    [ 12, Moves.FAKE_OUT ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.DETECT ],
    [ 21, Moves.NIGHT_SHADE ],
    [ 24, Moves.FURY_SWIPES ],
    [ 27, Moves.KNOCK_OFF ],
    [ 30, Moves.QUASH ],
    [ 33, Moves.SHADOW_CLAW ],
    [ 36, Moves.MEAN_LOOK ],
    [ 39, Moves.POWER_GEM ],
    [ 42, Moves.ZEN_HEADBUTT ],
    [ 45, Moves.SHADOW_BALL ],
    [ 48, Moves.FOUL_PLAY ],
  ],
  [Species.MAWILE]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.FAIRY_WIND ],
    [ 8, Moves.BATON_PASS ],
    [ 12, Moves.BITE ],
    [ 16, Moves.STOCKPILE ],
    [ 16, Moves.SPIT_UP ],
    [ 16, Moves.SWALLOW ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 24, Moves.IRON_DEFENSE ],
    [ 28, Moves.CRUNCH ],
    [ 32, Moves.SWEET_SCENT ],
    [ 36, Moves.IRON_HEAD ],
    [ 40, Moves.TAUNT ],
    [ 44, Moves.FAKE_TEARS ],
    [ 48, Moves.PLAY_ROUGH ],
  ],
  [Species.ARON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 4, Moves.METAL_CLAW ],
    [ 8, Moves.ROCK_TOMB ],
    [ 12, Moves.ROAR ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.PROTECT ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 28, Moves.IRON_HEAD ],
    [ 33, Moves.METAL_SOUND ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.AUTOTOMIZE ],
    [ 44, Moves.IRON_TAIL ],
    [ 48, Moves.IRON_DEFENSE ],
    [ 52, Moves.HEAVY_SLAM ],
    [ 56, Moves.DOUBLE_EDGE ],
    [ 60, Moves.METAL_BURST ],
  ],
  [Species.LAIRON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.ROCK_TOMB ],
    [ 12, Moves.ROAR ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.PROTECT ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 28, Moves.IRON_HEAD ],
    [ 35, Moves.METAL_SOUND ],
    [ 40, Moves.TAKE_DOWN ],
    [ 46, Moves.AUTOTOMIZE ],
    [ 52, Moves.IRON_TAIL ],
    [ 58, Moves.IRON_DEFENSE ],
    [ 64, Moves.HEAVY_SLAM ],
    [ 70, Moves.DOUBLE_EDGE ],
    [ 76, Moves.METAL_BURST ],
  ],
  [Species.AGGRON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.ROCK_TOMB ],
    [ 12, Moves.ROAR ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.PROTECT ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 28, Moves.IRON_HEAD ],
    [ 35, Moves.METAL_SOUND ],
    [ 40, Moves.TAKE_DOWN ],
    [ 48, Moves.AUTOTOMIZE ],
    [ 56, Moves.IRON_TAIL ],
    [ 64, Moves.IRON_DEFENSE ],
    [ 72, Moves.HEAVY_SLAM ],
    [ 80, Moves.DOUBLE_EDGE ],
    [ 88, Moves.METAL_BURST ],
  ],
  [Species.MEDITITE]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.WORK_UP ],
    [ 1, Moves.BIDE ],
    [ 9, Moves.DETECT ],
    [ 12, Moves.ENDURE ],
    [ 15, Moves.FEINT ],
    [ 17, Moves.FORCE_PALM ],
    [ 20, Moves.PSYBEAM ],
    [ 23, Moves.CALM_MIND ],
    [ 25, Moves.ZEN_HEADBUTT ],
    [ 28, Moves.HIGH_JUMP_KICK ],
    [ 31, Moves.PSYCH_UP ],
    [ 33, Moves.ACUPRESSURE ],
    [ 36, Moves.POWER_TRICK ],
    [ 39, Moves.REVERSAL ],
    [ 41, Moves.RECOVER ],
    [ 44, Moves.COUNTER ],
  ],
  [Species.MEDICHAM]: [
    [ 1, Moves.FIRE_PUNCH ],
    [ 1, Moves.ICE_PUNCH ],
    [ 1, Moves.THUNDER_PUNCH ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.DETECT ],
    [ 1, Moves.WORK_UP ],
    [ 1, Moves.BIDE ],
    [ 12, Moves.ENDURE ],
    [ 15, Moves.FEINT ],
    [ 17, Moves.FORCE_PALM ],
    [ 20, Moves.PSYBEAM ],
    [ 23, Moves.CALM_MIND ],
    [ 25, Moves.ZEN_HEADBUTT ],
    [ 28, Moves.HIGH_JUMP_KICK ],
    [ 31, Moves.PSYCH_UP ],
    [ 33, Moves.ACUPRESSURE ],
    [ 36, Moves.POWER_TRICK ],
    [ 47, Moves.RECOVER ],
    [ 53, Moves.COUNTER ],
    [ 53, Moves.AXE_KICK ],
  ],
  [Species.ELECTRIKE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 4, Moves.LEER ],
    [ 8, Moves.HOWL ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 16, Moves.SHOCK_WAVE ],
    [ 20, Moves.BITE ],
    [ 24, Moves.THUNDER_FANG ],
    [ 28, Moves.ROAR ],
    [ 32, Moves.DISCHARGE ],
    [ 36, Moves.CHARGE ],
    [ 40, Moves.WILD_CHARGE ],
    [ 44, Moves.THUNDER ],
  ],
  [Species.MANECTRIC]: [
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.HOWL ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 16, Moves.SHOCK_WAVE ],
    [ 20, Moves.BITE ],
    [ 24, Moves.THUNDER_FANG ],
    [ 30, Moves.ROAR ],
    [ 36, Moves.DISCHARGE ],
    [ 42, Moves.CHARGE ],
    [ 48, Moves.WILD_CHARGE ],
    [ 54, Moves.THUNDER ],
    [ 60, Moves.ELECTRIC_TERRAIN ],
  ],
  [Species.PLUSLE]: [
    [ RELEARN_MOVE, Moves.NUZZLE ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.HELPING_HAND ],
    [ 7, Moves.SPARK ],
    [ 10, Moves.ENCORE ],
    [ 13, Moves.SWITCHEROO ],
    [ 16, Moves.SWIFT ],
    [ 19, Moves.ELECTRO_BALL ],
    [ 22, Moves.COPYCAT ],
    [ 26, Moves.CHARGE ],
    [ 31, Moves.DISCHARGE ],
    [ 34, Moves.BATON_PASS ],
    [ 37, Moves.AGILITY ],
    [ 40, Moves.LAST_RESORT ],
    [ 43, Moves.THUNDER ],
    [ 46, Moves.NASTY_PLOT ],
    [ 49, Moves.ENTRAINMENT ],
  ],
  [Species.MINUN]: [
    [ RELEARN_MOVE, Moves.NUZZLE ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.HELPING_HAND ],
    [ 7, Moves.SPARK ],
    [ 10, Moves.ENCORE ],
    [ 13, Moves.SWITCHEROO ],
    [ 16, Moves.SWIFT ],
    [ 19, Moves.ELECTRO_BALL ],
    [ 22, Moves.COPYCAT ],
    [ 26, Moves.CHARGE ],
    [ 31, Moves.DISCHARGE ],
    [ 34, Moves.BATON_PASS ],
    [ 37, Moves.AGILITY ],
    [ 40, Moves.LAST_RESORT ],
    [ 43, Moves.THUNDER ],
    [ 46, Moves.NASTY_PLOT ],
    [ 49, Moves.ENTRAINMENT ],
  ],
  [Species.VOLBEAT]: [
    [ 1, Moves.FLASH ],
    [ 1, Moves.TACKLE ],
    [ 5, Moves.DOUBLE_TEAM ],
    [ 8, Moves.CONFUSE_RAY ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 15, Moves.STRUGGLE_BUG ],
    [ 19, Moves.MOONLIGHT ],
    [ 22, Moves.TAIL_GLOW ],
    [ 26, Moves.PROTECT ],
    [ 29, Moves.ZEN_HEADBUTT ],
    [ 33, Moves.HELPING_HAND ],
    [ 36, Moves.BUG_BUZZ ],
    [ 40, Moves.PLAY_ROUGH ],
    [ 43, Moves.DOUBLE_EDGE ],
    [ 47, Moves.INFESTATION ],
  ],
  [Species.ILLUMISE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PLAY_NICE ],
    [ 5, Moves.SWEET_SCENT ],
    [ 9, Moves.CHARM ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 15, Moves.STRUGGLE_BUG ],
    [ 19, Moves.MOONLIGHT ],
    [ 22, Moves.WISH ],
    [ 26, Moves.ENCORE ],
    [ 29, Moves.FLATTER ],
    [ 33, Moves.ZEN_HEADBUTT ],
    [ 36, Moves.HELPING_HAND ],
    [ 40, Moves.BUG_BUZZ ],
    [ 43, Moves.PLAY_ROUGH ],
    [ 47, Moves.INFESTATION ],
  ],
  [Species.ROSELIA]: [
    [ EVOLVE_MOVE, Moves.POISON_STING ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.WORRY_SEED ],
    [ 5, Moves.MEGA_DRAIN ],
    [ 10, Moves.LEECH_SEED ],
    [ 15, Moves.MAGICAL_LEAF ],
    [ 20, Moves.TOXIC_SPIKES ],
    [ 25, Moves.SWEET_SCENT ],
    [ 30, Moves.GIGA_DRAIN ],
    [ 35, Moves.SYNTHESIS ],
    [ 40, Moves.TOXIC ],
    [ 45, Moves.PETAL_BLIZZARD ],
    [ 50, Moves.AROMATHERAPY ],
    [ 55, Moves.INGRAIN ],
    [ 60, Moves.PETAL_DANCE ],
  ],
  [Species.GULPIN]: [
    [ 1, Moves.POUND ],
    [ 5, Moves.YAWN ],
    [ 8, Moves.POISON_GAS ],
    [ 10, Moves.SLUDGE ],
    [ 12, Moves.AMNESIA ],
    [ 17, Moves.ACID_SPRAY ],
    [ 20, Moves.ENCORE ],
    [ 25, Moves.TOXIC ],
    [ 28, Moves.STOCKPILE ],
    [ 28, Moves.SPIT_UP ],
    [ 28, Moves.SWALLOW ],
    [ 33, Moves.SLUDGE_BOMB ],
    [ 36, Moves.GASTRO_ACID ],
    [ 41, Moves.BELCH ],
    [ 44, Moves.PAIN_SPLIT ],
    [ 49, Moves.GUNK_SHOT ],
  ],
  [Species.SWALOT]: [
    [ EVOLVE_MOVE, Moves.BODY_SLAM ],
    [ 1, Moves.GUNK_SHOT ],
    [ 1, Moves.POUND ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.WRING_OUT ],
    [ 1, Moves.SLUDGE ],
    [ 12, Moves.AMNESIA ],
    [ 17, Moves.ACID_SPRAY ],
    [ 20, Moves.ENCORE ],
    [ 25, Moves.TOXIC ],
    [ 30, Moves.STOCKPILE ],
    [ 30, Moves.SPIT_UP ],
    [ 30, Moves.SWALLOW ],
    [ 37, Moves.SLUDGE_BOMB ],
    [ 42, Moves.GASTRO_ACID ],
    [ 49, Moves.BELCH ],
  ],
  [Species.CARVANHA]: [
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.LEER ],
    [ 4, Moves.POISON_FANG ],
    [ 8, Moves.FOCUS_ENERGY ],
    [ 12, Moves.SCARY_FACE ],
    [ 16, Moves.BITE ],
    [ 20, Moves.ICE_FANG ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.SWAGGER ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.AGILITY ],
    [ 40, Moves.LIQUIDATION ],
    [ 44, Moves.TAKE_DOWN ],
  ],
  [Species.SHARPEDO]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.LEER ],
    [ 1, Moves.POISON_FANG ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 12, Moves.SCARY_FACE ],
    [ 16, Moves.BITE ],
    [ 20, Moves.ICE_FANG ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.SWAGGER ],
    [ 34, Moves.CRUNCH ],
    [ 40, Moves.AGILITY ],
    [ 46, Moves.LIQUIDATION ],
    [ 52, Moves.TAKE_DOWN ],
  ],
  [Species.WAILMER]: [
    [ 1, Moves.SPLASH ],
    [ 1, Moves.TACKLE ], //Custom
    [ 3, Moves.GROWL ],
    [ 6, Moves.ASTONISH ],
    [ 12, Moves.WATER_GUN ],
    [ 15, Moves.MIST ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEAVY_SLAM ],
    [ 24, Moves.BRINE ],
    [ 27, Moves.WHIRLPOOL ],
    [ 30, Moves.DIVE ],
    [ 33, Moves.BOUNCE ],
    [ 36, Moves.BODY_SLAM ],
    [ 39, Moves.REST ],
    [ 42, Moves.AMNESIA ],
    [ 45, Moves.HYDRO_PUMP ],
    [ 48, Moves.WATER_SPOUT ],
  ],
  [Species.WAILORD]: [
    [ 1, Moves.SOAK ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.WATER_GUN ],
    [ 15, Moves.MIST ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEAVY_SLAM ],
    [ 24, Moves.BRINE ],
    [ 27, Moves.WHIRLPOOL ],
    [ 30, Moves.DIVE ],
    [ 33, Moves.BOUNCE ],
    [ 36, Moves.BODY_SLAM ],
    [ 39, Moves.REST ],
    [ 44, Moves.AMNESIA ],
    [ 49, Moves.HYDRO_PUMP ],
    [ 54, Moves.WATER_SPOUT ],
  ],
  [Species.NUMEL]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.TACKLE ],
    [ 5, Moves.EMBER ],
    [ 8, Moves.FOCUS_ENERGY ],
    [ 12, Moves.BULLDOZE ],
    [ 15, Moves.INCINERATE ],
    [ 19, Moves.AMNESIA ],
    [ 22, Moves.LAVA_PLUME ],
    [ 26, Moves.EARTH_POWER ],
    [ 29, Moves.CURSE ],
    [ 31, Moves.TAKE_DOWN ],
    [ 40, Moves.EARTHQUAKE ],
    [ 43, Moves.FLAMETHROWER ],
    [ 47, Moves.DOUBLE_EDGE ],
  ],
  [Species.CAMERUPT]: [
    [ EVOLVE_MOVE, Moves.ROCK_SLIDE ],
    [ 1, Moves.FISSURE ],
    [ 1, Moves.ERUPTION ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 12, Moves.BULLDOZE ],
    [ 15, Moves.INCINERATE ],
    [ 19, Moves.AMNESIA ],
    [ 22, Moves.LAVA_PLUME ],
    [ 26, Moves.EARTH_POWER ],
    [ 29, Moves.CURSE ],
    [ 31, Moves.TAKE_DOWN ],
    [ 39, Moves.YAWN ],
    [ 46, Moves.EARTHQUAKE ],
  ],
  [Species.TORKOAL]: [
    [ 1, Moves.SMOG ],
    [ 1, Moves.EMBER ],
    [ 4, Moves.WITHDRAW ],
    [ 8, Moves.RAPID_SPIN ],
    [ 12, Moves.SMOKESCREEN ],
    [ 16, Moves.CLEAR_SMOG ],
    [ 20, Moves.FLAME_WHEEL ],
    [ 24, Moves.PROTECT ],
    [ 28, Moves.LAVA_PLUME ],
    [ 32, Moves.BODY_SLAM ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 40, Moves.FLAMETHROWER ],
    [ 44, Moves.CURSE ],
    [ 48, Moves.HEAT_WAVE ],
    [ 52, Moves.AMNESIA ],
    [ 56, Moves.INFERNO ],
    [ 60, Moves.SHELL_SMASH ],
    [ 64, Moves.ERUPTION ],
  ],
  [Species.SPOINK]: [
    [ 1, Moves.SPLASH ],
    [ 5, Moves.CONFUSION ], //Custom, Moved from Level 7 to 5
    [ 10, Moves.GROWL ],
    [ 14, Moves.PSYBEAM ],
    [ 18, Moves.PSYCH_UP ],
    [ 22, Moves.CONFUSE_RAY ],
    [ 29, Moves.REST ],
    [ 29, Moves.POWER_GEM ],
    [ 33, Moves.SNORE ],
    [ 38, Moves.PSYSHOCK ],
    [ 40, Moves.PAYBACK ],
    [ 44, Moves.PSYCHIC ],
    [ 50, Moves.BOUNCE ],
  ],
  [Species.GRUMPIG]: [
    [ EVOLVE_MOVE, Moves.TEETER_DANCE ],
    [ 1, Moves.BELCH ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.PSYBEAM ],
    [ 18, Moves.PSYCH_UP ],
    [ 22, Moves.CONFUSE_RAY ],
    [ 26, Moves.ZEN_HEADBUTT ],
    [ 29, Moves.POWER_GEM ],
    [ 35, Moves.REST ],
    [ 35, Moves.SNORE ],
    [ 42, Moves.PSYSHOCK ],
    [ 46, Moves.PAYBACK ],
    [ 52, Moves.PSYCHIC ],
    [ 60, Moves.BOUNCE ],
  ],
  [Species.SPINDA]: [
    [ 1, Moves.TACKLE ],
    [ 5, Moves.COPYCAT ],
    [ 10, Moves.DIZZY_PUNCH ],
    [ 14, Moves.PSYBEAM ],
    [ 19, Moves.HYPNOSIS ],
    [ 23, Moves.BODY_SLAM ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.TEETER_DANCE ],
    [ 37, Moves.UPROAR ],
    [ 41, Moves.PSYCH_UP ],
    [ 46, Moves.DOUBLE_EDGE ],
    [ 50, Moves.FLAIL ],
    [ 55, Moves.THRASH ],
  ],
  [Species.TRAPINCH]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.FEINT_ATTACK ],
    [ 8, Moves.BITE ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.SAND_TOMB ],
    [ 20, Moves.BULLDOZE ],
    [ 24, Moves.DIG ],
    [ 28, Moves.CRUNCH ],
    [ 32, Moves.SANDSTORM ],
    [ 36, Moves.EARTH_POWER ],
    [ 40, Moves.EARTHQUAKE ],
    [ 44, Moves.SUPERPOWER ],
    [ 48, Moves.FISSURE ],
  ],
  [Species.VIBRAVA]: [
    [ EVOLVE_MOVE, Moves.DRAGON_BREATH ],
    [ RELEARN_MOVE, Moves.SUPERSONIC ],
    [ RELEARN_MOVE, Moves.FISSURE ],
    [ RELEARN_MOVE, Moves.CRUNCH ],
    [ RELEARN_MOVE, Moves.SUPERPOWER ],
    [ RELEARN_MOVE, Moves.ASTONISH ],
    [ RELEARN_MOVE, Moves.BULLDOZE ],
    [ 1, Moves.DIG ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.BITE ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.FEINT_ATTACK ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.SAND_TOMB ],
    [ 20, Moves.DRAGON_TAIL ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.BUG_BUZZ ],
    [ 32, Moves.SANDSTORM ],
    [ 38, Moves.EARTH_POWER ],
    [ 44, Moves.EARTHQUAKE ],
    [ 50, Moves.UPROAR ],
    [ 56, Moves.DRAGON_RUSH ],
    [ 62, Moves.BOOMBURST ],
  ],
  [Species.FLYGON]: [
    [ EVOLVE_MOVE, Moves.DRAGON_CLAW ],
    [ RELEARN_MOVE, Moves.BITE ],
    [ RELEARN_MOVE, Moves.FISSURE ],
    [ RELEARN_MOVE, Moves.DIG ],
    [ RELEARN_MOVE, Moves.CRUNCH ],
    [ RELEARN_MOVE, Moves.SUPERPOWER ],
    [ RELEARN_MOVE, Moves.ASTONISH ],
    [ RELEARN_MOVE, Moves.DRAGON_DANCE ],
    [ RELEARN_MOVE, Moves.FEINT ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.BULLDOZE ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.FEINT_ATTACK ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.SAND_TOMB ],
    [ 20, Moves.DRAGON_TAIL ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.BUG_BUZZ ],
    [ 32, Moves.SANDSTORM ],
    [ 38, Moves.EARTH_POWER ],
    [ 44, Moves.EARTHQUAKE ],
    [ 52, Moves.UPROAR ],
    [ 60, Moves.DRAGON_RUSH ],
    [ 68, Moves.BOOMBURST ],
  ],
  [Species.CACNEA]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 4, Moves.ABSORB ],
    [ 7, Moves.GROWTH ],
    [ 10, Moves.LEECH_SEED ],
    [ 13, Moves.SAND_ATTACK ],
    [ 16, Moves.BULLET_SEED ],
    [ 19, Moves.POWER_TRIP ],
    [ 22, Moves.INGRAIN ],
    [ 26, Moves.PAYBACK ],
    [ 30, Moves.SPIKES ],
    [ 34, Moves.SUCKER_PUNCH ],
    [ 38, Moves.PIN_MISSILE ],
    [ 42, Moves.ENERGY_BALL ],
    [ 46, Moves.COTTON_SPORE ],
    [ 50, Moves.SANDSTORM ],
    [ 54, Moves.DESTINY_BOND ],
  ],
  [Species.CACTURNE]: [
    [ EVOLVE_MOVE, Moves.SPIKY_SHIELD ],
    [ 1, Moves.DESTINY_BOND ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 10, Moves.LEECH_SEED ],
    [ 13, Moves.SAND_ATTACK ],
    [ 16, Moves.BULLET_SEED ],
    [ 19, Moves.POWER_TRIP ],
    [ 22, Moves.INGRAIN ],
    [ 26, Moves.PAYBACK ],
    [ 30, Moves.SPIKES ],
    [ 35, Moves.SUCKER_PUNCH ],
    [ 38, Moves.PIN_MISSILE ],
    [ 44, Moves.ENERGY_BALL ],
    [ 49, Moves.COTTON_SPORE ],
    [ 54, Moves.SANDSTORM ],
  ],
  [Species.SWABLU]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.DISARMING_VOICE ],
    [ 8, Moves.MIST ],
    [ 12, Moves.FURY_ATTACK ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.DRAGON_BREATH ],
    [ 24, Moves.SAFEGUARD ],
    [ 28, Moves.SING ],
    [ 32, Moves.COTTON_GUARD ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.MOONBLAST ],
    [ 44, Moves.PERISH_SONG ],
  ],
  [Species.ALTARIA]: [
    [ EVOLVE_MOVE, Moves.DRAGON_PULSE ],
    [ 1, Moves.PLUCK ],
    [ 1, Moves.PECK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.MIST ],
    [ 12, Moves.FURY_ATTACK ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.DRAGON_BREATH ],
    [ 24, Moves.SAFEGUARD ],
    [ 28, Moves.SING ],
    [ 32, Moves.COTTON_GUARD ],
    [ 38, Moves.TAKE_DOWN ],
    [ 44, Moves.MOONBLAST ],
    [ 50, Moves.PERISH_SONG ],
    [ 56, Moves.SKY_ATTACK ],
  ],
  [Species.ZANGOOSE]: [
    [ RELEARN_MOVE, Moves.DOUBLE_KICK ],
    [ RELEARN_MOVE, Moves.DISABLE ],
    [ RELEARN_MOVE, Moves.COUNTER ],
    [ RELEARN_MOVE, Moves.FURY_SWIPES ],
    [ RELEARN_MOVE, Moves.CURSE ],
    [ RELEARN_MOVE, Moves.FLAIL ],
    [ RELEARN_MOVE, Moves.BELLY_DRUM ],
    [ RELEARN_MOVE, Moves.FEINT ],
    [ RELEARN_MOVE, Moves.NIGHT_SLASH ],
    [ RELEARN_MOVE, Moves.DOUBLE_HIT ],
    [ RELEARN_MOVE, Moves.QUICK_GUARD ],
    [ RELEARN_MOVE, Moves.FINAL_GAMBIT ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 8, Moves.FURY_CUTTER ],
    [ 12, Moves.METAL_CLAW ],
    [ 15, Moves.HONE_CLAWS ],
    [ 19, Moves.SLASH ],
    [ 22, Moves.POWER_TRIP ],
    [ 26, Moves.CRUSH_CLAW ],
    [ 29, Moves.FALSE_SWIPE ],
    [ 33, Moves.SWITCHEROO ],
    [ 36, Moves.DETECT ],
    [ 40, Moves.X_SCISSOR ],
    [ 43, Moves.TAUNT ],
    [ 47, Moves.SWORDS_DANCE ],
    [ 50, Moves.CLOSE_COMBAT ],
  ],
  [Species.SEVIPER]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.SWAGGER ],
    [ 4, Moves.BITE ],
    [ 6, Moves.LICK ],
    [ 9, Moves.POISON_TAIL ],
    [ 11, Moves.FEINT ],
    [ 14, Moves.SCREECH ],
    [ 19, Moves.GLARE ],
    [ 21, Moves.POISON_FANG ],
    [ 24, Moves.VENOSHOCK ],
    [ 29, Moves.GASTRO_ACID ],
    [ 31, Moves.POISON_JAB ],
    [ 34, Moves.HAZE ],
    [ 39, Moves.CRUNCH ],
    [ 41, Moves.BELCH ],
    [ 44, Moves.COIL ],
    [ 46, Moves.SLUDGE_BOMB ],
  ],
  [Species.LUNATONE]: [
    [ 1, Moves.MOONBLAST ],
    [ 1, Moves.MOONLIGHT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.ROCK_THROW ],
    [ 5, Moves.HYPNOSIS ],
    [ 10, Moves.ROCK_POLISH ],
    [ 15, Moves.ROCK_SLIDE ],
    [ 20, Moves.PSYSHOCK ],
    [ 25, Moves.COSMIC_POWER ],
    [ 30, Moves.PSYCHIC ],
    [ 35, Moves.STONE_EDGE ],
    [ 40, Moves.FUTURE_SIGHT ],
    [ 45, Moves.MAGIC_ROOM ],
    [ 50, Moves.EXPLOSION ],
  ],
  [Species.SOLROCK]: [
    [ 1, Moves.FLARE_BLITZ ],
    [ 1, Moves.MORNING_SUN ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.ROCK_THROW ],
    [ 5, Moves.HYPNOSIS ],
    [ 10, Moves.ROCK_POLISH ],
    [ 15, Moves.ROCK_SLIDE ],
    [ 20, Moves.ZEN_HEADBUTT ],
    [ 25, Moves.COSMIC_POWER ],
    [ 30, Moves.PSYCHIC ],
    [ 35, Moves.STONE_EDGE ],
    [ 40, Moves.SOLAR_BEAM ],
    [ 45, Moves.WONDER_ROOM ],
    [ 50, Moves.EXPLOSION ],
  ],
  [Species.BARBOACH]: [
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.WATER_GUN ],
    [ 6, Moves.REST ],
    [ 6, Moves.SNORE ],
    [ 12, Moves.WATER_PULSE ],
    [ 18, Moves.AMNESIA ],
    [ 24, Moves.AQUA_TAIL ],
    [ 31, Moves.MUDDY_WATER ],
    [ 36, Moves.EARTHQUAKE ],
    [ 42, Moves.FUTURE_SIGHT ],
    [ 48, Moves.FISSURE ],
  ],
  [Species.WHISCASH]: [
    [ EVOLVE_MOVE, Moves.THRASH ],
    [ 1, Moves.BELCH ],
    [ 1, Moves.ZEN_HEADBUTT ],
    [ 1, Moves.TICKLE ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.REST ],
    [ 1, Moves.SNORE ],
    [ 12, Moves.WATER_PULSE ],
    [ 18, Moves.AMNESIA ],
    [ 24, Moves.AQUA_TAIL ],
    [ 33, Moves.MUDDY_WATER ],
    [ 40, Moves.EARTHQUAKE ],
    [ 48, Moves.FUTURE_SIGHT ],
    [ 56, Moves.FISSURE ],
  ],
  [Species.CORPHISH]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HARDEN ],
    [ 4, Moves.LEER ],
    [ 8, Moves.TAUNT ],
    [ 12, Moves.BUBBLE_BEAM ],
    [ 16, Moves.KNOCK_OFF ],
    [ 20, Moves.DOUBLE_HIT ],
    [ 24, Moves.PROTECT ],
    [ 28, Moves.NIGHT_SLASH ],
    [ 32, Moves.RAZOR_SHELL ],
    [ 36, Moves.SWORDS_DANCE ],
    [ 40, Moves.CRUNCH ],
    [ 44, Moves.CRABHAMMER ],
    [ 48, Moves.ENDEAVOR ],
    [ 52, Moves.GUILLOTINE ],
  ],
  [Species.CRAWDAUNT]: [
    [ EVOLVE_MOVE, Moves.SWIFT ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.LEER ],
    [ 1, Moves.TAUNT ],
    [ 12, Moves.BUBBLE_BEAM ],
    [ 16, Moves.KNOCK_OFF ],
    [ 20, Moves.DOUBLE_HIT ],
    [ 24, Moves.PROTECT ],
    [ 28, Moves.NIGHT_SLASH ],
    [ 34, Moves.RAZOR_SHELL ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 46, Moves.CRUNCH ],
    [ 52, Moves.CRABHAMMER ],
    [ 58, Moves.ENDEAVOR ],
    [ 64, Moves.GUILLOTINE ],
  ],
  [Species.BALTOY]: [
    [ 1, Moves.HARDEN ],
    [ 1, Moves.MUD_SLAP ],
    [ 3, Moves.RAPID_SPIN ],
    [ 6, Moves.CONFUSION ],
    [ 9, Moves.ROCK_TOMB ],
    [ 12, Moves.POWER_TRICK ],
    [ 15, Moves.PSYBEAM ],
    [ 18, Moves.ANCIENT_POWER ],
    [ 21, Moves.IMPRISON ],
    [ 24, Moves.COSMIC_POWER ],
    [ 27, Moves.EXTRASENSORY ],
    [ 30, Moves.EARTH_POWER ],
    [ 33, Moves.SELF_DESTRUCT ],
    [ 36, Moves.GUARD_SPLIT ],
    [ 36, Moves.POWER_SPLIT ],
    [ 39, Moves.SANDSTORM ],
    [ 42, Moves.EXPLOSION ],
  ],
  [Species.CLAYDOL]: [
    [ EVOLVE_MOVE, Moves.HYPER_BEAM ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.CONFUSION ],
    [ 9, Moves.ROCK_TOMB ],
    [ 12, Moves.POWER_TRICK ],
    [ 15, Moves.PSYBEAM ],
    [ 18, Moves.ANCIENT_POWER ],
    [ 21, Moves.IMPRISON ],
    [ 24, Moves.COSMIC_POWER ],
    [ 27, Moves.EXTRASENSORY ],
    [ 30, Moves.EARTH_POWER ],
    [ 33, Moves.SELF_DESTRUCT ],
    [ 38, Moves.GUARD_SPLIT ],
    [ 38, Moves.POWER_SPLIT ],
    [ 43, Moves.SANDSTORM ],
    [ 48, Moves.EXPLOSION ],
  ],
  [Species.LILEEP]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.CONSTRICT ],
    [ 4, Moves.ACID ],
    [ 8, Moves.CONFUSE_RAY ],
    [ 12, Moves.INGRAIN ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.AMNESIA ],
    [ 32, Moves.GASTRO_ACID ],
    [ 36, Moves.GIGA_DRAIN ],
    [ 41, Moves.STOCKPILE ],
    [ 41, Moves.SPIT_UP ],
    [ 41, Moves.SWALLOW ],
    [ 44, Moves.ENERGY_BALL ],
  ],
  [Species.CRADILY]: [
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.CONSTRICT ],
    [ 1, Moves.ACID ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 12, Moves.INGRAIN ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.AMNESIA ],
    [ 32, Moves.GASTRO_ACID ],
    [ 36, Moves.GIGA_DRAIN ],
    [ 43, Moves.STOCKPILE ],
    [ 43, Moves.SPIT_UP ],
    [ 43, Moves.SWALLOW ],
    [ 48, Moves.ENERGY_BALL ],
  ],
  [Species.ANORITH]: [
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.HARDEN ],
    [ 4, Moves.WATER_GUN ],
    [ 8, Moves.SMACK_DOWN ],
    [ 12, Moves.METAL_CLAW ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 20, Moves.BUG_BITE ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.CRUSH_CLAW ],
    [ 36, Moves.ROCK_BLAST ],
    [ 41, Moves.PROTECT ],
    [ 44, Moves.X_SCISSOR ],
  ],
  [Species.ARMALDO]: [
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SMACK_DOWN ],
    [ 12, Moves.METAL_CLAW ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 20, Moves.BUG_BITE ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.CRUSH_CLAW ],
    [ 36, Moves.ROCK_BLAST ],
    [ 43, Moves.PROTECT ],
    [ 48, Moves.X_SCISSOR ],
  ],
  [Species.FEEBAS]: [
    [ 1, Moves.SPLASH ],
    [ 15, Moves.TACKLE ],
    [ 25, Moves.FLAIL ],
  ],
  [Species.MILOTIC]: [
    [ EVOLVE_MOVE, Moves.WATER_PULSE ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WATER_SPORT ],
    [ 4, Moves.DISARMING_VOICE ],
    [ 8, Moves.TWISTER ],
    [ 12, Moves.AQUA_RING ],
    [ 16, Moves.ATTRACT ],
    [ 20, Moves.LIFE_DEW ],
    [ 24, Moves.DRAGON_TAIL ],
    [ 28, Moves.RECOVER ],
    [ 32, Moves.AQUA_TAIL ],
    [ 36, Moves.SAFEGUARD ],
    [ 40, Moves.SURF ],
    [ 44, Moves.RAIN_DANCE ],
    [ 48, Moves.COIL ],
    [ 52, Moves.HYDRO_PUMP ],
  ],
  [Species.CASTFORM]: [
    [ 1, Moves.TACKLE ],
    [ 10, Moves.WATER_GUN ],
    [ 10, Moves.EMBER ],
    [ 10, Moves.POWDER_SNOW ],
    [ 15, Moves.HEADBUTT ],
    [ 20, Moves.RAIN_DANCE ],
    [ 20, Moves.SUNNY_DAY ],
    [ 20, Moves.HAIL ],
    [ 25, Moves.WEATHER_BALL ],
    [ 35, Moves.HYDRO_PUMP ],
    [ 35, Moves.FIRE_BLAST ],
    [ 35, Moves.BLIZZARD ],
    [ 45, Moves.HURRICANE ],
  ],
  [Species.KECLEON]: [
    [ 1, Moves.THIEF ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.LICK ],
    [ 1, Moves.SCRATCH ],
    [ 4, Moves.BIND ],
    [ 7, Moves.SHADOW_SNEAK ],
    [ 10, Moves.FEINT ],
    [ 13, Moves.FURY_SWIPES ],
    [ 16, Moves.DISABLE ],
    [ 18, Moves.PSYBEAM ],
    [ 21, Moves.ANCIENT_POWER ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.DETECT ],
    [ 33, Moves.SHADOW_CLAW ],
    [ 38, Moves.SCREECH ],
    [ 42, Moves.SUBSTITUTE ],
    [ 46, Moves.SUCKER_PUNCH ],
    [ 50, Moves.FOUL_PLAY ],
  ],
  [Species.SHUPPET]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.PURSUIT ], //Custom
    [ 4, Moves.SCREECH ],
    [ 7, Moves.NIGHT_SHADE ],
    [ 10, Moves.SPITE ],
    [ 16, Moves.WILL_O_WISP ],
    [ 19, Moves.SHADOW_SNEAK ],
    [ 22, Moves.HEX ],
    [ 26, Moves.CURSE ],
    [ 30, Moves.SHADOW_BALL ],
    [ 34, Moves.ROLE_PLAY ],
    [ 38, Moves.SUCKER_PUNCH ],
    [ 42, Moves.TRICK ],
    [ 48, Moves.PHANTOM_FORCE ],
  ],
  [Species.BANETTE]: [
    [ EVOLVE_MOVE, Moves.KNOCK_OFF ],
    [ 1, Moves.SCREECH ],
    [ 1, Moves.NIGHT_SHADE ],
    [ 1, Moves.SPITE ],
    [ 16, Moves.WILL_O_WISP ],
    [ 19, Moves.SHADOW_SNEAK ],
    [ 22, Moves.HEX ],
    [ 26, Moves.CURSE ],
    [ 30, Moves.SHADOW_BALL ],
    [ 34, Moves.ROLE_PLAY ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 46, Moves.TRICK ],
    [ 53, Moves.PHANTOM_FORCE ],
  ],
  [Species.DUSKULL]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PURSUIT ], //Custom
    [ 4, Moves.DISABLE ],
    [ 8, Moves.SHADOW_SNEAK ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.NIGHT_SHADE ],
    [ 20, Moves.PAYBACK ],
    [ 24, Moves.WILL_O_WISP ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.HEX ],
    [ 36, Moves.CURSE ],
    [ 40, Moves.SHADOW_BALL ],
    [ 44, Moves.FUTURE_SIGHT ],
  ],
  [Species.DUSCLOPS]: [
    [ EVOLVE_MOVE, Moves.SHADOW_PUNCH ],
    [ 1, Moves.FIRE_PUNCH ],
    [ 1, Moves.ICE_PUNCH ],
    [ 1, Moves.THUNDER_PUNCH ],
    [ 1, Moves.GRAVITY ],
    [ 1, Moves.BIND ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.NIGHT_SHADE ],
    [ 20, Moves.PAYBACK ],
    [ 24, Moves.WILL_O_WISP ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.HEX ],
    [ 36, Moves.CURSE ],
    [ 42, Moves.SHADOW_BALL ],
    [ 48, Moves.FUTURE_SIGHT ],
  ],
  [Species.TROPIUS]: [
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 6, Moves.SWEET_SCENT ],
    [ 10, Moves.STOMP ],
    [ 16, Moves.MAGICAL_LEAF ],
    [ 21, Moves.WHIRLWIND ],
    [ 30, Moves.WIDE_GUARD ],
    [ 36, Moves.AIR_SLASH ],
    [ 41, Moves.BODY_SLAM ],
    [ 46, Moves.OUTRAGE ],
    [ 50, Moves.SYNTHESIS ],
    [ 56, Moves.SOLAR_BEAM ],
  ],
  [Species.CHIMECHO]: [
    [ 1, Moves.HEALING_WISH ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.SYNCHRONOISE ],
    [ 13, Moves.YAWN ],
    [ 16, Moves.STORED_POWER ],
    [ 19, Moves.TAKE_DOWN ],
    [ 22, Moves.EXTRASENSORY ],
    [ 27, Moves.HEAL_BELL ],
    [ 32, Moves.UPROAR ],
    [ 37, Moves.SAFEGUARD ],
    [ 42, Moves.DOUBLE_EDGE ],
    [ 47, Moves.HEAL_PULSE ],
  ],
  [Species.ABSOL]: [
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.LEER ],
    [ 5, Moves.DOUBLE_TEAM ],
    [ 10, Moves.KNOCK_OFF ],
    [ 15, Moves.DETECT ],
    [ 20, Moves.TAUNT ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.NIGHT_SLASH ],
    [ 35, Moves.FOCUS_ENERGY ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 45, Moves.SWORDS_DANCE ],
    [ 50, Moves.FUTURE_SIGHT ],
    [ 55, Moves.PERISH_SONG ],
  ],
  [Species.WYNAUT]: [
    [ 1, Moves.COUNTER ],
    [ 1, Moves.MIRROR_COAT ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.DESTINY_BOND ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.AMNESIA ],
  ],
  [Species.SNORUNT]: [
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.HEADBUTT ],
    [ 5, Moves.LEER ],
    [ 10, Moves.DOUBLE_TEAM ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.PROTECT ],
    [ 25, Moves.ICY_WIND ],
    [ 30, Moves.FROST_BREATH ],
    [ 35, Moves.BITE ],
    [ 40, Moves.ICE_FANG ],
    [ 45, Moves.SNOWSCAPE ],
    [ 50, Moves.WEATHER_BALL ],
    [ 55, Moves.CRUNCH ],
    [ 60, Moves.BLIZZARD ],
  ],
  [Species.GLALIE]: [
    [ EVOLVE_MOVE, Moves.FREEZE_DRY ],
    [ 1, Moves.SHEER_COLD ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.ICE_BALL ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.PROTECT ],
    [ 25, Moves.ICY_WIND ],
    [ 30, Moves.FROST_BREATH ],
    [ 35, Moves.BITE ],
    [ 40, Moves.ICE_FANG ],
    [ 47, Moves.SNOWSCAPE ],
    [ 54, Moves.WEATHER_BALL ],
    [ 61, Moves.CRUNCH ],
    [ 68, Moves.BLIZZARD ],
  ],
  [Species.SPHEAL]: [
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 4, Moves.GROWL ],
    [ 8, Moves.WATER_GUN ],
    [ 12, Moves.POWDER_SNOW ],
    [ 16, Moves.REST ],
    [ 20, Moves.SNORE ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.AURORA_BEAM ],
    [ 33, Moves.ENCORE ],
    [ 36, Moves.BODY_SLAM ],
    [ 40, Moves.SURF ],
    [ 44, Moves.BLIZZARD ],
    [ 48, Moves.HAIL ],
    [ 52, Moves.SHEER_COLD ],
  ],
  [Species.SEALEO]: [
    [ EVOLVE_MOVE, Moves.SWAGGER ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 12, Moves.POWDER_SNOW ],
    [ 16, Moves.REST ],
    [ 20, Moves.SNORE ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.AURORA_BEAM ],
    [ 35, Moves.ENCORE ],
    [ 40, Moves.BODY_SLAM ],
    [ 46, Moves.SURF ],
    [ 52, Moves.BLIZZARD ],
    [ 58, Moves.HAIL ],
    [ 64, Moves.SHEER_COLD ],
  ],
  [Species.WALREIN]: [
    [ 1, Moves.SWAGGER ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 12, Moves.POWDER_SNOW ],
    [ 16, Moves.REST ],
    [ 20, Moves.SNORE ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.AURORA_BEAM ],
    [ 35, Moves.ENCORE ],
    [ 40, Moves.BODY_SLAM ],
    [ 48, Moves.SURF ],
    [ 56, Moves.BLIZZARD ],
    [ 64, Moves.HAIL ],
    [ 72, Moves.SHEER_COLD ],
  ],
  [Species.CLAMPERL]: [
    [ 1, Moves.CLAMP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WHIRLPOOL ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 50, Moves.SHELL_SMASH ],
  ],
  [Species.HUNTAIL]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.SHELL_SMASH ],
    [ 1, Moves.WHIRLPOOL ],
    [ 1, Moves.BITE ],
    [ 5, Moves.SCREECH ],
    [ 9, Moves.SCARY_FACE ],
    [ 11, Moves.RAIN_DANCE ],
    [ 14, Moves.WATER_PULSE ],
    [ 16, Moves.ICE_FANG ],
    [ 19, Moves.BRINE ],
    [ 23, Moves.SUCKER_PUNCH ],
    [ 26, Moves.DIVE ],
    [ 29, Moves.BATON_PASS ],
    [ 34, Moves.CRUNCH ],
    [ 39, Moves.AQUA_TAIL ],
    [ 45, Moves.COIL ],
    [ 50, Moves.HYDRO_PUMP ],
  ],
  [Species.GOREBYSS]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.SHELL_SMASH ],
    [ 1, Moves.WHIRLPOOL ],
    [ 1, Moves.CONFUSION ],
    [ 5, Moves.RAIN_DANCE ],
    [ 9, Moves.AGILITY ],
    [ 11, Moves.DRAINING_KISS ],
    [ 14, Moves.WATER_PULSE ],
    [ 16, Moves.AMNESIA ],
    [ 19, Moves.AQUA_RING ],
    [ 23, Moves.SAFEGUARD ],
    [ 26, Moves.DIVE ],
    [ 29, Moves.BATON_PASS ],
    [ 34, Moves.PSYCHIC ],
    [ 39, Moves.AQUA_TAIL ],
    [ 45, Moves.COIL ],
    [ 50, Moves.HYDRO_PUMP ],
  ],
  [Species.RELICANTH]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.WATER_GUN ],
    [ 10, Moves.ANCIENT_POWER ],
    [ 15, Moves.YAWN ],
    [ 20, Moves.DIVE ],
    [ 25, Moves.TAKE_DOWN ],
    [ 30, Moves.AQUA_TAIL ],
    [ 35, Moves.REST ],
    [ 40, Moves.FLAIL ],
    [ 45, Moves.HYDRO_PUMP ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 55, Moves.HEAD_SMASH ],
  ],
  [Species.LUVDISC]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CHARM ],
    [ 4, Moves.WATER_GUN ],
    [ 7, Moves.AGILITY ],
    [ 13, Moves.WISH ],
    [ 17, Moves.WATER_PULSE ],
    [ 20, Moves.ATTRACT ],
    [ 22, Moves.DRAINING_KISS ],
    [ 26, Moves.FLAIL ],
    [ 31, Moves.SWEET_KISS ],
    [ 34, Moves.TAKE_DOWN ],
    [ 37, Moves.BABY_DOLL_EYES ],
    [ 40, Moves.AQUA_RING ],
    [ 42, Moves.SOAK ],
    [ 46, Moves.HYDRO_PUMP ],
    [ 49, Moves.SAFEGUARD ],
  ],
  [Species.BAGON]: [
    [ 1, Moves.EMBER ],
    [ 1, Moves.LEER ],
    [ 5, Moves.BITE ],
    [ 10, Moves.DRAGON_BREATH ],
    [ 15, Moves.HEADBUTT ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.CRUNCH ],
    [ 31, Moves.DRAGON_CLAW ],
    [ 35, Moves.ZEN_HEADBUTT ],
    [ 40, Moves.FOCUS_ENERGY ],
    [ 45, Moves.FLAMETHROWER ],
    [ 50, Moves.OUTRAGE ],
    [ 55, Moves.DOUBLE_EDGE ],
  ],
  [Species.SHELGON]: [
    [ EVOLVE_MOVE, Moves.PROTECT ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 15, Moves.HEADBUTT ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.CRUNCH ],
    [ 33, Moves.DRAGON_CLAW ],
    [ 39, Moves.ZEN_HEADBUTT ],
    [ 46, Moves.FOCUS_ENERGY ],
    [ 53, Moves.FLAMETHROWER ],
    [ 60, Moves.OUTRAGE ],
    [ 67, Moves.DOUBLE_EDGE ],
  ],
  [Species.SALAMENCE]: [
    [ EVOLVE_MOVE, Moves.FLY ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 1, Moves.DUAL_WINGBEAT ],
    [ 1, Moves.ROOST ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 15, Moves.HEADBUTT ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.CRUNCH ],
    [ 33, Moves.DRAGON_CLAW ],
    [ 39, Moves.ZEN_HEADBUTT ],
    [ 46, Moves.FOCUS_ENERGY ],
    [ 55, Moves.FLAMETHROWER ],
    [ 73, Moves.DOUBLE_EDGE ],
  ],
  [Species.BELDUM]: [
    [ 1, Moves.TACKLE ],
  ],
  [Species.METANG]: [
    [ EVOLVE_MOVE, Moves.CONFUSION ],
    [ EVOLVE_MOVE, Moves.METAL_CLAW ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BULLET_PUNCH ],
    [ 1, Moves.HONE_CLAWS ],
    [ 6, Moves.ZEN_HEADBUTT ],
    [ 12, Moves.MAGNET_RISE ],
    [ 18, Moves.FLASH_CANNON ],
    [ 26, Moves.TAKE_DOWN ],
    [ 34, Moves.PSYCHIC ],
    [ 42, Moves.SCARY_FACE ],
    [ 50, Moves.METEOR_MASH ],
    [ 58, Moves.IRON_DEFENSE ],
    [ 66, Moves.AGILITY ],
    [ 74, Moves.HYPER_BEAM ],
  ],
  [Species.METAGROSS]: [
    [ EVOLVE_MOVE, Moves.HAMMER_ARM ],
    [ RELEARN_MOVE, Moves.EXPLOSION ],
    [ RELEARN_MOVE, Moves.HONE_CLAWS ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.BULLET_PUNCH ],
    [ 1, Moves.TACKLE ],
    [ 6, Moves.ZEN_HEADBUTT ],
    [ 12, Moves.MAGNET_RISE ],
    [ 16, Moves.FLASH_CANNON ],
    [ 26, Moves.TAKE_DOWN ],
    [ 34, Moves.PSYCHIC ],
    [ 42, Moves.SCARY_FACE ],
    [ 52, Moves.METEOR_MASH ],
    [ 62, Moves.IRON_DEFENSE ],
    [ 72, Moves.AGILITY ],
    [ 82, Moves.HYPER_BEAM ],
  ],
  [Species.REGIROCK]: [
    [ 1, Moves.CHARGE_BEAM ],
    [ 1, Moves.ROCK_THROW ],
    [ 6, Moves.BULLDOZE ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 18, Moves.STOMP ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 30, Moves.CURSE ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 42, Moves.HAMMER_ARM ],
    [ 48, Moves.STONE_EDGE ],
    [ 54, Moves.SUPERPOWER ],
    [ 60, Moves.LOCK_ON ],
    [ 66, Moves.ZAP_CANNON ],
    [ 72, Moves.HYPER_BEAM ],
    [ 78, Moves.EXPLOSION ],
  ],
  [Species.REGICE]: [
    [ 1, Moves.CHARGE_BEAM ],
    [ 1, Moves.ICY_WIND ],
    [ 6, Moves.BULLDOZE ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 18, Moves.STOMP ],
    [ 24, Moves.ICE_BEAM ],
    [ 30, Moves.CURSE ],
    [ 36, Moves.AMNESIA ],
    [ 42, Moves.HAMMER_ARM ],
    [ 48, Moves.BLIZZARD ],
    [ 54, Moves.SUPERPOWER ],
    [ 60, Moves.LOCK_ON ],
    [ 66, Moves.ZAP_CANNON ],
    [ 72, Moves.HYPER_BEAM ],
    [ 78, Moves.EXPLOSION ],
  ],
  [Species.REGISTEEL]: [
    [ 1, Moves.CHARGE_BEAM ],
    [ 1, Moves.METAL_CLAW ],
    [ 6, Moves.BULLDOZE ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 18, Moves.STOMP ],
    [ 24, Moves.IRON_HEAD ],
    [ 24, Moves.FLASH_CANNON ],
    [ 30, Moves.CURSE ],
    [ 36, Moves.AMNESIA ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 42, Moves.HAMMER_ARM ],
    [ 48, Moves.HEAVY_SLAM ],
    [ 54, Moves.SUPERPOWER ],
    [ 60, Moves.LOCK_ON ],
    [ 66, Moves.ZAP_CANNON ],
    [ 72, Moves.HYPER_BEAM ],
    [ 78, Moves.EXPLOSION ],
  ],
  [Species.LATIAS]: [
    [ 1, Moves.STORED_POWER ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.PSYWAVE ],
    [ 5, Moves.HELPING_HAND ],
    [ 10, Moves.RECOVER ],
    [ 15, Moves.CONFUSION ],
    [ 20, Moves.TAILWIND ],
    [ 25, Moves.DRAGON_BREATH ],
    [ 30, Moves.WISH ],
    [ 35, Moves.MIST_BALL ],
    [ 40, Moves.ZEN_HEADBUTT ],
    [ 45, Moves.DRAGON_PULSE ],
    [ 50, Moves.HEAL_PULSE ],
    [ 55, Moves.REFLECT_TYPE ],
    [ 60, Moves.PSYCHIC ],
    [ 65, Moves.GUARD_SPLIT ],
    [ 70, Moves.HEALING_WISH ],
  ],
  [Species.LATIOS]: [
    [ 1, Moves.STORED_POWER ],
    [ 1, Moves.DRAGON_DANCE ],
    [ 1, Moves.HEAL_BLOCK ],
    [ 1, Moves.PSYWAVE ],
    [ 5, Moves.HELPING_HAND ],
    [ 10, Moves.RECOVER ],
    [ 15, Moves.CONFUSION ],
    [ 20, Moves.TAILWIND ],
    [ 25, Moves.DRAGON_BREATH ],
    [ 30, Moves.ALLY_SWITCH ],
    [ 35, Moves.LUSTER_PURGE ],
    [ 40, Moves.ZEN_HEADBUTT ],
    [ 45, Moves.DRAGON_PULSE ],
    [ 50, Moves.HEAL_PULSE ],
    [ 55, Moves.SIMPLE_BEAM ],
    [ 60, Moves.PSYCHIC ],
    [ 65, Moves.POWER_SPLIT ],
    [ 70, Moves.MEMENTO ],
  ],
  [Species.KYOGRE]: [
    [ 1, Moves.ORIGIN_PULSE ],
    [ 1, Moves.WATER_PULSE ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.BODY_SLAM ],
    [ 1, Moves.SCARY_FACE ],
    [ 9, Moves.AQUA_TAIL ],
    [ 18, Moves.CALM_MIND ],
    [ 27, Moves.MUDDY_WATER ],
    [ 36, Moves.ICE_BEAM ],
    [ 45, Moves.SHEER_COLD ],
    [ 54, Moves.AQUA_RING ],
    [ 72, Moves.HYDRO_PUMP ],
    [ 81, Moves.DOUBLE_EDGE ],
    [ 90, Moves.WATER_SPOUT ],
  ],
  [Species.GROUDON]: [
    [ 1, Moves.PRECIPICE_BLADES ],
    [ 1, Moves.MUD_SHOT ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.LAVA_PLUME ],
    [ 1, Moves.SCARY_FACE ],
    [ 9, Moves.EARTH_POWER ],
    [ 18, Moves.BULK_UP ],
    [ 27, Moves.EARTHQUAKE ],
    [ 36, Moves.HAMMER_ARM ],
    [ 45, Moves.FISSURE ],
    [ 54, Moves.REST ],
    [ 72, Moves.FIRE_BLAST ],
    [ 81, Moves.SOLAR_BEAM ],
    [ 90, Moves.ERUPTION ],
  ],
  [Species.RAYQUAZA]: [
    [ 1, Moves.DRAGON_ASCENT ],
    [ 1, Moves.TWISTER ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.AIR_SLASH ],
    [ 1, Moves.SCARY_FACE ],
    [ 9, Moves.CRUNCH ],
    [ 18, Moves.DRAGON_DANCE ],
    [ 27, Moves.EXTREME_SPEED ],
    [ 36, Moves.DRAGON_PULSE ],
    [ 45, Moves.HYPER_VOICE ],
    [ 54, Moves.REST ],
    [ 63, Moves.FLY ],
    [ 72, Moves.HURRICANE ],
    [ 81, Moves.OUTRAGE ],
    [ 90, Moves.HYPER_BEAM ],
  ],
  [Species.JIRACHI]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.WISH ],
    [ 7, Moves.SWIFT ],
    [ 21, Moves.LIFE_DEW ],
    [ 28, Moves.ZEN_HEADBUTT ],
    [ 35, Moves.GRAVITY ],
    [ 42, Moves.PSYCHIC ],
    [ 49, Moves.METEOR_MASH ],
    [ 56, Moves.HEALING_WISH ],
    [ 63, Moves.REST ],
    [ 70, Moves.FUTURE_SIGHT ],
    [ 77, Moves.DOUBLE_EDGE ],
    [ 84, Moves.COSMIC_POWER ],
    [ 91, Moves.LAST_RESORT ],
    [ 98, Moves.DOOM_DESIRE ],
  ],
  [Species.DEOXYS]: [
    [ 1, Moves.CONFUSION ], //Custom
    [ 1, Moves.LEER ],
    [ 1, Moves.WRAP ],
    [ 7, Moves.NIGHT_SHADE ],
    [ 13, Moves.TELEPORT ],
    [ 19, Moves.KNOCK_OFF ],
    [ 25, Moves.PSYSHOCK ],
    [ 31, Moves.PSYCHIC ],
    [ 37, Moves.GRAVITY ],
    [ 43, Moves.SKILL_SWAP ],
    [ 49, Moves.ZEN_HEADBUTT ],
    [ 55, Moves.COSMIC_POWER ],
    [ 61, Moves.RECOVER ],
    [ 67, Moves.PSYCHO_BOOST ],
    [ 73, Moves.HYPER_BEAM ],
  ],
  [Species.TURTWIG]: [
    [ 1, Moves.TACKLE ],
    [ 5, Moves.WITHDRAW ],
    [ 5, Moves.LEAFAGE ], //Custom, moved from 10 to 5, BDSP
    [ 9, Moves.GROWTH ], //Fill empty moveslot, from BDSP level 6
    [ 13, Moves.RAZOR_LEAF ],
    [ 17, Moves.CURSE ],
    [ 21, Moves.BITE ],
    [ 25, Moves.MEGA_DRAIN ],
    [ 29, Moves.LEECH_SEED ],
    [ 33, Moves.SYNTHESIS ],
    [ 37, Moves.CRUNCH ],
    [ 41, Moves.GIGA_DRAIN ],
    [ 45, Moves.LEAF_STORM ],
  ],
  [Species.GROTLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.LEAFAGE ],
    [ 13, Moves.RAZOR_LEAF ],
    [ 17, Moves.CURSE ],
    [ 22, Moves.BITE ],
    [ 27, Moves.MEGA_DRAIN ],
    [ 32, Moves.LEECH_SEED ],
    [ 37, Moves.SYNTHESIS ],
    [ 42, Moves.CRUNCH ],
    [ 47, Moves.GIGA_DRAIN ],
    [ 52, Moves.LEAF_STORM ],
  ],
  [Species.TORTERRA]: [
    [ EVOLVE_MOVE, Moves.EARTHQUAKE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.LEAFAGE ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.WOOD_HAMMER ],
    [ 17, Moves.CURSE ],
    [ 22, Moves.BITE ],
    [ 27, Moves.MEGA_DRAIN ],
    [ 33, Moves.LEECH_SEED ],
    [ 39, Moves.SYNTHESIS ],
    [ 45, Moves.CRUNCH ],
    [ 51, Moves.GIGA_DRAIN ],
    [ 57, Moves.LEAF_STORM ],
    [ 63, Moves.HEADLONG_RUSH ],
  ],
  [Species.CHIMCHAR]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.EMBER ], //Custom, moved from 7 to 5
    [ 9, Moves.TAUNT ],
    [ 15, Moves.FURY_SWIPES ],
    [ 17, Moves.FLAME_WHEEL ],
    [ 23, Moves.NASTY_PLOT ],
    [ 25, Moves.TORMENT ],
    [ 31, Moves.FACADE ],
    [ 33, Moves.FIRE_SPIN ],
    [ 39, Moves.ACROBATICS ],
    [ 41, Moves.SLACK_OFF ],
    [ 47, Moves.FLAMETHROWER ],
  ],
  [Species.MONFERNO]: [
    [ EVOLVE_MOVE, Moves.MACH_PUNCH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 9, Moves.TAUNT ],
    [ 16, Moves.FURY_SWIPES ],
    [ 19, Moves.FLAME_WHEEL ],
    [ 26, Moves.FEINT ],
    [ 29, Moves.TORMENT ],
    [ 36, Moves.CLOSE_COMBAT ],
    [ 39, Moves.FIRE_SPIN ],
    [ 46, Moves.ACROBATICS ],
    [ 49, Moves.SLACK_OFF ],
    [ 56, Moves.FLARE_BLITZ ],
  ],
  [Species.INFERNAPE]: [
    [ EVOLVE_MOVE, Moves.CLOSE_COMBAT ],
    [ RELEARN_MOVE, Moves.TAUNT ],
    [ RELEARN_MOVE, Moves.SLACK_OFF ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.MACH_PUNCH ],
    [ 16, Moves.FURY_SWIPES ],
    [ 19, Moves.FLAME_WHEEL ],
    [ 26, Moves.FEINT ],
    [ 29, Moves.TORMENT ],
    [ 42, Moves.FIRE_SPIN ],
    [ 47, Moves.FLARE_BLITZ ],
    [ 52, Moves.ACROBATICS ],
    [ 58, Moves.CALM_MIND ],
    [ 65, Moves.RAGING_FURY ],
  ],
  [Species.PIPLUP]: [
    [ 1, Moves.POUND ],
    [ 4, Moves.GROWL ],
    [ 5, Moves.WATER_GUN ], //Custom, moved from 8 to 5
    [ 11, Moves.CHARM ],
    [ 15, Moves.PECK ],
    [ 18, Moves.BUBBLE_BEAM ],
    [ 22, Moves.SWAGGER ],
    [ 25, Moves.FURY_ATTACK ],
    [ 29, Moves.BRINE ],
    [ 32, Moves.WHIRLPOOL ],
    [ 36, Moves.MIST ],
    [ 39, Moves.DRILL_PECK ],
    [ 43, Moves.HYDRO_PUMP ],
  ],
  [Species.PRINPLUP]: [
    [ EVOLVE_MOVE, Moves.METAL_CLAW ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 15, Moves.PECK ],
    [ 19, Moves.BUBBLE_BEAM ],
    [ 24, Moves.SWAGGER ],
    [ 28, Moves.FURY_ATTACK ],
    [ 33, Moves.BRINE ],
    [ 37, Moves.WHIRLPOOL ],
    [ 42, Moves.MIST ],
    [ 46, Moves.DRILL_PECK ],
    [ 50, Moves.HYDRO_PUMP ],
  ],
  [Species.EMPOLEON]: [
    [ EVOLVE_MOVE, Moves.AQUA_JET ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.METAL_CLAW ],
    [ 11, Moves.SWORDS_DANCE ],
    [ 15, Moves.PECK ],
    [ 19, Moves.BUBBLE_BEAM ],
    [ 24, Moves.SWAGGER ],
    [ 28, Moves.FURY_ATTACK ],
    [ 33, Moves.BRINE ],
    [ 39, Moves.WHIRLPOOL ],
    [ 46, Moves.MIST ],
    [ 52, Moves.DRILL_PECK ],
    [ 59, Moves.HYDRO_PUMP ],
    [ 66, Moves.WAVE_CRASH ],
  ],
  [Species.STARLY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 9, Moves.WING_ATTACK ],
    [ 13, Moves.DOUBLE_TEAM ],
    [ 17, Moves.ENDEAVOR ],
    [ 21, Moves.WHIRLWIND ],
    [ 25, Moves.AERIAL_ACE ],
    [ 29, Moves.TAKE_DOWN ],
    [ 33, Moves.AGILITY ],
    [ 37, Moves.BRAVE_BIRD ],
    [ 41, Moves.FINAL_GAMBIT ],
  ],
  [Species.STARAVIA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 9, Moves.WING_ATTACK ],
    [ 13, Moves.DOUBLE_TEAM ],
    [ 18, Moves.ENDEAVOR ],
    [ 23, Moves.WHIRLWIND ],
    [ 28, Moves.AERIAL_ACE ],
    [ 33, Moves.TAKE_DOWN ],
    [ 38, Moves.AGILITY ],
    [ 43, Moves.BRAVE_BIRD ],
    [ 48, Moves.FINAL_GAMBIT ],
  ],
  [Species.STARAPTOR]: [
    [ EVOLVE_MOVE, Moves.CLOSE_COMBAT ],
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 13, Moves.DOUBLE_TEAM ],
    [ 18, Moves.ENDEAVOR ],
    [ 23, Moves.WHIRLWIND ],
    [ 28, Moves.AERIAL_ACE ],
    [ 33, Moves.TAKE_DOWN ],
    [ 41, Moves.AGILITY ],
    [ 49, Moves.BRAVE_BIRD ],
    [ 57, Moves.FINAL_GAMBIT ],
  ],
  [Species.BIDOOF]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.DEFENSE_CURL ],
    [ 9, Moves.ROLLOUT ],
    [ 13, Moves.HEADBUTT ],
    [ 17, Moves.HYPER_FANG ],
    [ 21, Moves.YAWN ],
    [ 25, Moves.CRUNCH ],
    [ 29, Moves.TAKE_DOWN ],
    [ 33, Moves.SUPER_FANG ],
    [ 37, Moves.SWORDS_DANCE ],
    [ 41, Moves.AMNESIA ],
    [ 45, Moves.SUPERPOWER ],
    [ 49, Moves.CURSE ],
  ],
  [Species.BIBAREL]: [
    [ EVOLVE_MOVE, Moves.WATER_GUN ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.ROTOTILLER ],
    [ 5, Moves.DEFENSE_CURL ],
    [ 9, Moves.ROLLOUT ],
    [ 13, Moves.HEADBUTT ],
    [ 18, Moves.HYPER_FANG ],
    [ 23, Moves.YAWN ],
    [ 28, Moves.CRUNCH ],
    [ 33, Moves.TAKE_DOWN ],
    [ 38, Moves.SUPER_FANG ],
    [ 43, Moves.SWORDS_DANCE ],
    [ 48, Moves.AMNESIA ],
    [ 53, Moves.SUPERPOWER ],
    [ 58, Moves.CURSE ],
  ],
  [Species.KRICKETOT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.BIDE ],
    [ 6, Moves.STRUGGLE_BUG ],
    [ 16, Moves.BUG_BITE ],
  ],
  [Species.KRICKETUNE]: [
    [ EVOLVE_MOVE, Moves.FURY_CUTTER ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.BIDE ],
    [ 14, Moves.ABSORB ],
    [ 18, Moves.SING ],
    [ 22, Moves.FOCUS_ENERGY ],
    [ 26, Moves.SLASH ],
    [ 30, Moves.X_SCISSOR ],
    [ 34, Moves.SCREECH ],
    [ 36, Moves.FELL_STINGER ],
    [ 38, Moves.TAUNT ],
    [ 42, Moves.NIGHT_SLASH ],
    [ 44, Moves.STICKY_WEB ],
    [ 46, Moves.BUG_BUZZ ],
    [ 50, Moves.PERISH_SONG ],
  ],
  [Species.SHINX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 8, Moves.CHARGE ],
    [ 12, Moves.BITE ],
    [ 16, Moves.SPARK ],
    [ 20, Moves.ROAR ],
    [ 24, Moves.VOLT_SWITCH ],
    [ 28, Moves.SCARY_FACE ],
    [ 32, Moves.THUNDER_WAVE ],
    [ 36, Moves.CRUNCH ],
    [ 40, Moves.DISCHARGE ],
    [ 44, Moves.SWAGGER ],
    [ 48, Moves.WILD_CHARGE ],
  ],
  [Species.LUXIO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.CHARGE ],
    [ 12, Moves.BITE ],
    [ 18, Moves.SPARK ],
    [ 24, Moves.ROAR ],
    [ 31, Moves.VOLT_SWITCH ],
    [ 36, Moves.SCARY_FACE ],
    [ 42, Moves.THUNDER_WAVE ],
    [ 48, Moves.CRUNCH ],
    [ 54, Moves.DISCHARGE ],
    [ 60, Moves.SWAGGER ],
    [ 68, Moves.WILD_CHARGE ],
  ],
  [Species.LUXRAY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 12, Moves.BITE ],
    [ 18, Moves.SPARK ],
    [ 24, Moves.ROAR ],
    [ 33, Moves.VOLT_SWITCH ],
    [ 40, Moves.SCARY_FACE ],
    [ 48, Moves.THUNDER_WAVE ],
    [ 56, Moves.CRUNCH ],
    [ 64, Moves.DISCHARGE ],
    [ 72, Moves.SWAGGER ],
    [ 80, Moves.WILD_CHARGE ],
  ],
  [Species.BUDEW]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.WORRY_SEED ],
  ],
  [Species.ROSERADE]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.PETAL_DANCE ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.SYNTHESIS ],
    [ 1, Moves.INGRAIN ],
    [ 1, Moves.AROMATHERAPY ],
    [ 1, Moves.MAGICAL_LEAF ],
    [ 1, Moves.WORRY_SEED ],
    [ 1, Moves.TOXIC_SPIKES ],
    [ 1, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.GRASSY_TERRAIN ],
    [ 1, Moves.VENOM_DRENCH ],
  ],
  [Species.CRANIDOS]: [
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.LEER ],
    [ 6, Moves.FOCUS_ENERGY ],
    [ 10, Moves.PURSUIT ],
    [ 15, Moves.TAKE_DOWN ],
    [ 19, Moves.SCARY_FACE ],
    [ 24, Moves.ASSURANCE ],
    [ 28, Moves.CHIP_AWAY ],
    [ 33, Moves.ANCIENT_POWER ],
    [ 37, Moves.ZEN_HEADBUTT ],
    [ 42, Moves.SCREECH ],
    [ 46, Moves.HEAD_SMASH ],
  ],
  [Species.RAMPARDOS]: [
    [ EVOLVE_MOVE, Moves.ENDEAVOR ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.PURSUIT ],
    [ 15, Moves.TAKE_DOWN ],
    [ 19, Moves.SCARY_FACE ],
    [ 24, Moves.ASSURANCE ],
    [ 28, Moves.CHIP_AWAY ],
    [ 36, Moves.ANCIENT_POWER ],
    [ 43, Moves.ZEN_HEADBUTT ],
    [ 51, Moves.SCREECH ],
    [ 58, Moves.HEAD_SMASH ],
  ],
  [Species.SHIELDON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PROTECT ],
    [ 6, Moves.TAUNT ],
    [ 10, Moves.METAL_SOUND ],
    [ 15, Moves.TAKE_DOWN ],
    [ 19, Moves.IRON_DEFENSE ],
    [ 24, Moves.SWAGGER ],
    [ 28, Moves.ANCIENT_POWER ],
    [ 33, Moves.ENDURE ],
    [ 37, Moves.METAL_BURST ],
    [ 42, Moves.IRON_HEAD ],
    [ 46, Moves.HEAVY_SLAM ],
  ],
  [Species.BASTIODON]: [
    [ EVOLVE_MOVE, Moves.BLOCK ],
    [ RELEARN_MOVE, Moves.WIDE_GUARD ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.METAL_SOUND ],
    [ 15, Moves.TAKE_DOWN ],
    [ 19, Moves.IRON_DEFENSE ],
    [ 24, Moves.SWAGGER ],
    [ 28, Moves.ANCIENT_POWER ],
    [ 36, Moves.ENDURE ],
    [ 43, Moves.METAL_BURST ],
    [ 51, Moves.IRON_HEAD ],
    [ 58, Moves.HEAVY_SLAM ],
  ],
  [Species.BURMY]: [
    [ 1, Moves.PROTECT ],
    [ 1, Moves.STRUGGLE_BUG ], //Custom
    [ 10, Moves.TACKLE ],
    [ 15, Moves.BUG_BITE ],
    [ 20, Moves.STRING_SHOT ],
  ],
  [Species.WORMADAM]: [
    [ EVOLVE_MOVE, Moves.QUIVER_DANCE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SUCKER_PUNCH ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.PROTECT ],
    [ 10, Moves.TACKLE ],
    [ 20, Moves.STRING_SHOT ],
    [ 23, Moves.CONFUSION ],
    [ 26, Moves.RAZOR_LEAF ],
    [ 29, Moves.GROWTH ],
    [ 32, Moves.PSYBEAM ],
    [ 35, Moves.INFESTATION ],
    [ 38, Moves.FLAIL ],
    [ 41, Moves.ATTRACT ],
    [ 44, Moves.PSYCHIC ],
    [ 47, Moves.LEAF_STORM ],
    [ 50, Moves.BUG_BUZZ ],
  ],
  [Species.MOTHIM]: [
    [ EVOLVE_MOVE, Moves.QUIVER_DANCE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.BUG_BITE ],
    [ 10, Moves.PROTECT ],
    [ 15, Moves.BUG_BITE ],
    [ 20, Moves.STRING_SHOT ],
    [ 23, Moves.CONFUSION ],
    [ 26, Moves.GUST ],
    [ 29, Moves.POISON_POWDER ],
    [ 32, Moves.PSYBEAM ],
    [ 35, Moves.ROOST ],
    [ 38, Moves.STRUGGLE_BUG ],
    [ 41, Moves.AIR_SLASH ],
    [ 44, Moves.PSYCHIC ],
    [ 47, Moves.LUNGE ],
    [ 50, Moves.BUG_BUZZ ],
  ],
  [Species.COMBEE]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.STRUGGLE_BUG ],
  ],
  [Species.VESPIQUEN]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ 1, Moves.GUST ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.STRUGGLE_BUG ],
    [ 4, Moves.FURY_CUTTER ],
    [ 8, Moves.AROMATIC_MIST ],
    [ 12, Moves.FELL_STINGER ],
    [ 16, Moves.FURY_SWIPES ],
    [ 20, Moves.SWAGGER ],
    [ 24, Moves.ROOST ],
    [ 28, Moves.AIR_SLASH ],
    [ 32, Moves.POWER_GEM ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.ATTACK_ORDER ],
    [ 40, Moves.DEFEND_ORDER ],
    [ 40, Moves.HEAL_ORDER ],
    [ 44, Moves.DESTINY_BOND ],
  ],
  [Species.PACHIRISU]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.BIDE ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 9, Moves.CHARM ],
    [ 13, Moves.SPARK ],
    [ 17, Moves.ENDURE ],
    [ 19, Moves.NUZZLE ],
    [ 21, Moves.SWIFT ],
    [ 25, Moves.ELECTRO_BALL ],
    [ 29, Moves.SWEET_KISS ],
    [ 33, Moves.THUNDER_WAVE ],
    [ 37, Moves.SUPER_FANG ],
    [ 41, Moves.DISCHARGE ],
    [ 45, Moves.LAST_RESORT ],
    [ 49, Moves.THUNDER ],
  ],
  [Species.BUIZEL]: [
    [ 1, Moves.TACKLE ],
    [ 4, Moves.GROWL ],
    [ 7, Moves.SOAK ],
    [ 11, Moves.QUICK_ATTACK ],
    [ 15, Moves.WATER_GUN ],
    [ 18, Moves.BITE ],
    [ 21, Moves.SWIFT ],
    [ 24, Moves.AQUA_JET ],
    [ 27, Moves.DOUBLE_HIT ],
    [ 31, Moves.WHIRLPOOL ],
    [ 35, Moves.LIQUIDATION ],
    [ 38, Moves.AQUA_TAIL ],
    [ 41, Moves.AGILITY ],
    [ 45, Moves.HYDRO_PUMP ],
    [ 49, Moves.WAVE_CRASH ],
  ],
  [Species.FLOATZEL]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.SOAK ],
    [ 15, Moves.WATER_GUN ],
    [ 18, Moves.BITE ],
    [ 21, Moves.SWIFT ],
    [ 24, Moves.AQUA_JET ],
    [ 29, Moves.DOUBLE_HIT ],
    [ 35, Moves.WHIRLPOOL ],
    [ 41, Moves.LIQUIDATION ],
    [ 46, Moves.AQUA_TAIL ],
    [ 51, Moves.AGILITY ],
    [ 57, Moves.HYDRO_PUMP ],
    [ 62, Moves.WAVE_CRASH ],
  ],
  [Species.CHERUBI]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.MORNING_SUN ],
    [ 5, Moves.LEAFAGE ],
    [ 10, Moves.GROWTH ],
    [ 15, Moves.HELPING_HAND ],
    [ 20, Moves.MAGICAL_LEAF ],
    [ 26, Moves.LEECH_SEED ],
    [ 30, Moves.TAKE_DOWN ],
    [ 35, Moves.PETAL_BLIZZARD ],
    [ 40, Moves.WORRY_SEED ],
    [ 45, Moves.SOLAR_BEAM ],
  ],
  [Species.CHERRIM]: [
    [ EVOLVE_MOVE, Moves.SUNNY_DAY ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.MORNING_SUN ],
    [ 1, Moves.FLOWER_SHIELD ],
    [ 1, Moves.LEAFAGE ],
    [ 15, Moves.HELPING_HAND ],
    [ 20, Moves.MAGICAL_LEAF ],
    [ 28, Moves.LEECH_SEED ],
    [ 34, Moves.TAKE_DOWN ],
    [ 41, Moves.PETAL_BLIZZARD ],
    [ 48, Moves.WORRY_SEED ],
    [ 55, Moves.SOLAR_BEAM ],
    [ 62, Moves.PETAL_DANCE ],
  ],
  [Species.SHELLOS]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.MUD_SLAP ],
    [ 5, Moves.HARDEN ],
    [ 10, Moves.RECOVER ],
    [ 15, Moves.WATER_PULSE ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.BODY_SLAM ],
    [ 31, Moves.MUDDY_WATER ],
    [ 35, Moves.EARTH_POWER ],
    [ 40, Moves.RAIN_DANCE ],
    [ 45, Moves.MEMENTO ],
  ],
  [Species.GASTRODON]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.RECOVER ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.MUD_SPORT ],
    [ 15, Moves.WATER_PULSE ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.BODY_SLAM ],
    [ 33, Moves.MUDDY_WATER ],
    [ 39, Moves.EARTH_POWER ],
    [ 46, Moves.RAIN_DANCE ],
    [ 53, Moves.MEMENTO ],
  ],
  [Species.AMBIPOM]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.ASTONISH ],
    [ 11, Moves.BATON_PASS ],
    [ 15, Moves.TICKLE ],
    [ 18, Moves.FURY_SWIPES ],
    [ 22, Moves.SWIFT ],
    [ 25, Moves.SCREECH ],
    [ 29, Moves.AGILITY ],
    [ 32, Moves.DOUBLE_HIT ],
    [ 36, Moves.FLING ],
    [ 39, Moves.NASTY_PLOT ],
    [ 43, Moves.LAST_RESORT ],
  ],
  [Species.DRIFLOON]: [
    [ 1, Moves.MINIMIZE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.CONSTRICT ],
    [ 4, Moves.GUST ],
    [ 8, Moves.FOCUS_ENERGY ],
    [ 12, Moves.PAYBACK ],
    [ 16, Moves.HEX ],
    [ 20, Moves.SHADOW_BALL ],
    [ 24, Moves.STOCKPILE ],
    [ 24, Moves.SPIT_UP ],
    [ 24, Moves.SWALLOW ],
    [ 29, Moves.SELF_DESTRUCT ],
    [ 32, Moves.DESTINY_BOND ],
    [ 36, Moves.BATON_PASS ],
    [ 40, Moves.TAILWIND ],
    [ 44, Moves.EXPLOSION ],
  ],
  [Species.DRIFBLIM]: [
    [ EVOLVE_MOVE, Moves.PHANTOM_FORCE ],
    [ 1, Moves.GUST ],
    [ 1, Moves.MINIMIZE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.CONSTRICT ],
    [ 1, Moves.STRENGTH_SAP ],
    [ 12, Moves.PAYBACK ],
    [ 16, Moves.HEX ],
    [ 20, Moves.SHADOW_BALL ],
    [ 24, Moves.STOCKPILE ],
    [ 24, Moves.SPIT_UP ],
    [ 24, Moves.SWALLOW ],
    [ 31, Moves.SELF_DESTRUCT ],
    [ 36, Moves.DESTINY_BOND ],
    [ 42, Moves.BATON_PASS ],
    [ 48, Moves.TAILWIND ],
    [ 54, Moves.EXPLOSION ],
  ],
  [Species.BUNEARY]: [
    [ 1, Moves.FRUSTRATION ],
    [ 1, Moves.POUND ],
    [ 1, Moves.SPLASH ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 8, Moves.BABY_DOLL_EYES ],
    [ 12, Moves.AFTER_YOU ],
    [ 16, Moves.QUICK_ATTACK ],
    [ 20, Moves.DOUBLE_KICK ],
    [ 24, Moves.CHARM ],
    [ 28, Moves.BATON_PASS ],
    [ 32, Moves.HEADBUTT ],
    [ 36, Moves.AGILITY ],
    [ 40, Moves.ENTRAINMENT ],
    [ 44, Moves.FLATTER ],
    [ 48, Moves.BOUNCE ],
    [ 52, Moves.HEALING_WISH ],
  ],
  [Species.LOPUNNY]: [
    [ EVOLVE_MOVE, Moves.RETURN ],
    [ 1, Moves.POUND ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.MIRROR_COAT ],
    [ 1, Moves.MAGIC_COAT ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 1, Moves.ROTOTILLER ],
    [ 12, Moves.AFTER_YOU ],
    [ 16, Moves.QUICK_ATTACK ],
    [ 20, Moves.DOUBLE_KICK ],
    [ 24, Moves.CHARM ],
    [ 28, Moves.BATON_PASS ],
    [ 32, Moves.HEADBUTT ],
    [ 36, Moves.AGILITY ],
    [ 40, Moves.ENTRAINMENT ],
    [ 44, Moves.FLATTER ],
    [ 48, Moves.BOUNCE ],
    [ 52, Moves.HEALING_WISH ],
    [ 56, Moves.HIGH_JUMP_KICK ],
  ],
  [Species.MISMAGIUS]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.SPITE ],
    [ 1, Moves.PSYWAVE ],
    [ 1, Moves.LUCKY_CHANT ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.MAGICAL_LEAF ],
    [ 1, Moves.POWER_GEM ],
    [ 1, Moves.PHANTOM_FORCE ],
    [ 1, Moves.MYSTICAL_FIRE ],
  ],
  [Species.HONCHKROW]: [
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.HAZE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.SUCKER_PUNCH ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.QUASH ],
    [ 1, Moves.PURSUIT ],
    [ 25, Moves.SWAGGER ],
    [ 35, Moves.NASTY_PLOT ],
    [ 45, Moves.FOUL_PLAY ],
    [ 55, Moves.DARK_PULSE ],
    [ 65, Moves.COMEUPPANCE ],
  ],
  [Species.GLAMEOW]: [
    [ 1, Moves.FAKE_OUT ],
    [ 5, Moves.SCRATCH ],
    [ 8, Moves.GROWL ],
    [ 13, Moves.HYPNOSIS ],
    [ 17, Moves.AERIAL_ACE ],
    [ 20, Moves.FURY_SWIPES ],
    [ 25, Moves.CHARM ],
    [ 29, Moves.TAUNT ],
    [ 32, Moves.RETALIATE ],
    [ 37, Moves.SLASH ],
    [ 41, Moves.SUCKER_PUNCH ],
    [ 44, Moves.ATTRACT ],
    [ 48, Moves.HONE_CLAWS ],
    [ 50, Moves.PLAY_ROUGH ],
  ],
  [Species.PURUGLY]: [
    [ EVOLVE_MOVE, Moves.SWAGGER ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.PLAY_ROUGH ],
    [ 1, Moves.SUCKER_PUNCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 13, Moves.HYPNOSIS ],
    [ 17, Moves.AERIAL_ACE ],
    [ 20, Moves.FURY_SWIPES ],
    [ 25, Moves.CHARM ],
    [ 29, Moves.TAUNT ],
    [ 32, Moves.RETALIATE ],
    [ 37, Moves.SLASH ],
    [ 45, Moves.BODY_SLAM ],
    [ 52, Moves.ATTRACT ],
    [ 60, Moves.HONE_CLAWS ],
  ],
  [Species.CHINGLING]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.PSYWAVE ], //Custom
    [ 4, Moves.GROWL ],
    [ 7, Moves.ASTONISH ],
    [ 10, Moves.CONFUSION ],
    [ 13, Moves.YAWN ],
    [ 16, Moves.LAST_RESORT ],
    [ 19, Moves.ENTRAINMENT ],
    [ 32, Moves.UPROAR ],
  ],
  [Species.STUNKY]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.POISON_GAS ],
    [ 3, Moves.FEINT ],
    [ 6, Moves.SMOKESCREEN ],
    [ 9, Moves.ACID_SPRAY ],
    [ 12, Moves.FURY_SWIPES ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 18, Moves.BITE ],
    [ 21, Moves.VENOSHOCK ],
    [ 24, Moves.SCREECH ],
    [ 27, Moves.TOXIC ],
    [ 30, Moves.SUCKER_PUNCH ],
    [ 33, Moves.MEMENTO ],
    [ 36, Moves.NIGHT_SLASH ],
    [ 39, Moves.BELCH ],
    [ 42, Moves.EXPLOSION ],
  ],
  [Species.SKUNTANK]: [
    [ EVOLVE_MOVE, Moves.FLAMETHROWER ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.FEINT ],
    [ 12, Moves.FURY_SWIPES ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 18, Moves.BITE ],
    [ 21, Moves.VENOSHOCK ],
    [ 24, Moves.SCREECH ],
    [ 27, Moves.TOXIC ],
    [ 30, Moves.SUCKER_PUNCH ],
    [ 33, Moves.MEMENTO ],
    [ 38, Moves.NIGHT_SLASH ],
    [ 43, Moves.BELCH ],
    [ 48, Moves.EXPLOSION ],
  ],
  [Species.BRONZOR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CONFUSION ],
    [ 4, Moves.CONFUSE_RAY ],
    [ 8, Moves.PAYBACK ],
    [ 12, Moves.IMPRISON ],
    [ 16, Moves.GYRO_BALL ],
    [ 20, Moves.HYPNOSIS ],
    [ 24, Moves.SAFEGUARD ],
    [ 28, Moves.EXTRASENSORY ],
    [ 32, Moves.HEAVY_SLAM ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 40, Moves.METAL_SOUND ],
    [ 44, Moves.FUTURE_SIGHT ],
    [ 45, Moves.HEAL_BLOCK ],
  ],
  [Species.BRONZONG]: [
    [ EVOLVE_MOVE, Moves.BLOCK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.SUNNY_DAY ],
    [ 1, Moves.WEATHER_BALL ],
    [ 1, Moves.PAYBACK ],
    [ 12, Moves.IMPRISON ],
    [ 16, Moves.GYRO_BALL ],
    [ 20, Moves.HYPNOSIS ],
    [ 24, Moves.SAFEGUARD ],
    [ 28, Moves.EXTRASENSORY ],
    [ 32, Moves.HEAVY_SLAM ],
    [ 38, Moves.IRON_DEFENSE ],
    [ 44, Moves.METAL_SOUND ],
    [ 50, Moves.FUTURE_SIGHT ],
    [ 52, Moves.HEAL_BLOCK ],
    [ 56, Moves.RAIN_DANCE ],
  ],
  [Species.BONSLY]: [
    [ 1, Moves.FAKE_TEARS ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.TACKLE ], //Custom
    [ 4, Moves.FLAIL ],
    [ 8, Moves.ROCK_THROW ],
    [ 12, Moves.BLOCK ],
    [ 16, Moves.MIMIC ],
    [ 20, Moves.ROCK_TOMB ],
    [ 24, Moves.TEARFUL_LOOK ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.ROCK_SLIDE ],
    [ 36, Moves.LOW_KICK ],
    [ 40, Moves.COUNTER ],
    [ 44, Moves.DOUBLE_EDGE ],
  ],
  [Species.MIME_JR]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.BARRIER ],
    [ 1, Moves.TICKLE ], //USUM
    [ 4, Moves.BATON_PASS ],
    [ 8, Moves.ENCORE ],
    [ 12, Moves.CONFUSION ],
    [ 16, Moves.MIMIC ], //Custom, swapped with Role Play to be closer to USUM
    [ 20, Moves.PROTECT ],
    [ 24, Moves.RECYCLE ],
    [ 28, Moves.PSYBEAM ],
    [ 32, Moves.ROLE_PLAY ], //Custom, swapped with Mimic
    [ 36, Moves.LIGHT_SCREEN ],
    [ 36, Moves.REFLECT ],
    [ 36, Moves.SAFEGUARD ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 44, Moves.DAZZLING_GLEAM ],
    [ 48, Moves.PSYCHIC ],
    [ 52, Moves.TEETER_DANCE ],
  ],
  [Species.HAPPINY]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.COPYCAT ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 8, Moves.SWEET_KISS ],
    [ 12, Moves.DISARMING_VOICE ],
    [ 16, Moves.COVET ],
    [ 20, Moves.CHARM ],
  ],
  [Species.CHATOT]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.HYPER_VOICE ],
    [ 1, Moves.CHATTER ],
    [ 1, Moves.CONFIDE ],
    [ 5, Moves.GROWL ],
    [ 9, Moves.MIRROR_MOVE ],
    [ 13, Moves.SING ],
    [ 17, Moves.FURY_ATTACK ],
    [ 29, Moves.ROUND ],
    [ 33, Moves.MIMIC ],
    [ 37, Moves.ECHOED_VOICE ],
    [ 41, Moves.ROOST ],
    [ 45, Moves.UPROAR ],
    [ 49, Moves.SYNCHRONOISE ],
    [ 50, Moves.FEATHER_DANCE ],
  ],
  [Species.SPIRITOMB]: [
    [ 1, Moves.NIGHT_SHADE ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.PURSUIT ],
    [ 5, Moves.SHADOW_SNEAK ],
    [ 10, Moves.SPITE ],
    [ 15, Moves.PAYBACK ],
    [ 20, Moves.NASTY_PLOT ],
    [ 25, Moves.HEX ],
    [ 30, Moves.MEMENTO ],
    [ 35, Moves.SUCKER_PUNCH ],
    [ 40, Moves.CURSE ],
    [ 45, Moves.SHADOW_BALL ],
    [ 50, Moves.DARK_PULSE ],
    [ 55, Moves.HYPNOSIS ],
    [ 60, Moves.DREAM_EATER ],
  ],
  [Species.GIBLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SAND_TOMB ],
    [ 6, Moves.SAND_ATTACK ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 18, Moves.BULLDOZE ],
    [ 25, Moves.BITE ],
    [ 30, Moves.SLASH ],
    [ 36, Moves.DRAGON_CLAW ],
    [ 42, Moves.DIG ],
    [ 48, Moves.SANDSTORM ],
    [ 54, Moves.TAKE_DOWN ],
    [ 60, Moves.DRAGON_RUSH ],
  ],
  [Species.GABITE]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.SAND_TOMB ],
    [ 1, Moves.DUAL_CHOP ],
    [ 18, Moves.BULLDOZE ],
    [ 27, Moves.BITE ],
    [ 34, Moves.SLASH ],
    [ 42, Moves.DRAGON_CLAW ],
    [ 50, Moves.DIG ],
    [ 58, Moves.SANDSTORM ],
    [ 66, Moves.TAKE_DOWN ],
    [ 74, Moves.DRAGON_RUSH ],
  ],
  [Species.GARCHOMP]: [
    [ EVOLVE_MOVE, Moves.CRUNCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.SAND_TOMB ],
    [ 1, Moves.DUAL_CHOP ],
    [ 18, Moves.BULLDOZE ],
    [ 27, Moves.BITE ],
    [ 34, Moves.SLASH ],
    [ 42, Moves.DRAGON_CLAW ],
    [ 52, Moves.DIG ],
    [ 62, Moves.SANDSTORM ],
    [ 72, Moves.TAKE_DOWN ],
    [ 82, Moves.DRAGON_RUSH ],
  ],
  [Species.MUNCHLAX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LICK ],
    [ 1, Moves.ODOR_SLEUTH ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 8, Moves.RECYCLE ],
    [ 12, Moves.COVET ],
    [ 16, Moves.BITE ],
    [ 20, Moves.STOCKPILE ],
    [ 20, Moves.SWALLOW ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.BODY_SLAM ],
    [ 32, Moves.FLING ],
    [ 36, Moves.AMNESIA ],
    [ 40, Moves.METRONOME ],
    [ 44, Moves.FLAIL ],
    [ 48, Moves.BELLY_DRUM ],
    [ 52, Moves.LAST_RESORT ],
  ],
  [Species.RIOLU]: [
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.ENDURE ],
    [ 4, Moves.FEINT ],
    [ 8, Moves.METAL_CLAW ],
    [ 12, Moves.COUNTER ],
    [ 16, Moves.WORK_UP ],
    [ 20, Moves.ROCK_SMASH ],
    [ 24, Moves.VACUUM_WAVE ],
    [ 28, Moves.SCREECH ],
    [ 32, Moves.QUICK_GUARD ],
    [ 36, Moves.FORCE_PALM ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 44, Moves.HELPING_HAND ],
    [ 48, Moves.COPYCAT ],
    [ 52, Moves.FINAL_GAMBIT ],
    [ 56, Moves.REVERSAL ],
  ],
  [Species.LUCARIO]: [
    [ EVOLVE_MOVE, Moves.AURA_SPHERE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.SCREECH ],
    [ 1, Moves.REVERSAL ],
    [ 1, Moves.DETECT ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.VACUUM_WAVE ],
    [ 1, Moves.FINAL_GAMBIT ],
    [ 1, Moves.LIFE_DEW ],
    [ 12, Moves.COUNTER ],
    [ 16, Moves.WORK_UP ],
    [ 20, Moves.FORCE_PALM ],
    [ 24, Moves.CALM_MIND ],
    [ 28, Moves.METAL_SOUND ],
    [ 32, Moves.QUICK_GUARD ],
    [ 36, Moves.BONE_RUSH ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 44, Moves.HEAL_PULSE ],
    [ 48, Moves.METEOR_MASH ],
    [ 52, Moves.DRAGON_PULSE ],
    [ 56, Moves.EXTREME_SPEED ],
    [ 60, Moves.CLOSE_COMBAT ],
  ],
  [Species.HIPPOPOTAS]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 4, Moves.BITE ],
    [ 8, Moves.YAWN ],
    [ 12, Moves.SAND_TOMB ],
    [ 16, Moves.DIG ],
    [ 20, Moves.CRUNCH ],
    [ 24, Moves.SANDSTORM ],
    [ 28, Moves.TAKE_DOWN ],
    [ 32, Moves.ROAR ],
    [ 36, Moves.REST ],
    [ 40, Moves.EARTHQUAKE ],
    [ 44, Moves.DOUBLE_EDGE ],
    [ 48, Moves.FISSURE ],
    [ 52, Moves.SLACK_OFF ],
  ],
  [Species.HIPPOWDON]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 12, Moves.SAND_TOMB ],
    [ 16, Moves.DIG ],
    [ 20, Moves.CRUNCH ],
    [ 24, Moves.SANDSTORM ],
    [ 28, Moves.TAKE_DOWN ],
    [ 32, Moves.ROAR ],
    [ 38, Moves.REST ],
    [ 44, Moves.EARTHQUAKE ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 56, Moves.FISSURE ],
    [ 62, Moves.SLACK_OFF ],
  ],
  [Species.SKORUPI]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 3, Moves.HONE_CLAWS ],
    [ 6, Moves.FELL_STINGER ],
    [ 9, Moves.POISON_FANG ],
    [ 12, Moves.BITE ],
    [ 15, Moves.TOXIC_SPIKES ],
    [ 18, Moves.BUG_BITE ],
    [ 21, Moves.VENOSHOCK ],
    [ 24, Moves.KNOCK_OFF ],
    [ 27, Moves.SCARY_FACE ],
    [ 30, Moves.PIN_MISSILE ],
    [ 33, Moves.TOXIC ],
    [ 36, Moves.NIGHT_SLASH ],
    [ 39, Moves.CROSS_POISON ],
    [ 42, Moves.X_SCISSOR ],
    [ 45, Moves.ACUPRESSURE ],
    [ 48, Moves.CRUNCH ],
  ],
  [Species.DRAPION]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.HONE_CLAWS ],
    [ 1, Moves.FELL_STINGER ],
    [ 9, Moves.POISON_FANG ],
    [ 12, Moves.BITE ],
    [ 15, Moves.TOXIC_SPIKES ],
    [ 18, Moves.BUG_BITE ],
    [ 21, Moves.VENOSHOCK ],
    [ 24, Moves.KNOCK_OFF ],
    [ 27, Moves.SCARY_FACE ],
    [ 30, Moves.PIN_MISSILE ],
    [ 33, Moves.TOXIC ],
    [ 36, Moves.NIGHT_SLASH ],
    [ 39, Moves.CROSS_POISON ],
    [ 44, Moves.X_SCISSOR ],
    [ 49, Moves.ACUPRESSURE ],
    [ 54, Moves.CRUNCH ],
  ],
  [Species.CROAGUNK]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.MUD_SLAP ],
    [ 4, Moves.ASTONISH ],
    [ 8, Moves.TAUNT ],
    [ 12, Moves.FLATTER ],
    [ 16, Moves.LOW_KICK ],
    [ 20, Moves.VENOSHOCK ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.SWAGGER ],
    [ 32, Moves.POISON_JAB ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.NASTY_PLOT ],
    [ 44, Moves.SLUDGE_BOMB ],
    [ 48, Moves.BELCH ],
  ],
  [Species.TOXICROAK]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.ASTONISH ],
    [ 12, Moves.FLATTER ],
    [ 16, Moves.LOW_KICK ],
    [ 20, Moves.VENOSHOCK ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.SWAGGER ],
    [ 32, Moves.POISON_JAB ],
    [ 36, Moves.TOXIC ],
    [ 42, Moves.NASTY_PLOT ],
    [ 48, Moves.SLUDGE_BOMB ],
    [ 54, Moves.BELCH ],
  ],
  [Species.CARNIVINE]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.LEAFAGE ], //Custom
    [ 7, Moves.BITE ],
    [ 11, Moves.VINE_WHIP ],
    [ 17, Moves.SWEET_SCENT ],
    [ 21, Moves.INGRAIN ],
    [ 27, Moves.GRASS_KNOT ],
    [ 31, Moves.LEAF_TORNADO ],
    [ 37, Moves.STOCKPILE ],
    [ 37, Moves.SPIT_UP ],
    [ 37, Moves.SWALLOW ],
    [ 41, Moves.CRUNCH ],
    [ 47, Moves.SEED_BOMB ],
    [ 50, Moves.POWER_WHIP ],
  ],
  [Species.FINNEON]: [
    [ 1, Moves.POUND ],
    [ 6, Moves.WATER_GUN ],
    [ 13, Moves.RAIN_DANCE ],
    [ 17, Moves.GUST ],
    [ 22, Moves.WATER_PULSE ],
    [ 26, Moves.ATTRACT ],
    [ 29, Moves.SAFEGUARD ],
    [ 33, Moves.AQUA_RING ],
    [ 38, Moves.WHIRLPOOL ],
    [ 42, Moves.U_TURN ],
    [ 45, Moves.BOUNCE ],
    [ 49, Moves.TAILWIND ],
    [ 54, Moves.SOAK ],
  ],
  [Species.LUMINEON]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GUST ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SOAK ],
    [ 13, Moves.RAIN_DANCE ],
    [ 22, Moves.WATER_PULSE ],
    [ 26, Moves.ATTRACT ],
    [ 29, Moves.SAFEGUARD ],
    [ 35, Moves.AQUA_RING ],
    [ 42, Moves.WHIRLPOOL ],
    [ 48, Moves.U_TURN ],
    [ 53, Moves.BOUNCE ],
    [ 59, Moves.TAILWIND ],
  ],
  [Species.MANTYKE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.SUPERSONIC ],
    [ 8, Moves.WING_ATTACK ],
    [ 12, Moves.WATER_PULSE ],
    [ 16, Moves.WIDE_GUARD ],
    [ 20, Moves.AGILITY ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 28, Moves.HEADBUTT ],
    [ 32, Moves.AIR_SLASH ],
    [ 36, Moves.AQUA_RING ],
    [ 40, Moves.BOUNCE ],
    [ 44, Moves.TAKE_DOWN ],
    [ 48, Moves.HYDRO_PUMP ],
  ],
  [Species.SNOVER]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.POWDER_SNOW ],
    [ 5, Moves.LEAFAGE ],
    [ 10, Moves.MIST ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.RAZOR_LEAF ],
    [ 25, Moves.ICY_WIND ],
    [ 30, Moves.SWAGGER ],
    [ 35, Moves.INGRAIN ],
    [ 41, Moves.WOOD_HAMMER ],
    [ 45, Moves.BLIZZARD ],
    [ 50, Moves.SHEER_COLD ],
  ],
  [Species.ABOMASNOW]: [
    [ EVOLVE_MOVE, Moves.ICE_PUNCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.MIST ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.LEAFAGE ],
    [ 1, Moves.AURORA_VEIL ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.RAZOR_LEAF ],
    [ 25, Moves.ICY_WIND ],
    [ 30, Moves.SWAGGER ],
    [ 35, Moves.INGRAIN ],
    [ 43, Moves.WOOD_HAMMER ],
    [ 49, Moves.BLIZZARD ],
    [ 56, Moves.SHEER_COLD ],
  ],
  [Species.WEAVILE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.SLASH ],
    [ 1, Moves.BEAT_UP ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.ASSURANCE ],
    [ 1, Moves.ICE_SHARD ],
    [ 1, Moves.EMBARGO ],
    [ 18, Moves.METAL_CLAW ],
    [ 24, Moves.ICY_WIND ],
    [ 30, Moves.FURY_SWIPES ],
    [ 36, Moves.HONE_CLAWS ],
    [ 42, Moves.FLING ],
    [ 48, Moves.NASTY_PLOT ],
    [ 54, Moves.SCREECH ],
    [ 60, Moves.NIGHT_SLASH ],
    [ 66, Moves.DARK_PULSE ],
  ],
  [Species.MAGNEZONE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.BARRIER ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.MIRROR_COAT ],
    [ 1, Moves.MAGNETIC_FLUX ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 12, Moves.ELECTRO_BALL ],
    [ 16, Moves.GYRO_BALL ],
    [ 20, Moves.SPARK ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.MAGNET_RISE ],
    [ 34, Moves.FLASH_CANNON ],
    [ 40, Moves.DISCHARGE ],
    [ 46, Moves.METAL_SOUND ],
    [ 52, Moves.LIGHT_SCREEN ],
    [ 58, Moves.LOCK_ON ],
    [ 64, Moves.ZAP_CANNON ],
  ],
  [Species.LICKILICKY]: [
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.LICK ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.WRING_OUT ],
    [ 6, Moves.REST ],
    [ 18, Moves.WRAP ],
    [ 24, Moves.DISABLE ],
    [ 30, Moves.STOMP ],
    [ 36, Moves.KNOCK_OFF ],
    [ 42, Moves.SCREECH ],
    [ 48, Moves.SLAM ],
    [ 54, Moves.POWER_WHIP ],
    [ 60, Moves.BELLY_DRUM ],
  ],
  [Species.RHYPERIOR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.HAMMER_ARM ],
    [ 1, Moves.SMACK_DOWN ],
    [ 1, Moves.BULLDOZE ],
    [ 15, Moves.HORN_ATTACK ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.STOMP ],
    [ 30, Moves.ROCK_BLAST ],
    [ 35, Moves.DRILL_RUN ],
    [ 40, Moves.TAKE_DOWN ],
    [ 47, Moves.EARTHQUAKE ],
    [ 54, Moves.STONE_EDGE ],
    [ 61, Moves.MEGAHORN ],
    [ 68, Moves.HORN_DRILL ],
    [ 75, Moves.ROCK_WRECKER ],
  ],
  [Species.TANGROWTH]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.BLOCK ],
    [ 1, Moves.CONSTRICT ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.VINE_WHIP ],
    [ 20, Moves.POISON_POWDER ],
    [ 24, Moves.DOUBLE_HIT ],
    [ 28, Moves.KNOCK_OFF ],
    [ 32, Moves.GIGA_DRAIN ],
    [ 34, Moves.ANCIENT_POWER ],
    [ 36, Moves.SLEEP_POWDER ],
    [ 40, Moves.SLAM ],
    [ 44, Moves.TICKLE ],
    [ 48, Moves.POWER_WHIP ],
    [ 52, Moves.INGRAIN ],
    [ 56, Moves.GRASSY_TERRAIN ],
  ],
  [Species.ELECTIVIRE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.ION_DELUGE ],
    [ 12, Moves.SWIFT ],
    [ 16, Moves.SHOCK_WAVE ],
    [ 20, Moves.THUNDER_WAVE ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.THUNDER_PUNCH ],
    [ 34, Moves.DISCHARGE ],
    [ 40, Moves.LOW_KICK ],
    [ 46, Moves.THUNDERBOLT ],
    [ 52, Moves.LIGHT_SCREEN ],
    [ 58, Moves.THUNDER ],
    [ 64, Moves.GIGA_IMPACT ],
  ],
  [Species.MAGMORTAR]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.SMOG ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 16, Moves.FLAME_WHEEL ],
    [ 20, Moves.CONFUSE_RAY ],
    [ 24, Moves.SCARY_FACE ],
    [ 28, Moves.FIRE_PUNCH ],
    [ 34, Moves.LAVA_PLUME ],
    [ 40, Moves.LOW_KICK ],
    [ 46, Moves.FLAMETHROWER ],
    [ 52, Moves.SUNNY_DAY ],
    [ 58, Moves.FIRE_BLAST ],
    [ 64, Moves.HYPER_BEAM ],
  ],
  [Species.TOGEKISS]: [
    [ EVOLVE_MOVE, Moves.AIR_SLASH ],
    [ 1, Moves.POUND ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.METRONOME ],
    [ 1, Moves.SKY_ATTACK ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.EXTREME_SPEED ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.FOLLOW_ME ],
    [ 1, Moves.WISH ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.LAST_RESORT ],
    [ 1, Moves.AURA_SPHERE ],
    [ 1, Moves.AFTER_YOU ],
    [ 1, Moves.FAIRY_WIND ],
    [ 1, Moves.LIFE_DEW ],
  ],
  [Species.YANMEGA]: [
    [ RELEARN_MOVE, Moves.HYPNOSIS ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.AIR_SLASH ],
    [ 1, Moves.BUG_BUZZ ],
    [ 14, Moves.QUICK_ATTACK ],
    [ 17, Moves.DETECT ],
    [ 22, Moves.SUPERSONIC ],
    [ 27, Moves.UPROAR ],
    [ 30, Moves.BUG_BITE ],
    [ 33, Moves.ANCIENT_POWER ],
    [ 38, Moves.FEINT ],
    [ 43, Moves.SLASH ],
    [ 46, Moves.SCREECH ],
    [ 49, Moves.U_TURN ],
  ],
  [Species.LEAFEON]: [
    [ EVOLVE_MOVE, Moves.SAPPY_SEED ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.RAZOR_LEAF ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.LEECH_SEED ],
    [ 30, Moves.MAGICAL_LEAF ],
    [ 35, Moves.SYNTHESIS ],
    [ 40, Moves.SUNNY_DAY ],
    [ 45, Moves.GIGA_DRAIN ],
    [ 50, Moves.SWORDS_DANCE ],
    [ 55, Moves.LEAF_BLADE ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.GLACEON]: [
    [ EVOLVE_MOVE, Moves.FREEZY_FROST ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SWIFT ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.ICY_WIND ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.ICE_SHARD ],
    [ 30, Moves.BITE ],
    [ 35, Moves.ICE_FANG ],
    [ 40, Moves.SNOWSCAPE ],
    [ 45, Moves.FREEZE_DRY ],
    [ 50, Moves.MIRROR_COAT ],
    [ 55, Moves.BLIZZARD ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.GLISCOR]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POISON_JAB ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 16, Moves.FURY_CUTTER ],
    [ 19, Moves.KNOCK_OFF ],
    [ 22, Moves.ACROBATICS ],
    [ 27, Moves.NIGHT_SLASH ],
    [ 30, Moves.U_TURN ],
    [ 35, Moves.SCREECH ],
    [ 40, Moves.X_SCISSOR ],
    [ 45, Moves.CRABHAMMER ],
    [ 50, Moves.SWORDS_DANCE ],
  ],
  [Species.MAMOSWINE]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_HIT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.ODOR_SLEUTH ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.ICE_FANG ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.MIST ],
    [ 25, Moves.ENDURE ],
    [ 30, Moves.ICY_WIND ],
    [ 37, Moves.AMNESIA ],
    [ 44, Moves.TAKE_DOWN ],
    [ 51, Moves.EARTHQUAKE ],
    [ 58, Moves.BLIZZARD ],
    [ 65, Moves.THRASH ],
  ],
  [Species.PORYGON_Z]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.CONVERSION ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.MAGNET_RISE ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.TRICK_ROOM ],
    [ 15, Moves.THUNDER_SHOCK ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.CONVERSION_2 ],
    [ 30, Moves.AGILITY ],
    [ 35, Moves.RECOVER ],
    [ 40, Moves.DISCHARGE ],
    [ 45, Moves.TRI_ATTACK ],
    [ 50, Moves.MAGIC_COAT ],
    [ 55, Moves.LOCK_ON ],
    [ 60, Moves.ZAP_CANNON ],
    [ 65, Moves.HYPER_BEAM ],
  ],
  [Species.GALLADE]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PSYBEAM ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.PSYCHIC ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.DREAM_EATER ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.FUTURE_SIGHT ],
    [ 1, Moves.IMPRISON ],
    [ 1, Moves.CALM_MIND ],
    [ 1, Moves.LEAF_BLADE ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.SACRED_SWORD ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.DRAINING_KISS ],
    [ 1, Moves.LIFE_DEW ],
    [ 1, Moves.AQUA_CUTTER ],
    [ 9, Moves.HELPING_HAND ],
    [ 12, Moves.FEINT ],
    [ 15, Moves.TELEPORT ],
    [ 18, Moves.AERIAL_ACE ],
    [ 23, Moves.FALSE_SWIPE ],
    [ 28, Moves.PROTECT ],
    [ 35, Moves.SWORDS_DANCE ],
    [ 42, Moves.PSYCHO_CUT ],
    [ 49, Moves.HEAL_PULSE ],
    [ 56, Moves.WIDE_GUARD ],
    [ 56, Moves.QUICK_GUARD ],
    [ 63, Moves.CLOSE_COMBAT ],
  ],
  [Species.PROBOPASS]: [
    [ EVOLVE_MOVE, Moves.TRI_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.BLOCK ],
    [ 1, Moves.GRAVITY ],
    [ 1, Moves.MAGNET_RISE ],
    [ 1, Moves.WIDE_GUARD ],
    [ 1, Moves.MAGNETIC_FLUX ],
    [ 13, Moves.THUNDER_WAVE ],
    [ 16, Moves.REST ],
    [ 19, Moves.SPARK ],
    [ 22, Moves.ROCK_SLIDE ],
    [ 25, Moves.POWER_GEM ],
    [ 28, Moves.ROCK_BLAST ],
    [ 31, Moves.DISCHARGE ],
    [ 34, Moves.SANDSTORM ],
    [ 37, Moves.EARTH_POWER ],
    [ 40, Moves.STONE_EDGE ],
    [ 43, Moves.ZAP_CANNON ],
    [ 43, Moves.LOCK_ON ],
  ],
  [Species.DUSKNOIR]: [
    [ 1, Moves.FIRE_PUNCH ],
    [ 1, Moves.ICE_PUNCH ],
    [ 1, Moves.THUNDER_PUNCH ],
    [ 1, Moves.BIND ],
    [ 1, Moves.LEER ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.SHADOW_PUNCH ],
    [ 1, Moves.GRAVITY ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.NIGHT_SHADE ],
    [ 20, Moves.PAYBACK ],
    [ 24, Moves.WILL_O_WISP ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.HEX ],
    [ 36, Moves.CURSE ],
    [ 42, Moves.SHADOW_BALL ],
    [ 48, Moves.FUTURE_SIGHT ],
    [ 54, Moves.DESTINY_BOND ],
  ],
  [Species.FROSLASS]: [
    [ EVOLVE_MOVE, Moves.HEX ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.DESTINY_BOND ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.OMINOUS_WIND ],
    [ 15, Moves.ICE_SHARD ],
    [ 20, Moves.DRAINING_KISS ],
    [ 25, Moves.ICY_WIND ],
    [ 30, Moves.FROST_BREATH ],
    [ 35, Moves.CONFUSE_RAY ],
    [ 40, Moves.SNOWSCAPE ],
    [ 47, Moves.WILL_O_WISP ],
    [ 54, Moves.AURORA_VEIL ],
    [ 61, Moves.SHADOW_BALL ],
    [ 68, Moves.BLIZZARD ],
  ],
  [Species.ROTOM]: [
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.THUNDER_SHOCK ],
    [ 10, Moves.CONFUSE_RAY ],
    [ 15, Moves.CHARGE ],
    [ 20, Moves.ELECTRO_BALL ],
    [ 25, Moves.THUNDER_WAVE ],
    [ 30, Moves.SHOCK_WAVE ],
    [ 35, Moves.HEX ],
    [ 40, Moves.SUBSTITUTE ],
    [ 45, Moves.TRICK ],
    [ 50, Moves.DISCHARGE ],
    [ 55, Moves.UPROAR ],
  ],
  [Species.UXIE]: [
    [ RELEARN_MOVE, Moves.TRI_ATTACK ],
    [ RELEARN_MOVE, Moves.SNORE ],
    [ RELEARN_MOVE, Moves.SAFEGUARD ],
    [ RELEARN_MOVE, Moves.IRON_TAIL ],
    [ RELEARN_MOVE, Moves.PSYCHO_CUT ],
    [ RELEARN_MOVE, Moves.WONDER_ROOM ],
    [ RELEARN_MOVE, Moves.MAGIC_ROOM ],
    [ RELEARN_MOVE, Moves.ROUND ],
    [ RELEARN_MOVE, Moves.ALLY_SWITCH ],
    [ RELEARN_MOVE, Moves.EXPANDING_FORCE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.REST ],
    [ 7, Moves.SWIFT ],
    [ 14, Moves.ENDURE ],
    [ 21, Moves.PSYBEAM ],
    [ 28, Moves.IMPRISON ],
    [ 35, Moves.EXTRASENSORY ],
    [ 42, Moves.AMNESIA ],
    [ 49, Moves.PSYCHIC ],
    [ 56, Moves.YAWN ],
    [ 63, Moves.FUTURE_SIGHT ],
    [ 70, Moves.FLAIL ],
    [ 77, Moves.MEMENTO ],
    [ 84, Moves.MYSTICAL_POWER ],
  ],
  [Species.MESPRIT]: [
    [ RELEARN_MOVE, Moves.TRI_ATTACK ],
    [ RELEARN_MOVE, Moves.SNORE ],
    [ RELEARN_MOVE, Moves.SAFEGUARD ],
    [ RELEARN_MOVE, Moves.IRON_TAIL ],
    [ RELEARN_MOVE, Moves.PSYCHO_CUT ],
    [ RELEARN_MOVE, Moves.WONDER_ROOM ],
    [ RELEARN_MOVE, Moves.MAGIC_ROOM ],
    [ RELEARN_MOVE, Moves.ROUND ],
    [ RELEARN_MOVE, Moves.ALLY_SWITCH ],
    [ RELEARN_MOVE, Moves.EXPANDING_FORCE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.REST ],
    [ 7, Moves.SWIFT ],
    [ 14, Moves.PROTECT ],
    [ 21, Moves.PSYBEAM ],
    [ 28, Moves.IMPRISON ],
    [ 35, Moves.EXTRASENSORY ],
    [ 42, Moves.CHARM ],
    [ 49, Moves.PSYCHIC ],
    [ 56, Moves.FLATTER ],
    [ 63, Moves.FUTURE_SIGHT ],
    [ 70, Moves.COPYCAT ],
    [ 77, Moves.HEALING_WISH ],
    [ 84, Moves.MYSTICAL_POWER ],
  ],
  [Species.AZELF]: [
    [ RELEARN_MOVE, Moves.SELF_DESTRUCT ],
    [ RELEARN_MOVE, Moves.TRI_ATTACK ],
    [ RELEARN_MOVE, Moves.SNORE ],
    [ RELEARN_MOVE, Moves.SAFEGUARD ],
    [ RELEARN_MOVE, Moves.IRON_TAIL ],
    [ RELEARN_MOVE, Moves.PAYBACK ],
    [ RELEARN_MOVE, Moves.ASSURANCE ],
    [ RELEARN_MOVE, Moves.PSYCHO_CUT ],
    [ RELEARN_MOVE, Moves.WONDER_ROOM ],
    [ RELEARN_MOVE, Moves.MAGIC_ROOM ],
    [ RELEARN_MOVE, Moves.ROUND ],
    [ RELEARN_MOVE, Moves.ALLY_SWITCH ],
    [ RELEARN_MOVE, Moves.EXPANDING_FORCE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.REST ],
    [ 7, Moves.SWIFT ],
    [ 14, Moves.DETECT ],
    [ 21, Moves.PSYBEAM ],
    [ 28, Moves.IMPRISON ],
    [ 35, Moves.EXTRASENSORY ],
    [ 42, Moves.NASTY_PLOT ],
    [ 49, Moves.PSYCHIC ],
    [ 56, Moves.UPROAR ],
    [ 63, Moves.FUTURE_SIGHT ],
    [ 70, Moves.LAST_RESORT ],
    [ 77, Moves.EXPLOSION ],
    [ 84, Moves.MYSTICAL_POWER ],
  ],
  [Species.DIALGA]: [
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.METAL_CLAW ],
    [ 8, Moves.DRAGON_BREATH ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 24, Moves.SLASH ],
    [ 32, Moves.FLASH_CANNON ],
    [ 40, Moves.DRAGON_CLAW ],
    [ 48, Moves.AURA_SPHERE ],
    [ 56, Moves.POWER_GEM ],
    [ 64, Moves.METAL_BURST ],
    [ 72, Moves.EARTH_POWER ],
    [ 80, Moves.IRON_TAIL ],
    [ 88, Moves.ROAR_OF_TIME ],
  ],
  [Species.PALKIA]: [
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.WATER_PULSE ],
    [ 8, Moves.DRAGON_BREATH ],
    [ 16, Moves.ANCIENT_POWER ],
    [ 24, Moves.SLASH ],
    [ 32, Moves.AQUA_RING ],
    [ 48, Moves.AURA_SPHERE ],
    [ 56, Moves.POWER_GEM ],
    [ 64, Moves.AQUA_TAIL ],
    [ 72, Moves.EARTH_POWER ],
    [ 80, Moves.SPACIAL_REND ],
    [ 88, Moves.HYDRO_PUMP ],
  ],
  [Species.HEATRAN]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.FIRE_SPIN ],
    [ 6, Moves.METAL_CLAW ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 18, Moves.FIRE_FANG ],
    [ 24, Moves.SCARY_FACE ],
    [ 30, Moves.IRON_HEAD ],
    [ 36, Moves.CRUNCH ],
    [ 42, Moves.LAVA_PLUME ],
    [ 48, Moves.METAL_SOUND ],
    [ 54, Moves.EARTH_POWER ],
    [ 60, Moves.HEAT_WAVE ],
    [ 66, Moves.STONE_EDGE ],
    [ 72, Moves.MAGMA_STORM ],
  ],
  [Species.REGIGIGAS]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.DIZZY_PUNCH ],
    [ 1, Moves.FORESIGHT ],
    [ 6, Moves.PAYBACK ],
    [ 12, Moves.REVENGE ],
    [ 18, Moves.STOMP ],
    [ 24, Moves.PROTECT ],
    [ 30, Moves.KNOCK_OFF ],
    [ 36, Moves.MEGA_PUNCH ],
    [ 42, Moves.BODY_PRESS ],
    [ 48, Moves.WIDE_GUARD ],
    [ 54, Moves.ZEN_HEADBUTT ],
    [ 60, Moves.HEAVY_SLAM ],
    [ 66, Moves.HAMMER_ARM ],
    [ 72, Moves.GIGA_IMPACT ],
    [ 78, Moves.CRUSH_GRIP ],
  ],
  [Species.GIRATINA]: [
    [ 1, Moves.SHADOW_SNEAK ],
    [ 1, Moves.DEFOG ],
    [ 1, Moves.DRAGON_BREATH ], //USUM
    [ 7, Moves.OMINOUS_WIND ], //USUM
    [ 14, Moves.ANCIENT_POWER ],
    [ 21, Moves.HEX ],
    [ 28, Moves.SLASH ],
    [ 35, Moves.SCARY_FACE ],
    [ 42, Moves.SHADOW_CLAW ],
    [ 49, Moves.PAIN_SPLIT ],
    [ 56, Moves.AURA_SPHERE ],
    [ 63, Moves.DRAGON_CLAW ],
    [ 70, Moves.EARTH_POWER ],
    [ 77, Moves.SHADOW_FORCE ],
    [ 84, Moves.DESTINY_BOND ],
  ],
  [Species.CRESSELIA]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 6, Moves.MIST ],
    [ 12, Moves.AURORA_BEAM ],
    [ 18, Moves.PSYBEAM ],
    [ 24, Moves.ALLY_SWITCH ],
    [ 30, Moves.SLASH ],
    [ 36, Moves.PSYCHO_CUT ],
    [ 42, Moves.MOONLIGHT ],
    [ 48, Moves.SAFEGUARD ],
    [ 54, Moves.PSYCHIC ],
    [ 60, Moves.MOONBLAST ],
    [ 66, Moves.FUTURE_SIGHT ],
    [ 72, Moves.LUNAR_DANCE ],
    [ 72, Moves.LUNAR_BLESSING ],
  ],
  [Species.PHIONE]: [
    [ 1, Moves.WATER_GUN ],
    [ 9, Moves.CHARM ],
    [ 16, Moves.SUPERSONIC ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 31, Moves.ACID_ARMOR ],
    [ 39, Moves.WHIRLPOOL ],
    [ 46, Moves.WATER_PULSE ],
    [ 54, Moves.AQUA_RING ],
    [ 61, Moves.DIVE ],
    [ 69, Moves.RAIN_DANCE ],
    [ 75, Moves.TAKE_HEART ],
  ],
  [Species.MANAPHY]: [
    [ 1, Moves.HEART_SWAP ],
    [ 1, Moves.TAIL_GLOW ],
    [ 1, Moves.WATER_GUN ],
    [ 9, Moves.CHARM ],
    [ 16, Moves.SUPERSONIC ],
    [ 24, Moves.BUBBLE_BEAM ],
    [ 31, Moves.ACID_ARMOR ],
    [ 39, Moves.WHIRLPOOL ],
    [ 46, Moves.WATER_PULSE ],
    [ 54, Moves.AQUA_RING ],
    [ 61, Moves.DIVE ],
    [ 69, Moves.RAIN_DANCE ],
    [ 76, Moves.TAKE_HEART ],
  ],
  [Species.DARKRAI]: [
    [ 1, Moves.DISABLE ],
    [ 1, Moves.OMINOUS_WIND ],
    [ 1, Moves.PURSUIT ], //Custom
    [ 11, Moves.QUICK_ATTACK ],
    [ 20, Moves.HYPNOSIS ],
    [ 29, Moves.SUCKER_PUNCH ],
    [ 38, Moves.NIGHT_SHADE ],
    [ 47, Moves.DOUBLE_TEAM ],
    [ 57, Moves.HAZE ],
    [ 66, Moves.DARK_VOID ],
    [ 75, Moves.NASTY_PLOT ],
    [ 84, Moves.DREAM_EATER ],
    [ 93, Moves.DARK_PULSE ],
  ],
  [Species.SHAYMIN]: [
    [ 1, Moves.LEAFAGE ], //Custom
    [ 1, Moves.GROWTH ],
    [ 10, Moves.MAGICAL_LEAF ],
    [ 19, Moves.LEECH_SEED ],
    [ 28, Moves.SYNTHESIS ],
    [ 37, Moves.SWEET_SCENT ],
    [ 46, Moves.NATURAL_GIFT ],
    [ 55, Moves.WORRY_SEED ],
    [ 64, Moves.AROMATHERAPY ],
    [ 73, Moves.ENERGY_BALL ],
    [ 82, Moves.SWEET_KISS ],
    [ 91, Moves.HEALING_WISH ],
    [ 100, Moves.SEED_FLARE ],
  ],
  [Species.ARCEUS]: [
    [ 1, Moves.SEISMIC_TOSS ],
    [ 1, Moves.COSMIC_POWER ],
    [ 1, Moves.PUNISHMENT ],
    [ 10, Moves.GRAVITY ],
    [ 20, Moves.EARTH_POWER ],
    [ 30, Moves.HYPER_VOICE ],
    [ 40, Moves.EXTREME_SPEED ],
    [ 50, Moves.HEALING_WISH ],
    [ 60, Moves.FUTURE_SIGHT ],
    [ 70, Moves.RECOVER ],
    [ 80, Moves.HYPER_BEAM ],
    [ 90, Moves.PERISH_SONG ],
    [ 100, Moves.JUDGMENT ],
  ],
  [Species.VICTINI]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FLAME_CHARGE ],
    [ 1, Moves.V_CREATE ],
    [ 7, Moves.WORK_UP ],
    [ 14, Moves.INCINERATE ],
    [ 21, Moves.STORED_POWER ],
    [ 28, Moves.HEADBUTT ],
    [ 35, Moves.ENDURE ],
    [ 42, Moves.ZEN_HEADBUTT ],
    [ 49, Moves.INFERNO ],
    [ 56, Moves.REVERSAL ],
    [ 63, Moves.SEARING_SHOT ],
    [ 70, Moves.DOUBLE_EDGE ],
    [ 77, Moves.FLARE_BLITZ ],
    [ 84, Moves.OVERHEAT ],
    [ 91, Moves.FINAL_GAMBIT ],
  ],
  [Species.SNIVY]: [
    [ 1, Moves.TACKLE ],
    [ 4, Moves.LEER ],
    [ 5, Moves.VINE_WHIP ], //Custom, moved from 7 to 5
    [ 10, Moves.WRAP ],
    [ 13, Moves.GROWTH ],
    [ 16, Moves.MAGICAL_LEAF ],
    [ 19, Moves.LEECH_SEED ],
    [ 22, Moves.MEGA_DRAIN ],
    [ 25, Moves.SLAM ],
    [ 28, Moves.LEAF_BLADE ],
    [ 31, Moves.COIL ],
    [ 34, Moves.GIGA_DRAIN ],
    [ 37, Moves.GASTRO_ACID ],
    [ 40, Moves.LEAF_STORM ],
  ],
  [Species.SERVINE]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.LEER ],
    [ 13, Moves.GROWTH ],
    [ 16, Moves.MAGICAL_LEAF ],
    [ 20, Moves.LEECH_SEED ],
    [ 24, Moves.MEGA_DRAIN ],
    [ 28, Moves.SLAM ],
    [ 32, Moves.LEAF_BLADE ],
    [ 36, Moves.COIL ],
    [ 40, Moves.GIGA_DRAIN ],
    [ 44, Moves.GASTRO_ACID ],
    [ 48, Moves.LEAF_STORM ],
  ],
  [Species.SERPERIOR]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.LEER ],
    [ 13, Moves.GROWTH ],
    [ 16, Moves.MAGICAL_LEAF ],
    [ 20, Moves.LEECH_SEED ],
    [ 24, Moves.MEGA_DRAIN ],
    [ 28, Moves.SLAM ],
    [ 32, Moves.LEAF_BLADE ],
    [ 38, Moves.COIL ],
    [ 44, Moves.GIGA_DRAIN ],
    [ 50, Moves.GASTRO_ACID ],
    [ 56, Moves.LEAF_STORM ],
  ],
  [Species.TEPIG]: [
    [ 1, Moves.TACKLE ],
    [ 3, Moves.TAIL_WHIP ],
    [ 5, Moves.EMBER ], //Custom, moved from 7 to 5
    [ 9, Moves.ENDURE ],
    [ 13, Moves.DEFENSE_CURL ],
    [ 15, Moves.FLAME_CHARGE ],
    [ 19, Moves.SMOG ],
    [ 21, Moves.ROLLOUT ],
    [ 25, Moves.TAKE_DOWN ],
    [ 27, Moves.HEAT_CRASH ],
    [ 31, Moves.ASSURANCE ],
    [ 33, Moves.FLAMETHROWER ],
    [ 37, Moves.HEAD_SMASH ],
    [ 39, Moves.ROAR ],
    [ 43, Moves.FLARE_BLITZ ],
  ],
  [Species.PIGNITE]: [
    [ EVOLVE_MOVE, Moves.ARM_THRUST ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ENDURE ],
    [ 13, Moves.DEFENSE_CURL ],
    [ 15, Moves.FLAME_CHARGE ],
    [ 20, Moves.SMOG ],
    [ 23, Moves.ROLLOUT ],
    [ 28, Moves.TAKE_DOWN ],
    [ 31, Moves.HEAT_CRASH ],
    [ 36, Moves.ASSURANCE ],
    [ 39, Moves.FLAMETHROWER ],
    [ 44, Moves.HEAD_SMASH ],
    [ 47, Moves.ROAR ],
    [ 52, Moves.FLARE_BLITZ ],
  ],
  [Species.EMBOAR]: [
    [ RELEARN_MOVE, Moves.ENDURE ],
    [ RELEARN_MOVE, Moves.HAMMER_ARM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ARM_THRUST ],
    [ 13, Moves.DEFENSE_CURL ],
    [ 15, Moves.FLAME_CHARGE ],
    [ 20, Moves.SMOG ],
    [ 23, Moves.ROLLOUT ],
    [ 28, Moves.TAKE_DOWN ],
    [ 31, Moves.HEAT_CRASH ],
    [ 38, Moves.ASSURANCE ],
    [ 43, Moves.FLAMETHROWER ],
    [ 50, Moves.HEAD_SMASH ],
    [ 55, Moves.ROAR ],
    [ 62, Moves.FLARE_BLITZ ],
  ],
  [Species.OSHAWOTT]: [
    [ 1, Moves.TACKLE ],
    [ 5, Moves.TAIL_WHIP ],
    [ 5, Moves.WATER_GUN ], //Custom, moved from 7 to 5
    [ 11, Moves.SOAK ],
    [ 13, Moves.FOCUS_ENERGY ],
    [ 17, Moves.RAZOR_SHELL ],
    [ 19, Moves.FURY_CUTTER ],
    [ 23, Moves.WATER_PULSE ],
    [ 25, Moves.AERIAL_ACE ],
    [ 29, Moves.AQUA_JET ],
    [ 31, Moves.ENCORE ],
    [ 35, Moves.AQUA_TAIL ],
    [ 37, Moves.RETALIATE ],
    [ 41, Moves.SWORDS_DANCE ],
    [ 43, Moves.HYDRO_PUMP ],
  ],
  [Species.DEWOTT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SOAK ],
    [ 13, Moves.FOCUS_ENERGY ],
    [ 18, Moves.RAZOR_SHELL ],
    [ 21, Moves.FURY_CUTTER ],
    [ 26, Moves.WATER_PULSE ],
    [ 29, Moves.AERIAL_ACE ],
    [ 34, Moves.AQUA_JET ],
    [ 37, Moves.ENCORE ],
    [ 42, Moves.AQUA_TAIL ],
    [ 45, Moves.RETALIATE ],
    [ 50, Moves.SWORDS_DANCE ],
    [ 53, Moves.HYDRO_PUMP ],
  ],
  [Species.SAMUROTT]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.MEGAHORN ],
    [ 1, Moves.SOAK ],
    [ 13, Moves.FOCUS_ENERGY ],
    [ 18, Moves.RAZOR_SHELL ],
    [ 21, Moves.FURY_CUTTER ],
    [ 25, Moves.WATER_PULSE ],
    [ 29, Moves.AERIAL_ACE ],
    [ 34, Moves.AQUA_JET ],
    [ 39, Moves.ENCORE ],
    [ 46, Moves.AQUA_TAIL ],
    [ 51, Moves.RETALIATE ],
    [ 58, Moves.SWORDS_DANCE ],
    [ 63, Moves.HYDRO_PUMP ],
  ],
  [Species.PATRAT]: [
    [ 1, Moves.TACKLE ],
    [ 3, Moves.LEER ],
    [ 6, Moves.BITE ],
    [ 8, Moves.BIDE ],
    [ 11, Moves.DETECT ],
    [ 13, Moves.SAND_ATTACK ],
    [ 16, Moves.CRUNCH ],
    [ 18, Moves.HYPNOSIS ],
    [ 21, Moves.SUPER_FANG ],
    [ 23, Moves.AFTER_YOU ],
    [ 26, Moves.FOCUS_ENERGY ],
    [ 28, Moves.WORK_UP ],
    [ 31, Moves.HYPER_FANG ],
    [ 33, Moves.NASTY_PLOT ],
    [ 36, Moves.MEAN_LOOK ],
    [ 38, Moves.BATON_PASS ],
    [ 41, Moves.SLAM ],
  ],
  [Species.WATCHOG]: [
    [ EVOLVE_MOVE, Moves.CONFUSE_RAY ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.ROTOTILLER ],
    [ 8, Moves.BIDE ],
    [ 11, Moves.DETECT ],
    [ 13, Moves.SAND_ATTACK ],
    [ 16, Moves.CRUNCH ],
    [ 18, Moves.HYPNOSIS ],
    [ 22, Moves.SUPER_FANG ],
    [ 25, Moves.AFTER_YOU ],
    [ 29, Moves.FOCUS_ENERGY ],
    [ 32, Moves.PSYCH_UP ],
    [ 36, Moves.HYPER_FANG ],
    [ 39, Moves.NASTY_PLOT ],
    [ 43, Moves.MEAN_LOOK ],
    [ 46, Moves.BATON_PASS ],
    [ 50, Moves.SLAM ],
  ],
  [Species.LILLIPUP]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.WORK_UP ],
    [ 8, Moves.BITE ],
    [ 12, Moves.RETALIATE ],
    [ 17, Moves.BABY_DOLL_EYES ],
    [ 20, Moves.PLAY_ROUGH ],
    [ 24, Moves.CRUNCH ],
    [ 28, Moves.TAKE_DOWN ],
    [ 32, Moves.HELPING_HAND ],
    [ 36, Moves.REVERSAL ],
    [ 40, Moves.ROAR ],
    [ 44, Moves.LAST_RESORT ],
    [ 48, Moves.GIGA_IMPACT ],
  ],
  [Species.HERDIER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.WORK_UP ],
    [ 12, Moves.RETALIATE ],
    [ 19, Moves.BABY_DOLL_EYES ],
    [ 24, Moves.PLAY_ROUGH ],
    [ 30, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 42, Moves.HELPING_HAND ],
    [ 48, Moves.REVERSAL ],
    [ 54, Moves.ROAR ],
    [ 60, Moves.LAST_RESORT ],
    [ 66, Moves.GIGA_IMPACT ],
  ],
  [Species.STOUTLAND]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.WORK_UP ],
    [ 12, Moves.RETALIATE ],
    [ 19, Moves.BABY_DOLL_EYES ],
    [ 24, Moves.PLAY_ROUGH ],
    [ 30, Moves.CRUNCH ],
    [ 38, Moves.TAKE_DOWN ],
    [ 46, Moves.HELPING_HAND ],
    [ 54, Moves.REVERSAL ],
    [ 62, Moves.ROAR ],
    [ 70, Moves.LAST_RESORT ],
    [ 78, Moves.GIGA_IMPACT ],
  ],
  [Species.PURRLOIN]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.SAND_ATTACK ],
    [ 5, Moves.FAKE_OUT ],
    [ 12, Moves.FURY_SWIPES ],
    [ 16, Moves.TORMENT ],
    [ 21, Moves.ASSURANCE ],
    [ 24, Moves.HONE_CLAWS ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.NASTY_PLOT ],
    [ 36, Moves.NIGHT_SLASH ],
    [ 40, Moves.PLAY_ROUGH ],
  ],
  [Species.LIEPARD]: [
    [ 1, Moves.ASSIST ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 12, Moves.FURY_SWIPES ],
    [ 16, Moves.TORMENT ],
    [ 23, Moves.ASSURANCE ],
    [ 28, Moves.HONE_CLAWS ],
    [ 34, Moves.SUCKER_PUNCH ],
    [ 40, Moves.NASTY_PLOT ],
    [ 46, Moves.NIGHT_SLASH ],
    [ 52, Moves.PLAY_ROUGH ],
  ],
  [Species.PANSAGE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.PLAY_NICE ],
    [ 4, Moves.LEER ],
    [ 7, Moves.LICK ],
    [ 10, Moves.VINE_WHIP ],
    [ 13, Moves.FURY_SWIPES ],
    [ 16, Moves.LEECH_SEED ],
    [ 19, Moves.BITE ],
    [ 22, Moves.SEED_BOMB ],
    [ 25, Moves.TORMENT ],
    [ 28, Moves.FLING ],
    [ 31, Moves.ACROBATICS ],
    [ 34, Moves.GRASS_KNOT ],
    [ 37, Moves.RECYCLE ],
    [ 40, Moves.NATURAL_GIFT ],
    [ 43, Moves.CRUNCH ],
  ],
  [Species.SIMISAGE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.FURY_SWIPES ],
    [ 1, Moves.SEED_BOMB ],
  ],
  [Species.PANSEAR]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.PLAY_NICE ],
    [ 4, Moves.LEER ],
    [ 7, Moves.LICK ],
    [ 10, Moves.INCINERATE ],
    [ 13, Moves.FURY_SWIPES ],
    [ 16, Moves.YAWN ],
    [ 19, Moves.BITE ],
    [ 22, Moves.FLAME_BURST ],
    [ 25, Moves.AMNESIA ],
    [ 28, Moves.FLING ],
    [ 31, Moves.ACROBATICS ],
    [ 34, Moves.FIRE_BLAST ],
    [ 37, Moves.RECYCLE ],
    [ 40, Moves.NATURAL_GIFT ],
    [ 43, Moves.CRUNCH ],
  ],
  [Species.SIMISEAR]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.FURY_SWIPES ],
    [ 1, Moves.FLAME_BURST ],
  ],
  [Species.PANPOUR]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.PLAY_NICE ],
    [ 4, Moves.LEER ],
    [ 7, Moves.LICK ],
    [ 10, Moves.WATER_GUN ],
    [ 13, Moves.FURY_SWIPES ],
    [ 16, Moves.WATER_SPORT ],
    [ 19, Moves.BITE ],
    [ 22, Moves.SCALD ],
    [ 25, Moves.TAUNT ],
    [ 28, Moves.FLING ],
    [ 31, Moves.ACROBATICS ],
    [ 34, Moves.BRINE ],
    [ 37, Moves.RECYCLE ],
    [ 40, Moves.NATURAL_GIFT ],
    [ 43, Moves.CRUNCH ],
  ],
  [Species.SIMIPOUR]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.FURY_SWIPES ],
    [ 1, Moves.SCALD ],
  ],
  [Species.MUNNA]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.STORED_POWER ],
    [ 1, Moves.PSYWAVE ],
    [ 4, Moves.HYPNOSIS ],
    [ 8, Moves.PSYBEAM ],
    [ 12, Moves.IMPRISON ],
    [ 16, Moves.MOONLIGHT ],
    [ 20, Moves.MAGIC_COAT ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 28, Moves.CALM_MIND ],
    [ 32, Moves.YAWN ],
    [ 36, Moves.PSYCHIC ],
    [ 40, Moves.MOONBLAST ],
    [ 44, Moves.DREAM_EATER ],
    [ 48, Moves.FUTURE_SIGHT ],
    [ 52, Moves.WONDER_ROOM ],
  ],
  [Species.MUSHARNA]: [
    [ 1, Moves.PSYBEAM ],
    [ 1, Moves.PSYCHIC ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.LUCKY_CHANT ],
    [ 1, Moves.DREAM_EATER ],
    [ 1, Moves.MOONLIGHT ],
    [ 1, Moves.FUTURE_SIGHT ],
    [ 1, Moves.MAGIC_COAT ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.IMPRISON ],
    [ 1, Moves.CALM_MIND ],
    [ 1, Moves.ZEN_HEADBUTT ],
    [ 1, Moves.WONDER_ROOM ],
    [ 1, Moves.STORED_POWER ],
    [ 1, Moves.MOONBLAST ],
    [ 1, Moves.PSYCHIC_TERRAIN ],
  ],
  [Species.PIDOVE]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.LEER ],
    [ 8, Moves.QUICK_ATTACK ],
    [ 12, Moves.TAUNT ],
    [ 16, Moves.AIR_CUTTER ],
    [ 20, Moves.SWAGGER ],
    [ 24, Moves.FEATHER_DANCE ],
    [ 28, Moves.DETECT ],
    [ 32, Moves.AIR_SLASH ],
    [ 36, Moves.ROOST ],
    [ 40, Moves.TAILWIND ],
    [ 44, Moves.SKY_ATTACK ],
  ],
  [Species.TRANQUILL]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 12, Moves.TAUNT ],
    [ 16, Moves.AIR_CUTTER ],
    [ 20, Moves.SWAGGER ],
    [ 26, Moves.FEATHER_DANCE ],
    [ 34, Moves.DETECT ],
    [ 38, Moves.AIR_SLASH ],
    [ 44, Moves.ROOST ],
    [ 50, Moves.TAILWIND ],
    [ 56, Moves.SKY_ATTACK ],
  ],
  [Species.UNFEZANT]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 12, Moves.TAUNT ],
    [ 16, Moves.AIR_CUTTER ],
    [ 20, Moves.SWAGGER ],
    [ 26, Moves.FEATHER_DANCE ],
    [ 36, Moves.DETECT ],
    [ 42, Moves.AIR_SLASH ],
    [ 50, Moves.ROOST ],
    [ 58, Moves.TAILWIND ],
    [ 66, Moves.SKY_ATTACK ],
  ],
  [Species.BLITZLE]: [
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.TAIL_WHIP ],
    [ 8, Moves.CHARGE ],
    [ 11, Moves.SHOCK_WAVE ],
    [ 15, Moves.THUNDER_WAVE ],
    [ 18, Moves.FLAME_CHARGE ],
    [ 22, Moves.SPARK ],
    [ 25, Moves.STOMP ],
    [ 29, Moves.DISCHARGE ],
    [ 33, Moves.AGILITY ],
    [ 35, Moves.WILD_CHARGE ],
    [ 40, Moves.THRASH ],
  ],
  [Species.ZEBSTRIKA]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.ION_DELUGE ],
    [ 11, Moves.SHOCK_WAVE ],
    [ 18, Moves.FLAME_CHARGE ],
    [ 25, Moves.SPARK ],
    [ 31, Moves.STOMP ],
    [ 36, Moves.DISCHARGE ],
    [ 42, Moves.AGILITY ],
    [ 47, Moves.WILD_CHARGE ],
    [ 53, Moves.THRASH ],
  ],
  [Species.ROGGENROLA]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 4, Moves.HARDEN ],
    [ 8, Moves.STEALTH_ROCK ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.SMACK_DOWN ],
    [ 20, Moves.IRON_DEFENSE ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.ROCK_SLIDE ],
    [ 32, Moves.ROCK_BLAST ],
    [ 36, Moves.SANDSTORM ],
    [ 40, Moves.STONE_EDGE ],
    [ 44, Moves.EXPLOSION ],
  ],
  [Species.BOLDORE]: [
    [ EVOLVE_MOVE, Moves.POWER_GEM ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.STEALTH_ROCK ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.SMACK_DOWN ],
    [ 20, Moves.IRON_DEFENSE ],
    [ 24, Moves.HEADBUTT ],
    [ 30, Moves.ROCK_SLIDE ],
    [ 36, Moves.ROCK_BLAST ],
    [ 42, Moves.SANDSTORM ],
    [ 48, Moves.STONE_EDGE ],
    [ 54, Moves.EXPLOSION ],
  ],
  [Species.GIGALITH]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POWER_GEM ],
    [ 1, Moves.STEALTH_ROCK ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.SMACK_DOWN ],
    [ 20, Moves.IRON_DEFENSE ],
    [ 24, Moves.HEADBUTT ],
    [ 30, Moves.ROCK_SLIDE ],
    [ 36, Moves.ROCK_BLAST ],
    [ 42, Moves.SANDSTORM ],
    [ 48, Moves.STONE_EDGE ],
    [ 54, Moves.EXPLOSION ],
  ],
  [Species.WOOBAT]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.ATTRACT ],
    [ 5, Moves.CONFUSION ],
    [ 10, Moves.ENDEAVOR ],
    [ 15, Moves.AIR_CUTTER ],
    [ 20, Moves.IMPRISON ],
    [ 25, Moves.ASSURANCE ],
    [ 30, Moves.AMNESIA ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.PSYCHIC ],
    [ 45, Moves.CALM_MIND ],
    [ 50, Moves.FUTURE_SIGHT ],
    [ 55, Moves.SIMPLE_BEAM ],
  ],
  [Species.SWOOBAT]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.ATTRACT ],
    [ 1, Moves.ENDEAVOR ],
    [ 15, Moves.AIR_CUTTER ],
    [ 20, Moves.IMPRISON ],
    [ 25, Moves.ASSURANCE ],
    [ 30, Moves.AMNESIA ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.PSYCHIC ],
    [ 45, Moves.CALM_MIND ],
    [ 50, Moves.FUTURE_SIGHT ],
    [ 55, Moves.SIMPLE_BEAM ],
  ],
  [Species.DRILBUR]: [
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.MUD_SPORT ],
    [ 4, Moves.SCRATCH ],
    [ 8, Moves.HONE_CLAWS ],
    [ 12, Moves.FURY_SWIPES ],
    [ 16, Moves.METAL_CLAW ],
    [ 20, Moves.SANDSTORM ],
    [ 24, Moves.CRUSH_CLAW ],
    [ 28, Moves.ROCK_SLIDE ],
    [ 32, Moves.DIG ],
    [ 36, Moves.SWORDS_DANCE ],
    [ 40, Moves.DRILL_RUN ],
    [ 44, Moves.EARTHQUAKE ],
    [ 48, Moves.FISSURE ],
  ],
  [Species.EXCADRILL]: [
    [ EVOLVE_MOVE, Moves.HORN_DRILL ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.MUD_SPORT ],
    [ 1, Moves.ROTOTILLER ],
    [ 1, Moves.HONE_CLAWS ],
    [ 12, Moves.FURY_SWIPES ],
    [ 16, Moves.METAL_CLAW ],
    [ 20, Moves.SANDSTORM ],
    [ 24, Moves.CRUSH_CLAW ],
    [ 28, Moves.ROCK_SLIDE ],
    [ 34, Moves.DIG ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 46, Moves.DRILL_RUN ],
    [ 52, Moves.EARTHQUAKE ],
    [ 58, Moves.FISSURE ],
  ],
  [Species.AUDINO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.PLAY_NICE ],
    [ 4, Moves.DISARMING_VOICE ],
    [ 9, Moves.BABY_DOLL_EYES ],
    [ 12, Moves.HELPING_HAND ],
    [ 16, Moves.GROWL ],
    [ 20, Moves.ZEN_HEADBUTT ],
    [ 24, Moves.LIFE_DEW ],
    [ 28, Moves.AFTER_YOU ],
    [ 32, Moves.TAKE_DOWN ],
    [ 36, Moves.SIMPLE_BEAM ],
    [ 40, Moves.HYPER_VOICE ],
    [ 44, Moves.HEAL_PULSE ],
    [ 48, Moves.DOUBLE_EDGE ],
    [ 52, Moves.ENTRAINMENT ],
    [ 56, Moves.MISTY_TERRAIN ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.TIMBURR]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.LEER ],
    [ 4, Moves.LOW_KICK ],
    [ 8, Moves.ROCK_THROW ],
    [ 12, Moves.FOCUS_ENERGY ],
    [ 16, Moves.BULK_UP ],
    [ 20, Moves.ROCK_SLIDE ],
    [ 24, Moves.SLAM ],
    [ 28, Moves.SCARY_FACE ],
    [ 32, Moves.DYNAMIC_PUNCH ],
    [ 36, Moves.HAMMER_ARM ],
    [ 40, Moves.STONE_EDGE ],
    [ 44, Moves.SUPERPOWER ],
    [ 48, Moves.FOCUS_PUNCH ],
  ],
  [Species.GURDURR]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.BIDE ],
    [ 12, Moves.FOCUS_ENERGY ],
    [ 16, Moves.BULK_UP ],
    [ 20, Moves.ROCK_SLIDE ],
    [ 24, Moves.SLAM ],
    [ 30, Moves.SCARY_FACE ],
    [ 36, Moves.DYNAMIC_PUNCH ],
    [ 42, Moves.HAMMER_ARM ],
    [ 48, Moves.STONE_EDGE ],
    [ 54, Moves.SUPERPOWER ],
    [ 60, Moves.FOCUS_PUNCH ],
  ],
  [Species.CONKELDURR]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.BIDE ],
    [ 12, Moves.FOCUS_ENERGY ],
    [ 16, Moves.BULK_UP ],
    [ 20, Moves.ROCK_SLIDE ],
    [ 24, Moves.SLAM ],
    [ 30, Moves.SCARY_FACE ],
    [ 36, Moves.DYNAMIC_PUNCH ],
    [ 42, Moves.HAMMER_ARM ],
    [ 48, Moves.STONE_EDGE ],
    [ 54, Moves.SUPERPOWER ],
    [ 60, Moves.FOCUS_PUNCH ],
  ],
  [Species.TYMPOLE]: [
    [ 1, Moves.BUBBLE ], //USUM
    [ 1, Moves.GROWL ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 4, Moves.ACID ],
    [ 8, Moves.SUPERSONIC ],
    [ 12, Moves.MUD_SHOT ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.FLAIL ],
    [ 28, Moves.UPROAR ],
    [ 32, Moves.AQUA_RING ],
    [ 36, Moves.HYPER_VOICE ],
    [ 40, Moves.MUDDY_WATER ],
    [ 44, Moves.RAIN_DANCE ],
    [ 48, Moves.HYDRO_PUMP ],
  ],
  [Species.PALPITOAD]: [
    [ 1, Moves.BUBBLE ], //USUM
    [ 1, Moves.GROWL ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.ACID ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 12, Moves.MUD_SHOT ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.FLAIL ],
    [ 30, Moves.UPROAR ],
    [ 37, Moves.AQUA_RING ],
    [ 42, Moves.HYPER_VOICE ],
    [ 48, Moves.MUDDY_WATER ],
    [ 54, Moves.RAIN_DANCE ],
    [ 60, Moves.HYDRO_PUMP ],
  ],
  [Species.SEISMITOAD]: [
    [ EVOLVE_MOVE, Moves.DRAIN_PUNCH ],
    [ 1, Moves.BUBBLE ], //USUM
    [ 1, Moves.GROWL ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.ACID ],
    [ 1, Moves.GASTRO_ACID ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 12, Moves.MUD_SHOT ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.BUBBLE_BEAM ],
    [ 24, Moves.FLAIL ],
    [ 30, Moves.UPROAR ],
    [ 39, Moves.AQUA_RING ],
    [ 46, Moves.HYPER_VOICE ],
    [ 54, Moves.MUDDY_WATER ],
    [ 62, Moves.RAIN_DANCE ],
    [ 70, Moves.HYDRO_PUMP ],
  ],
  [Species.THROH]: [
    [ 1, Moves.ROCK_SMASH ], //Custom
    [ 1, Moves.LEER ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.MAT_BLOCK ],
    [ 1, Moves.BIND ],
    [ 5, Moves.FOCUS_ENERGY ],
    [ 10, Moves.CIRCLE_THROW ],
    [ 15, Moves.WIDE_GUARD ],
    [ 20, Moves.REVENGE ],
    [ 25, Moves.BULK_UP ],
    [ 30, Moves.STORM_THROW ],
    [ 35, Moves.VITAL_THROW ],
    [ 40, Moves.SEISMIC_TOSS ],
    [ 45, Moves.ENDURE ],
    [ 50, Moves.REVERSAL ],
    [ 55, Moves.SUPERPOWER ],
  ],
  [Species.SAWK]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.BIDE ],
    [ 5, Moves.FOCUS_ENERGY ],
    [ 10, Moves.DOUBLE_KICK ],
    [ 15, Moves.QUICK_GUARD ],
    [ 20, Moves.LOW_SWEEP ],
    [ 25, Moves.BULK_UP ],
    [ 30, Moves.RETALIATE ],
    [ 35, Moves.BRICK_BREAK ],
    [ 40, Moves.COUNTER ],
    [ 45, Moves.ENDURE ],
    [ 50, Moves.REVERSAL ],
    [ 55, Moves.CLOSE_COMBAT ],
  ],
  [Species.SEWADDLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 8, Moves.BUG_BITE ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 22, Moves.STRUGGLE_BUG ],
    [ 29, Moves.ENDURE ],
    [ 31, Moves.STICKY_WEB ],
    [ 36, Moves.BUG_BUZZ ],
    [ 43, Moves.FLAIL ],
  ],
  [Species.SWADLOON]: [
    [ EVOLVE_MOVE, Moves.PROTECT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.GRASS_WHISTLE ],
    [ 22, Moves.STRUGGLE_BUG ],
    [ 29, Moves.ENDURE ],
    [ 31, Moves.STICKY_WEB ],
    [ 36, Moves.BUG_BUZZ ],
    [ 43, Moves.FLAIL ],
  ],
  [Species.LEAVANNY]: [
    [ EVOLVE_MOVE, Moves.SLASH ],
    [ RELEARN_MOVE, Moves.BUG_BITE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 22, Moves.STRUGGLE_BUG ],
    [ 29, Moves.FELL_STINGER ],
    [ 32, Moves.HELPING_HAND ],
    [ 36, Moves.LEAF_BLADE ],
    [ 39, Moves.X_SCISSOR ],
    [ 43, Moves.ENTRAINMENT ],
    [ 46, Moves.SWORDS_DANCE ],
    [ 50, Moves.LEAF_STORM ],
  ],
  [Species.VENIPEDE]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 4, Moves.ROLLOUT ],
    [ 8, Moves.PROTECT ],
    [ 12, Moves.POISON_TAIL ],
    [ 16, Moves.SCREECH ],
    [ 20, Moves.BUG_BITE ],
    [ 24, Moves.VENOSHOCK ],
    [ 28, Moves.TAKE_DOWN ],
    [ 32, Moves.AGILITY ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.VENOM_DRENCH ],
    [ 44, Moves.DOUBLE_EDGE ],
  ],
  [Species.WHIRLIPEDE]: [
    [ EVOLVE_MOVE, Moves.IRON_DEFENSE ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ROLLOUT ],
    [ 12, Moves.POISON_TAIL ],
    [ 16, Moves.SCREECH ],
    [ 20, Moves.BUG_BITE ],
    [ 26, Moves.VENOSHOCK ],
    [ 32, Moves.TAKE_DOWN ],
    [ 38, Moves.AGILITY ],
    [ 44, Moves.TOXIC ],
    [ 50, Moves.VENOM_DRENCH ],
    [ 56, Moves.DOUBLE_EDGE ],
  ],
  [Species.SCOLIPEDE]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 12, Moves.POISON_TAIL ],
    [ 16, Moves.SCREECH ],
    [ 20, Moves.BUG_BITE ],
    [ 26, Moves.VENOSHOCK ],
    [ 34, Moves.TAKE_DOWN ],
    [ 42, Moves.AGILITY ],
    [ 50, Moves.TOXIC ],
    [ 58, Moves.VENOM_DRENCH ],
    [ 66, Moves.DOUBLE_EDGE ],
    [ 74, Moves.MEGAHORN ],
  ],
  [Species.COTTONEE]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HELPING_HAND ],
    [ 3, Moves.FAIRY_WIND ],
    [ 6, Moves.STUN_SPORE ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 18, Moves.GROWTH ],
    [ 21, Moves.POISON_POWDER ],
    [ 24, Moves.GIGA_DRAIN ],
    [ 27, Moves.CHARM ],
    [ 30, Moves.LEECH_SEED ],
    [ 33, Moves.COTTON_SPORE ],
    [ 36, Moves.ENERGY_BALL ],
    [ 39, Moves.SUNNY_DAY ],
    [ 42, Moves.ENDEAVOR ],
    [ 45, Moves.COTTON_GUARD ],
    [ 48, Moves.SOLAR_BEAM ],
  ],
  [Species.WHIMSICOTT]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.SOLAR_BEAM ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.COTTON_SPORE ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.SUNNY_DAY ],
    [ 1, Moves.MEMENTO ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.ENDEAVOR ],
    [ 1, Moves.TAILWIND ],
    [ 1, Moves.ENERGY_BALL ],
    [ 1, Moves.COTTON_GUARD ],
    [ 1, Moves.HURRICANE ],
    [ 1, Moves.FAIRY_WIND ],
    [ 1, Moves.MOONBLAST ],
  ],
  [Species.PETILIL]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 3, Moves.HELPING_HAND ],
    [ 6, Moves.STUN_SPORE ],
    [ 9, Moves.MEGA_DRAIN ],
    [ 12, Moves.CHARM ],
    [ 15, Moves.MAGICAL_LEAF ],
    [ 18, Moves.SLEEP_POWDER ],
    [ 21, Moves.GIGA_DRAIN ],
    [ 24, Moves.LEECH_SEED ],
    [ 27, Moves.AFTER_YOU ],
    [ 30, Moves.ENERGY_BALL ],
    [ 33, Moves.SYNTHESIS ],
    [ 36, Moves.SUNNY_DAY ],
    [ 39, Moves.ENTRAINMENT ],
    [ 42, Moves.LEAF_STORM ],
  ],
  [Species.LILLIGANT]: [
    [ EVOLVE_MOVE, Moves.PETAL_DANCE ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.SLEEP_POWDER ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.SYNTHESIS ],
    [ 1, Moves.SUNNY_DAY ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.TEETER_DANCE ],
    [ 1, Moves.ENERGY_BALL ],
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.QUIVER_DANCE ],
    [ 1, Moves.ENTRAINMENT ],
    [ 1, Moves.AFTER_YOU ],
    [ 1, Moves.PETAL_BLIZZARD ],
    [ 5, Moves.MAGICAL_LEAF ],
  ],
  [Species.BASCULIN]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.TACKLE ],
    [ 8, Moves.FLAIL ],
    [ 12, Moves.AQUA_JET ],
    [ 16, Moves.BITE ],
    [ 20, Moves.SCARY_FACE ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.SOAK ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.FINAL_GAMBIT ],
    [ 44, Moves.WAVE_CRASH ],
    [ 48, Moves.THRASH ],
    [ 52, Moves.DOUBLE_EDGE ],
    [ 56, Moves.HEAD_SMASH ],
  ],
  [Species.SANDILE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.POWER_TRIP ],
    [ 3, Moves.SAND_ATTACK ],
    [ 6, Moves.HONE_CLAWS ],
    [ 9, Moves.SAND_TOMB ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.BITE ],
    [ 18, Moves.TORMENT ],
    [ 21, Moves.DIG ],
    [ 24, Moves.SWAGGER ],
    [ 27, Moves.CRUNCH ],
    [ 30, Moves.SANDSTORM ],
    [ 33, Moves.FOUL_PLAY ],
    [ 36, Moves.EARTHQUAKE ],
    [ 39, Moves.THRASH ],
  ],
  [Species.KROKOROK]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.HONE_CLAWS ],
    [ 1, Moves.POWER_TRIP ],
    [ 9, Moves.SAND_TOMB ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.BITE ],
    [ 18, Moves.TORMENT ],
    [ 21, Moves.DIG ],
    [ 24, Moves.SWAGGER ],
    [ 27, Moves.CRUNCH ],
    [ 32, Moves.SANDSTORM ],
    [ 35, Moves.FOUL_PLAY ],
    [ 42, Moves.EARTHQUAKE ],
    [ 47, Moves.THRASH ],
  ],
  [Species.KROOKODILE]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.HONE_CLAWS ],
    [ 1, Moves.POWER_TRIP ],
    [ 9, Moves.SAND_TOMB ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.BITE ],
    [ 18, Moves.TORMENT ],
    [ 21, Moves.DIG ],
    [ 24, Moves.SWAGGER ],
    [ 27, Moves.CRUNCH ],
    [ 32, Moves.SANDSTORM ],
    [ 35, Moves.FOUL_PLAY ],
    [ 44, Moves.EARTHQUAKE ],
    [ 51, Moves.THRASH ],
    [ 58, Moves.OUTRAGE ],
  ],
  [Species.DARUMAKA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.EMBER ],
    [ 4, Moves.TAUNT ],
    [ 8, Moves.BITE ],
    [ 12, Moves.INCINERATE ],
    [ 16, Moves.WORK_UP ],
    [ 20, Moves.FIRE_FANG ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.FIRE_PUNCH ],
    [ 32, Moves.UPROAR ],
    [ 36, Moves.BELLY_DRUM ],
    [ 40, Moves.FLARE_BLITZ ],
    [ 44, Moves.THRASH ],
    [ 48, Moves.SUPERPOWER ],
  ],
  [Species.DARMANITAN]: [
    [ EVOLVE_MOVE, Moves.HAMMER_ARM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.TAUNT ],
    [ 12, Moves.INCINERATE ],
    [ 16, Moves.WORK_UP ],
    [ 20, Moves.FIRE_FANG ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.FIRE_PUNCH ],
    [ 32, Moves.UPROAR ],
    [ 38, Moves.BELLY_DRUM ],
    [ 44, Moves.FLARE_BLITZ ],
    [ 50, Moves.THRASH ],
    [ 56, Moves.SUPERPOWER ],
  ],
  [Species.MARACTUS]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.INGRAIN ],
    [ 1, Moves.AFTER_YOU ],
    [ 1, Moves.SPIKY_SHIELD ],
    [ 4, Moves.GROWTH ],
    [ 8, Moves.MEGA_DRAIN ],
    [ 12, Moves.LEECH_SEED ],
    [ 16, Moves.SUCKER_PUNCH ],
    [ 20, Moves.PIN_MISSILE ],
    [ 24, Moves.GIGA_DRAIN ],
    [ 28, Moves.SWEET_SCENT ],
    [ 32, Moves.SYNTHESIS ],
    [ 36, Moves.PETAL_BLIZZARD ],
    [ 40, Moves.COTTON_SPORE ],
    [ 44, Moves.SUNNY_DAY ],
    [ 48, Moves.SOLAR_BEAM ],
    [ 52, Moves.ACUPRESSURE ],
    [ 56, Moves.PETAL_DANCE ],
    [ 60, Moves.COTTON_GUARD ],
  ],
  [Species.DWEBBLE]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.FURY_CUTTER ],
    [ 4, Moves.WITHDRAW ],
    [ 8, Moves.SMACK_DOWN ],
    [ 12, Moves.BUG_BITE ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.SLASH ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 28, Moves.STEALTH_ROCK ],
    [ 32, Moves.ROCK_BLAST ],
    [ 36, Moves.X_SCISSOR ],
    [ 40, Moves.ROCK_POLISH ],
    [ 44, Moves.SHELL_SMASH ],
    [ 48, Moves.ROCK_WRECKER ],
  ],
  [Species.CRUSTLE]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.SMACK_DOWN ],
    [ 12, Moves.BUG_BITE ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.SLASH ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 28, Moves.STEALTH_ROCK ],
    [ 32, Moves.ROCK_BLAST ],
    [ 38, Moves.X_SCISSOR ],
    [ 44, Moves.ROCK_POLISH ],
    [ 50, Moves.SHELL_SMASH ],
    [ 56, Moves.ROCK_WRECKER ],
  ],
  [Species.SCRAGGY]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 4, Moves.PAYBACK ],
    [ 8, Moves.HEADBUTT ],
    [ 12, Moves.SAND_ATTACK ],
    [ 16, Moves.FACADE ],
    [ 20, Moves.PROTECT ],
    [ 24, Moves.BEAT_UP ],
    [ 28, Moves.SCARY_FACE ],
    [ 32, Moves.BRICK_BREAK ],
    [ 36, Moves.SWAGGER ],
    [ 40, Moves.CRUNCH ],
    [ 44, Moves.HIGH_JUMP_KICK ],
    [ 48, Moves.FOCUS_PUNCH ],
    [ 52, Moves.HEAD_SMASH ],
  ],
  [Species.SCRAFTY]: [
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.PAYBACK ],
    [ 1, Moves.FEINT_ATTACK ],
    [ 12, Moves.SAND_ATTACK ],
    [ 16, Moves.FACADE ],
    [ 20, Moves.PROTECT ],
    [ 24, Moves.BEAT_UP ],
    [ 28, Moves.SCARY_FACE ],
    [ 32, Moves.BRICK_BREAK ],
    [ 36, Moves.SWAGGER ],
    [ 42, Moves.CRUNCH ],
    [ 48, Moves.HIGH_JUMP_KICK ],
    [ 54, Moves.FOCUS_PUNCH ],
    [ 60, Moves.HEAD_SMASH ],
  ],
  [Species.SIGILYPH]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.CONFUSION ],
    [ 5, Moves.GRAVITY ],
    [ 10, Moves.HYPNOSIS ],
    [ 15, Moves.AIR_CUTTER ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.WHIRLWIND ],
    [ 30, Moves.COSMIC_POWER ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.PSYCHIC ],
    [ 45, Moves.TAILWIND ],
    [ 50, Moves.LIGHT_SCREEN ],
    [ 50, Moves.REFLECT ],
    [ 55, Moves.SKY_ATTACK ],
    [ 60, Moves.SKILL_SWAP ],
  ],
  [Species.YAMASK]: [
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.HEAL_BLOCK ],
    [ 4, Moves.HAZE ],
    [ 8, Moves.NIGHT_SHADE ],
    [ 12, Moves.DISABLE ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.CRAFTY_SHIELD ],
    [ 24, Moves.HEX ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.GRUDGE ],
    [ 36, Moves.CURSE ],
    [ 40, Moves.SHADOW_BALL ],
    [ 44, Moves.DARK_PULSE ],
    [ 48, Moves.GUARD_SPLIT ],
    [ 48, Moves.POWER_SPLIT ],
    [ 52, Moves.DESTINY_BOND ],
  ],
  [Species.COFAGRIGUS]: [
    [ EVOLVE_MOVE, Moves.SHADOW_CLAW ],
    [ 1, Moves.NIGHT_SHADE ],
    [ 1, Moves.HAZE ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.HEAL_BLOCK ],
    [ 12, Moves.DISABLE ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.CRAFTY_SHIELD ],
    [ 24, Moves.HEX ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.GRUDGE ],
    [ 38, Moves.CURSE ],
    [ 44, Moves.SHADOW_BALL ],
    [ 50, Moves.DARK_PULSE ],
    [ 56, Moves.GUARD_SPLIT ],
    [ 56, Moves.POWER_SPLIT ],
    [ 62, Moves.DESTINY_BOND ],
  ],
  [Species.TIRTOUGA]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.BIDE ],
    [ 3, Moves.PROTECT ],
    [ 6, Moves.AQUA_JET ],
    [ 9, Moves.SMACK_DOWN ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 15, Moves.BITE ],
    [ 18, Moves.WIDE_GUARD ],
    [ 21, Moves.BRINE ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 27, Moves.CRUNCH ],
    [ 30, Moves.CURSE ],
    [ 33, Moves.IRON_DEFENSE ],
    [ 36, Moves.AQUA_TAIL ],
    [ 39, Moves.RAIN_DANCE ],
    [ 42, Moves.HYDRO_PUMP ],
    [ 45, Moves.SHELL_SMASH ],
  ],
  [Species.CARRACOSTA]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.BIDE ],
    [ 9, Moves.SMACK_DOWN ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 15, Moves.BITE ],
    [ 18, Moves.WIDE_GUARD ],
    [ 21, Moves.BRINE ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 27, Moves.CRUNCH ],
    [ 30, Moves.CURSE ],
    [ 33, Moves.IRON_DEFENSE ],
    [ 36, Moves.AQUA_TAIL ],
    [ 41, Moves.RAIN_DANCE ],
    [ 46, Moves.HYDRO_PUMP ],
    [ 51, Moves.SHELL_SMASH ],
  ],
  [Species.ARCHEN]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 3, Moves.ROCK_THROW ],
    [ 6, Moves.WING_ATTACK ],
    [ 9, Moves.DRAGON_BREATH ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 15, Moves.PLUCK ],
    [ 18, Moves.QUICK_GUARD ],
    [ 21, Moves.U_TURN ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 27, Moves.SCARY_FACE ],
    [ 30, Moves.CRUNCH ],
    [ 33, Moves.AGILITY ],
    [ 36, Moves.TAILWIND ],
    [ 39, Moves.DRAGON_CLAW ],
    [ 42, Moves.THRASH ],
    [ 45, Moves.ENDEAVOR ],
  ],
  [Species.ARCHEOPS]: [
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 9, Moves.DRAGON_BREATH ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 15, Moves.PLUCK ],
    [ 18, Moves.QUICK_GUARD ],
    [ 21, Moves.U_TURN ],
    [ 24, Moves.ROCK_SLIDE ],
    [ 27, Moves.SCARY_FACE ],
    [ 30, Moves.CRUNCH ],
    [ 33, Moves.AGILITY ],
    [ 36, Moves.TAILWIND ],
    [ 41, Moves.DRAGON_CLAW ],
    [ 46, Moves.THRASH ],
    [ 51, Moves.ENDEAVOR ],
  ],
  [Species.TRUBBISH]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.POISON_GAS ],
    [ 3, Moves.RECYCLE ],
    [ 6, Moves.ACID_SPRAY ],
    [ 9, Moves.AMNESIA ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 15, Moves.TOXIC_SPIKES ],
    [ 18, Moves.SLUDGE ],
    [ 21, Moves.STOCKPILE ],
    [ 21, Moves.SWALLOW ],
    [ 24, Moves.TAKE_DOWN ],
    [ 27, Moves.SLUDGE_BOMB ],
    [ 30, Moves.TOXIC ],
    [ 33, Moves.BELCH ],
    [ 37, Moves.PAIN_SPLIT ],
    [ 39, Moves.GUNK_SHOT ],
    [ 42, Moves.EXPLOSION ],
  ],
  [Species.GARBODOR]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.ACID_SPRAY ],
    [ 9, Moves.AMNESIA ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 15, Moves.TOXIC_SPIKES ],
    [ 18, Moves.SLUDGE ],
    [ 21, Moves.STOCKPILE ],
    [ 21, Moves.SWALLOW ],
    [ 24, Moves.BODY_SLAM ],
    [ 27, Moves.SLUDGE_BOMB ],
    [ 30, Moves.TOXIC ],
    [ 33, Moves.BELCH ],
    [ 39, Moves.PAIN_SPLIT ],
    [ 43, Moves.GUNK_SHOT ],
    [ 48, Moves.EXPLOSION ],
  ],
  [Species.ZORUA]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 4, Moves.TORMENT ],
    [ 8, Moves.HONE_CLAWS ],
    [ 12, Moves.FURY_SWIPES ],
    [ 16, Moves.SCARY_FACE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.KNOCK_OFF ],
    [ 28, Moves.FAKE_TEARS ],
    [ 32, Moves.AGILITY ],
    [ 36, Moves.IMPRISON ],
    [ 40, Moves.NIGHT_DAZE ],
    [ 44, Moves.NASTY_PLOT ],
    [ 48, Moves.FOUL_PLAY ],
  ],
  [Species.ZOROARK]: [
    [ EVOLVE_MOVE, Moves.NIGHT_SLASH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.TORMENT ],
    [ 1, Moves.U_TURN ],
    [ 1, Moves.HONE_CLAWS ],
    [ 1, Moves.PURSUIT ],
    [ 12, Moves.FURY_SWIPES ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.KNOCK_OFF ],
    [ 28, Moves.FAKE_TEARS ],
    [ 34, Moves.AGILITY ],
    [ 40, Moves.IMPRISON ],
    [ 46, Moves.NIGHT_DAZE ],
    [ 52, Moves.NASTY_PLOT ],
    [ 58, Moves.FOUL_PLAY ],
  ],
  [Species.MINCCINO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 4, Moves.HELPING_HAND ],
    [ 8, Moves.ECHOED_VOICE ],
    [ 12, Moves.SING ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.SWIFT ],
    [ 24, Moves.ENCORE ],
    [ 28, Moves.AFTER_YOU ],
    [ 32, Moves.TAIL_SLAP ],
    [ 36, Moves.TICKLE ],
    [ 40, Moves.SLAM ],
    [ 44, Moves.HYPER_VOICE ],
    [ 48, Moves.LAST_RESORT ],
  ],
  [Species.CINCCINO]: [
    [ EVOLVE_MOVE, Moves.TAIL_SLAP ],
    [ RELEARN_MOVE, Moves.SLAM ],
    [ RELEARN_MOVE, Moves.SWIFT ],
    [ RELEARN_MOVE, Moves.ENCORE ],
    [ RELEARN_MOVE, Moves.HELPING_HAND ],
    [ RELEARN_MOVE, Moves.HYPER_VOICE ],
    [ RELEARN_MOVE, Moves.TICKLE ],
    [ RELEARN_MOVE, Moves.ROCK_BLAST ],
    [ RELEARN_MOVE, Moves.LAST_RESORT ],
    [ RELEARN_MOVE, Moves.AFTER_YOU ],
    [ RELEARN_MOVE, Moves.ECHOED_VOICE ],
    [ RELEARN_MOVE, Moves.BABY_DOLL_EYES ],
    [ 1, Moves.BULLET_SEED ],
    [ 1, Moves.SING ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.POUND ],
  ],
  [Species.GOTHITA]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.CONFUSION ],
    [ 4, Moves.PLAY_NICE ],
    [ 8, Moves.TICKLE ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.PSYSHOCK ],
    [ 24, Moves.HYPNOSIS ],
    [ 28, Moves.FAKE_TEARS ],
    [ 33, Moves.PSYCH_UP ],
    [ 34, Moves.HEAL_BLOCK ],
    [ 36, Moves.PSYCHIC ],
    [ 40, Moves.FLATTER ],
    [ 44, Moves.FUTURE_SIGHT ],
    [ 48, Moves.MAGIC_ROOM ],
  ],
  [Species.GOTHORITA]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.TICKLE ],
    [ 1, Moves.PLAY_NICE ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.PSYSHOCK ],
    [ 24, Moves.HYPNOSIS ],
    [ 28, Moves.FAKE_TEARS ],
    [ 34, Moves.HEAL_BLOCK ],
    [ 35, Moves.PSYCH_UP ],
    [ 46, Moves.FLATTER ],
    [ 52, Moves.FUTURE_SIGHT ],
    [ 58, Moves.MAGIC_ROOM ],
  ],
  [Species.GOTHITELLE]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.TICKLE ],
    [ 1, Moves.PLAY_NICE ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.PSYSHOCK ],
    [ 24, Moves.HYPNOSIS ],
    [ 28, Moves.FAKE_TEARS ],
    [ 34, Moves.HEAL_BLOCK ],
    [ 35, Moves.PSYCH_UP ],
    [ 40, Moves.PSYCHIC ],
    [ 48, Moves.FLATTER ],
    [ 56, Moves.FUTURE_SIGHT ],
    [ 64, Moves.MAGIC_ROOM ],
  ],
  [Species.SOLOSIS]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.PSYWAVE ],
    [ 4, Moves.RECOVER ],
    [ 8, Moves.ENDEAVOR ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.PSYSHOCK ],
    [ 24, Moves.LIGHT_SCREEN ],
    [ 24, Moves.REFLECT ],
    [ 28, Moves.ALLY_SWITCH ],
    [ 33, Moves.PAIN_SPLIT ],
    [ 36, Moves.PSYCHIC ],
    [ 40, Moves.SKILL_SWAP ],
    [ 44, Moves.FUTURE_SIGHT ],
    [ 46, Moves.HEAL_BLOCK ],
    [ 48, Moves.WONDER_ROOM ],
  ],
  [Species.DUOSION]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.RECOVER ],
    [ 1, Moves.PSYWAVE ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ENDEAVOR ],
    [ 1, Moves.SNATCH ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.PSYSHOCK ],
    [ 24, Moves.LIGHT_SCREEN ],
    [ 24, Moves.REFLECT ],
    [ 28, Moves.ALLY_SWITCH ],
    [ 35, Moves.PAIN_SPLIT ],
    [ 40, Moves.PSYCHIC ],
    [ 46, Moves.SKILL_SWAP ],
    [ 50, Moves.HEAL_BLOCK ],
    [ 52, Moves.FUTURE_SIGHT ],
    [ 58, Moves.WONDER_ROOM ],
  ],
  [Species.REUNICLUS]: [
    [ EVOLVE_MOVE, Moves.HAMMER_ARM ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.RECOVER ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ENDEAVOR ],
    [ 1, Moves.DIZZY_PUNCH ],
    [ 1, Moves.PSYWAVE ],
    [ 1, Moves.SNATCH ],
    [ 12, Moves.PSYBEAM ],
    [ 16, Moves.CHARM ],
    [ 20, Moves.PSYSHOCK ],
    [ 24, Moves.LIGHT_SCREEN ],
    [ 24, Moves.REFLECT ],
    [ 28, Moves.ALLY_SWITCH ],
    [ 35, Moves.PAIN_SPLIT ],
    [ 40, Moves.PSYCHIC ],
    [ 48, Moves.SKILL_SWAP ],
    [ 54, Moves.HEAL_BLOCK ],
    [ 56, Moves.FUTURE_SIGHT ],
    [ 64, Moves.WONDER_ROOM ],
  ],
  [Species.DUCKLETT]: [
    [ 1, Moves.WATER_GUN ],
    [ 6, Moves.DEFOG ],
    [ 9, Moves.WING_ATTACK ],
    [ 13, Moves.WATER_PULSE ],
    [ 15, Moves.AERIAL_ACE ],
    [ 19, Moves.BUBBLE_BEAM ],
    [ 21, Moves.FEATHER_DANCE ],
    [ 24, Moves.AQUA_RING ],
    [ 27, Moves.AIR_SLASH ],
    [ 30, Moves.ROOST ],
    [ 34, Moves.RAIN_DANCE ],
    [ 37, Moves.TAILWIND ],
    [ 41, Moves.BRAVE_BIRD ],
    [ 46, Moves.HURRICANE ],
  ],
  [Species.SWANNA]: [
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DEFOG ],
    [ 13, Moves.WATER_PULSE ],
    [ 15, Moves.AERIAL_ACE ],
    [ 19, Moves.BUBBLE_BEAM ],
    [ 21, Moves.FEATHER_DANCE ],
    [ 24, Moves.AQUA_RING ],
    [ 27, Moves.AIR_SLASH ],
    [ 30, Moves.ROOST ],
    [ 34, Moves.RAIN_DANCE ],
    [ 40, Moves.TAILWIND ],
    [ 47, Moves.BRAVE_BIRD ],
    [ 55, Moves.HURRICANE ],
  ],
  [Species.VANILLITE]: [
    [ 1, Moves.HARDEN ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.POWDER_SNOW ], //Custom
    [ 4, Moves.TAUNT ],
    [ 8, Moves.MIST ],
    [ 12, Moves.ICY_WIND ],
    [ 16, Moves.AVALANCHE ],
    [ 20, Moves.HAIL ],
    [ 24, Moves.ICICLE_SPEAR ],
    [ 28, Moves.UPROAR ],
    [ 32, Moves.ACID_ARMOR ],
    [ 36, Moves.MIRROR_COAT ],
    [ 40, Moves.ICE_BEAM ],
    [ 44, Moves.BLIZZARD ],
    [ 48, Moves.SHEER_COLD ],
  ],
  [Species.VANILLISH]: [
    [ 1, Moves.MIST ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.ASTONISH ],
    [ 12, Moves.ICY_WIND ],
    [ 16, Moves.AVALANCHE ],
    [ 20, Moves.HAIL ],
    [ 24, Moves.ICICLE_SPEAR ],
    [ 28, Moves.UPROAR ],
    [ 32, Moves.ACID_ARMOR ],
    [ 38, Moves.MIRROR_COAT ],
    [ 44, Moves.ICE_BEAM ],
    [ 50, Moves.BLIZZARD ],
    [ 56, Moves.SHEER_COLD ],
  ],
  [Species.VANILLUXE]: [
    [ 1, Moves.MIST ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.WEATHER_BALL ],
    [ 1, Moves.ICICLE_CRASH ],
    [ 1, Moves.FREEZE_DRY ],
    [ 12, Moves.ICY_WIND ],
    [ 16, Moves.AVALANCHE ],
    [ 20, Moves.HAIL ],
    [ 24, Moves.ICICLE_SPEAR ],
    [ 28, Moves.UPROAR ],
    [ 32, Moves.ACID_ARMOR ],
    [ 38, Moves.MIRROR_COAT ],
    [ 44, Moves.ICE_BEAM ],
    [ 52, Moves.BLIZZARD ],
    [ 60, Moves.SHEER_COLD ],
  ],
  [Species.DEERLING]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CAMOUFLAGE ],
    [ 4, Moves.GROWL ],
    [ 7, Moves.SAND_ATTACK ],
    [ 10, Moves.DOUBLE_KICK ],
    [ 13, Moves.LEECH_SEED ],
    [ 16, Moves.BULLET_SEED ],
    [ 20, Moves.TAKE_DOWN ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 28, Moves.ENERGY_BALL ],
    [ 32, Moves.CHARM ],
    [ 37, Moves.DOUBLE_EDGE ],
    [ 42, Moves.SOLAR_BEAM ],
  ],
  [Species.SAWSBUCK]: [
    [ EVOLVE_MOVE, Moves.HORN_LEECH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.CAMOUFLAGE ],
    [ 1, Moves.MEGAHORN ],
    [ 10, Moves.DOUBLE_KICK ],
    [ 13, Moves.LEECH_SEED ],
    [ 16, Moves.BULLET_SEED ],
    [ 20, Moves.TAKE_DOWN ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 28, Moves.ENERGY_BALL ],
    [ 36, Moves.CHARM ],
    [ 44, Moves.DOUBLE_EDGE ],
    [ 52, Moves.SOLAR_BEAM ],
  ],
  [Species.EMOLGA]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.NUZZLE ],
    [ 5, Moves.DOUBLE_TEAM ],
    [ 10, Moves.QUICK_ATTACK ],
    [ 15, Moves.THUNDER_SHOCK ],
    [ 20, Moves.CHARGE ],
    [ 25, Moves.ACROBATICS ],
    [ 30, Moves.SPARK ],
    [ 35, Moves.ENCORE ],
    [ 40, Moves.VOLT_SWITCH ],
    [ 45, Moves.LIGHT_SCREEN ],
    [ 50, Moves.DISCHARGE ],
    [ 55, Moves.AGILITY ],
  ],
  [Species.KARRABLAST]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 4, Moves.FURY_CUTTER ],
    [ 8, Moves.ENDURE ],
    [ 12, Moves.FALSE_SWIPE ],
    [ 16, Moves.ACID_SPRAY ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.FLAIL ],
    [ 28, Moves.SCARY_FACE ],
    [ 32, Moves.X_SCISSOR ],
    [ 36, Moves.SWORDS_DANCE ],
    [ 40, Moves.TAKE_DOWN ],
    [ 44, Moves.BUG_BUZZ ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.ESCAVALIER]: [
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.ENDURE ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.QUICK_GUARD ],
    [ 1, Moves.FELL_STINGER ],
    [ 1, Moves.TWINEEDLE ],
    [ 12, Moves.FALSE_SWIPE ],
    [ 16, Moves.ACID_SPRAY ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.REVERSAL ],
    [ 28, Moves.IRON_DEFENSE ],
    [ 32, Moves.X_SCISSOR ],
    [ 36, Moves.SWORDS_DANCE ],
    [ 40, Moves.IRON_HEAD ],
    [ 44, Moves.BUG_BUZZ ],
    [ 48, Moves.GIGA_IMPACT ],
    [ 52, Moves.METAL_BURST ],
  ],
  [Species.FOONGUS]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.GROWTH ],
    [ 8, Moves.STUN_SPORE ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.SYNTHESIS ],
    [ 20, Moves.CLEAR_SMOG ],
    [ 24, Moves.SWEET_SCENT ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 32, Moves.INGRAIN ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.RAGE_POWDER ],
    [ 44, Moves.SOLAR_BEAM ],
    [ 48, Moves.SPORE ],
  ],
  [Species.AMOONGUSS]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.BIDE ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.SYNTHESIS ],
    [ 20, Moves.CLEAR_SMOG ],
    [ 24, Moves.SWEET_SCENT ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 32, Moves.INGRAIN ],
    [ 36, Moves.TOXIC ],
    [ 42, Moves.RAGE_POWDER ],
    [ 48, Moves.SOLAR_BEAM ],
    [ 54, Moves.SPORE ],
  ],
  [Species.FRILLISH]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.WATER_SPORT ],
    [ 4, Moves.POISON_STING ],
    [ 8, Moves.NIGHT_SHADE ],
    [ 12, Moves.WATER_PULSE ],
    [ 16, Moves.RAIN_DANCE ],
    [ 20, Moves.HEX ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.RECOVER ],
    [ 32, Moves.SHADOW_BALL ],
    [ 36, Moves.WHIRLPOOL ],
    [ 41, Moves.HYDRO_PUMP ],
    [ 44, Moves.DESTINY_BOND ],
    [ 48, Moves.WATER_SPOUT ],
  ],
  [Species.JELLICENT]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WATER_SPORT ],
    [ 1, Moves.WRING_OUT ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.NIGHT_SHADE ],
    [ 1, Moves.ACID_ARMOR ],
    [ 12, Moves.WATER_PULSE ],
    [ 16, Moves.RAIN_DANCE ],
    [ 20, Moves.HEX ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.RECOVER ],
    [ 32, Moves.SHADOW_BALL ],
    [ 36, Moves.WHIRLPOOL ],
    [ 43, Moves.HYDRO_PUMP ],
    [ 48, Moves.DESTINY_BOND ],
    [ 54, Moves.WATER_SPOUT ],
  ],
  [Species.ALOMOMOLA]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.WATER_SPORT ],
    [ 5, Moves.AQUA_RING ],
    [ 9, Moves.AQUA_JET ],
    [ 13, Moves.HELPING_HAND ],
    [ 13, Moves.WIDE_GUARD ],
    [ 21, Moves.PROTECT ],
    [ 25, Moves.WATER_PULSE ],
    [ 29, Moves.HEALING_WISH ],
    [ 33, Moves.SOAK ],
    [ 37, Moves.WISH ],
    [ 41, Moves.BRINE ],
    [ 45, Moves.SAFEGUARD ],
    [ 49, Moves.WHIRLPOOL ],
    [ 55, Moves.HYDRO_PUMP ],
  ],
  [Species.JOLTIK]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.SPIDER_WEB ],
    [ 4, Moves.ELECTROWEB ],
    [ 8, Moves.BUG_BITE ],
    [ 12, Moves.STRING_SHOT ],
    [ 16, Moves.THUNDER_WAVE ],
    [ 20, Moves.ELECTRO_BALL ],
    [ 24, Moves.AGILITY ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.SLASH ],
    [ 37, Moves.DISCHARGE ],
    [ 40, Moves.SCREECH ],
    [ 44, Moves.GASTRO_ACID ],
    [ 48, Moves.BUG_BUZZ ],
  ],
  [Species.GALVANTULA]: [
    [ EVOLVE_MOVE, Moves.STICKY_WEB ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.ELECTROWEB ],
    [ 1, Moves.SPIDER_WEB ],
    [ 12, Moves.STRING_SHOT ],
    [ 16, Moves.THUNDER_WAVE ],
    [ 20, Moves.ELECTRO_BALL ],
    [ 24, Moves.AGILITY ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 32, Moves.SLASH ],
    [ 39, Moves.DISCHARGE ],
    [ 44, Moves.SCREECH ],
    [ 50, Moves.GASTRO_ACID ],
    [ 56, Moves.BUG_BUZZ ],
  ],
  [Species.FERROSEED]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.METAL_CLAW ],
    [ 10, Moves.PIN_MISSILE ],
    [ 15, Moves.INGRAIN ],
    [ 20, Moves.FLASH_CANNON ],
    [ 25, Moves.IRON_HEAD ],
    [ 30, Moves.SELF_DESTRUCT ],
    [ 35, Moves.IRON_DEFENSE ],
    [ 41, Moves.CURSE ],
    [ 45, Moves.GYRO_BALL ],
    [ 50, Moves.EXPLOSION ],
  ],
  [Species.FERROTHORN]: [
    [ EVOLVE_MOVE, Moves.POWER_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PIN_MISSILE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.ROCK_CLIMB ],
    [ 15, Moves.INGRAIN ],
    [ 20, Moves.FLASH_CANNON ],
    [ 25, Moves.IRON_HEAD ],
    [ 30, Moves.SELF_DESTRUCT ],
    [ 35, Moves.IRON_DEFENSE ],
    [ 43, Moves.CURSE ],
    [ 49, Moves.GYRO_BALL ],
    [ 56, Moves.EXPLOSION ],
  ],
  [Species.KLINK]: [
    [ 1, Moves.VISE_GRIP ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 4, Moves.BIND ],
    [ 8, Moves.CHARGE ],
    [ 12, Moves.CHARGE_BEAM ],
    [ 16, Moves.METAL_SOUND ],
    [ 20, Moves.AUTOTOMIZE ],
    [ 24, Moves.DISCHARGE ],
    [ 28, Moves.SCREECH ],
    [ 32, Moves.GEAR_GRIND ],
    [ 36, Moves.LOCK_ON ],
    [ 40, Moves.SHIFT_GEAR ],
    [ 44, Moves.ZAP_CANNON ],
    [ 48, Moves.HYPER_BEAM ],
  ],
  [Species.KLANG]: [
    [ 1, Moves.VISE_GRIP ],
    [ 1, Moves.BIND ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.CHARGE ],
    [ 12, Moves.CHARGE_BEAM ],
    [ 16, Moves.METAL_SOUND ],
    [ 20, Moves.AUTOTOMIZE ],
    [ 24, Moves.DISCHARGE ],
    [ 28, Moves.SCREECH ],
    [ 32, Moves.GEAR_GRIND ],
    [ 36, Moves.LOCK_ON ],
    [ 42, Moves.SHIFT_GEAR ],
    [ 48, Moves.ZAP_CANNON ],
    [ 54, Moves.HYPER_BEAM ],
  ],
  [Species.KLINKLANG]: [
    [ 1, Moves.VISE_GRIP ],
    [ 1, Moves.BIND ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.MAGNETIC_FLUX ],
    [ 1, Moves.GEAR_UP ],
    [ 12, Moves.CHARGE_BEAM ],
    [ 16, Moves.METAL_SOUND ],
    [ 20, Moves.AUTOTOMIZE ],
    [ 24, Moves.DISCHARGE ],
    [ 28, Moves.SCREECH ],
    [ 32, Moves.GEAR_GRIND ],
    [ 36, Moves.LOCK_ON ],
    [ 42, Moves.SHIFT_GEAR ],
    [ 48, Moves.ZAP_CANNON ],
    [ 56, Moves.HYPER_BEAM ],
    [ 64, Moves.ELECTRIC_TERRAIN ],
  ],
  [Species.TYNAMO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.SPARK ],
    [ 1, Moves.CHARGE_BEAM ],
  ],
  [Species.EELEKTRIK]: [
    [ EVOLVE_MOVE, Moves.CRUNCH ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.SPARK ],
    [ 1, Moves.CHARGE_BEAM ],
    [ 1, Moves.ION_DELUGE ],
    [ 9, Moves.BIND ],
    [ 19, Moves.ACID ],
    [ 29, Moves.DISCHARGE ],
    [ 44, Moves.THUNDERBOLT ],
    [ 49, Moves.ACID_SPRAY ],
    [ 54, Moves.COIL ],
    [ 59, Moves.WILD_CHARGE ],
    [ 64, Moves.GASTRO_ACID ],
    [ 69, Moves.ZAP_CANNON ],
    [ 74, Moves.THRASH ],
  ],
  [Species.EELEKTROSS]: [
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.THRASH ],
    [ 1, Moves.ACID ],
    [ 1, Moves.ZAP_CANNON ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.CRUSH_CLAW ],
    [ 1, Moves.GASTRO_ACID ],
    [ 1, Moves.DISCHARGE ],
    [ 1, Moves.COIL ],
    [ 5, Moves.WILD_CHARGE ],
  ],
  [Species.ELGYEM]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.CONFUSION ],
    [ 6, Moves.IMPRISON ],
    [ 8, Moves.HEAL_BLOCK ],
    [ 12, Moves.TELEPORT ],
    [ 18, Moves.PSYBEAM ],
    [ 24, Moves.GUARD_SPLIT ],
    [ 24, Moves.POWER_SPLIT ],
    [ 30, Moves.HEADBUTT ],
    [ 36, Moves.ZEN_HEADBUTT ],
    [ 43, Moves.RECOVER ],
    [ 48, Moves.CALM_MIND ],
    [ 54, Moves.WONDER_ROOM ],
    [ 60, Moves.PSYCHIC ],
  ],
  [Species.BEHEEYEM]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.SYNCHRONOISE ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.IMPRISON ],
    [ 1, Moves.PSYCHIC_TERRAIN ],
    [ 8, Moves.HEAL_BLOCK ],
    [ 18, Moves.PSYBEAM ],
    [ 24, Moves.GUARD_SPLIT ],
    [ 24, Moves.POWER_SPLIT ],
    [ 30, Moves.HEADBUTT ],
    [ 36, Moves.ZEN_HEADBUTT ],
    [ 45, Moves.RECOVER ],
    [ 52, Moves.CALM_MIND ],
    [ 60, Moves.WONDER_ROOM ],
    [ 68, Moves.PSYCHIC ],
  ],
  [Species.LITWICK]: [
    [ 1, Moves.SMOG ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.EMBER ],
    [ 8, Moves.MINIMIZE ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.HEX ],
    [ 20, Moves.WILL_O_WISP ],
    [ 24, Moves.FIRE_SPIN ],
    [ 28, Moves.NIGHT_SHADE ],
    [ 32, Moves.CURSE ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.INFERNO ],
    [ 44, Moves.IMPRISON ],
    [ 48, Moves.PAIN_SPLIT ],
    [ 52, Moves.OVERHEAT ],
    [ 56, Moves.MEMENTO ],
  ],
  [Species.LAMPENT]: [
    [ 1, Moves.EMBER ],
    [ 1, Moves.MINIMIZE ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.ASTONISH ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.HEX ],
    [ 20, Moves.WILL_O_WISP ],
    [ 24, Moves.FIRE_SPIN ],
    [ 28, Moves.NIGHT_SHADE ],
    [ 32, Moves.CURSE ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.INFERNO ],
    [ 46, Moves.IMPRISON ],
    [ 52, Moves.PAIN_SPLIT ],
    [ 58, Moves.OVERHEAT ],
    [ 64, Moves.MEMENTO ],
  ],
  [Species.CHANDELURE]: [
    [ 1, Moves.EMBER ],
    [ 1, Moves.FIRE_SPIN ],
    [ 1, Moves.NIGHT_SHADE ],
    [ 1, Moves.MINIMIZE ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.CURSE ],
    [ 1, Moves.PAIN_SPLIT ],
    [ 1, Moves.SHADOW_BALL ],
    [ 1, Moves.WILL_O_WISP ],
    [ 1, Moves.MEMENTO ],
    [ 1, Moves.IMPRISON ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.OVERHEAT ],
    [ 1, Moves.HEX ],
    [ 1, Moves.INFERNO ],
  ],
  [Species.AXEW]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 3, Moves.BITE ],
    [ 6, Moves.FALSE_SWIPE ],
    [ 9, Moves.ASSURANCE ],
    [ 12, Moves.TAUNT ],
    [ 15, Moves.SLASH ],
    [ 18, Moves.DRAGON_CLAW ],
    [ 21, Moves.SCARY_FACE ],
    [ 24, Moves.CRUNCH ],
    [ 27, Moves.DRAGON_DANCE ],
    [ 30, Moves.DUAL_CHOP ],
    [ 33, Moves.FOCUS_ENERGY ],
    [ 36, Moves.DRAGON_PULSE ],
    [ 39, Moves.SWORDS_DANCE ],
    [ 42, Moves.OUTRAGE ],
    [ 45, Moves.GUILLOTINE ],
    [ 48, Moves.GIGA_IMPACT ],
  ],
  [Species.FRAXURE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 1, Moves.DUAL_CHOP ],
    [ 9, Moves.ASSURANCE ],
    [ 12, Moves.TAUNT ],
    [ 15, Moves.SLASH ],
    [ 18, Moves.DRAGON_CLAW ],
    [ 21, Moves.SCARY_FACE ],
    [ 24, Moves.CRUNCH ],
    [ 27, Moves.DRAGON_DANCE ],
    [ 30, Moves.BREAKING_SWIPE ],
    [ 33, Moves.FOCUS_ENERGY ],
    [ 36, Moves.DRAGON_PULSE ],
    [ 41, Moves.SWORDS_DANCE ],
    [ 46, Moves.OUTRAGE ],
    [ 51, Moves.GUILLOTINE ],
    [ 56, Moves.GIGA_IMPACT ],
  ],
  [Species.HAXORUS]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 1, Moves.DUAL_CHOP ],
    [ 9, Moves.ASSURANCE ],
    [ 12, Moves.TAUNT ],
    [ 15, Moves.SLASH ],
    [ 18, Moves.DRAGON_CLAW ],
    [ 21, Moves.SCARY_FACE ],
    [ 24, Moves.CRUNCH ],
    [ 27, Moves.DRAGON_DANCE ],
    [ 30, Moves.BREAKING_SWIPE ],
    [ 33, Moves.FOCUS_ENERGY ],
    [ 36, Moves.DRAGON_PULSE ],
    [ 41, Moves.SWORDS_DANCE ],
    [ 46, Moves.OUTRAGE ],
    [ 53, Moves.GUILLOTINE ],
    [ 60, Moves.GIGA_IMPACT ],
  ],
  [Species.CUBCHOO]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.POWDER_SNOW ],
    [ 3, Moves.ENDURE ],
    [ 6, Moves.FURY_SWIPES ],
    [ 9, Moves.ICY_WIND ],
    [ 12, Moves.PLAY_NICE ],
    [ 15, Moves.BRINE ],
    [ 18, Moves.FROST_BREATH ],
    [ 21, Moves.SLASH ],
    [ 24, Moves.FLAIL ],
    [ 27, Moves.CHARM ],
    [ 30, Moves.SNOWSCAPE ],
    [ 33, Moves.THRASH ],
    [ 36, Moves.REST ],
    [ 39, Moves.BLIZZARD ],
    [ 42, Moves.SHEER_COLD ],
  ],
  [Species.BEARTIC]: [
    [ EVOLVE_MOVE, Moves.ICICLE_CRASH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FURY_SWIPES ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.ENDURE ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.BIDE ],
    [ 9, Moves.ICY_WIND ],
    [ 12, Moves.PLAY_NICE ],
    [ 15, Moves.BRINE ],
    [ 18, Moves.FROST_BREATH ],
    [ 21, Moves.SLASH ],
    [ 24, Moves.FLAIL ],
    [ 27, Moves.SWAGGER ],
    [ 30, Moves.SNOWSCAPE ],
    [ 33, Moves.THRASH ],
    [ 36, Moves.REST ],
    [ 41, Moves.BLIZZARD ],
    [ 46, Moves.SHEER_COLD ],
    [ 51, Moves.SUPERPOWER ],
  ],
  [Species.CRYOGONAL]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.ICE_SHARD ],
    [ 4, Moves.CONFUSE_RAY ],
    [ 8, Moves.RAPID_SPIN ],
    [ 12, Moves.ICY_WIND ],
    [ 16, Moves.MIST ],
    [ 16, Moves.HAZE ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 24, Moves.AURORA_BEAM ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.NIGHT_SLASH ],
    [ 36, Moves.FREEZE_DRY ],
    [ 40, Moves.LIGHT_SCREEN ],
    [ 40, Moves.REFLECT ],
    [ 44, Moves.RECOVER ],
    [ 48, Moves.ICE_BEAM ],
    [ 52, Moves.ACID_ARMOR ],
    [ 56, Moves.SOLAR_BEAM ],
    [ 60, Moves.SHEER_COLD ],
  ],
  [Species.SHELMET]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.PROTECT ],
    [ 4, Moves.ACID ],
    [ 8, Moves.CURSE ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.STRUGGLE_BUG ],
    [ 20, Moves.YAWN ],
    [ 24, Moves.ACID_ARMOR ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 32, Moves.GUARD_SWAP ],
    [ 36, Moves.BODY_SLAM ],
    [ 40, Moves.RECOVER ],
    [ 44, Moves.BUG_BUZZ ],
    [ 48, Moves.FINAL_GAMBIT ],
  ],
  [Species.ACCELGOR]: [
    [ 1, Moves.BODY_SLAM ],
    [ 1, Moves.ACID ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.ACID_ARMOR ],
    [ 1, Moves.CURSE ],
    [ 1, Moves.YAWN ],
    [ 1, Moves.GUARD_SWAP ],
    [ 1, Moves.ACID_SPRAY ],
    [ 1, Moves.WATER_SHURIKEN ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.STRUGGLE_BUG ],
    [ 20, Moves.SWIFT ],
    [ 24, Moves.AGILITY ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 32, Moves.POWER_SWAP ],
    [ 36, Moves.U_TURN ],
    [ 40, Moves.RECOVER ],
    [ 44, Moves.BUG_BUZZ ],
    [ 48, Moves.FINAL_GAMBIT ],
    [ 52, Moves.TOXIC ],
  ],
  [Species.STUNFISK]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.MUD_SPORT ],
    [ 5, Moves.ENDURE ],
    [ 10, Moves.MUD_SHOT ],
    [ 15, Moves.REVENGE ],
    [ 20, Moves.CHARGE ],
    [ 25, Moves.SUCKER_PUNCH ],
    [ 30, Moves.ELECTRIC_TERRAIN ],
    [ 35, Moves.BOUNCE ],
    [ 40, Moves.MUDDY_WATER ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.FLAIL ],
    [ 55, Moves.FISSURE ],
  ],
  [Species.MIENFOO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.DETECT ],
    [ 5, Moves.FAKE_OUT ],
    [ 10, Moves.REVERSAL ],
    [ 15, Moves.FURY_SWIPES ],
    [ 20, Moves.QUICK_GUARD ],
    [ 25, Moves.FORCE_PALM ],
    [ 30, Moves.U_TURN ],
    [ 35, Moves.DRAIN_PUNCH ],
    [ 40, Moves.HONE_CLAWS ],
    [ 45, Moves.AURA_SPHERE ],
    [ 51, Moves.BOUNCE ],
    [ 55, Moves.CALM_MIND ],
    [ 60, Moves.HIGH_JUMP_KICK ],
  ],
  [Species.MIENSHAO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.REVERSAL ],
    [ 1, Moves.DETECT ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.QUICK_GUARD ],
    [ 15, Moves.FURY_SWIPES ],
    [ 20, Moves.WIDE_GUARD ],
    [ 25, Moves.FORCE_PALM ],
    [ 30, Moves.U_TURN ],
    [ 35, Moves.DRAIN_PUNCH ],
    [ 40, Moves.HONE_CLAWS ],
    [ 45, Moves.AURA_SPHERE ],
    [ 53, Moves.BOUNCE ],
    [ 59, Moves.CALM_MIND ],
    [ 66, Moves.HIGH_JUMP_KICK ],
  ],
  [Species.DRUDDIGON]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.BITE ],
    [ 10, Moves.DRAGON_TAIL ],
    [ 15, Moves.METAL_CLAW ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.DRAGON_CLAW ],
    [ 35, Moves.HONE_CLAWS ],
    [ 40, Moves.CRUNCH ],
    [ 45, Moves.IRON_HEAD ],
    [ 50, Moves.OUTRAGE ],
    [ 55, Moves.SUPERPOWER ],
  ],
  [Species.GOLETT]: [
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 8, Moves.POUND ],
    [ 12, Moves.SHADOW_PUNCH ],
    [ 16, Moves.CURSE ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 24, Moves.STOMPING_TANTRUM ],
    [ 28, Moves.IRON_DEFENSE ],
    [ 32, Moves.MEGA_PUNCH ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.HEAVY_SLAM ],
    [ 44, Moves.PHANTOM_FORCE ],
    [ 48, Moves.HAMMER_ARM ],
    [ 52, Moves.EARTHQUAKE ],
    [ 56, Moves.DYNAMIC_PUNCH ],
  ],
  [Species.GOLURK]: [
    [ RELEARN_MOVE, Moves.MUD_SLAP ],
    [ RELEARN_MOVE, Moves.FOCUS_PUNCH ],
    [ 1, Moves.POUND ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.HIGH_HORSEPOWER ],
    [ 12, Moves.SHADOW_PUNCH ],
    [ 16, Moves.CURSE ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 24, Moves.STOMPING_TANTRUM ],
    [ 28, Moves.IRON_DEFENSE ],
    [ 32, Moves.MEGA_PUNCH ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.HEAVY_SLAM ],
    [ 46, Moves.PHANTOM_FORCE ],
    [ 52, Moves.HAMMER_ARM ],
    [ 58, Moves.EARTHQUAKE ],
    [ 64, Moves.DYNAMIC_PUNCH ],
  ],
  [Species.PAWNIARD]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.FURY_CUTTER ],
    [ 10, Moves.METAL_CLAW ],
    [ 15, Moves.TORMENT ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.ASSURANCE ],
    [ 30, Moves.METAL_SOUND ],
    [ 35, Moves.SLASH ],
    [ 40, Moves.NIGHT_SLASH ],
    [ 45, Moves.IRON_DEFENSE ],
    [ 50, Moves.RETALIATE ],
    [ 55, Moves.IRON_HEAD ],
    [ 60, Moves.SWORDS_DANCE ],
    [ 65, Moves.GUILLOTINE ],
  ],
  [Species.BISHARP]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.METAL_BURST ],
    [ 15, Moves.TORMENT ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.ASSURANCE ],
    [ 30, Moves.METAL_SOUND ],
    [ 35, Moves.SLASH ],
    [ 40, Moves.NIGHT_SLASH ],
    [ 45, Moves.IRON_DEFENSE ],
    [ 50, Moves.RETALIATE ],
    [ 57, Moves.IRON_HEAD ],
    [ 64, Moves.SWORDS_DANCE ],
    [ 71, Moves.GUILLOTINE ],
  ],
  [Species.BOUFFALANT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PURSUIT ],
    [ 5, Moves.FOCUS_ENERGY ],
    [ 10, Moves.FURY_ATTACK ],
    [ 15, Moves.REVENGE ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.HORN_ATTACK ],
    [ 30, Moves.REVERSAL ],
    [ 35, Moves.THROAT_CHOP ],
    [ 40, Moves.HEAD_CHARGE ],
    [ 45, Moves.SWORDS_DANCE ],
    [ 50, Moves.MEGAHORN ],
    [ 55, Moves.GIGA_IMPACT ],
  ],
  [Species.RUFFLET]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 6, Moves.HONE_CLAWS ],
    [ 12, Moves.WING_ATTACK ],
    [ 18, Moves.TAILWIND ],
    [ 24, Moves.SCARY_FACE ],
    [ 30, Moves.AERIAL_ACE ],
    [ 36, Moves.SLASH ],
    [ 42, Moves.WHIRLWIND ],
    [ 48, Moves.CRUSH_CLAW ],
    [ 55, Moves.AIR_SLASH ],
    [ 60, Moves.DEFOG ],
    [ 66, Moves.THRASH ],
    [ 72, Moves.BRAVE_BIRD ],
  ],
  [Species.BRAVIARY]: [
    [ EVOLVE_MOVE, Moves.SUPERPOWER ],
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.SKY_ATTACK ],
    [ 1, Moves.HONE_CLAWS ],
    [ 18, Moves.TAILWIND ],
    [ 24, Moves.SCARY_FACE ],
    [ 30, Moves.AERIAL_ACE ],
    [ 36, Moves.SLASH ],
    [ 42, Moves.WHIRLWIND ],
    [ 48, Moves.CRUSH_CLAW ],
    [ 57, Moves.AIR_SLASH ],
    [ 64, Moves.DEFOG ],
    [ 72, Moves.THRASH ],
  ],
  [Species.VULLABY]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 6, Moves.FLATTER ],
    [ 12, Moves.PLUCK ],
    [ 18, Moves.TAILWIND ],
    [ 24, Moves.KNOCK_OFF ],
    [ 30, Moves.IRON_DEFENSE ],
    [ 36, Moves.WHIRLWIND ],
    [ 42, Moves.AIR_SLASH ],
    [ 48, Moves.DARK_PULSE ],
    [ 54, Moves.NASTY_PLOT ],
    [ 60, Moves.DEFOG ],
    [ 66, Moves.ATTRACT ],
    [ 72, Moves.BRAVE_BIRD ],
  ],
  [Species.MANDIBUZZ]: [
    [ EVOLVE_MOVE, Moves.BONE_RUSH ],
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 1, Moves.TOXIC ],
    [ 1, Moves.SKY_ATTACK ],
    [ 1, Moves.FLATTER ],
    [ 1, Moves.PLUCK ],
    [ 18, Moves.TAILWIND ],
    [ 24, Moves.KNOCK_OFF ],
    [ 30, Moves.IRON_DEFENSE ],
    [ 36, Moves.WHIRLWIND ],
    [ 42, Moves.AIR_SLASH ],
    [ 48, Moves.DARK_PULSE ],
    [ 57, Moves.NASTY_PLOT ],
    [ 64, Moves.DEFOG ],
    [ 72, Moves.ATTRACT ],
    [ 80, Moves.BRAVE_BIRD ],
  ],
  [Species.HEATMOR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LICK ],
    [ 5, Moves.FURY_SWIPES ],
    [ 10, Moves.INCINERATE ],
    [ 15, Moves.BUG_BITE ],
    [ 20, Moves.STOCKPILE ],
    [ 20, Moves.SPIT_UP ],
    [ 20, Moves.SWALLOW ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.BIND ],
    [ 35, Moves.FIRE_LASH ],
    [ 40, Moves.HONE_CLAWS ],
    [ 45, Moves.AMNESIA ],
    [ 50, Moves.FIRE_SPIN ],
    [ 55, Moves.INFERNO ],
    [ 60, Moves.FLARE_BLITZ ],
  ],
  [Species.DURANT]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.FURY_CUTTER ],
    [ 4, Moves.VISE_GRIP ],
    [ 8, Moves.METAL_CLAW ],
    [ 12, Moves.BEAT_UP ],
    [ 16, Moves.BUG_BITE ],
    [ 20, Moves.BITE ],
    [ 24, Moves.AGILITY ],
    [ 28, Moves.DIG ],
    [ 32, Moves.X_SCISSOR ],
    [ 36, Moves.CRUNCH ],
    [ 40, Moves.METAL_SOUND ],
    [ 44, Moves.IRON_HEAD ],
    [ 48, Moves.ENTRAINMENT ],
    [ 52, Moves.IRON_DEFENSE ],
    [ 56, Moves.GUILLOTINE ],
  ],
  [Species.DEINO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 4, Moves.DRAGON_BREATH ],
    [ 8, Moves.BITE ],
    [ 12, Moves.ROAR ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.WORK_UP ],
    [ 28, Moves.SLAM ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.SCARY_FACE ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 44, Moves.BODY_SLAM ],
    [ 48, Moves.HYPER_VOICE ],
    [ 52, Moves.DRAGON_RUSH ],
    [ 56, Moves.NASTY_PLOT ],
    [ 60, Moves.OUTRAGE ],
  ],
  [Species.ZWEILOUS]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.DOUBLE_HIT ],
    [ 12, Moves.ROAR ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.WORK_UP ],
    [ 28, Moves.SLAM ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.SCARY_FACE ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 44, Moves.BODY_SLAM ],
    [ 48, Moves.HYPER_VOICE ],
    [ 54, Moves.DRAGON_RUSH ],
    [ 60, Moves.NASTY_PLOT ],
    [ 66, Moves.OUTRAGE ],
  ],
  [Species.HYDREIGON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.DOUBLE_HIT ],
    [ 12, Moves.ROAR ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.WORK_UP ],
    [ 28, Moves.SLAM ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.SCARY_FACE ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 44, Moves.BODY_SLAM ],
    [ 48, Moves.HYPER_VOICE ],
    [ 54, Moves.DRAGON_RUSH ],
    [ 60, Moves.NASTY_PLOT ],
    [ 68, Moves.OUTRAGE ],
    [ 76, Moves.HYPER_BEAM ],
  ],
  [Species.LARVESTA]: [
    [ 1, Moves.EMBER ],
    [ 1, Moves.STRING_SHOT ],
    [ 6, Moves.FLAME_CHARGE ],
    [ 12, Moves.STRUGGLE_BUG ],
    [ 18, Moves.FLAME_WHEEL ],
    [ 24, Moves.BUG_BITE ],
    [ 30, Moves.SCREECH ],
    [ 36, Moves.LEECH_LIFE ],
    [ 42, Moves.BUG_BUZZ ],
    [ 48, Moves.TAKE_DOWN ],
    [ 54, Moves.AMNESIA ],
    [ 60, Moves.DOUBLE_EDGE ],
    [ 66, Moves.FLARE_BLITZ ],
  ],
  [Species.VOLCARONA]: [
    [ EVOLVE_MOVE, Moves.QUIVER_DANCE ],
    [ 1, Moves.GUST ],
    [ 1, Moves.WHIRLWIND ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.FIRE_SPIN ],
    [ 1, Moves.FLARE_BLITZ ],
    [ 1, Moves.FLAME_CHARGE ],
    [ 1, Moves.STRUGGLE_BUG ],
    [ 1, Moves.FIERY_DANCE ],
    [ 18, Moves.FLAME_WHEEL ],
    [ 24, Moves.BUG_BITE ],
    [ 30, Moves.SCREECH ],
    [ 36, Moves.LEECH_LIFE ],
    [ 42, Moves.BUG_BUZZ ],
    [ 48, Moves.HEAT_WAVE ],
    [ 54, Moves.AMNESIA ],
    [ 62, Moves.HURRICANE ],
    [ 70, Moves.FIRE_BLAST ],
    [ 78, Moves.RAGE_POWDER ],
  ],
  [Species.COBALION]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.WORK_UP ],
    [ 7, Moves.METAL_CLAW ],
    [ 14, Moves.QUICK_GUARD ],
    [ 21, Moves.DOUBLE_KICK ],
    [ 28, Moves.RETALIATE ],
    [ 35, Moves.METAL_BURST ],
    [ 42, Moves.TAKE_DOWN ],
    [ 49, Moves.SACRED_SWORD ],
    [ 56, Moves.SWORDS_DANCE ],
    [ 63, Moves.IRON_HEAD ],
    [ 70, Moves.CLOSE_COMBAT ],
  ],
  [Species.TERRAKION]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.WORK_UP ],
    [ 7, Moves.SMACK_DOWN ],
    [ 14, Moves.QUICK_GUARD ],
    [ 21, Moves.DOUBLE_KICK ],
    [ 28, Moves.RETALIATE ],
    [ 35, Moves.ROCK_SLIDE ],
    [ 42, Moves.TAKE_DOWN ],
    [ 49, Moves.SACRED_SWORD ],
    [ 56, Moves.SWORDS_DANCE ],
    [ 63, Moves.STONE_EDGE ],
    [ 70, Moves.CLOSE_COMBAT ],
  ],
  [Species.VIRIZION]: [
    [ RELEARN_MOVE, Moves.TAKE_DOWN ],
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.WORK_UP ],
    [ 7, Moves.MAGICAL_LEAF ],
    [ 14, Moves.QUICK_GUARD ],
    [ 21, Moves.DOUBLE_KICK ],
    [ 28, Moves.RETALIATE ],
    [ 35, Moves.GIGA_DRAIN ],
    [ 42, Moves.TAKE_DOWN ],
    [ 49, Moves.SACRED_SWORD ],
    [ 56, Moves.SWORDS_DANCE ],
    [ 63, Moves.LEAF_BLADE ],
    [ 70, Moves.CLOSE_COMBAT ],
  ],
  [Species.TORNADUS]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.LEER ],
    [ 10, Moves.SWAGGER ],
    [ 15, Moves.BITE ],
    [ 20, Moves.AIR_CUTTER ],
    [ 25, Moves.AGILITY ],
    [ 30, Moves.TAILWIND ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.CRUNCH ],
    [ 45, Moves.EXTRASENSORY ],
    [ 50, Moves.UPROAR ],
    [ 55, Moves.HAMMER_ARM ],
    [ 60, Moves.RAIN_DANCE ],
    [ 65, Moves.HURRICANE ],
    [ 70, Moves.THRASH ],
    [ 77, Moves.BLEAKWIND_STORM ],
  ],
  [Species.THUNDURUS]: [
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.LEER ],
    [ 10, Moves.SWAGGER ],
    [ 15, Moves.BITE ],
    [ 20, Moves.SHOCK_WAVE ],
    [ 25, Moves.AGILITY ],
    [ 30, Moves.CHARGE ],
    [ 31, Moves.HEAL_BLOCK ],
    [ 35, Moves.VOLT_SWITCH ],
    [ 40, Moves.CRUNCH ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.UPROAR ],
    [ 55, Moves.HAMMER_ARM ],
    [ 60, Moves.RAIN_DANCE ],
    [ 65, Moves.THUNDER ],
    [ 70, Moves.THRASH ],
    [ 75, Moves.WILDBOLT_STORM ],
  ],
  [Species.RESHIRAM]: [
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 8, Moves.SLASH ],
    [ 16, Moves.CRUNCH ],
    [ 24, Moves.EXTRASENSORY ],
    [ 32, Moves.DRAGON_PULSE ],
    [ 40, Moves.FLAMETHROWER ],
    [ 48, Moves.FUSION_FLARE ],
    [ 56, Moves.HYPER_VOICE ],
    [ 64, Moves.FIRE_BLAST ],
    [ 72, Moves.IMPRISON ],
    [ 80, Moves.OUTRAGE ],
    [ 88, Moves.BLUE_FLARE ],
  ],
  [Species.ZEKROM]: [
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 8, Moves.SLASH ],
    [ 16, Moves.CRUNCH ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 32, Moves.DRAGON_CLAW ],
    [ 40, Moves.THUNDERBOLT ],
    [ 48, Moves.FUSION_BOLT ],
    [ 56, Moves.HYPER_VOICE ],
    [ 64, Moves.THUNDER ],
    [ 72, Moves.IMPRISON ],
    [ 80, Moves.OUTRAGE ],
    [ 88, Moves.BOLT_STRIKE ],
  ],
  [Species.LANDORUS]: [
    [ 1, Moves.SAND_TOMB ],
    [ 1, Moves.SMACK_DOWN ],
    [ 5, Moves.LEER ],
    [ 10, Moves.BLOCK ],
    [ 15, Moves.BULLDOZE ],
    [ 20, Moves.ROCK_TOMB ],
    [ 30, Moves.IMPRISON ],
    [ 35, Moves.ROCK_SLIDE ],
    [ 40, Moves.EARTH_POWER ],
    [ 45, Moves.EXTRASENSORY ],
    [ 50, Moves.STONE_EDGE ],
    [ 55, Moves.HAMMER_ARM ],
    [ 60, Moves.SANDSTORM ],
    [ 65, Moves.EARTHQUAKE ],
    [ 70, Moves.OUTRAGE ],
    [ 75, Moves.FISSURE ],
    [ 80, Moves.SANDSEAR_STORM ],
  ],
  [Species.KYUREM]: [
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 1, Moves.FREEZE_DRY ],
    [ 8, Moves.SLASH ],
    [ 16, Moves.ENDEAVOR ],
    [ 24, Moves.DRAGON_PULSE ],
    [ 32, Moves.ICE_BEAM ],
    [ 40, Moves.HYPER_VOICE ],
    [ 48, Moves.SCARY_FACE ],
    [ 56, Moves.BLIZZARD ],
    [ 64, Moves.IMPRISON ],
    [ 72, Moves.OUTRAGE ],
    [ 80, Moves.GLACIATE ],
    [ 88, Moves.SHEER_COLD ],
  ],
  [Species.KELDEO]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.WORK_UP ],
    [ 1, Moves.SECRET_SWORD ],
    [ 7, Moves.BUBBLE_BEAM ],
    [ 14, Moves.QUICK_GUARD ],
    [ 21, Moves.DOUBLE_KICK ],
    [ 28, Moves.RETALIATE ],
    [ 35, Moves.AQUA_TAIL ],
    [ 42, Moves.TAKE_DOWN ],
    [ 49, Moves.SACRED_SWORD ],
    [ 56, Moves.SWORDS_DANCE ],
    [ 63, Moves.HYDRO_PUMP ],
    [ 70, Moves.CLOSE_COMBAT ],
  ],
  [Species.MELOETTA]: [
    [ 1, Moves.SING ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.ROUND ],
    [ 21, Moves.TEETER_DANCE ],
    [ 26, Moves.ACROBATICS ],
    [ 31, Moves.PSYBEAM ],
    [ 36, Moves.ECHOED_VOICE ],
    [ 43, Moves.U_TURN ],
    [ 50, Moves.RELIC_SONG ],
    [ 57, Moves.PSYCHIC ],
    [ 64, Moves.HYPER_VOICE ],
    [ 71, Moves.ROLE_PLAY ],
    [ 78, Moves.CLOSE_COMBAT ],
    [ 85, Moves.PERISH_SONG ],
  ],
  [Species.GENESECT]: [
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FURY_CUTTER ],
    [ 7, Moves.SCREECH ],
    [ 14, Moves.METAL_CLAW ],
    [ 21, Moves.FELL_STINGER ],
    [ 28, Moves.FLAME_CHARGE ],
    [ 35, Moves.METAL_SOUND ],
    [ 42, Moves.X_SCISSOR ],
    [ 49, Moves.MAGNET_RISE ],
    [ 56, Moves.BUG_BUZZ ],
    [ 63, Moves.SIMPLE_BEAM ],
    [ 70, Moves.ZAP_CANNON ],
    [ 77, Moves.LOCK_ON ],
    [ 84, Moves.TECHNO_BLAST ],
    [ 91, Moves.SELF_DESTRUCT ],
  ],
  [Species.CHESPIN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.VINE_WHIP ],
    [ 8, Moves.ROLLOUT ],
    [ 11, Moves.BITE ],
    [ 15, Moves.LEECH_SEED ],
    [ 18, Moves.PIN_MISSILE ],
    [ 27, Moves.TAKE_DOWN ],
    [ 32, Moves.SEED_BOMB ],
    [ 35, Moves.MUD_SHOT ],
    [ 42, Moves.BODY_SLAM ],
    [ 45, Moves.PAIN_SPLIT ],
    [ 48, Moves.WOOD_HAMMER ],
  ],
  [Species.QUILLADIN]: [
    [ EVOLVE_MOVE, Moves.NEEDLE_ARM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.VINE_WHIP ],
    [ 8, Moves.ROLLOUT ],
    [ 11, Moves.BITE ],
    [ 15, Moves.LEECH_SEED ],
    [ 20, Moves.SEED_BOMB ],
    [ 24, Moves.PIN_MISSILE ],
    [ 29, Moves.TAKE_DOWN ],
    [ 34, Moves.MUD_SHOT ],
    [ 38, Moves.BULK_UP ],
    [ 43, Moves.BODY_SLAM ],
    [ 47, Moves.PAIN_SPLIT ],
    [ 53, Moves.WOOD_HAMMER ],
  ],
  [Species.CHESNAUGHT]: [
    [ EVOLVE_MOVE, Moves.SPIKY_SHIELD ],
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.HAMMER_ARM ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.NEEDLE_ARM ],
    [ 11, Moves.BITE ],
    [ 15, Moves.LEECH_SEED ],
    [ 19, Moves.PIN_MISSILE ],
    [ 29, Moves.TAKE_DOWN ],
    [ 35, Moves.SEED_BOMB ],
    [ 41, Moves.MUD_SHOT ],
    [ 48, Moves.BULK_UP ],
    [ 54, Moves.BODY_SLAM ],
    [ 60, Moves.PAIN_SPLIT ],
    [ 66, Moves.WOOD_HAMMER ],
    [ 78, Moves.GIGA_IMPACT ],
  ],
  [Species.FENNEKIN]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.EMBER ],
    [ 11, Moves.HOWL ],
    [ 14, Moves.FLAME_CHARGE ],
    [ 17, Moves.PSYBEAM ],
    [ 20, Moves.FIRE_SPIN ],
    [ 25, Moves.LIGHT_SCREEN ],
    [ 31, Moves.PSYSHOCK ],
    [ 35, Moves.FLAMETHROWER ],
    [ 38, Moves.WILL_O_WISP ],
    [ 41, Moves.PSYCHIC ],
    [ 43, Moves.SUNNY_DAY ],
    [ 48, Moves.FIRE_BLAST ],
  ],
  [Species.BRAIXEN]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.EMBER ],
    [ 11, Moves.HOWL ],
    [ 14, Moves.FLAME_CHARGE ],
    [ 18, Moves.PSYBEAM ],
    [ 22, Moves.FIRE_SPIN ],
    [ 28, Moves.LIGHT_SCREEN ],
    [ 36, Moves.PSYSHOCK ],
    [ 41, Moves.FLAMETHROWER ],
    [ 45, Moves.WILL_O_WISP ],
    [ 49, Moves.PSYCHIC ],
    [ 52, Moves.SUNNY_DAY ],
    [ 56, Moves.MAGIC_ROOM ],
    [ 59, Moves.FIRE_BLAST ],
  ],
  [Species.DELPHOX]: [
    [ EVOLVE_MOVE, Moves.MYSTICAL_FIRE ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SHADOW_BALL ],
    [ 1, Moves.FUTURE_SIGHT ],
    [ 1, Moves.ROLE_PLAY ],
    [ 1, Moves.HOWL ],
    [ 1, Moves.SWITCHEROO ],
    [ 14, Moves.FLAME_CHARGE ],
    [ 18, Moves.PSYBEAM ],
    [ 22, Moves.FIRE_SPIN ],
    [ 28, Moves.LIGHT_SCREEN ],
    [ 38, Moves.PSYSHOCK ],
    [ 45, Moves.FLAMETHROWER ],
    [ 51, Moves.WILL_O_WISP ],
    [ 57, Moves.PSYCHIC ],
    [ 62, Moves.SUNNY_DAY ],
    [ 68, Moves.MAGIC_ROOM ],
    [ 74, Moves.FIRE_BLAST ],
  ],
  [Species.FROAKIE]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.WATER_GUN ],
    [ 8, Moves.QUICK_ATTACK ],
    [ 10, Moves.LICK ],
    [ 14, Moves.WATER_PULSE ],
    [ 18, Moves.SMOKESCREEN ],
    [ 21, Moves.ROUND ],
    [ 25, Moves.FLING ],
    [ 29, Moves.SMACK_DOWN ],
    [ 35, Moves.SUBSTITUTE ],
    [ 39, Moves.BOUNCE ],
    [ 43, Moves.DOUBLE_TEAM ],
    [ 48, Moves.HYDRO_PUMP ],
  ],
  [Species.FROGADIER]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 8, Moves.QUICK_ATTACK ],
    [ 10, Moves.LICK ],
    [ 14, Moves.WATER_PULSE ],
    [ 19, Moves.SMOKESCREEN ],
    [ 23, Moves.ROUND ],
    [ 28, Moves.FLING ],
    [ 33, Moves.SMACK_DOWN ],
    [ 40, Moves.SUBSTITUTE ],
    [ 45, Moves.BOUNCE ],
    [ 50, Moves.DOUBLE_TEAM ],
    [ 56, Moves.HYDRO_PUMP ],
  ],
  [Species.GRENINJA]: [
    [ EVOLVE_MOVE, Moves.WATER_SHURIKEN ],
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.HAZE ],
    [ 1, Moves.MAT_BLOCK ],
    [ 1, Moves.ROLE_PLAY ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 10, Moves.LICK ],
    [ 14, Moves.WATER_PULSE ],
    [ 19, Moves.SMOKESCREEN ],
    [ 23, Moves.SHADOW_SNEAK ],
    [ 28, Moves.SPIKES ],
    [ 33, Moves.AERIAL_ACE ],
    [ 42, Moves.SUBSTITUTE ],
    [ 49, Moves.EXTRASENSORY ],
    [ 56, Moves.DOUBLE_TEAM ],
    [ 68, Moves.HYDRO_PUMP ],
  ],
  [Species.BUNNELBY]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.MUD_SLAP ],
    [ 3, Moves.TACKLE ],
    [ 6, Moves.LASER_FOCUS ],
    [ 9, Moves.QUICK_ATTACK ],
    [ 12, Moves.MUD_SHOT ],
    [ 15, Moves.FLAIL ],
    [ 18, Moves.DOUBLE_KICK ],
    [ 21, Moves.BULLDOZE ],
    [ 24, Moves.DIG ],
    [ 27, Moves.BOUNCE ],
    [ 30, Moves.TAKE_DOWN ],
    [ 33, Moves.SWORDS_DANCE ],
    [ 36, Moves.EARTHQUAKE ],
    [ 39, Moves.SUPER_FANG ],
  ],
  [Species.DIGGERSBY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.LASER_FOCUS ],
    [ 1, Moves.ROTOTILLER ],
    [ 9, Moves.QUICK_ATTACK ],
    [ 12, Moves.MUD_SHOT ],
    [ 15, Moves.FLAIL ],
    [ 18, Moves.DOUBLE_KICK ],
    [ 23, Moves.BULLDOZE ],
    [ 28, Moves.DIG ],
    [ 33, Moves.BOUNCE ],
    [ 38, Moves.TAKE_DOWN ],
    [ 43, Moves.SWORDS_DANCE ],
    [ 48, Moves.EARTHQUAKE ],
    [ 53, Moves.SUPER_FANG ],
    [ 58, Moves.HAMMER_ARM ],
  ],
  [Species.FLETCHLING]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 5, Moves.QUICK_ATTACK ],
    [ 10, Moves.EMBER ],
    [ 15, Moves.FLAIL ],
    [ 20, Moves.ACROBATICS ],
    [ 25, Moves.AGILITY ],
    [ 30, Moves.AERIAL_ACE ],
    [ 35, Moves.TAILWIND ],
    [ 40, Moves.STEEL_WING ],
    [ 45, Moves.ROOST ],
    [ 50, Moves.FLY ],
  ],
  [Species.FLETCHINDER]: [
    [ EVOLVE_MOVE, Moves.FLAME_CHARGE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FEINT ],
    [ 15, Moves.FLAIL ],
    [ 22, Moves.ACROBATICS ],
    [ 29, Moves.AGILITY ],
    [ 36, Moves.AERIAL_ACE ],
    [ 43, Moves.TAILWIND ],
    [ 50, Moves.STEEL_WING ],
    [ 57, Moves.ROOST ],
    [ 64, Moves.FLY ],
  ],
  [Species.TALONFLAME]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.FLARE_BLITZ ],
    [ 1, Moves.FLAME_CHARGE ],
    [ 15, Moves.FLAIL ],
    [ 22, Moves.ACROBATICS ],
    [ 29, Moves.AGILITY ],
    [ 38, Moves.AERIAL_ACE ],
    [ 47, Moves.TAILWIND ],
    [ 56, Moves.STEEL_WING ],
    [ 65, Moves.ROOST ],
    [ 74, Moves.FLY ],
    [ 83, Moves.BRAVE_BIRD ],
  ],
  [Species.SCATTERBUG]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 6, Moves.STUN_SPORE ],
    [ 15, Moves.BUG_BITE ],
  ],
  [Species.SPEWPA]: [
    [ EVOLVE_MOVE, Moves.PROTECT ],
    [ 1, Moves.HARDEN ],
  ],
  [Species.VIVILLON]: [
    [ EVOLVE_MOVE, Moves.GUST ],
    [ 1, Moves.POISON_POWDER ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.SLEEP_POWDER ],
    [ 1, Moves.STRUGGLE_BUG ],
    [ 1, Moves.POWDER ],
    [ 12, Moves.LIGHT_SCREEN ],
    [ 17, Moves.PSYBEAM ],
    [ 21, Moves.SUPERSONIC ],
    [ 25, Moves.DRAINING_KISS ],
    [ 31, Moves.SAFEGUARD ],
    [ 35, Moves.BUG_BUZZ ],
    [ 45, Moves.QUIVER_DANCE ],
    [ 50, Moves.HURRICANE ],
  ],
  [Species.LITLEO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 5, Moves.EMBER ],
    [ 8, Moves.WORK_UP ],
    [ 11, Moves.HEADBUTT ],
    [ 15, Moves.NOBLE_ROAR ],
    [ 20, Moves.TAKE_DOWN ],
    [ 23, Moves.FIRE_FANG ],
    [ 28, Moves.ENDEAVOR ],
    [ 33, Moves.ECHOED_VOICE ],
    [ 36, Moves.FLAMETHROWER ],
    [ 39, Moves.CRUNCH ],
    [ 43, Moves.HYPER_VOICE ],
    [ 46, Moves.INCINERATE ],
    [ 50, Moves.OVERHEAT ],
  ],
  [Species.PYROAR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.HYPER_BEAM ],
    [ 1, Moves.WORK_UP ],
    [ 11, Moves.HEADBUTT ],
    [ 15, Moves.NOBLE_ROAR ],
    [ 20, Moves.TAKE_DOWN ],
    [ 23, Moves.FIRE_FANG ],
    [ 28, Moves.ENDEAVOR ],
    [ 33, Moves.ECHOED_VOICE ],
    [ 38, Moves.FLAMETHROWER ],
    [ 42, Moves.CRUNCH ],
    [ 48, Moves.HYPER_VOICE ],
    [ 51, Moves.INCINERATE ],
    [ 57, Moves.OVERHEAT ],
  ],
  [Species.FLABEBE]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 6, Moves.FAIRY_WIND ],
    [ 10, Moves.SAFEGUARD ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.WISH ],
    [ 22, Moves.MAGICAL_LEAF ],
    [ 24, Moves.GRASSY_TERRAIN ],
    [ 28, Moves.PETAL_BLIZZARD ],
    [ 33, Moves.SYNTHESIS ],
    [ 37, Moves.MISTY_TERRAIN ],
    [ 41, Moves.MOONBLAST ],
    [ 45, Moves.PETAL_DANCE ],
    [ 48, Moves.SOLAR_BEAM ],
  ],
  [Species.FLOETTE]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FAIRY_WIND ],
    [ 10, Moves.SAFEGUARD ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.WISH ],
    [ 25, Moves.MAGICAL_LEAF ],
    [ 27, Moves.GRASSY_TERRAIN ],
    [ 33, Moves.PETAL_BLIZZARD ],
    [ 38, Moves.SYNTHESIS ],
    [ 43, Moves.MISTY_TERRAIN ],
    [ 46, Moves.MOONBLAST ],
    [ 51, Moves.PETAL_DANCE ],
    [ 58, Moves.SOLAR_BEAM ],
  ],
  [Species.FLORGES]: [
    [ 1, Moves.SOLAR_BEAM ],
    [ 1, Moves.PETAL_DANCE ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.SYNTHESIS ],
    [ 1, Moves.WISH ],
    [ 1, Moves.LUCKY_CHANT ],
    [ 1, Moves.MAGICAL_LEAF ],
    [ 1, Moves.GRASS_KNOT ],
    [ 1, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.GRASSY_TERRAIN ],
    [ 1, Moves.MISTY_TERRAIN ],
    [ 5, Moves.MOONBLAST ],
  ],
  [Species.SKIDDO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWTH ],
    [ 7, Moves.VINE_WHIP ],
    [ 9, Moves.TAIL_WHIP ],
    [ 12, Moves.LEECH_SEED ],
    [ 13, Moves.RAZOR_LEAF ],
    [ 16, Moves.WORRY_SEED ],
    [ 20, Moves.SYNTHESIS ],
    [ 22, Moves.TAKE_DOWN ],
    [ 26, Moves.BULLDOZE ],
    [ 30, Moves.SEED_BOMB ],
    [ 34, Moves.BULK_UP ],
    [ 38, Moves.DOUBLE_EDGE ],
    [ 42, Moves.HORN_LEECH ],
    [ 45, Moves.LEAF_BLADE ],
  ],
  [Species.GOGOAT]: [
    [ EVOLVE_MOVE, Moves.AERIAL_ACE ],
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.EARTHQUAKE ],
    [ 12, Moves.LEECH_SEED ],
    [ 13, Moves.RAZOR_LEAF ],
    [ 16, Moves.WORRY_SEED ],
    [ 20, Moves.SYNTHESIS ],
    [ 22, Moves.TAKE_DOWN ],
    [ 26, Moves.BULLDOZE ],
    [ 30, Moves.SEED_BOMB ],
    [ 34, Moves.BULK_UP ],
    [ 40, Moves.DOUBLE_EDGE ],
    [ 47, Moves.HORN_LEECH ],
    [ 55, Moves.LEAF_BLADE ],
    [ 58, Moves.MILK_DRINK ],
  ],
  [Species.PANCHAM]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.ARM_THRUST ],
    [ 8, Moves.TAUNT ],
    [ 12, Moves.CIRCLE_THROW ],
    [ 16, Moves.LOW_SWEEP ],
    [ 20, Moves.WORK_UP ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.VITAL_THROW ],
    [ 33, Moves.CRUNCH ],
    [ 36, Moves.BODY_SLAM ],
    [ 40, Moves.PARTING_SHOT ],
    [ 44, Moves.ENTRAINMENT ],
  ],
  [Species.PANGORO]: [
    [ EVOLVE_MOVE, Moves.NIGHT_SLASH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.ARM_THRUST ],
    [ 1, Moves.BULLET_PUNCH ],
    [ 12, Moves.CIRCLE_THROW ],
    [ 16, Moves.LOW_SWEEP ],
    [ 20, Moves.WORK_UP ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.VITAL_THROW ],
    [ 35, Moves.CRUNCH ],
    [ 40, Moves.BODY_SLAM ],
    [ 46, Moves.PARTING_SHOT ],
    [ 52, Moves.ENTRAINMENT ],
    [ 58, Moves.HAMMER_ARM ],
  ],
  [Species.FURFROU]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.SAND_ATTACK ],
    [ 9, Moves.BABY_DOLL_EYES ],
    [ 12, Moves.HEADBUTT ],
    [ 15, Moves.TAIL_WHIP ],
    [ 22, Moves.BITE ],
    [ 27, Moves.ODOR_SLEUTH ],
    [ 33, Moves.RETALIATE ],
    [ 35, Moves.TAKE_DOWN ],
    [ 38, Moves.CHARM ],
    [ 42, Moves.SUCKER_PUNCH ],
    [ 48, Moves.COTTON_GUARD ],
  ],
  [Species.ESPURR]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 3, Moves.FAKE_OUT ],
    [ 6, Moves.DISARMING_VOICE ],
    [ 9, Moves.CONFUSION ],
    [ 18, Moves.COVET ],
    [ 21, Moves.PSYBEAM ],
    [ 30, Moves.LIGHT_SCREEN ],
    [ 30, Moves.REFLECT ],
    [ 33, Moves.PSYSHOCK ],
  ],
  [Species.MEOWSTIC]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.MEAN_LOOK ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 9, Moves.CONFUSION ],
    [ 12, Moves.HELPING_HAND ],
    [ 15, Moves.CHARM ],
    [ 18, Moves.COVET ],
    [ 21, Moves.PSYBEAM ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 29, Moves.ROLE_PLAY ],
    [ 34, Moves.LIGHT_SCREEN ],
    [ 34, Moves.REFLECT ],
    [ 39, Moves.PSYSHOCK ],
    [ 44, Moves.IMPRISON ],
    [ 49, Moves.QUICK_GUARD ],
    [ 54, Moves.PSYCHIC ],
    [ 59, Moves.MISTY_TERRAIN ],
  ],
  [Species.HONEDGE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FURY_CUTTER ],
    [ 4, Moves.SHADOW_SNEAK ],
    [ 8, Moves.AUTOTOMIZE ],
    [ 12, Moves.AERIAL_ACE ],
    [ 16, Moves.METAL_SOUND ],
    [ 20, Moves.SLASH ],
    [ 24, Moves.NIGHT_SLASH ],
    [ 28, Moves.RETALIATE ],
    [ 32, Moves.IRON_DEFENSE ],
    [ 36, Moves.IRON_HEAD ],
    [ 40, Moves.POWER_TRICK ],
    [ 44, Moves.SWORDS_DANCE ],
    [ 48, Moves.SACRED_SWORD ],
  ],
  [Species.DOUBLADE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 1, Moves.AUTOTOMIZE ],
    [ 12, Moves.AERIAL_ACE ],
    [ 16, Moves.METAL_SOUND ],
    [ 20, Moves.SLASH ],
    [ 24, Moves.NIGHT_SLASH ],
    [ 28, Moves.RETALIATE ],
    [ 32, Moves.IRON_DEFENSE ],
    [ 38, Moves.IRON_HEAD ],
    [ 44, Moves.POWER_TRICK ],
    [ 50, Moves.SWORDS_DANCE ],
    [ 56, Moves.SACRED_SWORD ],
  ],
  [Species.AEGISLASH]: [
    [ EVOLVE_MOVE, Moves.KINGS_SHIELD ],
    [ 1, Moves.SWORDS_DANCE ],
    [ 1, Moves.PURSUIT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SLASH ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.METAL_SOUND ],
    [ 1, Moves.AERIAL_ACE ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.POWER_TRICK ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 1, Moves.IRON_HEAD ],
    [ 1, Moves.HEAD_SMASH ],
    [ 1, Moves.AUTOTOMIZE ],
    [ 1, Moves.RETALIATE ],
    [ 1, Moves.SACRED_SWORD ],
  ],
  [Species.SPRITZEE]: [
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.FAIRY_WIND ],
    [ 3, Moves.SWEET_KISS ],
    [ 6, Moves.ECHOED_VOICE ],
    [ 9, Moves.DRAINING_KISS ],
    [ 12, Moves.AROMATHERAPY ],
    [ 18, Moves.ATTRACT ],
    [ 21, Moves.FLAIL ],
    [ 24, Moves.MISTY_TERRAIN ],
    [ 27, Moves.PSYCHIC ],
    [ 30, Moves.CHARM ],
    [ 33, Moves.CALM_MIND ],
    [ 36, Moves.MOONBLAST ],
    [ 39, Moves.SKILL_SWAP ],
  ],
  [Species.AROMATISSE]: [
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 1, Moves.HEAL_PULSE ],
    [ 1, Moves.FAIRY_WIND ],
    [ 1, Moves.AROMATIC_MIST ],
    [ 9, Moves.DISARMING_VOICE ],
    [ 12, Moves.AROMATHERAPY ],
    [ 15, Moves.DRAINING_KISS ],
    [ 18, Moves.ATTRACT ],
    [ 21, Moves.FLAIL ],
    [ 24, Moves.MISTY_TERRAIN ],
    [ 27, Moves.PSYCHIC ],
    [ 30, Moves.CHARM ],
    [ 33, Moves.CALM_MIND ],
    [ 36, Moves.MOONBLAST ],
    [ 39, Moves.SKILL_SWAP ],
    [ 42, Moves.PSYCH_UP ],
  ],
  [Species.SWIRLIX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SWEET_SCENT ],
    [ 3, Moves.PLAY_NICE ],
    [ 6, Moves.FAIRY_WIND ],
    [ 9, Moves.AROMATHERAPY ],
    [ 12, Moves.DRAINING_KISS ],
    [ 15, Moves.FAKE_TEARS ],
    [ 18, Moves.ROUND ],
    [ 21, Moves.STRING_SHOT ],
    [ 24, Moves.COTTON_SPORE ],
    [ 27, Moves.ENERGY_BALL ],
    [ 30, Moves.WISH ],
    [ 33, Moves.PLAY_ROUGH ],
    [ 36, Moves.COTTON_GUARD ],
    [ 39, Moves.ENDEAVOR ],
  ],
  [Species.SLURPUFF]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.FAIRY_WIND ],
    [ 1, Moves.PLAY_NICE ],
    [ 9, Moves.AROMATHERAPY ],
    [ 12, Moves.DRAINING_KISS ],
    [ 15, Moves.FAKE_TEARS ],
    [ 18, Moves.ROUND ],
    [ 21, Moves.STRING_SHOT ],
    [ 24, Moves.COTTON_SPORE ],
    [ 27, Moves.ENERGY_BALL ],
    [ 30, Moves.WISH ],
    [ 33, Moves.PLAY_ROUGH ],
    [ 36, Moves.COTTON_GUARD ],
    [ 39, Moves.ENDEAVOR ],
    [ 42, Moves.STICKY_WEB ],
  ],
  [Species.INKAY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PECK ],
    [ 1, Moves.CONSTRICT ],
    [ 3, Moves.HYPNOSIS ],
    [ 6, Moves.WRAP ],
    [ 9, Moves.PAYBACK ],
    [ 12, Moves.PLUCK ],
    [ 15, Moves.PSYBEAM ],
    [ 18, Moves.SWAGGER ],
    [ 21, Moves.SLASH ],
    [ 24, Moves.NIGHT_SLASH ],
    [ 27, Moves.PSYCHO_CUT ],
    [ 31, Moves.SWITCHEROO ],
    [ 33, Moves.FOUL_PLAY ],
    [ 36, Moves.TOPSY_TURVY ],
    [ 39, Moves.SUPERPOWER ],
  ],
  [Species.MALAMAR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.PECK ],
    [ 1, Moves.CONSTRICT ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.REVERSAL ],
    [ 9, Moves.PAYBACK ],
    [ 12, Moves.PLUCK ],
    [ 15, Moves.PSYBEAM ],
    [ 18, Moves.SWAGGER ],
    [ 21, Moves.SLASH ],
    [ 24, Moves.NIGHT_SLASH ],
    [ 27, Moves.PSYCHO_CUT ],
    [ 33, Moves.SWITCHEROO ],
    [ 37, Moves.FOUL_PLAY ],
    [ 42, Moves.TOPSY_TURVY ],
    [ 47, Moves.SUPERPOWER ],
  ],
  [Species.BINACLE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.MUD_SLAP ],
    [ 4, Moves.WITHDRAW ],
    [ 8, Moves.WATER_GUN ],
    [ 12, Moves.FURY_CUTTER ],
    [ 16, Moves.FURY_SWIPES ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 24, Moves.ROCK_POLISH ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.HONE_CLAWS ],
    [ 36, Moves.RAZOR_SHELL ],
    [ 40, Moves.SHELL_SMASH ],
    [ 44, Moves.CROSS_CHOP ],
  ],
  [Species.BARBARACLE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.SKULL_BASH ],
    [ 1, Moves.MUD_SLAP ],
    [ 12, Moves.FURY_CUTTER ],
    [ 16, Moves.FURY_SWIPES ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 24, Moves.ROCK_POLISH ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.HONE_CLAWS ],
    [ 36, Moves.RAZOR_SHELL ],
    [ 42, Moves.SHELL_SMASH ],
    [ 48, Moves.CROSS_CHOP ],
    [ 54, Moves.STONE_EDGE ],
  ],
  [Species.SKRELP]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOKESCREEN ],
    [ 5, Moves.ACID ],
    [ 10, Moves.WATER_GUN ],
    [ 15, Moves.TAIL_WHIP ],
    [ 20, Moves.DOUBLE_TEAM ],
    [ 25, Moves.POISON_TAIL ],
    [ 30, Moves.WATER_PULSE ],
    [ 35, Moves.TOXIC ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 45, Moves.AQUA_TAIL ],
    [ 50, Moves.SLUDGE_BOMB ],
    [ 55, Moves.HYDRO_PUMP ],
  ],
  [Species.DRAGALGE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ACID ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.FEINT_ATTACK ],
    [ 15, Moves.TAIL_WHIP ],
    [ 20, Moves.DOUBLE_TEAM ],
    [ 25, Moves.POISON_TAIL ],
    [ 30, Moves.WATER_PULSE ],
    [ 35, Moves.TOXIC ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 45, Moves.AQUA_TAIL ],
    [ 52, Moves.SLUDGE_BOMB ],
    [ 59, Moves.HYDRO_PUMP ],
    [ 66, Moves.OUTRAGE ],
  ],
  [Species.CLAUNCHER]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SPLASH ],
    [ 10, Moves.FLAIL ],
    [ 15, Moves.AQUA_JET ],
    [ 20, Moves.SMACK_DOWN ],
    [ 25, Moves.HONE_CLAWS ],
    [ 30, Moves.WATER_PULSE ],
    [ 35, Moves.SWORDS_DANCE ],
    [ 40, Moves.AURA_SPHERE ],
    [ 45, Moves.BOUNCE ],
    [ 50, Moves.MUDDY_WATER ],
    [ 55, Moves.CRABHAMMER ],
  ],
  [Species.CLAWITZER]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.DARK_PULSE ],
    [ 1, Moves.DRAGON_PULSE ],
    [ 1, Moves.HEAL_PULSE ],
    [ 15, Moves.AQUA_JET ],
    [ 20, Moves.SMACK_DOWN ],
    [ 25, Moves.HONE_CLAWS ],
    [ 30, Moves.WATER_PULSE ],
    [ 35, Moves.SWORDS_DANCE ],
    [ 42, Moves.AURA_SPHERE ],
    [ 49, Moves.BOUNCE ],
    [ 56, Moves.MUDDY_WATER ],
    [ 63, Moves.CRABHAMMER ],
  ],
  [Species.HELIOPTILE]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.MUD_SLAP ],
    [ 4, Moves.POUND ],
    [ 8, Moves.THUNDER_SHOCK ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 16, Moves.CHARGE ],
    [ 20, Moves.BULLDOZE ],
    [ 24, Moves.VOLT_SWITCH ],
    [ 28, Moves.PARABOLIC_CHARGE ],
    [ 32, Moves.THUNDER_WAVE ],
    [ 36, Moves.THUNDERBOLT ],
    [ 40, Moves.ELECTRIFY ],
    [ 44, Moves.THUNDER ],
  ],
  [Species.HELIOLISK]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.RAZOR_WIND ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDERBOLT ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.THUNDER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.DISCHARGE ],
    [ 1, Moves.VOLT_SWITCH ],
    [ 1, Moves.BULLDOZE ],
    [ 1, Moves.PARABOLIC_CHARGE ],
    [ 1, Moves.ELECTRIFY ],
    [ 1, Moves.EERIE_IMPULSE ],
  ],
  [Species.TYRUNT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 4, Moves.ROAR ],
    [ 8, Moves.ANCIENT_POWER ],
    [ 12, Moves.CHARM ],
    [ 16, Moves.BITE ],
    [ 20, Moves.DRAGON_TAIL ],
    [ 24, Moves.STOMP ],
    [ 28, Moves.ROCK_SLIDE ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.DRAGON_CLAW ],
    [ 40, Moves.THRASH ],
    [ 44, Moves.EARTHQUAKE ],
    [ 48, Moves.HORN_DRILL ],
  ],
  [Species.TYRANTRUM]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.ROAR ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 12, Moves.CHARM ],
    [ 16, Moves.BITE ],
    [ 20, Moves.DRAGON_TAIL ],
    [ 24, Moves.STOMP ],
    [ 28, Moves.ROCK_SLIDE ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.DRAGON_CLAW ],
    [ 42, Moves.THRASH ],
    [ 48, Moves.EARTHQUAKE ],
    [ 54, Moves.HORN_DRILL ],
    [ 60, Moves.GIGA_IMPACT ],
    [ 66, Moves.HEAD_SMASH ],
  ],
  [Species.AMAURA]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.POWDER_SNOW ],
    [ 4, Moves.ENCORE ],
    [ 8, Moves.ANCIENT_POWER ],
    [ 12, Moves.ICY_WIND ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.MIST ],
    [ 24, Moves.AURORA_BEAM ],
    [ 28, Moves.THUNDER_WAVE ],
    [ 32, Moves.NATURE_POWER ],
    [ 36, Moves.FREEZE_DRY ],
    [ 40, Moves.ICE_BEAM ],
    [ 44, Moves.LIGHT_SCREEN ],
    [ 48, Moves.HAIL ],
    [ 52, Moves.BLIZZARD ],
    [ 56, Moves.HYPER_BEAM ],
  ],
  [Species.AURORUS]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.ANCIENT_POWER ],
    [ 12, Moves.ICY_WIND ],
    [ 16, Moves.ROUND ],
    [ 20, Moves.MIST ],
    [ 24, Moves.AURORA_BEAM ],
    [ 28, Moves.THUNDER_WAVE ],
    [ 32, Moves.NATURE_POWER ],
    [ 36, Moves.FREEZE_DRY ],
    [ 42, Moves.ICE_BEAM ],
    [ 48, Moves.LIGHT_SCREEN ],
    [ 54, Moves.HAIL ],
    [ 60, Moves.BLIZZARD ],
    [ 66, Moves.HYPER_BEAM ],
  ],
  [Species.SYLVEON]: [
    [ EVOLVE_MOVE, Moves.SPARKLY_SWIRL ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.COVET ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.SAND_ATTACK ],
    [ 10, Moves.DISARMING_VOICE ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.BABY_DOLL_EYES ],
    [ 25, Moves.SWIFT ],
    [ 30, Moves.LIGHT_SCREEN ],
    [ 35, Moves.DRAINING_KISS ],
    [ 40, Moves.MISTY_TERRAIN ],
    [ 45, Moves.SKILL_SWAP ],
    [ 50, Moves.PSYCH_UP ],
    [ 55, Moves.MOONBLAST ],
    [ 60, Moves.LAST_RESORT ],
  ],
  [Species.HAWLUCHA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HONE_CLAWS ],
    [ 4, Moves.WING_ATTACK ],
    [ 8, Moves.DETECT ],
    [ 12, Moves.AERIAL_ACE ],
    [ 16, Moves.ENCORE ],
    [ 20, Moves.FEATHER_DANCE ],
    [ 24, Moves.BRICK_BREAK ],
    [ 28, Moves.BOUNCE ],
    [ 32, Moves.TAUNT ],
    [ 36, Moves.ROOST ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 44, Moves.FLYING_PRESS ],
    [ 48, Moves.HIGH_JUMP_KICK ],
    [ 52, Moves.ENDEAVOR ],
    [ 56, Moves.SKY_ATTACK ],
  ],
  [Species.DEDENNE]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.NUZZLE ],
    [ 5, Moves.TACKLE ],
    [ 10, Moves.CHARGE ],
    [ 15, Moves.THUNDER_SHOCK ],
    [ 20, Moves.CHARM ],
    [ 25, Moves.PARABOLIC_CHARGE ],
    [ 30, Moves.VOLT_SWITCH ],
    [ 35, Moves.REST ],
    [ 35, Moves.SNORE ],
    [ 40, Moves.DISCHARGE ],
    [ 45, Moves.PLAY_ROUGH ],
    [ 50, Moves.SUPER_FANG ],
    [ 55, Moves.ENTRAINMENT ],
    [ 60, Moves.THUNDER ],
  ],
  [Species.CARBINK]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.GUARD_SPLIT ],
    [ 10, Moves.SMACK_DOWN ],
    [ 15, Moves.FLAIL ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.ROCK_POLISH ],
    [ 30, Moves.LIGHT_SCREEN ],
    [ 35, Moves.ROCK_SLIDE ],
    [ 40, Moves.SKILL_SWAP ],
    [ 45, Moves.POWER_GEM ],
    [ 50, Moves.STEALTH_ROCK ],
    [ 55, Moves.MOONBLAST ],
    [ 60, Moves.STONE_EDGE ],
  ],
  [Species.GOOMY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ABSORB ],
    [ 5, Moves.WATER_GUN ],
    [ 10, Moves.DRAGON_BREATH ],
    [ 15, Moves.PROTECT ],
    [ 20, Moves.FLAIL ],
    [ 25, Moves.WATER_PULSE ],
    [ 30, Moves.RAIN_DANCE ],
    [ 35, Moves.DRAGON_PULSE ],
    [ 41, Moves.CURSE ],
    [ 45, Moves.BODY_SLAM ],
    [ 50, Moves.MUDDY_WATER ],
  ],
  [Species.SLIGGOO]: [
    [ EVOLVE_MOVE, Moves.ACID_SPRAY ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.ACID_ARMOR ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 15, Moves.PROTECT ],
    [ 20, Moves.FLAIL ],
    [ 25, Moves.WATER_PULSE ],
    [ 30, Moves.RAIN_DANCE ],
    [ 35, Moves.DRAGON_PULSE ],
    [ 43, Moves.CURSE ],
    [ 49, Moves.BODY_SLAM ],
    [ 56, Moves.MUDDY_WATER ],
  ],
  [Species.GOODRA]: [
    [ EVOLVE_MOVE, Moves.AQUA_TAIL ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.POISON_TAIL ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.ACID_SPRAY ],
    [ 1, Moves.TEARFUL_LOOK ],
    [ 15, Moves.PROTECT ],
    [ 20, Moves.FLAIL ],
    [ 25, Moves.WATER_PULSE ],
    [ 30, Moves.RAIN_DANCE ],
    [ 43, Moves.CURSE ],
    [ 49, Moves.BODY_SLAM ],
    [ 58, Moves.MUDDY_WATER ],
    [ 67, Moves.POWER_WHIP ],
  ],
  [Species.KLEFKI]: [
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.TACKLE ],
    [ 8, Moves.FAIRY_WIND ],
    [ 12, Moves.TORMENT ],
    [ 16, Moves.FAIRY_LOCK ],
    [ 20, Moves.METAL_SOUND ],
    [ 24, Moves.DRAINING_KISS ],
    [ 28, Moves.RECYCLE ],
    [ 32, Moves.IMPRISON ],
    [ 36, Moves.FLASH_CANNON ],
    [ 40, Moves.PLAY_ROUGH ],
    [ 44, Moves.MAGIC_ROOM ],
    [ 48, Moves.FOUL_PLAY ],
    [ 50, Moves.HEAL_BLOCK ],
    [ 52, Moves.LAST_RESORT ],
  ],
  [Species.PHANTUMP]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.BRANCH_POKE ],
    [ 8, Moves.LEECH_SEED ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.HEX ],
    [ 24, Moves.GROWTH ],
    [ 28, Moves.HORN_LEECH ],
    [ 32, Moves.CURSE ],
    [ 36, Moves.PHANTOM_FORCE ],
    [ 40, Moves.INGRAIN ],
    [ 44, Moves.WOOD_HAMMER ],
    [ 48, Moves.DESTINY_BOND ],
    [ 52, Moves.FORESTS_CURSE ],
  ],
  [Species.TREVENANT]: [
    [ EVOLVE_MOVE, Moves.SHADOW_CLAW ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.BRANCH_POKE ],
    [ 12, Moves.CONFUSE_RAY ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.HEX ],
    [ 24, Moves.GROWTH ],
    [ 28, Moves.HORN_LEECH ],
    [ 32, Moves.CURSE ],
    [ 36, Moves.PHANTOM_FORCE ],
    [ 40, Moves.INGRAIN ],
    [ 44, Moves.WOOD_HAMMER ],
    [ 48, Moves.DESTINY_BOND ],
    [ 52, Moves.FORESTS_CURSE ],
  ],
  [Species.PUMPKABOO]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.TRICK_OR_TREAT ],
    [ 1, Moves.LEAFAGE ], //Custom
    [ 4, Moves.SHADOW_SNEAK ],
    [ 8, Moves.CONFUSE_RAY ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 16, Moves.LEECH_SEED ],
    [ 20, Moves.BULLET_SEED ],
    [ 24, Moves.SCARY_FACE ],
    [ 28, Moves.WORRY_SEED ],
    [ 32, Moves.SEED_BOMB ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.TRICK ],
    [ 44, Moves.PAIN_SPLIT ],
  ],
  [Species.GOURGEIST]: [
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.EXPLOSION ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 1, Moves.TRICK_OR_TREAT ],
    [ 1, Moves.MOONBLAST ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 16, Moves.LEECH_SEED ],
    [ 20, Moves.BULLET_SEED ],
    [ 24, Moves.SCARY_FACE ],
    [ 28, Moves.WORRY_SEED ],
    [ 32, Moves.SEED_BOMB ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.TRICK ],
    [ 44, Moves.PAIN_SPLIT ],
    [ 48, Moves.PHANTOM_FORCE ],
  ],
  [Species.BERGMITE]: [
    [ 1, Moves.HARDEN ],
    [ 1, Moves.RAPID_SPIN ],
    [ 3, Moves.TACKLE ],
    [ 6, Moves.POWDER_SNOW ],
    [ 9, Moves.CURSE ],
    [ 12, Moves.ICY_WIND ],
    [ 15, Moves.PROTECT ],
    [ 18, Moves.AVALANCHE ],
    [ 21, Moves.BITE ],
    [ 24, Moves.ICE_FANG ],
    [ 27, Moves.IRON_DEFENSE ],
    [ 30, Moves.RECOVER ],
    [ 33, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 39, Moves.BLIZZARD ],
    [ 42, Moves.DOUBLE_EDGE ],
  ],
  [Species.AVALUGG]: [
    [ EVOLVE_MOVE, Moves.BODY_SLAM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.WIDE_GUARD ],
    [ 9, Moves.CURSE ],
    [ 12, Moves.ICY_WIND ],
    [ 15, Moves.PROTECT ],
    [ 18, Moves.AVALANCHE ],
    [ 21, Moves.BITE ],
    [ 24, Moves.ICE_FANG ],
    [ 27, Moves.IRON_DEFENSE ],
    [ 30, Moves.RECOVER ],
    [ 33, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 41, Moves.BLIZZARD ],
    [ 46, Moves.DOUBLE_EDGE ],
    [ 51, Moves.ICICLE_CRASH ],
  ],
  [Species.NOIBAT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ABSORB ],
    [ 4, Moves.GUST ],
    [ 8, Moves.SUPERSONIC ],
    [ 12, Moves.DOUBLE_TEAM ],
    [ 16, Moves.WING_ATTACK ],
    [ 20, Moves.BITE ],
    [ 24, Moves.AIR_CUTTER ],
    [ 28, Moves.WHIRLWIND ],
    [ 32, Moves.SUPER_FANG ],
    [ 36, Moves.AIR_SLASH ],
    [ 40, Moves.SCREECH ],
    [ 44, Moves.ROOST ],
    [ 49, Moves.TAILWIND ],
    [ 52, Moves.HURRICANE ],
  ],
  [Species.NOIVERN]: [
    [ EVOLVE_MOVE, Moves.DRAGON_PULSE ],
    [ 1, Moves.GUST ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MOONLIGHT ],
    [ 12, Moves.DOUBLE_TEAM ],
    [ 16, Moves.WING_ATTACK ],
    [ 20, Moves.BITE ],
    [ 24, Moves.AIR_CUTTER ],
    [ 28, Moves.WHIRLWIND ],
    [ 32, Moves.SUPER_FANG ],
    [ 36, Moves.AIR_SLASH ],
    [ 40, Moves.SCREECH ],
    [ 44, Moves.ROOST ],
    [ 51, Moves.TAILWIND ],
    [ 56, Moves.HURRICANE ],
    [ 62, Moves.BOOMBURST ],
  ],
  [Species.XERNEAS]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GRAVITY ],
    [ 5, Moves.LIGHT_SCREEN ],
    [ 10, Moves.AURORA_BEAM ],
    [ 15, Moves.NATURE_POWER ],
    [ 20, Moves.NIGHT_SLASH ],
    [ 25, Moves.AROMATHERAPY ],
    [ 30, Moves.PSYCH_UP ],
    [ 35, Moves.HORN_LEECH ],
    [ 40, Moves.MISTY_TERRAIN ],
    [ 45, Moves.INGRAIN ],
    [ 50, Moves.TAKE_DOWN ],
    [ 55, Moves.GEOMANCY ],
    [ 60, Moves.MOONBLAST ],
    [ 65, Moves.HEAL_PULSE ],
    [ 70, Moves.MEGAHORN ],
    [ 75, Moves.CLOSE_COMBAT ],
    [ 80, Moves.OUTRAGE ],
    [ 85, Moves.GIGA_IMPACT ],
  ],
  [Species.YVELTAL]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.RAZOR_WIND ],
    [ 5, Moves.TAUNT ],
    [ 10, Moves.SNARL ],
    [ 15, Moves.DISABLE ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 25, Moves.TAILWIND ],
    [ 30, Moves.ROOST ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.DARK_PULSE ],
    [ 45, Moves.PSYCHIC ],
    [ 50, Moves.OBLIVION_WING ],
    [ 55, Moves.PHANTOM_FORCE ],
    [ 60, Moves.FOUL_PLAY ],
    [ 65, Moves.DRAGON_RUSH ],
    [ 70, Moves.HURRICANE ],
    [ 75, Moves.FOCUS_BLAST ],
    [ 80, Moves.SKY_ATTACK ],
    [ 85, Moves.HYPER_BEAM ],
  ],
  [Species.ZYGARDE]: [
    [ 1, Moves.BIND ],
    [ 1, Moves.BITE ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.BULLDOZE ],
    [ 1, Moves.THOUSAND_ARROWS ],
    [ 1, Moves.THOUSAND_WAVES ],
    [ 1, Moves.CORE_ENFORCER ],
    [ 8, Moves.HAZE ],
    [ 16, Moves.DIG ],
    [ 24, Moves.SAFEGUARD ],
    [ 32, Moves.CRUNCH ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 48, Moves.LANDS_WRATH ],
    [ 56, Moves.GLARE ],
    [ 64, Moves.SANDSTORM ],
    [ 72, Moves.COIL ],
    [ 80, Moves.EARTHQUAKE ],
    [ 88, Moves.OUTRAGE ],
  ],
  [Species.DIANCIE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.DIAMOND_STORM ],
    [ 7, Moves.GUARD_SPLIT ],
    [ 14, Moves.SMACK_DOWN ],
    [ 21, Moves.FLAIL ],
    [ 28, Moves.ANCIENT_POWER ],
    [ 35, Moves.ROCK_POLISH ],
    [ 42, Moves.LIGHT_SCREEN ],
    [ 49, Moves.ROCK_SLIDE ],
    [ 56, Moves.SKILL_SWAP ],
    [ 63, Moves.POWER_GEM ],
    [ 70, Moves.STEALTH_ROCK ],
    [ 77, Moves.MOONBLAST ],
    [ 84, Moves.STONE_EDGE ],
  ],
  [Species.HOOPA]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.DESTINY_BOND ],
    [ 1, Moves.ALLY_SWITCH ],
    [ 6, Moves.ASTONISH ],
    [ 10, Moves.TRICK ],
    [ 15, Moves.LIGHT_SCREEN ],
    [ 19, Moves.PSYBEAM ],
    [ 25, Moves.SKILL_SWAP ],
    [ 29, Moves.GUARD_SPLIT ],
    [ 29, Moves.POWER_SPLIT ],
    [ 35, Moves.PHANTOM_FORCE ],
    [ 46, Moves.ZEN_HEADBUTT ],
    [ 50, Moves.TRICK_ROOM ],
    [ 50, Moves.WONDER_ROOM ],
    [ 55, Moves.SHADOW_BALL ],
    [ 68, Moves.NASTY_PLOT ],
    [ 75, Moves.PSYCHIC ],
    [ 85, Moves.HYPERSPACE_HOLE ],
  ],
  [Species.VOLCANION]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.FIRE_SPIN ],
    [ 1, Moves.STEAM_ERUPTION ],
    [ 6, Moves.LEER ],
    [ 12, Moves.WEATHER_BALL ],
    [ 18, Moves.FLAME_CHARGE ],
    [ 24, Moves.WATER_PULSE ],
    [ 30, Moves.SCARY_FACE ],
    [ 36, Moves.INCINERATE ],
    [ 42, Moves.STOMP ],
    [ 48, Moves.SCALD ],
    [ 54, Moves.TAKE_DOWN ],
    [ 60, Moves.MIST ],
    [ 60, Moves.HAZE ],
    [ 66, Moves.HYDRO_PUMP ],
    [ 78, Moves.FLARE_BLITZ ],
    [ 84, Moves.OVERHEAT ],
    [ 90, Moves.EXPLOSION ],
  ],
  [Species.ROWLET]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.LEAFAGE ],
    [ 6, Moves.ASTONISH ],
    [ 9, Moves.PECK ],
    [ 12, Moves.SHADOW_SNEAK ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 18, Moves.SYNTHESIS ],
    [ 21, Moves.PLUCK ],
    [ 24, Moves.NASTY_PLOT ],
    [ 27, Moves.SUCKER_PUNCH ],
    [ 30, Moves.LEAF_BLADE ],
    [ 33, Moves.FEATHER_DANCE ],
    [ 36, Moves.BRAVE_BIRD ],
  ],
  [Species.DARTRIX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.LEAFAGE ],
    [ 9, Moves.PECK ],
    [ 12, Moves.SHADOW_SNEAK ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.SYNTHESIS ],
    [ 25, Moves.PLUCK ],
    [ 30, Moves.NASTY_PLOT ],
    [ 35, Moves.SUCKER_PUNCH ],
    [ 40, Moves.LEAF_BLADE ],
    [ 45, Moves.FEATHER_DANCE ],
    [ 50, Moves.BRAVE_BIRD ],
  ],
  [Species.DECIDUEYE]: [
    [ EVOLVE_MOVE, Moves.SPIRIT_SHACKLE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.SPITE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.U_TURN ],
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.PHANTOM_FORCE ],
    [ 1, Moves.LEAFAGE ],
    [ 9, Moves.PECK ],
    [ 12, Moves.SHADOW_SNEAK ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.SYNTHESIS ],
    [ 25, Moves.PLUCK ],
    [ 30, Moves.NASTY_PLOT ],
    [ 37, Moves.SUCKER_PUNCH ],
    [ 44, Moves.LEAF_BLADE ],
    [ 51, Moves.FEATHER_DANCE ],
    [ 58, Moves.BRAVE_BIRD ],
  ],
  [Species.LITTEN]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.EMBER ],
    [ 6, Moves.LICK ],
    [ 9, Moves.ROAR ],
    [ 12, Moves.FURY_SWIPES ],
    [ 15, Moves.BITE ],
    [ 18, Moves.DOUBLE_KICK ],
    [ 21, Moves.FIRE_FANG ],
    [ 24, Moves.SCARY_FACE ],
    [ 27, Moves.SWAGGER ],
    [ 30, Moves.FLAMETHROWER ],
    [ 33, Moves.THRASH ],
    [ 36, Moves.FLARE_BLITZ ],
  ],
  [Species.TORRACAT]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.LICK ],
    [ 9, Moves.ROAR ],
    [ 12, Moves.FURY_SWIPES ],
    [ 15, Moves.BITE ],
    [ 20, Moves.DOUBLE_KICK ],
    [ 25, Moves.FIRE_FANG ],
    [ 30, Moves.SCARY_FACE ],
    [ 35, Moves.SWAGGER ],
    [ 40, Moves.FLAMETHROWER ],
    [ 45, Moves.THRASH ],
    [ 50, Moves.FLARE_BLITZ ],
  ],
  [Species.INCINEROAR]: [
    [ EVOLVE_MOVE, Moves.DARKEST_LARIAT ],
    [ RELEARN_MOVE, Moves.SCRATCH ],
    [ RELEARN_MOVE, Moves.GROWL ],
    [ RELEARN_MOVE, Moves.THROAT_CHOP ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.CROSS_CHOP ],
    [ 1, Moves.BULK_UP ],
    [ 9, Moves.ROAR ],
    [ 12, Moves.FURY_SWIPES ],
    [ 15, Moves.BITE ],
    [ 20, Moves.DOUBLE_KICK ],
    [ 25, Moves.FIRE_FANG ],
    [ 30, Moves.SCARY_FACE ],
    [ 32, Moves.SWAGGER ],
    [ 44, Moves.FLAMETHROWER ],
    [ 51, Moves.THRASH ],
    [ 58, Moves.FLARE_BLITZ ],
  ],
  [Species.POPPLIO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.WATER_GUN ],
    [ 6, Moves.DISARMING_VOICE ],
    [ 9, Moves.AQUA_JET ],
    [ 12, Moves.BABY_DOLL_EYES ],
    [ 15, Moves.ICY_WIND ],
    [ 18, Moves.SING ],
    [ 21, Moves.BUBBLE_BEAM ],
    [ 24, Moves.ENCORE ],
    [ 27, Moves.MISTY_TERRAIN ],
    [ 30, Moves.HYPER_VOICE ],
    [ 33, Moves.MOONBLAST ],
    [ 36, Moves.HYDRO_PUMP ],
  ],
  [Species.BRIONNE]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 9, Moves.AQUA_JET ],
    [ 12, Moves.BABY_DOLL_EYES ],
    [ 15, Moves.ICY_WIND ],
    [ 20, Moves.SING ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.ENCORE ],
    [ 35, Moves.MISTY_TERRAIN ],
    [ 40, Moves.HYPER_VOICE ],
    [ 45, Moves.MOONBLAST ],
    [ 50, Moves.HYDRO_PUMP ],
  ],
  [Species.PRIMARINA]: [
    [ EVOLVE_MOVE, Moves.SPARKLING_ARIA ],
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 9, Moves.AQUA_JET ],
    [ 12, Moves.BABY_DOLL_EYES ],
    [ 15, Moves.ICY_WIND ],
    [ 20, Moves.SING ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.ENCORE ],
    [ 37, Moves.MISTY_TERRAIN ],
    [ 44, Moves.HYPER_VOICE ],
    [ 51, Moves.MOONBLAST ],
    [ 58, Moves.HYDRO_PUMP ],
  ],
  [Species.PIKIPEK]: [
    [ 1, Moves.PECK ],
    [ 3, Moves.GROWL ],
    [ 7, Moves.ECHOED_VOICE ],
    [ 9, Moves.ROCK_SMASH ],
    [ 13, Moves.SUPERSONIC ],
    [ 15, Moves.PLUCK ],
    [ 19, Moves.ROOST ],
    [ 21, Moves.FURY_ATTACK ],
    [ 25, Moves.SCREECH ],
    [ 27, Moves.DRILL_PECK ],
    [ 31, Moves.BULLET_SEED ],
    [ 33, Moves.FEATHER_DANCE ],
    [ 37, Moves.HYPER_VOICE ],
  ],
  [Species.TRUMBEAK]: [
    [ RELEARN_MOVE, Moves.ECHOED_VOICE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.ROCK_BLAST ],
    [ 13, Moves.SUPERSONIC ],
    [ 16, Moves.PLUCK ],
    [ 21, Moves.ROOST ],
    [ 24, Moves.FURY_ATTACK ],
    [ 29, Moves.SCREECH ],
    [ 32, Moves.DRILL_PECK ],
    [ 37, Moves.BULLET_SEED ],
    [ 40, Moves.FEATHER_DANCE ],
    [ 45, Moves.HYPER_VOICE ],
  ],
  [Species.TOUCANNON]: [
    [ EVOLVE_MOVE, Moves.BEAK_BLAST ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.ROCK_BLAST ],
    [ 1, Moves.ECHOED_VOICE ],
    [ 13, Moves.SUPERSONIC ],
    [ 16, Moves.PLUCK ],
    [ 21, Moves.ROOST ],
    [ 24, Moves.FURY_ATTACK ],
    [ 30, Moves.SCREECH ],
    [ 34, Moves.DRILL_PECK ],
    [ 40, Moves.BULLET_SEED ],
    [ 44, Moves.FEATHER_DANCE ],
    [ 50, Moves.HYPER_VOICE ],
  ],
  [Species.YUNGOOS]: [
    [ 1, Moves.TACKLE ],
    [ 3, Moves.LEER ],
    [ 7, Moves.PAYBACK ],
    [ 10, Moves.SAND_ATTACK ],
    [ 13, Moves.WORK_UP ],
    [ 19, Moves.BITE ],
    [ 22, Moves.MUD_SLAP ],
    [ 25, Moves.SUPER_FANG ],
    [ 28, Moves.TAKE_DOWN ],
    [ 31, Moves.SCARY_FACE ],
    [ 34, Moves.CRUNCH ],
    [ 37, Moves.YAWN ],
    [ 40, Moves.THRASH ],
    [ 43, Moves.REST ],
  ],
  [Species.GUMSHOOS]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PAYBACK ],
    [ 1, Moves.PURSUIT ],
    [ 13, Moves.WORK_UP ],
    [ 19, Moves.BITE ],
    [ 23, Moves.MUD_SLAP ],
    [ 27, Moves.SUPER_FANG ],
    [ 31, Moves.TAKE_DOWN ],
    [ 35, Moves.SCARY_FACE ],
    [ 39, Moves.CRUNCH ],
    [ 43, Moves.YAWN ],
    [ 47, Moves.THRASH ],
    [ 52, Moves.REST ],
  ],
  [Species.GRUBBIN]: [
    [ 1, Moves.VISE_GRIP ],
    [ 1, Moves.MUD_SLAP ],
    [ 5, Moves.STRING_SHOT ],
    [ 10, Moves.BUG_BITE ],
    [ 15, Moves.BITE ],
    [ 21, Moves.SPARK ],
    [ 25, Moves.STICKY_WEB ],
    [ 30, Moves.X_SCISSOR ],
    [ 35, Moves.CRUNCH ],
    [ 40, Moves.DIG ],
  ],
  [Species.CHARJABUG]: [
    [ EVOLVE_MOVE, Moves.CHARGE ],
    [ 1, Moves.VISE_GRIP ],
    [ 1, Moves.STRING_SHOT ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.BUG_BITE ],
    [ 15, Moves.BITE ],
    [ 23, Moves.SPARK ],
    [ 29, Moves.STICKY_WEB ],
    [ 36, Moves.X_SCISSOR ],
    [ 43, Moves.CRUNCH ],
    [ 50, Moves.DIG ],
    [ 57, Moves.IRON_DEFENSE ],
    [ 64, Moves.DISCHARGE ],
  ],
  [Species.VIKAVOLT]: [
    [ EVOLVE_MOVE, Moves.THUNDERBOLT ],
    [ RELEARN_MOVE, Moves.VISE_GRIP ],
    [ RELEARN_MOVE, Moves.DIG ],
    [ RELEARN_MOVE, Moves.MUD_SLAP ],
    [ RELEARN_MOVE, Moves.IRON_DEFENSE ],
    [ RELEARN_MOVE, Moves.X_SCISSOR ],
    [ RELEARN_MOVE, Moves.BUG_BITE ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.DISCHARGE ],
    [ 1, Moves.STRING_SHOT ],
    [ 15, Moves.BITE ],
    [ 23, Moves.SPARK ],
    [ 29, Moves.STICKY_WEB ],
    [ 36, Moves.BUG_BUZZ ],
    [ 43, Moves.GUILLOTINE ],
    [ 50, Moves.FLY ],
    [ 57, Moves.AGILITY ],
    [ 64, Moves.ZAP_CANNON ],
  ],
  [Species.CRABRAWLER]: [
    [ 1, Moves.BUBBLE ],
    [ 1, Moves.VISE_GRIP ],
    [ 5, Moves.ROCK_SMASH ],
    [ 9, Moves.LEER ],
    [ 13, Moves.BUBBLE_BEAM ],
    [ 17, Moves.PROTECT ],
    [ 22, Moves.BRICK_BREAK ],
    [ 25, Moves.SLAM ],
    [ 29, Moves.PAYBACK ],
    [ 33, Moves.REVERSAL ],
    [ 37, Moves.CRABHAMMER ],
    [ 42, Moves.IRON_DEFENSE ],
    [ 45, Moves.DYNAMIC_PUNCH ],
    [ 49, Moves.CLOSE_COMBAT ],
  ],
  [Species.CRABOMINABLE]: [
    [ EVOLVE_MOVE, Moves.ICE_PUNCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.BUBBLE ],
    [ 1, Moves.PURSUIT ],
    [ 17, Moves.BUBBLE_BEAM ],
    [ 22, Moves.BRICK_BREAK ],
    [ 25, Moves.SLAM ],
    [ 29, Moves.AVALANCHE ],
    [ 33, Moves.REVERSAL ],
    [ 37, Moves.ICE_HAMMER ],
    [ 42, Moves.IRON_DEFENSE ],
    [ 45, Moves.DYNAMIC_PUNCH ],
    [ 49, Moves.CLOSE_COMBAT ],
  ],
  [Species.ORICORIO]: [
    [ 1, Moves.POUND ],
    [ 4, Moves.GROWL ],
    [ 6, Moves.PECK ],
    [ 10, Moves.HELPING_HAND ],
    [ 13, Moves.AIR_CUTTER ],
    [ 16, Moves.BATON_PASS ],
    [ 20, Moves.FEATHER_DANCE ],
    [ 23, Moves.ACROBATICS ],
    [ 26, Moves.TEETER_DANCE ],
    [ 30, Moves.ROOST ],
    [ 33, Moves.FLATTER ],
    [ 36, Moves.AIR_SLASH ],
    [ 40, Moves.REVELATION_DANCE ],
    [ 43, Moves.AGILITY ],
    [ 47, Moves.HURRICANE ],
  ],
  [Species.CUTIEFLY]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.FAIRY_WIND ],
    [ 6, Moves.STUN_SPORE ],
    [ 12, Moves.SWEET_SCENT ],
    [ 18, Moves.DRAINING_KISS ],
    [ 24, Moves.STRUGGLE_BUG ],
    [ 30, Moves.COVET ],
    [ 36, Moves.SWITCHEROO ],
    [ 42, Moves.DAZZLING_GLEAM ],
    [ 48, Moves.BUG_BUZZ ],
    [ 54, Moves.QUIVER_DANCE ],
  ],
  [Species.RIBOMBEE]: [
    [ EVOLVE_MOVE, Moves.POLLEN_PUFF ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.FAIRY_WIND ],
    [ 18, Moves.DRAINING_KISS ],
    [ 24, Moves.STRUGGLE_BUG ],
    [ 32, Moves.COVET ],
    [ 40, Moves.SWITCHEROO ],
    [ 48, Moves.DAZZLING_GLEAM ],
    [ 56, Moves.BUG_BUZZ ],
    [ 64, Moves.QUIVER_DANCE ],
  ],
  [Species.ROCKRUFF]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.SAND_ATTACK ],
    [ 8, Moves.DOUBLE_TEAM ],
    [ 12, Moves.ROCK_THROW ],
    [ 16, Moves.HOWL ],
    [ 20, Moves.BITE ],
    [ 24, Moves.ROCK_TOMB ],
    [ 28, Moves.ROAR ],
    [ 32, Moves.ROCK_SLIDE ],
    [ 36, Moves.CRUNCH ],
    [ 40, Moves.SCARY_FACE ],
    [ 44, Moves.STEALTH_ROCK ],
    [ 48, Moves.STONE_EDGE ],
  ],
  [Species.LYCANROC]: [
    [ EVOLVE_MOVE, Moves.SUCKER_PUNCH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.QUICK_GUARD ],
    [ 1, Moves.ACCELEROCK ],
    [ 12, Moves.ROCK_THROW ],
    [ 16, Moves.HOWL ],
    [ 20, Moves.BITE ],
    [ 24, Moves.ROCK_TOMB ],
    [ 30, Moves.ROAR ],
    [ 36, Moves.ROCK_SLIDE ],
    [ 42, Moves.CRUNCH ],
    [ 48, Moves.SCARY_FACE ],
    [ 54, Moves.STEALTH_ROCK ],
    [ 60, Moves.STONE_EDGE ],
  ],
  [Species.WISHIWASHI]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.HELPING_HAND ],
    [ 8, Moves.BEAT_UP ],
    [ 12, Moves.BRINE ],
    [ 16, Moves.TEARFUL_LOOK ],
    [ 20, Moves.DIVE ],
    [ 24, Moves.SOAK ],
    [ 28, Moves.UPROAR ],
    [ 32, Moves.AQUA_TAIL ],
    [ 36, Moves.AQUA_RING ],
    [ 40, Moves.ENDEAVOR ],
    [ 44, Moves.HYDRO_PUMP ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.MAREANIE]: [
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.PECK ],
    [ 5, Moves.WIDE_GUARD ],
    [ 10, Moves.BITE ],
    [ 15, Moves.VENOSHOCK ],
    [ 20, Moves.RECOVER ],
    [ 25, Moves.PIN_MISSILE ],
    [ 30, Moves.TOXIC_SPIKES ],
    [ 35, Moves.LIQUIDATION ],
    [ 40, Moves.ACID_SPRAY ],
    [ 45, Moves.POISON_JAB ],
    [ 50, Moves.TOXIC ],
  ],
  [Species.TOXAPEX]: [
    [ EVOLVE_MOVE, Moves.BANEFUL_BUNKER ],
    [ 1, Moves.POISON_STING ],
    [ 1, Moves.BITE ],
    [ 1, Moves.PECK ],
    [ 1, Moves.WIDE_GUARD ],
    [ 15, Moves.VENOSHOCK ],
    [ 20, Moves.RECOVER ],
    [ 25, Moves.PIN_MISSILE ],
    [ 30, Moves.TOXIC_SPIKES ],
    [ 35, Moves.LIQUIDATION ],
    [ 42, Moves.ACID_SPRAY ],
    [ 49, Moves.POISON_JAB ],
    [ 56, Moves.TOXIC ],
  ],
  [Species.MUDBRAY]: [
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.ROCK_SMASH ],
    [ 4, Moves.IRON_DEFENSE ],
    [ 8, Moves.DOUBLE_KICK ],
    [ 12, Moves.BULLDOZE ],
    [ 16, Moves.STOMP ],
    [ 20, Moves.STRENGTH ],
    [ 24, Moves.COUNTER ],
    [ 28, Moves.HIGH_HORSEPOWER ],
    [ 32, Moves.HEAVY_SLAM ],
    [ 36, Moves.EARTHQUAKE ],
    [ 40, Moves.MEGA_KICK ],
    [ 44, Moves.SUPERPOWER ],
  ],
  [Species.MUDSDALE]: [
    [ 1, Moves.DOUBLE_KICK ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 12, Moves.BULLDOZE ],
    [ 16, Moves.STOMP ],
    [ 20, Moves.STRENGTH ],
    [ 24, Moves.COUNTER ],
    [ 28, Moves.HIGH_HORSEPOWER ],
    [ 34, Moves.HEAVY_SLAM ],
    [ 40, Moves.EARTHQUAKE ],
    [ 46, Moves.MEGA_KICK ],
    [ 52, Moves.SUPERPOWER ],
  ],
  [Species.DEWPIDER]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.INFESTATION ],
    [ 1, Moves.WATER_SPORT ],
    [ 4, Moves.BUG_BITE ],
    [ 8, Moves.BITE ],
    [ 12, Moves.BUBBLE_BEAM ],
    [ 16, Moves.AQUA_RING ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.CRUNCH ],
    [ 28, Moves.SOAK ],
    [ 32, Moves.ENTRAINMENT ],
    [ 36, Moves.LUNGE ],
    [ 40, Moves.LIQUIDATION ],
    [ 44, Moves.LEECH_LIFE ],
    [ 48, Moves.MIRROR_COAT ],
  ],
  [Species.ARAQUANID]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.BUG_BITE ],
    [ 1, Moves.WIDE_GUARD ],
    [ 1, Moves.INFESTATION ],
    [ 1, Moves.SPIDER_WEB ],
    [ 12, Moves.BUBBLE_BEAM ],
    [ 16, Moves.AQUA_RING ],
    [ 20, Moves.HEADBUTT ],
    [ 26, Moves.CRUNCH ],
    [ 32, Moves.SOAK ],
    [ 38, Moves.ENTRAINMENT ],
    [ 44, Moves.LUNGE ],
    [ 50, Moves.LIQUIDATION ],
    [ 56, Moves.LEECH_LIFE ],
    [ 62, Moves.MIRROR_COAT ],
  ],
  [Species.FOMANTIS]: [
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.LEAFAGE ],
    [ 5, Moves.GROWTH ],
    [ 10, Moves.INGRAIN ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.SWEET_SCENT ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.X_SCISSOR ],
    [ 35, Moves.SYNTHESIS ],
    [ 40, Moves.LEAF_BLADE ],
    [ 45, Moves.SUNNY_DAY ],
    [ 50, Moves.SOLAR_BEAM ],
  ],
  [Species.LURANTIS]: [
    [ EVOLVE_MOVE, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.SOLAR_BEAM ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.INGRAIN ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.LEAFAGE ],
    [ 1, Moves.DUAL_CHOP ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.SWEET_SCENT ],
    [ 25, Moves.SLASH ],
    [ 30, Moves.X_SCISSOR ],
    [ 37, Moves.SYNTHESIS ],
    [ 44, Moves.LEAF_BLADE ],
    [ 51, Moves.SUNNY_DAY ],
    [ 63, Moves.SOLAR_BLADE ],
  ],
  [Species.MORELULL]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.CONFUSE_RAY ],
    [ 8, Moves.INGRAIN ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.SLEEP_POWDER ],
    [ 20, Moves.MOONLIGHT ],
    [ 25, Moves.STRENGTH_SAP ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 32, Moves.DAZZLING_GLEAM ],
    [ 36, Moves.SPORE ],
    [ 40, Moves.MOONBLAST ],
    [ 44, Moves.DREAM_EATER ],
  ],
  [Species.SHIINOTIC]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.INGRAIN ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.FLASH ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.SLEEP_POWDER ],
    [ 20, Moves.MOONLIGHT ],
    [ 27, Moves.STRENGTH_SAP ],
    [ 32, Moves.GIGA_DRAIN ],
    [ 38, Moves.DAZZLING_GLEAM ],
    [ 44, Moves.SPORE ],
    [ 50, Moves.MOONBLAST ],
    [ 56, Moves.DREAM_EATER ],
  ],
  [Species.SALANDIT]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.POISON_GAS ],
    [ 5, Moves.SMOG ],
    [ 10, Moves.EMBER ],
    [ 15, Moves.POISON_FANG ],
    [ 20, Moves.SWEET_SCENT ],
    [ 25, Moves.NASTY_PLOT ],
    [ 30, Moves.INCINERATE ],
    [ 35, Moves.VENOSHOCK ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 45, Moves.FLAMETHROWER ],
    [ 50, Moves.TOXIC ],
    [ 55, Moves.ENDEAVOR ],
  ],
  [Species.SALAZZLE]: [
    [ EVOLVE_MOVE, Moves.FIRE_LASH ],
    [ 1, Moves.POUND ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.SWAGGER ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.TORMENT ],
    [ 1, Moves.KNOCK_OFF ],
    [ 1, Moves.ENDEAVOR ],
    [ 1, Moves.CAPTIVATE ],
    [ 15, Moves.POISON_FANG ],
    [ 20, Moves.SWEET_SCENT ],
    [ 25, Moves.NASTY_PLOT ],
    [ 30, Moves.INCINERATE ],
    [ 37, Moves.VENOSHOCK ],
    [ 44, Moves.DRAGON_PULSE ],
    [ 51, Moves.FLAMETHROWER ],
    [ 58, Moves.TOXIC ],
  ],
  [Species.STUFFUL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.BABY_DOLL_EYES ],
    [ 8, Moves.PAYBACK ],
    [ 12, Moves.BRUTAL_SWING ],
    [ 16, Moves.ENDURE ],
    [ 20, Moves.STRENGTH ],
    [ 24, Moves.TAKE_DOWN ],
    [ 28, Moves.FLAIL ],
    [ 32, Moves.HAMMER_ARM ],
    [ 36, Moves.THRASH ],
    [ 40, Moves.PAIN_SPLIT ],
    [ 44, Moves.DOUBLE_EDGE ],
    [ 48, Moves.SUPERPOWER ],
  ],
  [Species.BEWEAR]: [
    [ EVOLVE_MOVE, Moves.BIND ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PAYBACK ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 12, Moves.BRUTAL_SWING ],
    [ 16, Moves.ENDURE ],
    [ 20, Moves.STRENGTH ],
    [ 24, Moves.TAKE_DOWN ],
    [ 30, Moves.FLAIL ],
    [ 36, Moves.HAMMER_ARM ],
    [ 42, Moves.THRASH ],
    [ 48, Moves.PAIN_SPLIT ],
    [ 54, Moves.DOUBLE_EDGE ],
    [ 60, Moves.SUPERPOWER ],
  ],
  [Species.BOUNSWEET]: [
    [ 1, Moves.SPLASH ],
    [ 1, Moves.LEAFAGE ], //Custom
    [ 4, Moves.PLAY_NICE ],
    [ 8, Moves.RAPID_SPIN ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 16, Moves.SWEET_SCENT ],
    [ 20, Moves.MAGICAL_LEAF ],
    [ 24, Moves.FLAIL ],
    [ 28, Moves.TEETER_DANCE ],
    [ 32, Moves.AROMATIC_MIST ],
  ],
  [Species.STEENEE]: [
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.PLAY_NICE ],
    [ 16, Moves.SWEET_SCENT ],
    [ 22, Moves.MAGICAL_LEAF ],
    [ 28, Moves.STOMP ],
    [ 34, Moves.TEETER_DANCE ],
    [ 40, Moves.AROMATIC_MIST ],
    [ 46, Moves.LEAF_STORM ],
  ],
  [Species.TSAREENA]: [
    [ EVOLVE_MOVE, Moves.TROP_KICK ],
    [ 1, Moves.RAZOR_LEAF ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.SWAGGER ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.POWER_WHIP ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.PUNISHMENT ],
    [ 16, Moves.SWEET_SCENT ],
    [ 22, Moves.MAGICAL_LEAF ],
    [ 28, Moves.STOMP ],
    [ 34, Moves.TEETER_DANCE ],
    [ 40, Moves.AROMATIC_MIST ],
    [ 46, Moves.LEAF_STORM ],
    [ 58, Moves.HIGH_JUMP_KICK ],
  ],
  [Species.COMFEY]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.GROWTH ],
    [ 3, Moves.VINE_WHIP ],
    [ 6, Moves.HELPING_HAND ],
    [ 9, Moves.DRAINING_KISS ],
    [ 12, Moves.FLOWER_SHIELD ],
    [ 15, Moves.MAGICAL_LEAF ],
    [ 18, Moves.SYNTHESIS ],
    [ 21, Moves.LEECH_SEED ],
    [ 24, Moves.GRASS_KNOT ],
    [ 27, Moves.SWEET_KISS ],
    [ 30, Moves.FLORAL_HEALING ],
    [ 33, Moves.PETAL_BLIZZARD ],
    [ 36, Moves.AROMATHERAPY ],
    [ 39, Moves.PLAY_ROUGH ],
    [ 42, Moves.SWEET_SCENT ],
    [ 45, Moves.PETAL_DANCE ],
    [ 48, Moves.GRASSY_TERRAIN ],
  ],
  [Species.ORANGURU]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.TAUNT ],
    [ 5, Moves.AFTER_YOU ],
    [ 10, Moves.CALM_MIND ],
    [ 15, Moves.STORED_POWER ],
    [ 20, Moves.PSYCH_UP ],
    [ 25, Moves.QUASH ],
    [ 30, Moves.NASTY_PLOT ],
    [ 35, Moves.ZEN_HEADBUTT ],
    [ 40, Moves.TRICK_ROOM ],
    [ 45, Moves.PSYCHIC ],
    [ 50, Moves.INSTRUCT ],
    [ 55, Moves.FOUL_PLAY ],
    [ 60, Moves.FUTURE_SIGHT ],
  ],
  [Species.PASSIMIAN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 5, Moves.ROCK_SMASH ],
    [ 10, Moves.FOCUS_ENERGY ],
    [ 15, Moves.BEAT_UP ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.TAKE_DOWN ],
    [ 30, Moves.FLING ],
    [ 35, Moves.BULK_UP ],
    [ 40, Moves.THRASH ],
    [ 45, Moves.DOUBLE_EDGE ],
    [ 50, Moves.CLOSE_COMBAT ],
    [ 55, Moves.REVERSAL ],
    [ 60, Moves.GIGA_IMPACT ],
  ],
  [Species.WIMPOD]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.STRUGGLE_BUG ],
  ],
  [Species.GOLISOPOD]: [
    [ EVOLVE_MOVE, Moves.FIRST_IMPRESSION ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.SPITE ],
    [ 1, Moves.STRUGGLE_BUG ],
    [ 4, Moves.ROCK_SMASH ],
    [ 8, Moves.FURY_CUTTER ],
    [ 12, Moves.MUD_SHOT ],
    [ 16, Moves.BUG_BITE ],
    [ 20, Moves.IRON_DEFENSE ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.RAZOR_SHELL ],
    [ 36, Moves.PIN_MISSILE ],
    [ 40, Moves.SWORDS_DANCE ],
    [ 44, Moves.LIQUIDATION ],
  ],
  [Species.SANDYGAST]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.ASTONISH ],
    [ 10, Moves.SAND_TOMB ],
    [ 15, Moves.MEGA_DRAIN ],
    [ 20, Moves.SAND_ATTACK ],
    [ 25, Moves.BULLDOZE ],
    [ 30, Moves.HYPNOSIS ],
    [ 35, Moves.GIGA_DRAIN ],
    [ 40, Moves.IRON_DEFENSE ],
    [ 45, Moves.SHADOW_BALL ],
    [ 50, Moves.EARTH_POWER ],
    [ 55, Moves.SHORE_UP ],
    [ 60, Moves.SANDSTORM ],
  ],
  [Species.PALOSSAND]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.SAND_TOMB ],
    [ 15, Moves.MEGA_DRAIN ],
    [ 20, Moves.SAND_ATTACK ],
    [ 25, Moves.BULLDOZE ],
    [ 30, Moves.HYPNOSIS ],
    [ 35, Moves.GIGA_DRAIN ],
    [ 40, Moves.IRON_DEFENSE ],
    [ 47, Moves.SHADOW_BALL ],
    [ 54, Moves.EARTH_POWER ],
    [ 61, Moves.SHORE_UP ],
    [ 68, Moves.SANDSTORM ],
  ],
  [Species.PYUKUMUKU]: [
    [ 1, Moves.COUNTER ], //Custom, Moved from Level 20 to 1
    [ 1, Moves.HARDEN ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.MUD_SPORT ],
    [ 1, Moves.WATER_SPORT ],
    [ 5, Moves.HELPING_HAND ],
    [ 10, Moves.TAUNT ],
    [ 15, Moves.SAFEGUARD ],
    [ 20, Moves.MIRROR_COAT ], //Custom
    [ 25, Moves.PURIFY ],
    [ 30, Moves.CURSE ],
    [ 35, Moves.GASTRO_ACID ],
    [ 40, Moves.PAIN_SPLIT ],
    [ 45, Moves.RECOVER ],
    [ 50, Moves.SOAK ],
    [ 55, Moves.TOXIC ],
    [ 60, Moves.MEMENTO ],
  ],
  [Species.TYPE_NULL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.IMPRISON ],
    [ 5, Moves.AERIAL_ACE ],
    [ 10, Moves.SCARY_FACE ],
    [ 15, Moves.DOUBLE_HIT ],
    [ 20, Moves.METAL_SOUND ],
    [ 25, Moves.CRUSH_CLAW ],
    [ 30, Moves.AIR_SLASH ],
    [ 35, Moves.TRI_ATTACK ],
    [ 40, Moves.X_SCISSOR ],
    [ 45, Moves.IRON_HEAD ],
    [ 50, Moves.TAKE_DOWN ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 60, Moves.HEAL_BLOCK ],
  ],
  [Species.SILVALLY]: [
    [ EVOLVE_MOVE, Moves.MULTI_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.EXPLOSION ],
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.IMPRISON ],
    [ 1, Moves.POISON_FANG ],
    [ 1, Moves.AERIAL_ACE ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.IRON_HEAD ],
    [ 1, Moves.HEAL_BLOCK ],
    [ 15, Moves.DOUBLE_HIT ],
    [ 20, Moves.METAL_SOUND ],
    [ 25, Moves.CRUSH_CLAW ],
    [ 30, Moves.AIR_SLASH ],
    [ 35, Moves.TRI_ATTACK ],
    [ 40, Moves.X_SCISSOR ],
    [ 45, Moves.CRUNCH ],
    [ 50, Moves.TAKE_DOWN ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 60, Moves.PARTING_SHOT ],
  ],
  [Species.MINIOR]: [
    [ 1, Moves.TACKLE ],
    [ 3, Moves.DEFENSE_CURL ],
    [ 8, Moves.ROLLOUT ],
    [ 10, Moves.CONFUSE_RAY ],
    [ 15, Moves.SWIFT ],
    [ 17, Moves.ANCIENT_POWER ],
    [ 22, Moves.SELF_DESTRUCT ],
    [ 24, Moves.STEALTH_ROCK ],
    [ 29, Moves.TAKE_DOWN ],
    [ 31, Moves.AUTOTOMIZE ],
    [ 36, Moves.COSMIC_POWER ],
    [ 38, Moves.POWER_GEM ],
    [ 43, Moves.DOUBLE_EDGE ],
    [ 45, Moves.SHELL_SMASH ],
    [ 50, Moves.EXPLOSION ],
  ],
  [Species.KOMALA]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 6, Moves.STOCKPILE ],
    [ 6, Moves.SPIT_UP ],
    [ 6, Moves.SWALLOW ],
    [ 11, Moves.RAPID_SPIN ],
    [ 16, Moves.YAWN ],
    [ 21, Moves.SLAM ],
    [ 26, Moves.FLAIL ],
    [ 31, Moves.SUCKER_PUNCH ],
    [ 36, Moves.PSYCH_UP ],
    [ 41, Moves.WOOD_HAMMER ],
    [ 46, Moves.THRASH ],
  ],
  [Species.TURTONATOR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOG ],
    [ 4, Moves.EMBER ],
    [ 8, Moves.PROTECT ],
    [ 12, Moves.ENDURE ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.INCINERATE ],
    [ 24, Moves.IRON_DEFENSE ],
    [ 28, Moves.DRAGON_PULSE ],
    [ 32, Moves.BODY_SLAM ],
    [ 36, Moves.FLAMETHROWER ],
    [ 40, Moves.SHELL_TRAP ],
    [ 44, Moves.SHELL_SMASH ],
    [ 48, Moves.OVERHEAT ],
    [ 52, Moves.EXPLOSION ],
  ],
  [Species.TOGEDEMARU]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.NUZZLE ],
    [ 5, Moves.DEFENSE_CURL ],
    [ 10, Moves.CHARGE ],
    [ 15, Moves.THUNDER_SHOCK ],
    [ 20, Moves.FELL_STINGER ],
    [ 25, Moves.SPARK ],
    [ 30, Moves.PIN_MISSILE ],
    [ 35, Moves.MAGNET_RISE ],
    [ 40, Moves.ZING_ZAP ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.ELECTRIC_TERRAIN ],
    [ 55, Moves.WILD_CHARGE ],
    [ 60, Moves.SPIKY_SHIELD ],
  ],
  [Species.MIMIKYU]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.WOOD_HAMMER ],
    [ 6, Moves.SHADOW_SNEAK ],
    [ 12, Moves.DOUBLE_TEAM ],
    [ 18, Moves.BABY_DOLL_EYES ],
    [ 24, Moves.MIMIC ],
    [ 30, Moves.HONE_CLAWS ],
    [ 36, Moves.SLASH ],
    [ 42, Moves.SHADOW_CLAW ],
    [ 48, Moves.CHARM ],
    [ 54, Moves.PLAY_ROUGH ],
    [ 60, Moves.PAIN_SPLIT ],
  ],
  [Species.BRUXISH]: [
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.ASTONISH ],
    [ 9, Moves.CONFUSION ],
    [ 12, Moves.BITE ],
    [ 17, Moves.AQUA_JET ],
    [ 20, Moves.DISABLE ],
    [ 25, Moves.PSYSHOCK ],
    [ 28, Moves.CRUNCH ],
    [ 33, Moves.AQUA_TAIL ],
    [ 36, Moves.SCREECH ],
    [ 41, Moves.PSYCHIC_FANGS ],
    [ 44, Moves.WAVE_CRASH ],
  ],
  [Species.DRAMPA]: [
    [ 1, Moves.ECHOED_VOICE ],
    [ 1, Moves.PLAY_NICE ],
    [ 5, Moves.TWISTER ],
    [ 10, Moves.PROTECT ],
    [ 15, Moves.GLARE ],
    [ 20, Moves.SAFEGUARD ],
    [ 25, Moves.DRAGON_BREATH ],
    [ 30, Moves.EXTRASENSORY ],
    [ 35, Moves.DRAGON_PULSE ],
    [ 40, Moves.LIGHT_SCREEN ],
    [ 45, Moves.FLY ],
    [ 50, Moves.HYPER_VOICE ],
    [ 55, Moves.OUTRAGE ],
  ],
  [Species.DHELMISE]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.RAPID_SPIN ],
    [ 4, Moves.ASTONISH ],
    [ 8, Moves.WRAP ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 16, Moves.GROWTH ],
    [ 20, Moves.GYRO_BALL ],
    [ 24, Moves.SWITCHEROO ],
    [ 28, Moves.GIGA_DRAIN ],
    [ 32, Moves.WHIRLPOOL ],
    [ 36, Moves.HEAVY_SLAM ],
    [ 40, Moves.SLAM ],
    [ 44, Moves.SHADOW_BALL ],
    [ 48, Moves.METAL_SOUND ],
    [ 52, Moves.ANCHOR_SHOT ],
    [ 56, Moves.ENERGY_BALL ],
    [ 60, Moves.PHANTOM_FORCE ],
    [ 64, Moves.POWER_WHIP ],
  ],
  [Species.JANGMO_O]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.PROTECT ],
    [ 8, Moves.DRAGON_TAIL ],
    [ 12, Moves.SCARY_FACE ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.WORK_UP ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.IRON_DEFENSE ],
    [ 32, Moves.DRAGON_CLAW ],
    [ 36, Moves.NOBLE_ROAR ],
    [ 40, Moves.DRAGON_DANCE ],
    [ 44, Moves.OUTRAGE ],
  ],
  [Species.HAKAMO_O]: [
    [ EVOLVE_MOVE, Moves.SKY_UPPERCUT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 1, Moves.BIDE ],
    [ 12, Moves.SCARY_FACE ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.WORK_UP ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.IRON_DEFENSE ],
    [ 32, Moves.DRAGON_CLAW ],
    [ 38, Moves.NOBLE_ROAR ],
    [ 44, Moves.DRAGON_DANCE ],
    [ 50, Moves.OUTRAGE ],
    [ 56, Moves.CLOSE_COMBAT ],
  ],
  [Species.KOMMO_O]: [
    [ EVOLVE_MOVE, Moves.CLANGING_SCALES ],
    [ RELEARN_MOVE, Moves.BELLY_DRUM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 1, Moves.BIDE ],
    [ 1, Moves.SKY_UPPERCUT ],
    [ 12, Moves.SCARY_FACE ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.WORK_UP ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.IRON_DEFENSE ],
    [ 32, Moves.DRAGON_CLAW ],
    [ 38, Moves.NOBLE_ROAR ],
    [ 44, Moves.DRAGON_DANCE ],
    [ 52, Moves.OUTRAGE ],
    [ 60, Moves.CLOSE_COMBAT ],
    [ 68, Moves.CLANGOROUS_SOUL ],
    [ 76, Moves.BOOMBURST ],
  ],
  [Species.TAPU_KOKO]: [
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 5, Moves.WITHDRAW ],
    [ 10, Moves.FAIRY_WIND ],
    [ 15, Moves.FALSE_SWIPE ],
    [ 20, Moves.SPARK ],
    [ 25, Moves.SHOCK_WAVE ],
    [ 30, Moves.CHARGE ],
    [ 35, Moves.AGILITY ],
    [ 40, Moves.SCREECH ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.MEAN_LOOK ],
    [ 55, Moves.NATURES_MADNESS ],
    [ 60, Moves.WILD_CHARGE ],
    [ 65, Moves.BRAVE_BIRD ],
    [ 70, Moves.POWER_SWAP ],
    [ 75, Moves.ELECTRIC_TERRAIN ],
  ],
  [Species.TAPU_LELE]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.WITHDRAW ],
    [ 10, Moves.AROMATHERAPY ],
    [ 15, Moves.DRAINING_KISS ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.FLATTER ],
    [ 30, Moves.AROMATIC_MIST ],
    [ 35, Moves.SWEET_SCENT ],
    [ 40, Moves.EXTRASENSORY ],
    [ 45, Moves.PSYSHOCK ],
    [ 50, Moves.MEAN_LOOK ],
    [ 55, Moves.NATURES_MADNESS ],
    [ 60, Moves.MOONBLAST ],
    [ 65, Moves.TICKLE ],
    [ 70, Moves.SKILL_SWAP ],
    [ 75, Moves.PSYCHIC_TERRAIN ],
  ],
  [Species.TAPU_BULU]: [
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.LEAFAGE ],
    [ 5, Moves.WITHDRAW ],
    [ 10, Moves.DISABLE ],
    [ 15, Moves.LEECH_SEED ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 25, Moves.WHIRLWIND ],
    [ 30, Moves.HORN_ATTACK ],
    [ 35, Moves.SCARY_FACE ],
    [ 40, Moves.HORN_LEECH ],
    [ 45, Moves.ZEN_HEADBUTT ],
    [ 50, Moves.MEAN_LOOK ],
    [ 55, Moves.NATURES_MADNESS ],
    [ 60, Moves.WOOD_HAMMER ],
    [ 65, Moves.MEGAHORN ],
    [ 70, Moves.SKULL_BASH ],
    [ 75, Moves.GRASSY_TERRAIN ],
  ],
  [Species.TAPU_FINI]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 5, Moves.WITHDRAW ],
    [ 10, Moves.MIST ],
    [ 10, Moves.HAZE ],
    [ 15, Moves.AQUA_RING ],
    [ 20, Moves.WATER_PULSE ],
    [ 25, Moves.BRINE ],
    [ 30, Moves.DEFOG ],
    [ 35, Moves.HEAL_PULSE ],
    [ 40, Moves.SURF ],
    [ 45, Moves.MUDDY_WATER ],
    [ 50, Moves.MEAN_LOOK ],
    [ 55, Moves.NATURES_MADNESS ],
    [ 60, Moves.MOONBLAST ],
    [ 65, Moves.HYDRO_PUMP ],
    [ 70, Moves.SOAK ],
    [ 75, Moves.MISTY_TERRAIN ],
  ],
  [Species.COSMOG]: [
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.SPLASH ],
    [ 1, Moves.STORED_POWER ], //Custom
  ],
  [Species.COSMOEM]: [
    [ EVOLVE_MOVE, Moves.COSMIC_POWER ],
    [ 1, Moves.TELEPORT ],
  ],
  [Species.SOLGALEO]: [
    [ EVOLVE_MOVE, Moves.SUNSTEEL_STRIKE ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.COSMIC_POWER ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 1, Moves.WAKE_UP_SLAP ],
    [ 7, Moves.IRON_HEAD ],
    [ 14, Moves.METAL_SOUND ],
    [ 21, Moves.ZEN_HEADBUTT ],
    [ 28, Moves.FLASH_CANNON ],
    [ 35, Moves.MORNING_SUN ],
    [ 42, Moves.CRUNCH ],
    [ 49, Moves.METAL_BURST ],
    [ 56, Moves.WILD_CHARGE ],
    [ 63, Moves.SOLAR_BEAM ],
    [ 70, Moves.FLARE_BLITZ ],
    [ 77, Moves.WIDE_GUARD ],
    [ 84, Moves.GIGA_IMPACT ],
  ],
  [Species.LUNALA]: [
    [ EVOLVE_MOVE, Moves.MOONGEIST_BEAM ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.TELEPORT ],
    [ 1, Moves.COSMIC_POWER ],
    [ 7, Moves.NIGHT_SHADE ],
    [ 14, Moves.CONFUSE_RAY ],
    [ 21, Moves.AIR_SLASH ],
    [ 28, Moves.SHADOW_BALL ],
    [ 35, Moves.MOONLIGHT ],
    [ 42, Moves.NIGHT_DAZE ],
    [ 49, Moves.MAGIC_COAT ],
    [ 56, Moves.MOONBLAST ],
    [ 63, Moves.PHANTOM_FORCE ],
    [ 70, Moves.DREAM_EATER ],
    [ 77, Moves.WIDE_GUARD ],
    [ 84, Moves.HYPER_BEAM ],
  ],
  [Species.NIHILEGO]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.CONSTRICT ],
    [ 5, Moves.ACID ],
    [ 10, Moves.TICKLE ],
    [ 15, Moves.ACID_SPRAY ],
    [ 20, Moves.CLEAR_SMOG ],
    [ 25, Moves.GUARD_SPLIT ],
    [ 25, Moves.POWER_SPLIT ],
    [ 30, Moves.VENOSHOCK ],
    [ 35, Moves.HEADBUTT ],
    [ 40, Moves.TOXIC_SPIKES ],
    [ 45, Moves.VENOM_DRENCH ],
    [ 50, Moves.POWER_GEM ],
    [ 55, Moves.STEALTH_ROCK ],
    [ 60, Moves.MIRROR_COAT ],
    [ 65, Moves.WONDER_ROOM ],
    [ 70, Moves.HEAD_SMASH ],
  ],
  [Species.BUZZWOLE]: [
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POWER_UP_PUNCH ],
    [ 5, Moves.TAUNT ],
    [ 10, Moves.FELL_STINGER ],
    [ 15, Moves.VITAL_THROW ],
    [ 20, Moves.BULK_UP ],
    [ 25, Moves.ENDURE ],
    [ 30, Moves.REVERSAL ],
    [ 35, Moves.MEGA_PUNCH ],
    [ 40, Moves.LUNGE ],
    [ 45, Moves.FOCUS_ENERGY ],
    [ 50, Moves.DYNAMIC_PUNCH ],
    [ 55, Moves.COUNTER ],
    [ 60, Moves.HAMMER_ARM ],
    [ 65, Moves.SUPERPOWER ],
    [ 70, Moves.FOCUS_PUNCH ],
  ],
  [Species.PHEROMOSA]: [
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.FEINT ],
    [ 5, Moves.LEER ],
    [ 10, Moves.QUICK_GUARD ],
    [ 15, Moves.BUG_BITE ],
    [ 20, Moves.LOW_KICK ],
    [ 25, Moves.DOUBLE_KICK ],
    [ 30, Moves.TRIPLE_KICK ],
    [ 35, Moves.STOMP ],
    [ 40, Moves.AGILITY ],
    [ 45, Moves.LUNGE ],
    [ 50, Moves.BOUNCE ],
    [ 55, Moves.SPEED_SWAP ],
    [ 60, Moves.BUG_BUZZ ],
    [ 65, Moves.QUIVER_DANCE ],
    [ 70, Moves.HIGH_JUMP_KICK ],
  ],
  [Species.XURKITREE]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 5, Moves.CHARGE ],
    [ 10, Moves.THUNDER_WAVE ],
    [ 15, Moves.INGRAIN ],
    [ 20, Moves.SPARK ],
    [ 25, Moves.SHOCK_WAVE ],
    [ 30, Moves.HYPNOSIS ],
    [ 35, Moves.EERIE_IMPULSE ],
    [ 40, Moves.THUNDER_PUNCH ],
    [ 45, Moves.DISCHARGE ],
    [ 50, Moves.MAGNET_RISE ],
    [ 55, Moves.THUNDERBOLT ],
    [ 60, Moves.ELECTRIC_TERRAIN ],
    [ 65, Moves.POWER_WHIP ],
    [ 70, Moves.ZAP_CANNON ],
  ],
  [Species.CELESTEELA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ABSORB ],
    [ 5, Moves.HARDEN ],
    [ 10, Moves.WIDE_GUARD ],
    [ 15, Moves.MEGA_DRAIN ],
    [ 20, Moves.SMACK_DOWN ],
    [ 25, Moves.INGRAIN ],
    [ 30, Moves.AUTOTOMIZE ],
    [ 35, Moves.GIGA_DRAIN ],
    [ 40, Moves.FLASH_CANNON ],
    [ 45, Moves.METAL_SOUND ],
    [ 50, Moves.IRON_DEFENSE ],
    [ 55, Moves.LEECH_SEED ],
    [ 60, Moves.HEAVY_SLAM ],
    [ 65, Moves.DOUBLE_EDGE ],
    [ 70, Moves.SKULL_BASH ],
  ],
  [Species.KARTANA]: [
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.VACUUM_WAVE ],
    [ 5, Moves.RAZOR_LEAF ],
    [ 10, Moves.FALSE_SWIPE ],
    [ 15, Moves.CUT ],
    [ 20, Moves.AIR_CUTTER ],
    [ 25, Moves.AERIAL_ACE ],
    [ 30, Moves.DETECT ],
    [ 35, Moves.NIGHT_SLASH ],
    [ 40, Moves.SYNTHESIS ],
    [ 45, Moves.LASER_FOCUS ],
    [ 50, Moves.DEFOG ],
    [ 55, Moves.LEAF_BLADE ],
    [ 60, Moves.SACRED_SWORD ],
    [ 65, Moves.SWORDS_DANCE ],
    [ 70, Moves.GUILLOTINE ],
  ],
  [Species.GUZZLORD]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 5, Moves.STOCKPILE ],
    [ 5, Moves.SWALLOW ],
    [ 10, Moves.KNOCK_OFF ],
    [ 15, Moves.STOMP ],
    [ 20, Moves.STOMPING_TANTRUM ],
    [ 25, Moves.WIDE_GUARD ],
    [ 30, Moves.CRUNCH ],
    [ 35, Moves.BODY_SLAM ],
    [ 40, Moves.GASTRO_ACID ],
    [ 45, Moves.HAMMER_ARM ],
    [ 50, Moves.HEAVY_SLAM ],
    [ 55, Moves.DRAGON_RUSH ],
    [ 60, Moves.BELCH ],
    [ 65, Moves.THRASH ],
    [ 70, Moves.GIGA_IMPACT ],
  ],
  [Species.NECROZMA]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.MORNING_SUN ],
    [ 1, Moves.MOONLIGHT ],
    [ 1, Moves.GRAVITY ],
    [ 1, Moves.CHARGE_BEAM ],
    [ 1, Moves.MIRROR_SHOT ],
    [ 8, Moves.STEALTH_ROCK ],
    [ 16, Moves.SLASH ],
    [ 24, Moves.NIGHT_SLASH ],
    [ 32, Moves.PSYCHO_CUT ],
    [ 40, Moves.STORED_POWER ],
    [ 48, Moves.ROCK_BLAST ],
    [ 56, Moves.IRON_DEFENSE ],
    [ 64, Moves.POWER_GEM ],
    [ 72, Moves.PHOTON_GEYSER ],
    [ 80, Moves.AUTOTOMIZE ],
    [ 88, Moves.PRISMATIC_LASER ],
  ],
  [Species.MAGEARNA]: [
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.GYRO_BALL ],
    [ 1, Moves.DISARMING_VOICE ], //Custom
    [ 1, Moves.CRAFTY_SHIELD ],
    [ 1, Moves.GEAR_UP ],
    [ 6, Moves.DEFENSE_CURL ],
    [ 12, Moves.ROLLOUT ],
    [ 18, Moves.IRON_DEFENSE ],
    [ 24, Moves.MAGNETIC_FLUX ],
    [ 30, Moves.PSYBEAM ],
    [ 36, Moves.AURORA_BEAM ],
    [ 42, Moves.LOCK_ON ],
    [ 48, Moves.SHIFT_GEAR ],
    [ 54, Moves.TRICK ],
    [ 60, Moves.IRON_HEAD ],
    [ 66, Moves.AURA_SPHERE ],
    [ 72, Moves.FLASH_CANNON ],
    [ 78, Moves.PAIN_SPLIT ],
    [ 84, Moves.ZAP_CANNON ],
    [ 90, Moves.FLEUR_CANNON ],
  ],
  [Species.MARSHADOW]: [
    [ 1, Moves.FIRE_PUNCH ],
    [ 1, Moves.ICE_PUNCH ],
    [ 1, Moves.THUNDER_PUNCH ],
    [ 1, Moves.COUNTER ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 1, Moves.PURSUIT ],
    [ 9, Moves.ROLE_PLAY ],
    [ 18, Moves.SHADOW_PUNCH ],
    [ 27, Moves.FORCE_PALM ],
    [ 36, Moves.ASSURANCE ],
    [ 45, Moves.SUCKER_PUNCH ],
    [ 54, Moves.DRAIN_PUNCH ],
    [ 63, Moves.PSYCH_UP ],
    [ 72, Moves.SPECTRAL_THIEF ],
    [ 81, Moves.LASER_FOCUS ],
    [ 90, Moves.ENDEAVOR ],
    [ 99, Moves.CLOSE_COMBAT ],
  ],
  [Species.POIPOLE]: [
    [ RELEARN_MOVE, Moves.DRAGON_PULSE ], //Custom, made relearn
    [ 1, Moves.GROWL ],
    [ 1, Moves.ACID ],
    [ 1, Moves.PECK ],
    [ 1, Moves.HELPING_HAND ],
    [ 7, Moves.FURY_ATTACK ],
    [ 14, Moves.FELL_STINGER ],
    [ 21, Moves.CHARM ],
    [ 28, Moves.VENOSHOCK ],
    [ 35, Moves.VENOM_DRENCH ],
    [ 42, Moves.NASTY_PLOT ],
    [ 49, Moves.POISON_JAB ],
    [ 56, Moves.GASTRO_ACID ],
    [ 63, Moves.TOXIC ],
  ],
  [Species.NAGANADEL]: [
    [ EVOLVE_MOVE, Moves.AIR_CUTTER ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ACID ],
    [ 1, Moves.PECK ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.AIR_SLASH ],
    [ 1, Moves.DRAGON_PULSE ],
    [ 7, Moves.FURY_ATTACK ],
    [ 14, Moves.FELL_STINGER ],
    [ 21, Moves.CHARM ],
    [ 28, Moves.VENOSHOCK ],
    [ 35, Moves.VENOM_DRENCH ],
    [ 42, Moves.NASTY_PLOT ],
    [ 49, Moves.POISON_JAB ],
    [ 56, Moves.GASTRO_ACID ],
    [ 63, Moves.TOXIC ],
    [ 70, Moves.DRAGON_RUSH ],
  ],
  [Species.STAKATAKA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.ROCK_THROW ],
    [ 10, Moves.PROTECT ],
    [ 15, Moves.STOMP ],
    [ 20, Moves.BLOCK ],
    [ 25, Moves.ROCK_SLIDE ],
    [ 30, Moves.WIDE_GUARD ],
    [ 35, Moves.AUTOTOMIZE ],
    [ 40, Moves.ROCK_BLAST ],
    [ 45, Moves.MAGNET_RISE ],
    [ 50, Moves.IRON_DEFENSE ],
    [ 55, Moves.IRON_HEAD ],
    [ 60, Moves.TAKE_DOWN ],
    [ 65, Moves.STEALTH_ROCK ],
    [ 70, Moves.DOUBLE_EDGE ],
  ],
  [Species.BLACEPHALON]: [
    [ 1, Moves.FIRE_SPIN ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.LIGHT_SCREEN ],
    [ 10, Moves.EMBER ],
    [ 15, Moves.NIGHT_SHADE ],
    [ 20, Moves.CONFUSE_RAY ],
    [ 25, Moves.MAGIC_COAT ],
    [ 30, Moves.INCINERATE ],
    [ 35, Moves.HYPNOSIS ],
    [ 40, Moves.MYSTICAL_FIRE ],
    [ 45, Moves.SHADOW_BALL ],
    [ 50, Moves.CALM_MIND ],
    [ 55, Moves.WILL_O_WISP ],
    [ 60, Moves.TRICK ],
    [ 65, Moves.FIRE_BLAST ],
    [ 70, Moves.MIND_BLOWN ],
  ],
  [Species.ZERAORA]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.SPARK ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.SNARL ],
    [ 1, Moves.POWER_UP_PUNCH ],
    [ 8, Moves.FURY_SWIPES ],
    [ 16, Moves.QUICK_GUARD ],
    [ 24, Moves.SLASH ],
    [ 32, Moves.VOLT_SWITCH ],
    [ 40, Moves.CHARGE ],
    [ 48, Moves.THUNDER_PUNCH ],
    [ 56, Moves.HONE_CLAWS ],
    [ 64, Moves.DISCHARGE ],
    [ 72, Moves.WILD_CHARGE ],
    [ 80, Moves.AGILITY ],
    [ 88, Moves.PLASMA_FISTS ],
    [ 96, Moves.CLOSE_COMBAT ],
  ],
  [Species.MELTAN]: [
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.HARDEN ],
    [ 8, Moves.TAIL_WHIP ],
    [ 16, Moves.HEADBUTT ],
    [ 24, Moves.THUNDER_WAVE ],
    [ 32, Moves.ACID_ARMOR ],
    [ 40, Moves.FLASH_CANNON ],
  ],
  [Species.MELMETAL]: [
    [ EVOLVE_MOVE, Moves.THUNDER_PUNCH ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.HARDEN ],
    [ 24, Moves.THUNDER_WAVE ],
    [ 32, Moves.ACID_ARMOR ],
    [ 40, Moves.FLASH_CANNON ],
    [ 48, Moves.MEGA_PUNCH ],
    [ 56, Moves.PROTECT ],
    [ 64, Moves.DISCHARGE ],
    [ 72, Moves.DYNAMIC_PUNCH ],
    [ 80, Moves.SUPERPOWER ],
    [ 88, Moves.DOUBLE_IRON_BASH ],
    [ 96, Moves.HYPER_BEAM ],
  ],
  [Species.GROOKEY]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.BRANCH_POKE ], //Custom, moved from 6 to 5
    [ 8, Moves.TAUNT ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 17, Moves.SCREECH ],
    [ 20, Moves.KNOCK_OFF ],
    [ 24, Moves.SLAM ],
    [ 28, Moves.UPROAR ],
    [ 32, Moves.WOOD_HAMMER ],
    [ 36, Moves.ENDEAVOR ],
  ],
  [Species.THWACKEY]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_HIT ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.BRANCH_POKE ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 19, Moves.SCREECH ],
    [ 24, Moves.KNOCK_OFF ],
    [ 30, Moves.SLAM ],
    [ 36, Moves.UPROAR ],
    [ 42, Moves.WOOD_HAMMER ],
    [ 48, Moves.ENDEAVOR ],
  ],
  [Species.RILLABOOM]: [
    [ EVOLVE_MOVE, Moves.DRUM_BEATING ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TAUNT ],
    [ 1, Moves.DOUBLE_HIT ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 1, Moves.GRASSY_TERRAIN ],
    [ 1, Moves.BRANCH_POKE ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 19, Moves.SCREECH ],
    [ 24, Moves.KNOCK_OFF ],
    [ 30, Moves.SLAM ],
    [ 38, Moves.UPROAR ],
    [ 46, Moves.WOOD_HAMMER ],
    [ 54, Moves.ENDEAVOR ],
    [ 62, Moves.BOOMBURST ],
  ],
  [Species.SCORBUNNY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.EMBER ], //Custom, moved from 6 to 5
    [ 8, Moves.QUICK_ATTACK ],
    [ 12, Moves.DOUBLE_KICK ],
    [ 17, Moves.FLAME_CHARGE ],
    [ 20, Moves.AGILITY ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.COUNTER ],
    [ 32, Moves.BOUNCE ],
    [ 36, Moves.DOUBLE_EDGE ],
  ],
  [Species.RABOOT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 12, Moves.DOUBLE_KICK ],
    [ 19, Moves.FLAME_CHARGE ],
    [ 24, Moves.AGILITY ],
    [ 30, Moves.HEADBUTT ],
    [ 36, Moves.COUNTER ],
    [ 42, Moves.BOUNCE ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.CINDERACE]: [
    [ EVOLVE_MOVE, Moves.PYRO_BALL ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FEINT ],
    [ 12, Moves.DOUBLE_KICK ],
    [ 19, Moves.FLAME_CHARGE ],
    [ 24, Moves.AGILITY ],
    [ 30, Moves.HEADBUTT ],
    [ 38, Moves.COUNTER ],
    [ 46, Moves.BOUNCE ],
    [ 54, Moves.DOUBLE_EDGE ],
    [ 62, Moves.COURT_CHANGE ],
  ],
  [Species.SOBBLE]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.WATER_GUN ], //Custom, moved from 6 to 5
    [ 8, Moves.BIND ],
    [ 12, Moves.WATER_PULSE ],
    [ 17, Moves.TEARFUL_LOOK ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 24, Moves.U_TURN ],
    [ 28, Moves.LIQUIDATION ],
    [ 32, Moves.SOAK ],
    [ 36, Moves.RAIN_DANCE ],
  ],
  [Species.DRIZZILE]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.BIND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 12, Moves.WATER_PULSE ],
    [ 19, Moves.TEARFUL_LOOK ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 30, Moves.U_TURN ],
    [ 36, Moves.LIQUIDATION ],
    [ 42, Moves.SOAK ],
    [ 48, Moves.RAIN_DANCE ],
  ],
  [Species.INTELEON]: [
    [ EVOLVE_MOVE, Moves.SNIPE_SHOT ],
    [ 1, Moves.POUND ],
    [ 1, Moves.BIND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ACROBATICS ],
    [ 12, Moves.WATER_PULSE ],
    [ 19, Moves.TEARFUL_LOOK ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 30, Moves.U_TURN ],
    [ 38, Moves.LIQUIDATION ],
    [ 46, Moves.SOAK ],
    [ 54, Moves.RAIN_DANCE ],
    [ 62, Moves.HYDRO_PUMP ],
  ],
  [Species.SKWOVET]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.BITE ],
    [ 10, Moves.STUFF_CHEEKS ],
    [ 15, Moves.STOCKPILE ],
    [ 15, Moves.SPIT_UP ],
    [ 15, Moves.SWALLOW ],
    [ 20, Moves.BODY_SLAM ],
    [ 25, Moves.REST ],
    [ 30, Moves.COUNTER ],
    [ 35, Moves.BULLET_SEED ],
    [ 40, Moves.SUPER_FANG ],
    [ 45, Moves.BELCH ],
  ],
  [Species.GREEDENT]: [
    [ EVOLVE_MOVE, Moves.COVET ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.STUFF_CHEEKS ],
    [ 15, Moves.STOCKPILE ],
    [ 15, Moves.SPIT_UP ],
    [ 15, Moves.SWALLOW ],
    [ 20, Moves.BODY_SLAM ],
    [ 27, Moves.REST ],
    [ 34, Moves.COUNTER ],
    [ 41, Moves.BULLET_SEED ],
    [ 48, Moves.SUPER_FANG ],
    [ 55, Moves.BELCH ],
  ],
  [Species.ROOKIDEE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 4, Moves.POWER_TRIP ],
    [ 8, Moves.HONE_CLAWS ],
    [ 12, Moves.FURY_ATTACK ],
    [ 16, Moves.PLUCK ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.SCARY_FACE ],
    [ 28, Moves.DRILL_PECK ],
    [ 32, Moves.SWAGGER ],
    [ 36, Moves.BRAVE_BIRD ],
  ],
  [Species.CORVISQUIRE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.HONE_CLAWS ],
    [ 1, Moves.POWER_TRIP ],
    [ 12, Moves.FURY_ATTACK ],
    [ 16, Moves.PLUCK ],
    [ 22, Moves.TAUNT ],
    [ 28, Moves.SCARY_FACE ],
    [ 34, Moves.DRILL_PECK ],
    [ 40, Moves.SWAGGER ],
    [ 46, Moves.BRAVE_BIRD ],
  ],
  [Species.CORVIKNIGHT]: [
    [ EVOLVE_MOVE, Moves.STEEL_WING ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.SCREECH ],
    [ 1, Moves.METAL_SOUND ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.HONE_CLAWS ],
    [ 1, Moves.POWER_TRIP ],
    [ 12, Moves.FURY_ATTACK ],
    [ 16, Moves.PLUCK ],
    [ 22, Moves.TAUNT ],
    [ 28, Moves.SCARY_FACE ],
    [ 34, Moves.DRILL_PECK ],
    [ 42, Moves.SWAGGER ],
    [ 50, Moves.BRAVE_BIRD ],
  ],
  [Species.BLIPBUG]: [
    [ 1, Moves.STRUGGLE_BUG ],
  ],
  [Species.DOTTLER]: [
    [ EVOLVE_MOVE, Moves.CONFUSION ],
    [ EVOLVE_MOVE, Moves.LIGHT_SCREEN ],
    [ EVOLVE_MOVE, Moves.REFLECT ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.REFLECT ],
    [ 1, Moves.STRUGGLE_BUG ],
  ],
  [Species.ORBEETLE]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.REFLECT ],
    [ 1, Moves.STRUGGLE_BUG ],
    [ 4, Moves.CONFUSE_RAY ],
    [ 8, Moves.MAGIC_COAT ],
    [ 12, Moves.AGILITY ],
    [ 16, Moves.PSYBEAM ],
    [ 20, Moves.HYPNOSIS ],
    [ 24, Moves.ALLY_SWITCH ],
    [ 28, Moves.BUG_BUZZ ],
    [ 32, Moves.MIRROR_COAT ],
    [ 36, Moves.PSYCHIC ],
    [ 40, Moves.AFTER_YOU ],
    [ 44, Moves.CALM_MIND ],
    [ 48, Moves.PSYCHIC_TERRAIN ],
  ],
  [Species.NICKIT]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.BEAT_UP ],
    [ 8, Moves.HONE_CLAWS ],
    [ 12, Moves.SNARL ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.NASTY_PLOT ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.NIGHT_SLASH ],
    [ 32, Moves.TAIL_SLAP ],
    [ 36, Moves.FOUL_PLAY ],
  ],
  [Species.THIEVUL]: [
    [ EVOLVE_MOVE, Moves.THIEF ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.BEAT_UP ],
    [ 1, Moves.HONE_CLAWS ],
    [ 12, Moves.SNARL ],
    [ 16, Moves.ASSURANCE ],
    [ 22, Moves.NASTY_PLOT ],
    [ 28, Moves.SUCKER_PUNCH ],
    [ 34, Moves.NIGHT_SLASH ],
    [ 40, Moves.TAIL_SLAP ],
    [ 46, Moves.FOUL_PLAY ],
    [ 52, Moves.PARTING_SHOT ],
  ],
  [Species.GOSSIFLEUR]: [
    [ 1, Moves.SING ],
    [ 1, Moves.LEAFAGE ],
    [ 4, Moves.RAPID_SPIN ],
    [ 8, Moves.SWEET_SCENT ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 16, Moves.ROUND ],
    [ 21, Moves.LEAF_TORNADO ],
    [ 24, Moves.SYNTHESIS ],
    [ 28, Moves.HYPER_VOICE ],
    [ 32, Moves.AROMATHERAPY ],
    [ 36, Moves.LEAF_STORM ],
  ],
  [Species.ELDEGOSS]: [
    [ EVOLVE_MOVE, Moves.COTTON_SPORE ],
    [ 1, Moves.SING ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.LEAFAGE ],
    [ 12, Moves.RAZOR_LEAF ],
    [ 16, Moves.ROUND ],
    [ 23, Moves.LEAF_TORNADO ],
    [ 28, Moves.SYNTHESIS ],
    [ 34, Moves.HYPER_VOICE ],
    [ 40, Moves.AROMATHERAPY ],
    [ 46, Moves.LEAF_STORM ],
    [ 52, Moves.COTTON_GUARD ],
  ],
  [Species.WOOLOO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 4, Moves.DEFENSE_CURL ],
    [ 8, Moves.COPYCAT ],
    [ 12, Moves.GUARD_SPLIT ],
    [ 16, Moves.DOUBLE_KICK ],
    [ 21, Moves.HEADBUTT ],
    [ 25, Moves.TAKE_DOWN ],
    [ 28, Moves.GUARD_SWAP ],
    [ 32, Moves.REVERSAL ],
    [ 36, Moves.COTTON_GUARD ],
    [ 40, Moves.DOUBLE_EDGE ],
  ],
  [Species.DUBWOOL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.COPYCAT ],
    [ 12, Moves.GUARD_SPLIT ],
    [ 16, Moves.DOUBLE_KICK ],
    [ 21, Moves.HEADBUTT ],
    [ 27, Moves.TAKE_DOWN ],
    [ 32, Moves.GUARD_SWAP ],
    [ 38, Moves.REVERSAL ],
    [ 44, Moves.COTTON_GUARD ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 56, Moves.LAST_RESORT ],
  ],
  [Species.CHEWTLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 7, Moves.BITE ],
    [ 14, Moves.PROTECT ],
    [ 21, Moves.HEADBUTT ],
    [ 28, Moves.COUNTER ],
    [ 35, Moves.JAW_LOCK ],
    [ 42, Moves.LIQUIDATION ],
    [ 49, Moves.BODY_SLAM ],
  ],
  [Species.DREDNAW]: [
    [ EVOLVE_MOVE, Moves.ROCK_TOMB ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.ROCK_POLISH ],
    [ 1, Moves.RAZOR_SHELL ],
    [ 21, Moves.HEADBUTT ],
    [ 30, Moves.COUNTER ],
    [ 39, Moves.JAW_LOCK ],
    [ 48, Moves.LIQUIDATION ],
    [ 57, Moves.BODY_SLAM ],
    [ 66, Moves.HEAD_SMASH ],
  ],
  [Species.YAMPER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.NUZZLE ],
    [ 10, Moves.BITE ],
    [ 15, Moves.ROAR ],
    [ 20, Moves.SPARK ],
    [ 26, Moves.CHARM ],
    [ 30, Moves.CRUNCH ],
    [ 35, Moves.CHARGE ],
    [ 40, Moves.WILD_CHARGE ],
    [ 45, Moves.PLAY_ROUGH ],
  ],
  [Species.BOLTUND]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.ELECTRIFY ],
    [ 1, Moves.NUZZLE ],
    [ 15, Moves.ROAR ],
    [ 20, Moves.SPARK ],
    [ 28, Moves.CHARM ],
    [ 34, Moves.CRUNCH ],
    [ 41, Moves.CHARGE ],
    [ 48, Moves.WILD_CHARGE ],
    [ 55, Moves.PLAY_ROUGH ],
    [ 62, Moves.ELECTRIC_TERRAIN ],
  ],
  [Species.ROLYCOLY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOKESCREEN ],
    [ 5, Moves.RAPID_SPIN ],
    [ 10, Moves.SMACK_DOWN ],
    [ 15, Moves.ROCK_POLISH ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.INCINERATE ],
    [ 30, Moves.STEALTH_ROCK ],
    [ 35, Moves.HEAT_CRASH ],
    [ 40, Moves.ROCK_BLAST ],
  ],
  [Species.CARKOL]: [
    [ EVOLVE_MOVE, Moves.FLAME_CHARGE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.SMACK_DOWN ],
    [ 15, Moves.ROCK_POLISH ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 27, Moves.INCINERATE ],
    [ 35, Moves.STEALTH_ROCK ],
    [ 41, Moves.HEAT_CRASH ],
    [ 48, Moves.ROCK_BLAST ],
    [ 55, Moves.STONE_EDGE ],
  ],
  [Species.COALOSSAL]: [
    [ EVOLVE_MOVE, Moves.TAR_SHOT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.SMACK_DOWN ],
    [ 1, Moves.FLAME_CHARGE ],
    [ 15, Moves.ROCK_POLISH ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 27, Moves.INCINERATE ],
    [ 37, Moves.STEALTH_ROCK ],
    [ 45, Moves.HEAT_CRASH ],
    [ 54, Moves.ROCK_BLAST ],
    [ 63, Moves.STONE_EDGE ],
  ],
  [Species.APPLIN]: [
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.LEAFAGE ], //Custom
  ],
  [Species.FLAPPLE]: [
    [ EVOLVE_MOVE, Moves.WING_ATTACK ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.TWISTER ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.ACID_SPRAY ],
    [ 8, Moves.ACROBATICS ],
    [ 12, Moves.LEECH_SEED ],
    [ 16, Moves.PROTECT ],
    [ 20, Moves.DRAGON_BREATH ],
    [ 24, Moves.DRAGON_DANCE ],
    [ 28, Moves.DRAGON_PULSE ],
    [ 32, Moves.GRAV_APPLE ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 40, Moves.FLY ],
    [ 44, Moves.DRAGON_RUSH ],
  ],
  [Species.APPLETUN]: [
    [ EVOLVE_MOVE, Moves.HEADBUTT ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.CURSE ],
    [ 8, Moves.STOMP ],
    [ 12, Moves.LEECH_SEED ],
    [ 16, Moves.PROTECT ],
    [ 20, Moves.BULLET_SEED ],
    [ 24, Moves.RECOVER ],
    [ 28, Moves.APPLE_ACID ],
    [ 32, Moves.BODY_SLAM ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 44, Moves.ENERGY_BALL ],
  ],
  [Species.SILICOBRA]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.MUD_SLAP ], //Custom
    [ 5, Moves.MINIMIZE ],
    [ 10, Moves.BRUTAL_SWING ],
    [ 15, Moves.BULLDOZE ],
    [ 20, Moves.HEADBUTT ],
    [ 25, Moves.GLARE ],
    [ 30, Moves.DIG ],
    [ 35, Moves.SANDSTORM ],
    [ 40, Moves.SLAM ],
    [ 45, Moves.COIL ],
    [ 50, Moves.SAND_TOMB ],
  ],
  [Species.SANDACONDA]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.MINIMIZE ],
    [ 1, Moves.BRUTAL_SWING ],
    [ 15, Moves.BULLDOZE ],
    [ 20, Moves.HEADBUTT ],
    [ 25, Moves.GLARE ],
    [ 30, Moves.DIG ],
    [ 35, Moves.SANDSTORM ],
    [ 42, Moves.SLAM ],
    [ 49, Moves.COIL ],
    [ 51, Moves.SAND_TOMB ],
  ],
  [Species.CRAMORANT]: [
    [ RELEARN_MOVE, Moves.BELCH ],
    [ 1, Moves.PECK ],
    [ 1, Moves.STOCKPILE ],
    [ 1, Moves.SPIT_UP ],
    [ 1, Moves.SWALLOW ],
    [ 7, Moves.WATER_GUN ],
    [ 14, Moves.FURY_ATTACK ],
    [ 21, Moves.PLUCK ],
    [ 28, Moves.DIVE ],
    [ 35, Moves.DRILL_PECK ],
    [ 42, Moves.AMNESIA ],
    [ 49, Moves.THRASH ],
    [ 56, Moves.HYDRO_PUMP ],
  ],
  [Species.ARROKUDA]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.AQUA_JET ],
    [ 6, Moves.FURY_ATTACK ],
    [ 12, Moves.BITE ],
    [ 18, Moves.AGILITY ],
    [ 24, Moves.DIVE ],
    [ 30, Moves.FOCUS_ENERGY ],
    [ 36, Moves.CRUNCH ],
    [ 42, Moves.LIQUIDATION ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.BARRASKEWDA]: [
    [ 1, Moves.FURY_ATTACK ],
    [ 1, Moves.BITE ],
    [ 1, Moves.PECK ],
    [ 1, Moves.AQUA_JET ],
    [ 1, Moves.THROAT_CHOP ],
    [ 18, Moves.AGILITY ],
    [ 24, Moves.DIVE ],
    [ 32, Moves.FOCUS_ENERGY ],
    [ 40, Moves.CRUNCH ],
    [ 48, Moves.LIQUIDATION ],
    [ 56, Moves.DOUBLE_EDGE ],
  ],
  [Species.TOXEL]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.ACID ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.BELCH ],
    [ 1, Moves.NUZZLE ],
    [ 1, Moves.TEARFUL_LOOK ],
  ],
  [Species.TOXTRICITY]: [
    [ EVOLVE_MOVE, Moves.SPARK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ACID ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.FLAIL ],
    [ 1, Moves.ACID_SPRAY ],
    [ 1, Moves.BELCH ],
    [ 1, Moves.NOBLE_ROAR ],
    [ 1, Moves.NUZZLE ],
    [ 1, Moves.TEARFUL_LOOK ],
    [ 4, Moves.CHARGE ],
    [ 8, Moves.SHOCK_WAVE ],
    [ 12, Moves.SCARY_FACE ],
    [ 16, Moves.TAUNT ],
    [ 24, Moves.SCREECH ],
    [ 28, Moves.SWAGGER ],
    [ 32, Moves.TOXIC ],
    [ 36, Moves.DISCHARGE ],
    [ 40, Moves.POISON_JAB ],
    [ 44, Moves.OVERDRIVE ],
    [ 48, Moves.BOOMBURST ],
    [ 52, Moves.SHIFT_GEAR ],
  ],
  [Species.SIZZLIPEDE]: [
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 5, Moves.WRAP ],
    [ 10, Moves.BITE ],
    [ 15, Moves.FLAME_WHEEL ],
    [ 20, Moves.BUG_BITE ],
    [ 25, Moves.COIL ],
    [ 30, Moves.SLAM ],
    [ 35, Moves.FIRE_SPIN ],
    [ 40, Moves.CRUNCH ],
    [ 45, Moves.FIRE_LASH ],
    [ 50, Moves.LUNGE ],
    [ 55, Moves.BURN_UP ],
  ],
  [Species.CENTISKORCH]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.BITE ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.INFERNO ],
    [ 15, Moves.FLAME_WHEEL ],
    [ 20, Moves.BUG_BITE ],
    [ 25, Moves.COIL ],
    [ 32, Moves.SLAM ],
    [ 39, Moves.FIRE_SPIN ],
    [ 46, Moves.CRUNCH ],
    [ 53, Moves.FIRE_LASH ],
    [ 60, Moves.LUNGE ],
    [ 67, Moves.BURN_UP ],
  ],
  [Species.CLOBBOPUS]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_SMASH ],
    [ 5, Moves.FEINT ],
    [ 10, Moves.BIND ],
    [ 15, Moves.DETECT ],
    [ 20, Moves.BRICK_BREAK ],
    [ 25, Moves.BULK_UP ],
    [ 30, Moves.SUBMISSION ],
    [ 35, Moves.TAUNT ],
    [ 40, Moves.REVERSAL ],
    [ 45, Moves.SUPERPOWER ],
  ],
  [Species.GRAPPLOCT]: [
    [ EVOLVE_MOVE, Moves.OCTOLOCK ],
    [ 1, Moves.BIND ],
    [ 1, Moves.LEER ],
    [ 1, Moves.OCTAZOOKA ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.FEINT ],
    [ 15, Moves.DETECT ],
    [ 20, Moves.BRICK_BREAK ],
    [ 25, Moves.BULK_UP ],
    [ 30, Moves.SUBMISSION ],
    [ 35, Moves.TAUNT ],
    [ 40, Moves.REVERSAL ],
    [ 45, Moves.SUPERPOWER ],
    [ 50, Moves.TOPSY_TURVY ],
  ],
  [Species.SINISTEA]: [
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.ABSORB ], //Custom
    [ 6, Moves.AROMATIC_MIST ],
    [ 12, Moves.MEGA_DRAIN ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 30, Moves.SWEET_SCENT ],
    [ 36, Moves.GIGA_DRAIN ],
    [ 42, Moves.NASTY_PLOT ],
    [ 48, Moves.SHADOW_BALL ],
    [ 54, Moves.MEMENTO ],
    [ 60, Moves.SHELL_SMASH ],
  ],
  [Species.POLTEAGEIST]: [
    [ EVOLVE_MOVE, Moves.TEATIME ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.AROMATIC_MIST ],
    [ 1, Moves.STRENGTH_SAP ],
    [ 18, Moves.PROTECT ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 30, Moves.SWEET_SCENT ],
    [ 36, Moves.GIGA_DRAIN ],
    [ 42, Moves.NASTY_PLOT ],
    [ 48, Moves.SHADOW_BALL ],
    [ 54, Moves.MEMENTO ],
    [ 60, Moves.SHELL_SMASH ],
    [ 66, Moves.CURSE ],
  ],
  [Species.HATENNA]: [
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.PLAY_NICE ],
    [ 5, Moves.LIFE_DEW ],
    [ 10, Moves.DISARMING_VOICE ],
    [ 15, Moves.AROMATIC_MIST ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.HEAL_PULSE ],
    [ 30, Moves.DAZZLING_GLEAM ],
    [ 35, Moves.CALM_MIND ],
    [ 40, Moves.PSYCHIC ],
    [ 45, Moves.HEALING_WISH ],
  ],
  [Species.HATTREM]: [
    [ EVOLVE_MOVE, Moves.BRUTAL_SWING ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.LIFE_DEW ],
    [ 15, Moves.AROMATIC_MIST ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.HEAL_PULSE ],
    [ 30, Moves.DAZZLING_GLEAM ],
    [ 37, Moves.CALM_MIND ],
    [ 44, Moves.PSYCHIC ],
    [ 51, Moves.HEALING_WISH ],
  ],
  [Species.HATTERENE]: [
    [ EVOLVE_MOVE, Moves.PSYCHO_CUT ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.BRUTAL_SWING ],
    [ 1, Moves.LIFE_DEW ],
    [ 15, Moves.AROMATIC_MIST ],
    [ 20, Moves.PSYBEAM ],
    [ 25, Moves.HEAL_PULSE ],
    [ 30, Moves.DAZZLING_GLEAM ],
    [ 37, Moves.CALM_MIND ],
    [ 46, Moves.PSYCHIC ],
    [ 55, Moves.HEALING_WISH ],
    [ 64, Moves.MAGIC_POWDER ],
  ],
  [Species.IMPIDIMP]: [
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.CONFIDE ],
    [ 4, Moves.BITE ],
    [ 8, Moves.FLATTER ],
    [ 12, Moves.FAKE_TEARS ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.SWAGGER ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.TORMENT ],
    [ 33, Moves.DARK_PULSE ],
    [ 36, Moves.NASTY_PLOT ],
    [ 40, Moves.PLAY_ROUGH ],
    [ 44, Moves.FOUL_PLAY ],
  ],
  [Species.MORGREM]: [
    [ EVOLVE_MOVE, Moves.FALSE_SURRENDER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.FLATTER ],
    [ 1, Moves.CONFIDE ],
    [ 12, Moves.FAKE_TEARS ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.SWAGGER ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.TORMENT ],
    [ 35, Moves.DARK_PULSE ],
    [ 40, Moves.NASTY_PLOT ],
    [ 46, Moves.PLAY_ROUGH ],
    [ 52, Moves.FOUL_PLAY ],
  ],
  [Species.GRIMMSNARL]: [
    [ EVOLVE_MOVE, Moves.SPIRIT_BREAK ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.FLATTER ],
    [ 1, Moves.BULK_UP ],
    [ 1, Moves.CONFIDE ],
    [ 1, Moves.FALSE_SURRENDER ],
    [ 12, Moves.FAKE_TEARS ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.SWAGGER ],
    [ 24, Moves.SUCKER_PUNCH ],
    [ 28, Moves.TORMENT ],
    [ 35, Moves.DARK_PULSE ],
    [ 40, Moves.NASTY_PLOT ],
    [ 48, Moves.PLAY_ROUGH ],
    [ 56, Moves.FOUL_PLAY ],
    [ 64, Moves.HAMMER_ARM ],
  ],
  [Species.OBSTAGOON]: [
    [ EVOLVE_MOVE, Moves.OBSTRUCT ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PIN_MISSILE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.SUBMISSION ],
    [ 1, Moves.LICK ],
    [ 1, Moves.CROSS_CHOP ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.SWITCHEROO ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 9, Moves.SNARL ],
    [ 12, Moves.HEADBUTT ],
    [ 15, Moves.HONE_CLAWS ],
    [ 18, Moves.FURY_SWIPES ],
    [ 23, Moves.REST ],
    [ 28, Moves.TAKE_DOWN ],
    [ 35, Moves.SCARY_FACE ],
    [ 42, Moves.COUNTER ],
    [ 49, Moves.TAUNT ],
    [ 56, Moves.DOUBLE_EDGE ],
  ],
  [Species.PERRSERKER]: [
    [ EVOLVE_MOVE, Moves.IRON_HEAD ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.METAL_BURST ],
    [ 1, Moves.HONE_CLAWS ],
    [ 12, Moves.PAY_DAY ],
    [ 16, Moves.METAL_CLAW ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.SWAGGER ],
    [ 31, Moves.FURY_SWIPES ],
    [ 36, Moves.SCREECH ],
    [ 42, Moves.SLASH ],
    [ 48, Moves.METAL_SOUND ],
    [ 54, Moves.THRASH ],
  ],
  [Species.CURSOLA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.PERISH_SONG ],
    [ 1, Moves.ASTONISH ],
    [ 15, Moves.SPITE ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.HEX ],
    [ 30, Moves.CURSE ],
    [ 35, Moves.STRENGTH_SAP ],
    [ 40, Moves.POWER_GEM ],
    [ 45, Moves.NIGHT_SHADE ],
    [ 50, Moves.GRUDGE ],
    [ 55, Moves.MIRROR_COAT ],
  ],
  [Species.SIRFETCHD]: [
    [ EVOLVE_MOVE, Moves.IRON_DEFENSE ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.FIRST_IMPRESSION ],
    [ 15, Moves.ROCK_SMASH ],
    [ 20, Moves.BRUTAL_SWING ],
    [ 25, Moves.DETECT ],
    [ 30, Moves.KNOCK_OFF ],
    [ 35, Moves.DEFOG ],
    [ 40, Moves.BRICK_BREAK ],
    [ 45, Moves.SWORDS_DANCE ],
    [ 50, Moves.SLAM ],
    [ 55, Moves.LEAF_BLADE ],
    [ 60, Moves.FINAL_GAMBIT ],
    [ 65, Moves.BRAVE_BIRD ],
    [ 70, Moves.METEOR_ASSAULT ],
  ],
  [Species.MR_RIME]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.MIMIC ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.REFLECT ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.ROLE_PLAY ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.SLACK_OFF ],
    [ 1, Moves.FAKE_TEARS ],
    [ 1, Moves.BLOCK ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.ICE_SHARD ],
    [ 1, Moves.AFTER_YOU ],
    [ 1, Moves.MISTY_TERRAIN ],
    [ 1, Moves.DAZZLING_GLEAM ],
    [ 12, Moves.CONFUSION ],
    [ 16, Moves.ALLY_SWITCH ],
    [ 20, Moves.ICY_WIND ],
    [ 24, Moves.DOUBLE_KICK ],
    [ 28, Moves.PSYBEAM ],
    [ 32, Moves.HYPNOSIS ],
    [ 36, Moves.MIRROR_COAT ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 44, Moves.FREEZE_DRY ],
    [ 48, Moves.PSYCHIC ],
    [ 52, Moves.TEETER_DANCE ],
  ],
  [Species.RUNERIGUS]: [
    [ EVOLVE_MOVE, Moves.SHADOW_CLAW ],
    [ 1, Moves.NIGHT_SHADE ],
    [ 1, Moves.HAZE ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SCARY_FACE ],
    [ 1, Moves.ASTONISH ],
    [ 12, Moves.DISABLE ],
    [ 16, Moves.BRUTAL_SWING ],
    [ 20, Moves.CRAFTY_SHIELD ],
    [ 24, Moves.HEX ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.SLAM ],
    [ 38, Moves.CURSE ],
    [ 44, Moves.SHADOW_BALL ],
    [ 50, Moves.EARTHQUAKE ],
    [ 56, Moves.GUARD_SPLIT ],
    [ 56, Moves.POWER_SPLIT ],
    [ 62, Moves.DESTINY_BOND ],
  ],
  [Species.MILCERY]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.AROMATIC_MIST ],
    [ 5, Moves.SWEET_KISS ],
    [ 10, Moves.SWEET_SCENT ],
    [ 15, Moves.DRAINING_KISS ],
    [ 20, Moves.AROMATHERAPY ],
    [ 25, Moves.ATTRACT ],
    [ 30, Moves.ACID_ARMOR ],
    [ 35, Moves.DAZZLING_GLEAM ],
    [ 40, Moves.RECOVER ],
    [ 45, Moves.MISTY_TERRAIN ],
    [ 50, Moves.ENTRAINMENT ],
  ],
  [Species.ALCREMIE]: [
    [ EVOLVE_MOVE, Moves.DECORATE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.AROMATIC_MIST ],
    [ 15, Moves.DRAINING_KISS ],
    [ 20, Moves.AROMATHERAPY ],
    [ 25, Moves.ATTRACT ],
    [ 30, Moves.ACID_ARMOR ],
    [ 35, Moves.DAZZLING_GLEAM ],
    [ 40, Moves.RECOVER ],
    [ 45, Moves.MISTY_TERRAIN ],
    [ 50, Moves.ENTRAINMENT ],
  ],
  [Species.FALINKS]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PROTECT ],
    [ 5, Moves.ROCK_SMASH ],
    [ 10, Moves.FOCUS_ENERGY ],
    [ 15, Moves.HEADBUTT ],
    [ 20, Moves.BULK_UP ],
    [ 25, Moves.ENDURE ],
    [ 30, Moves.REVERSAL ],
    [ 35, Moves.FIRST_IMPRESSION ],
    [ 40, Moves.NO_RETREAT ],
    [ 45, Moves.IRON_DEFENSE ],
    [ 50, Moves.CLOSE_COMBAT ],
    [ 55, Moves.MEGAHORN ],
    [ 60, Moves.COUNTER ],
  ],
  [Species.PINCURCHIN]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 5, Moves.WATER_GUN ],
    [ 10, Moves.CHARGE ],
    [ 15, Moves.FURY_ATTACK ],
    [ 20, Moves.SPARK ],
    [ 25, Moves.BUBBLE_BEAM ],
    [ 30, Moves.RECOVER ],
    [ 35, Moves.CURSE ],
    [ 40, Moves.ELECTRIC_TERRAIN ],
    [ 45, Moves.POISON_JAB ],
    [ 50, Moves.ZING_ZAP ],
    [ 55, Moves.ACUPRESSURE ],
    [ 60, Moves.DISCHARGE ],
  ],
  [Species.SNOM]: [
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.STRUGGLE_BUG ],
  ],
  [Species.FROSMOTH]: [
    [ EVOLVE_MOVE, Moves.ICY_WIND ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.ATTRACT ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.STRUGGLE_BUG ],
    [ 4, Moves.STUN_SPORE ],
    [ 8, Moves.INFESTATION ],
    [ 12, Moves.MIST ],
    [ 16, Moves.DEFOG ],
    [ 21, Moves.FEATHER_DANCE ],
    [ 24, Moves.AURORA_BEAM ],
    [ 28, Moves.SNOWSCAPE ],
    [ 32, Moves.BUG_BUZZ ],
    [ 36, Moves.AURORA_VEIL ],
    [ 40, Moves.BLIZZARD ],
    [ 44, Moves.TAILWIND ],
    [ 48, Moves.WIDE_GUARD ],
    [ 52, Moves.QUIVER_DANCE ],
  ],
  [Species.STONJOURNER]: [
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.BLOCK ],
    [ 6, Moves.ROCK_POLISH ],
    [ 12, Moves.ROCK_TOMB ],
    [ 18, Moves.GRAVITY ],
    [ 24, Moves.STOMP ],
    [ 30, Moves.STEALTH_ROCK ],
    [ 36, Moves.ROCK_SLIDE ],
    [ 42, Moves.BODY_SLAM ],
    [ 48, Moves.WIDE_GUARD ],
    [ 54, Moves.HEAVY_SLAM ],
    [ 60, Moves.STONE_EDGE ],
    [ 66, Moves.MEGA_KICK ],
  ],
  [Species.EISCUE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POWDER_SNOW ],
    [ 6, Moves.MIST ],
    [ 12, Moves.WEATHER_BALL ],
    [ 18, Moves.ICY_WIND ],
    [ 24, Moves.HEADBUTT ],
    [ 30, Moves.AMNESIA ],
    [ 36, Moves.FREEZE_DRY ],
    [ 42, Moves.SNOWSCAPE ],
    [ 48, Moves.AURORA_VEIL ],
    [ 54, Moves.SURF ],
    [ 60, Moves.BLIZZARD ],
  ],
  [Species.INDEEDEE]: [
    [ 1, Moves.STORED_POWER ],
    [ 1, Moves.PLAY_NICE ],
    [ 5, Moves.ENCORE ],
    [ 10, Moves.DISARMING_VOICE ],
    [ 15, Moves.PSYBEAM ],
    [ 20, Moves.HELPING_HAND ],
    [ 25, Moves.AFTER_YOU ],
    [ 30, Moves.HEALING_WISH ],
    [ 35, Moves.PSYCHIC ],
    [ 40, Moves.CALM_MIND ],
    [ 45, Moves.POWER_SPLIT ],
    [ 50, Moves.PSYCHIC_TERRAIN ],
    [ 55, Moves.LAST_RESORT ],
  ],
  [Species.MORPEKO]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 5, Moves.LEER ],
    [ 10, Moves.POWER_TRIP ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.FLATTER ],
    [ 25, Moves.BITE ],
    [ 30, Moves.SPARK ],
    [ 35, Moves.TORMENT ],
    [ 40, Moves.AGILITY ],
    [ 45, Moves.BULLET_SEED ],
    [ 50, Moves.CRUNCH ],
    [ 55, Moves.AURA_WHEEL ],
    [ 60, Moves.THRASH ],
  ],
  [Species.CUFANT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.ROLLOUT ],
    [ 10, Moves.ROCK_SMASH ],
    [ 15, Moves.BULLDOZE ],
    [ 20, Moves.STOMP ],
    [ 25, Moves.IRON_DEFENSE ],
    [ 30, Moves.DIG ],
    [ 35, Moves.STRENGTH ],
    [ 40, Moves.IRON_HEAD ],
    [ 45, Moves.PLAY_ROUGH ],
    [ 50, Moves.HIGH_HORSEPOWER ],
    [ 55, Moves.SUPERPOWER ],
  ],
  [Species.COPPERAJAH]: [
    [ EVOLVE_MOVE, Moves.HEAVY_SLAM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.ROCK_SMASH ],
    [ 15, Moves.BULLDOZE ],
    [ 20, Moves.STOMP ],
    [ 25, Moves.IRON_DEFENSE ],
    [ 30, Moves.DIG ],
    [ 37, Moves.STRENGTH ],
    [ 44, Moves.IRON_HEAD ],
    [ 51, Moves.PLAY_ROUGH ],
    [ 58, Moves.HIGH_HORSEPOWER ],
    [ 65, Moves.SUPERPOWER ],
  ],
  [Species.DRACOZOLT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 7, Moves.CHARGE ],
    [ 14, Moves.AERIAL_ACE ],
    [ 21, Moves.ANCIENT_POWER ],
    [ 28, Moves.PLUCK ],
    [ 35, Moves.DRAGON_TAIL ],
    [ 42, Moves.STOMP ],
    [ 49, Moves.SLAM ],
    [ 56, Moves.DISCHARGE ],
    [ 63, Moves.BOLT_BEAK ],
    [ 70, Moves.DRAGON_PULSE ],
    [ 77, Moves.DRAGON_RUSH ],
  ],
  [Species.ARCTOZOLT]: [
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.POWDER_SNOW ],
    [ 7, Moves.CHARGE ],
    [ 14, Moves.ECHOED_VOICE ],
    [ 21, Moves.ANCIENT_POWER ],
    [ 28, Moves.PLUCK ],
    [ 35, Moves.AVALANCHE ],
    [ 42, Moves.FREEZE_DRY ],
    [ 49, Moves.SLAM ],
    [ 56, Moves.DISCHARGE ],
    [ 63, Moves.BOLT_BEAK ],
    [ 70, Moves.ICICLE_CRASH ],
    [ 77, Moves.BLIZZARD ],
  ],
  [Species.DRACOVISH]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 7, Moves.PROTECT ],
    [ 14, Moves.BRUTAL_SWING ],
    [ 21, Moves.ANCIENT_POWER ],
    [ 28, Moves.BITE ],
    [ 35, Moves.DRAGON_BREATH ],
    [ 42, Moves.STOMP ],
    [ 49, Moves.SUPER_FANG ],
    [ 56, Moves.CRUNCH ],
    [ 63, Moves.FISHIOUS_REND ],
    [ 70, Moves.DRAGON_PULSE ],
    [ 77, Moves.DRAGON_RUSH ],
  ],
  [Species.ARCTOVISH]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.POWDER_SNOW ],
    [ 7, Moves.PROTECT ],
    [ 14, Moves.ICY_WIND ],
    [ 21, Moves.ANCIENT_POWER ],
    [ 28, Moves.BITE ],
    [ 35, Moves.AURORA_VEIL ],
    [ 42, Moves.FREEZE_DRY ],
    [ 49, Moves.SUPER_FANG ],
    [ 56, Moves.CRUNCH ],
    [ 63, Moves.FISHIOUS_REND ],
    [ 70, Moves.ICICLE_CRASH ],
    [ 77, Moves.BLIZZARD ],
  ],
  [Species.DURALUDON]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.METAL_CLAW ],
    [ 6, Moves.ROCK_SMASH ],
    [ 12, Moves.HONE_CLAWS ],
    [ 18, Moves.METAL_SOUND ],
    [ 24, Moves.BREAKING_SWIPE ],
    [ 30, Moves.DRAGON_TAIL ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 42, Moves.LASER_FOCUS ],
    [ 48, Moves.DRAGON_CLAW ],
    [ 54, Moves.FLASH_CANNON ],
    [ 60, Moves.METAL_BURST ],
    [ 66, Moves.HYPER_BEAM ],
  ],
  [Species.DREEPY]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.INFESTATION ],
  ],
  [Species.DRAKLOAK]: [
    [ EVOLVE_MOVE, Moves.DRAGON_PULSE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.INFESTATION ],
    [ 6, Moves.LOCK_ON ],
    [ 12, Moves.ASSURANCE ],
    [ 18, Moves.HEX ],
    [ 24, Moves.AGILITY ],
    [ 30, Moves.DOUBLE_HIT ],
    [ 36, Moves.U_TURN ],
    [ 42, Moves.DRAGON_DANCE ],
    [ 48, Moves.PHANTOM_FORCE ],
    [ 54, Moves.TAKE_DOWN ],
    [ 61, Moves.DRAGON_RUSH ],
    [ 66, Moves.DOUBLE_EDGE ],
    [ 72, Moves.LAST_RESORT ],
  ],
  [Species.DRAGAPULT]: [
    [ EVOLVE_MOVE, Moves.DRAGON_DARTS ],
    [ 1, Moves.BITE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.INFESTATION ],
    [ 6, Moves.LOCK_ON ],
    [ 12, Moves.ASSURANCE ],
    [ 18, Moves.HEX ],
    [ 24, Moves.AGILITY ],
    [ 30, Moves.DOUBLE_HIT ],
    [ 36, Moves.U_TURN ],
    [ 42, Moves.DRAGON_DANCE ],
    [ 48, Moves.PHANTOM_FORCE ],
    [ 54, Moves.TAKE_DOWN ],
    [ 63, Moves.DRAGON_RUSH ],
    [ 70, Moves.DOUBLE_EDGE ],
    [ 78, Moves.LAST_RESORT ],
  ],
  [Species.ZACIAN]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.HOWL ],
    [ 1, Moves.QUICK_GUARD ],
    [ 1, Moves.SACRED_SWORD ],
    [ 11, Moves.SLASH ],
    [ 22, Moves.SWORDS_DANCE ],
    [ 33, Moves.IRON_HEAD ],
    [ 44, Moves.NOBLE_ROAR ],
    [ 55, Moves.CRUNCH ],
    [ 66, Moves.MOONBLAST ],
    [ 77, Moves.CLOSE_COMBAT ],
    [ 88, Moves.GIGA_IMPACT ],
  ],
  [Species.ZAMAZENTA]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.HOWL ],
    [ 1, Moves.WIDE_GUARD ],
    [ 11, Moves.SLASH ],
    [ 22, Moves.IRON_DEFENSE ],
    [ 33, Moves.IRON_HEAD ],
    [ 44, Moves.METAL_BURST ],
    [ 55, Moves.CRUNCH ],
    [ 66, Moves.MOONBLAST ],
    [ 77, Moves.CLOSE_COMBAT ],
    [ 88, Moves.GIGA_IMPACT ],
  ],
  [Species.ETERNATUS]: [
    [ 1, Moves.AGILITY ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.POISON_TAIL ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 8, Moves.TOXIC ],
    [ 16, Moves.VENOSHOCK ],
    [ 24, Moves.DRAGON_DANCE ],
    [ 32, Moves.CROSS_POISON ],
    [ 40, Moves.DRAGON_PULSE ],
    [ 48, Moves.FLAMETHROWER ],
    [ 56, Moves.DYNAMAX_CANNON ],
    [ 64, Moves.COSMIC_POWER ],
    [ 72, Moves.RECOVER ],
    [ 80, Moves.HYPER_BEAM ],
    [ 88, Moves.OUTRAGE ],
  ],
  [Species.KUBFU]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_SMASH ],
    [ 4, Moves.ENDURE ],
    [ 8, Moves.FOCUS_ENERGY ],
    [ 12, Moves.AERIAL_ACE ],
    [ 16, Moves.SCARY_FACE ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.BRICK_BREAK ],
    [ 28, Moves.DETECT ],
    [ 32, Moves.BULK_UP ],
    [ 36, Moves.IRON_HEAD ],
    [ 40, Moves.DYNAMIC_PUNCH ],
    [ 44, Moves.COUNTER ],
    [ 48, Moves.CLOSE_COMBAT ],
    [ 52, Moves.FOCUS_PUNCH ],
  ],
  [Species.URSHIFU]: [
    [ EVOLVE_MOVE, Moves.WICKED_BLOW ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.ENDURE ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.SUCKER_PUNCH ],
    [ 12, Moves.AERIAL_ACE ],
    [ 16, Moves.SCARY_FACE ],
    [ 20, Moves.HEADBUTT ],
    [ 24, Moves.BRICK_BREAK ],
    [ 28, Moves.DETECT ],
    [ 32, Moves.BULK_UP ],
    [ 36, Moves.IRON_HEAD ],
    [ 40, Moves.DYNAMIC_PUNCH ],
    [ 44, Moves.COUNTER ],
    [ 48, Moves.CLOSE_COMBAT ],
    [ 52, Moves.FOCUS_PUNCH ],
  ],
  [Species.ZARUDE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.BIND ],
    [ 6, Moves.LEER ],
    [ 12, Moves.VINE_WHIP ],
    [ 18, Moves.GROWTH ],
    [ 24, Moves.FURY_SWIPES ],
    [ 30, Moves.SCARY_FACE ],
    [ 36, Moves.GRASS_KNOT ],
    [ 42, Moves.BITE ],
    [ 48, Moves.U_TURN ],
    [ 54, Moves.SWAGGER ],
    [ 60, Moves.ENERGY_BALL ],
    [ 66, Moves.SYNTHESIS ],
    [ 72, Moves.HAMMER_ARM ],
    [ 78, Moves.THRASH ],
    [ 84, Moves.POWER_WHIP ],
    [ 90, Moves.JUNGLE_HEALING ],
  ],
  [Species.REGIELEKI]: [
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.RAPID_SPIN ],
    [ 6, Moves.ELECTROWEB ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 18, Moves.SHOCK_WAVE ],
    [ 24, Moves.THUNDER_WAVE ],
    [ 30, Moves.EXTREME_SPEED ],
    [ 36, Moves.THUNDER_CAGE ],
    [ 42, Moves.THUNDERBOLT ],
    [ 48, Moves.MAGNET_RISE ],
    [ 54, Moves.THRASH ],
    [ 60, Moves.LOCK_ON ],
    [ 66, Moves.ZAP_CANNON ],
    [ 72, Moves.HYPER_BEAM ],
    [ 78, Moves.EXPLOSION ],
  ],
  [Species.REGIDRAGO]: [
    [ 1, Moves.TWISTER ],
    [ 6, Moves.BITE ],
    [ 12, Moves.ANCIENT_POWER ],
    [ 18, Moves.DRAGON_BREATH ],
    [ 30, Moves.CRUNCH ],
    [ 36, Moves.DRAGON_CLAW ],
    [ 42, Moves.HAMMER_ARM ],
    [ 48, Moves.DRAGON_DANCE ],
    [ 54, Moves.THRASH ],
    [ 60, Moves.FOCUS_ENERGY ],
    [ 66, Moves.DRAGON_ENERGY ],
    [ 72, Moves.HYPER_BEAM ],
    [ 78, Moves.EXPLOSION ],
  ],
  [Species.GLASTRIER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 6, Moves.DOUBLE_KICK ],
    [ 12, Moves.AVALANCHE ],
    [ 18, Moves.STOMP ],
    [ 24, Moves.TORMENT ],
    [ 30, Moves.MIST ],
    [ 36, Moves.ICICLE_CRASH ],
    [ 42, Moves.TAKE_DOWN ],
    [ 48, Moves.IRON_DEFENSE ],
    [ 54, Moves.THRASH ],
    [ 60, Moves.TAUNT ],
    [ 66, Moves.DOUBLE_EDGE ],
    [ 72, Moves.SWORDS_DANCE ],
  ],
  [Species.SPECTRIER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 6, Moves.DOUBLE_KICK ],
    [ 12, Moves.HEX ],
    [ 18, Moves.STOMP ],
    [ 24, Moves.CONFUSE_RAY ],
    [ 30, Moves.HAZE ],
    [ 36, Moves.SHADOW_BALL ],
    [ 42, Moves.TAKE_DOWN ],
    [ 48, Moves.AGILITY ],
    [ 54, Moves.THRASH ],
    [ 60, Moves.DISABLE ],
    [ 66, Moves.DOUBLE_EDGE ],
    [ 72, Moves.NASTY_PLOT ],
  ],
  [Species.CALYREX]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.CONFUSION ],
    [ 8, Moves.LIFE_DEW ],
    [ 16, Moves.GIGA_DRAIN ],
    [ 24, Moves.PSYSHOCK ],
    [ 32, Moves.HELPING_HAND ],
    [ 40, Moves.GRASSY_TERRAIN ],
    [ 40, Moves.PSYCHIC_TERRAIN ],
    [ 48, Moves.ENERGY_BALL ],
    [ 56, Moves.PSYCHIC ],
    [ 64, Moves.LEECH_SEED ],
    [ 72, Moves.HEAL_PULSE ],
    [ 80, Moves.SOLAR_BEAM ],
    [ 88, Moves.FUTURE_SIGHT ],
  ],
  [Species.WYRDEER]: [
    [ EVOLVE_MOVE, Moves.PSYSHIELD_BASH ],
    [ 1, Moves.TACKLE ],
    [ 3, Moves.LEER ],
    [ 7, Moves.ASTONISH ],
    [ 10, Moves.HYPNOSIS ],
    [ 13, Moves.STOMP ],
    [ 16, Moves.SAND_ATTACK ],
    [ 21, Moves.TAKE_DOWN ],
    [ 23, Moves.CONFUSE_RAY ],
    [ 27, Moves.CALM_MIND ],
    [ 32, Moves.ROLE_PLAY ],
    [ 37, Moves.ZEN_HEADBUTT ],
    [ 49, Moves.IMPRISON ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 62, Moves.MEGAHORN ],
  ],
  [Species.KLEAVOR]: [
    [ EVOLVE_MOVE, Moves.STONE_AXE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 4, Moves.FURY_CUTTER ],
    [ 8, Moves.FALSE_SWIPE ],
    [ 12, Moves.SMACK_DOWN ],
    [ 16, Moves.DOUBLE_TEAM ],
    [ 20, Moves.DOUBLE_HIT ],
    [ 24, Moves.SLASH ],
    [ 28, Moves.FOCUS_ENERGY ],
    [ 32, Moves.AGILITY ],
    [ 36, Moves.ROCK_SLIDE ],
    [ 40, Moves.X_SCISSOR ],
    [ 44, Moves.SWORDS_DANCE ],
  ],
  [Species.URSALUNA]: [
    [ EVOLVE_MOVE, Moves.HEADLONG_RUSH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.FAKE_TEARS ],
    [ 1, Moves.COVET ],
    [ 8, Moves.FURY_SWIPES ],
    [ 13, Moves.PAYBACK ],
    [ 17, Moves.SWEET_SCENT ],
    [ 22, Moves.SLASH ],
    [ 25, Moves.PLAY_NICE ],
    [ 29, Moves.PLAY_ROUGH ],
    [ 35, Moves.SCARY_FACE ],
    [ 41, Moves.REST ],
    [ 41, Moves.SNORE ],
    [ 48, Moves.HIGH_HORSEPOWER ],
    [ 56, Moves.THRASH ],
    [ 64, Moves.HAMMER_ARM ],
  ],
  [Species.BASCULEGION]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SHADOW_BALL ],
    [ 1, Moves.PHANTOM_FORCE ],
    [ 4, Moves.TACKLE ],
    [ 8, Moves.FLAIL ],
    [ 12, Moves.AQUA_JET ],
    [ 16, Moves.BITE ],
    [ 20, Moves.SCARY_FACE ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.SOAK ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.UPROAR ],
    [ 44, Moves.WAVE_CRASH ],
    [ 48, Moves.THRASH ],
    [ 52, Moves.DOUBLE_EDGE ],
    [ 56, Moves.HEAD_SMASH ],
  ],
  [Species.SNEASLER]: [
    [ EVOLVE_MOVE, Moves.DIRE_CLAW ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_SMASH ],
    [ 1, Moves.FLING ],
    [ 6, Moves.TAUNT ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 18, Moves.METAL_CLAW ],
    [ 24, Moves.POISON_JAB ],
    [ 30, Moves.BRICK_BREAK ],
    [ 36, Moves.HONE_CLAWS ],
    [ 42, Moves.SLASH ],
    [ 48, Moves.AGILITY ],
    [ 54, Moves.SCREECH ],
    [ 60, Moves.CLOSE_COMBAT ],
  ],
  [Species.OVERQWIL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POISON_STING ],
    [ 4, Moves.HARDEN ],
    [ 8, Moves.BITE ],
    [ 12, Moves.FELL_STINGER ],
    [ 16, Moves.MINIMIZE ],
    [ 20, Moves.SPIKES ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.BARB_BARRAGE ],
    [ 32, Moves.PIN_MISSILE ],
    [ 36, Moves.TOXIC_SPIKES ],
    [ 40, Moves.STOCKPILE ],
    [ 40, Moves.SPIT_UP ],
    [ 44, Moves.TOXIC ],
    [ 48, Moves.CRUNCH ],
    [ 52, Moves.ACUPRESSURE ],
    [ 56, Moves.DESTINY_BOND ],
  ],
  [Species.ENAMORUS]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.FAIRY_WIND ],
    [ 5, Moves.TORMENT ],
    [ 10, Moves.FLATTER ],
    [ 15, Moves.TWISTER ],
    [ 20, Moves.DRAINING_KISS ],
    [ 25, Moves.IRON_DEFENSE ],
    [ 30, Moves.IMPRISON ],
    [ 35, Moves.MYSTICAL_FIRE ],
    [ 40, Moves.DAZZLING_GLEAM ],
    [ 45, Moves.EXTRASENSORY ],
    [ 50, Moves.UPROAR ],
    [ 55, Moves.SUPERPOWER ],
    [ 60, Moves.HEALING_WISH ],
    [ 65, Moves.MOONBLAST ],
    [ 70, Moves.OUTRAGE ],
    [ 75, Moves.SPRINGTIDE_STORM ],
  ],
  [Species.SPRIGATITO]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.LEAFAGE ],
    [ 7, Moves.BITE ],
    [ 10, Moves.HONE_CLAWS ],
    [ 13, Moves.MAGICAL_LEAF ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 17, Moves.SEED_BOMB ],
    [ 21, Moves.U_TURN ],
    [ 25, Moves.WORRY_SEED ],
    [ 28, Moves.SLASH ],
    [ 32, Moves.ENERGY_BALL ],
    [ 36, Moves.PLAY_ROUGH ],
  ],
  [Species.FLORAGATO]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.LEAFAGE ],
    [ 7, Moves.BITE ],
    [ 10, Moves.HONE_CLAWS ],
    [ 13, Moves.MAGICAL_LEAF ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.SEED_BOMB ],
    [ 24, Moves.U_TURN ],
    [ 28, Moves.WORRY_SEED ],
    [ 33, Moves.SLASH ],
    [ 38, Moves.ENERGY_BALL ],
    [ 42, Moves.PLAY_ROUGH ],
    [ 46, Moves.LEAF_STORM ],
  ],
  [Species.MEOWSCARADA]: [
    [ EVOLVE_MOVE, Moves.FLOWER_TRICK ],
    [ RELEARN_MOVE, Moves.DOUBLE_TEAM ],
    [ RELEARN_MOVE, Moves.TRICK ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.LEAFAGE ],
    [ 7, Moves.BITE ],
    [ 10, Moves.HONE_CLAWS ],
    [ 13, Moves.MAGICAL_LEAF ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 20, Moves.SEED_BOMB ],
    [ 24, Moves.U_TURN ],
    [ 29, Moves.WORRY_SEED ],
    [ 33, Moves.SLASH ],
    [ 38, Moves.NIGHT_SLASH ],
    [ 42, Moves.ENERGY_BALL ],
    [ 47, Moves.PLAY_ROUGH ],
    [ 52, Moves.KNOCK_OFF ],
    [ 58, Moves.GRASSY_TERRAIN ],
    [ 64, Moves.LEAF_STORM ],
  ],
  [Species.FUECOCO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 7, Moves.ROUND ],
    [ 12, Moves.BITE ],
    [ 15, Moves.INCINERATE ],
    [ 17, Moves.YAWN ],
    [ 21, Moves.SNARL ],
    [ 25, Moves.ROAR ],
    [ 28, Moves.FLAMETHROWER ],
    [ 32, Moves.HYPER_VOICE ],
    [ 36, Moves.FIRE_BLAST ],
  ],
  [Species.CROCALOR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 7, Moves.LICK ],
    [ 10, Moves.ROUND ],
    [ 12, Moves.BITE ],
    [ 15, Moves.YAWN ],
    [ 17, Moves.INCINERATE ],
    [ 24, Moves.SNARL ],
    [ 28, Moves.ROAR ],
    [ 32, Moves.FLAMETHROWER ],
    [ 38, Moves.HYPER_VOICE ],
    [ 42, Moves.WILL_O_WISP ],
    [ 47, Moves.FIRE_BLAST ],
  ],
  [Species.SKELEDIRGE]: [
    [ EVOLVE_MOVE, Moves.TORCH_SONG ],
    [ RELEARN_MOVE, Moves.SING ],
    [ RELEARN_MOVE, Moves.YAWN ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 7, Moves.LICK ],
    [ 10, Moves.ROUND ],
    [ 12, Moves.SCARY_FACE ],
    [ 15, Moves.BITE ],
    [ 17, Moves.INCINERATE ],
    [ 24, Moves.SNARL ],
    [ 28, Moves.ROAR ],
    [ 32, Moves.FLAMETHROWER ],
    [ 38, Moves.SHADOW_BALL ],
    [ 42, Moves.HYPER_VOICE ],
    [ 47, Moves.WILL_O_WISP ],
    [ 47, Moves.HEX ],
    [ 58, Moves.FIRE_BLAST ],
    [ 64, Moves.OVERHEAT ],
  ],
  [Species.QUAXLY]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 7, Moves.WORK_UP ],
    [ 10, Moves.WING_ATTACK ],
    [ 13, Moves.AQUA_JET ],
    [ 17, Moves.DOUBLE_HIT ],
    [ 21, Moves.AQUA_CUTTER ],
    [ 24, Moves.AIR_SLASH ],
    [ 28, Moves.FOCUS_ENERGY ],
    [ 31, Moves.ACROBATICS ],
    [ 35, Moves.LIQUIDATION ],
  ],
  [Species.QUAXWELL]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DOUBLE_HIT ],
    [ 7, Moves.WORK_UP ],
    [ 10, Moves.WING_ATTACK ],
    [ 13, Moves.AQUA_JET ],
    [ 17, Moves.WATER_PULSE ],
    [ 19, Moves.LOW_SWEEP ],
    [ 23, Moves.AQUA_CUTTER ],
    [ 27, Moves.AIR_SLASH ],
    [ 32, Moves.FOCUS_ENERGY ],
    [ 38, Moves.ACROBATICS ],
    [ 43, Moves.LIQUIDATION ],
    [ 48, Moves.FEATHER_DANCE ],
  ],
  [Species.QUAQUAVAL]: [
    [ EVOLVE_MOVE, Moves.AQUA_STEP ],
    [ RELEARN_MOVE, Moves.COUNTER ],
    [ 1, Moves.POUND ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.DOUBLE_HIT ],
    [ 7, Moves.WORK_UP ],
    [ 10, Moves.WING_ATTACK ],
    [ 13, Moves.AQUA_JET ],
    [ 17, Moves.WATER_PULSE ],
    [ 17, Moves.LOW_SWEEP ],
    [ 21, Moves.AQUA_CUTTER ],
    [ 27, Moves.AIR_SLASH ],
    [ 32, Moves.FOCUS_ENERGY ],
    [ 38, Moves.MEGA_KICK ],
    [ 43, Moves.ACROBATICS ],
    [ 47, Moves.LIQUIDATION ],
    [ 52, Moves.FEATHER_DANCE ],
    [ 58, Moves.CLOSE_COMBAT ],
    [ 64, Moves.WAVE_CRASH ],
  ],
  [Species.LECHONK]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.DISARMING_VOICE ],
    [ 8, Moves.ECHOED_VOICE ],
    [ 12, Moves.MUD_SHOT ],
    [ 15, Moves.COVET ],
    [ 17, Moves.DIG ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.YAWN ],
    [ 27, Moves.TAKE_DOWN ],
    [ 30, Moves.WORK_UP ],
    [ 32, Moves.UPROAR ],
    [ 35, Moves.DOUBLE_EDGE ],
  ],
  [Species.OINKOLOGNE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.DISARMING_VOICE ],
    [ 8, Moves.ECHOED_VOICE ],
    [ 12, Moves.MUD_SHOT ],
    [ 15, Moves.COVET ],
    [ 17, Moves.DIG ],
    [ 23, Moves.HEADBUTT ],
    [ 26, Moves.TAKE_DOWN ],
    [ 27, Moves.YAWN ],
    [ 34, Moves.WORK_UP ],
    [ 38, Moves.UPROAR ],
    [ 42, Moves.DOUBLE_EDGE ],
    [ 48, Moves.EARTH_POWER ],
    [ 54, Moves.BELCH ],
  ],
  [Species.TAROUNTULA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 5, Moves.STRUGGLE_BUG ],
    [ 8, Moves.ASSURANCE ],
    [ 11, Moves.FEINT ],
    [ 14, Moves.BUG_BITE ],
    [ 18, Moves.BLOCK ],
    [ 22, Moves.COUNTER ],
    [ 25, Moves.HEADBUTT ],
    [ 29, Moves.STICKY_WEB ],
    [ 33, Moves.GASTRO_ACID ],
    [ 36, Moves.CIRCLE_THROW ],
    [ 40, Moves.THROAT_CHOP ],
    [ 44, Moves.SKITTER_SMACK ],
  ],
  [Species.SPIDOPS]: [
    [ EVOLVE_MOVE, Moves.SILK_TRAP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.STRING_SHOT ],
    [ 5, Moves.STRUGGLE_BUG ],
    [ 8, Moves.ASSURANCE ],
    [ 11, Moves.FEINT ],
    [ 14, Moves.BUG_BITE ],
    [ 19, Moves.BLOCK ],
    [ 24, Moves.COUNTER ],
    [ 28, Moves.HEADBUTT ],
    [ 33, Moves.STICKY_WEB ],
    [ 37, Moves.GASTRO_ACID ],
    [ 41, Moves.CIRCLE_THROW ],
    [ 45, Moves.THROAT_CHOP ],
    [ 49, Moves.SKITTER_SMACK ],
  ],
  [Species.NYMBLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 4, Moves.STRUGGLE_BUG ],
    [ 6, Moves.ASTONISH ],
    [ 9, Moves.ASSURANCE ],
    [ 11, Moves.DOUBLE_KICK ],
    [ 14, Moves.SCREECH ],
    [ 18, Moves.ENDURE ],
    [ 22, Moves.BUG_BITE ],
    [ 26, Moves.FEINT ],
    [ 30, Moves.AGILITY ],
    [ 38, Moves.SUCKER_PUNCH ],
    [ 41, Moves.FIRST_IMPRESSION ],
  ],
  [Species.LOKIX]: [
    [ EVOLVE_MOVE, Moves.LUNGE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.DETECT ],
    [ 4, Moves.STRUGGLE_BUG ],
    [ 6, Moves.ASTONISH ],
    [ 9, Moves.ASSURANCE ],
    [ 11, Moves.DOUBLE_KICK ],
    [ 14, Moves.SCREECH ],
    [ 18, Moves.ENDURE ],
    [ 22, Moves.BUG_BITE ],
    [ 28, Moves.FEINT ],
    [ 32, Moves.AGILITY ],
    [ 36, Moves.THROAT_CHOP ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 44, Moves.FIRST_IMPRESSION ],
    [ 48, Moves.BOUNCE ],
    [ 53, Moves.AXE_KICK ],
  ],
  [Species.PAWMI]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.THUNDER_SHOCK ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 8, Moves.CHARGE ],
    [ 12, Moves.NUZZLE ],
    [ 15, Moves.DIG ],
    [ 19, Moves.BITE ],
    [ 23, Moves.SPARK ],
    [ 27, Moves.THUNDER_WAVE ],
    [ 31, Moves.ENTRAINMENT ],
    [ 35, Moves.SLAM ],
    [ 38, Moves.DISCHARGE ],
    [ 40, Moves.AGILITY ],
    [ 44, Moves.WILD_CHARGE ],
  ],
  [Species.PAWMO]: [
    [ EVOLVE_MOVE, Moves.ARM_THRUST ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.THUNDER_SHOCK ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 8, Moves.CHARGE ],
    [ 12, Moves.NUZZLE ],
    [ 15, Moves.DIG ],
    [ 19, Moves.BITE ],
    [ 23, Moves.SPARK ],
    [ 27, Moves.THUNDER_WAVE ],
    [ 32, Moves.SLAM ],
    [ 38, Moves.ENTRAINMENT ],
    [ 42, Moves.DISCHARGE ],
    [ 46, Moves.AGILITY ],
    [ 52, Moves.WILD_CHARGE ],
  ],
  [Species.PAWMOT]: [
    [ EVOLVE_MOVE, Moves.REVIVAL_BLESSING ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.WILD_CHARGE ],
    [ 3, Moves.THUNDER_SHOCK ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 8, Moves.CHARGE ],
    [ 12, Moves.NUZZLE ],
    [ 15, Moves.DIG ],
    [ 19, Moves.BITE ],
    [ 23, Moves.SPARK ],
    [ 25, Moves.ARM_THRUST ],
    [ 29, Moves.THUNDER_WAVE ],
    [ 33, Moves.SLAM ],
    [ 39, Moves.ENTRAINMENT ],
    [ 44, Moves.CLOSE_COMBAT ],
    [ 49, Moves.DISCHARGE ],
    [ 54, Moves.AGILITY ],
    [ 60, Moves.DOUBLE_SHOCK ],
  ],
  [Species.TANDEMAUS]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 5, Moves.ECHOED_VOICE ],
    [ 8, Moves.HELPING_HAND ],
    [ 11, Moves.SUPER_FANG ],
    [ 14, Moves.DOUBLE_HIT ],
    [ 18, Moves.BULLET_SEED ],
    [ 22, Moves.ENCORE ],
    [ 26, Moves.PLAY_ROUGH ],
    [ 30, Moves.HYPER_VOICE ],
    [ 33, Moves.CHARM ],
    [ 37, Moves.BEAT_UP ],
    [ 41, Moves.COPYCAT ],
    [ 46, Moves.POPULATION_BOMB ],
  ],
  [Species.MAUSHOLD]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.FOLLOW_ME ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 1, Moves.TIDY_UP ],
    [ 5, Moves.ECHOED_VOICE ],
    [ 8, Moves.HELPING_HAND ],
    [ 11, Moves.SUPER_FANG ],
    [ 14, Moves.DOUBLE_HIT ],
    [ 18, Moves.BULLET_SEED ],
    [ 22, Moves.ENCORE ],
    [ 29, Moves.PLAY_ROUGH ],
    [ 33, Moves.HYPER_VOICE ],
    [ 37, Moves.CHARM ],
    [ 41, Moves.BEAT_UP ],
    [ 46, Moves.COPYCAT ],
    [ 53, Moves.POPULATION_BOMB ],
  ],
  [Species.FIDOUGH]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.LICK ],
    [ 6, Moves.TAIL_WHIP ],
    [ 8, Moves.COVET ],
    [ 11, Moves.BITE ],
    [ 15, Moves.BABY_DOLL_EYES ],
    [ 18, Moves.PLAY_ROUGH ],
    [ 22, Moves.WORK_UP ],
    [ 26, Moves.BATON_PASS ],
    [ 30, Moves.ROAR ],
    [ 33, Moves.DOUBLE_EDGE ],
    [ 36, Moves.CHARM ],
    [ 40, Moves.CRUNCH ],
    [ 45, Moves.LAST_RESORT ],
  ],
  [Species.DACHSBUN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.LICK ],
    [ 6, Moves.TAIL_WHIP ],
    [ 8, Moves.COVET ],
    [ 11, Moves.BITE ],
    [ 15, Moves.BABY_DOLL_EYES ],
    [ 18, Moves.PLAY_ROUGH ],
    [ 22, Moves.WORK_UP ],
    [ 29, Moves.BATON_PASS ],
    [ 33, Moves.ROAR ],
    [ 38, Moves.DOUBLE_EDGE ],
    [ 42, Moves.CHARM ],
    [ 47, Moves.CRUNCH ],
    [ 53, Moves.LAST_RESORT ],
  ],
  [Species.SMOLIV]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SWEET_SCENT ],
    [ 5, Moves.ABSORB ],
    [ 7, Moves.GROWTH ],
    [ 10, Moves.RAZOR_LEAF ],
    [ 13, Moves.HELPING_HAND ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 23, Moves.GRASSY_TERRAIN ],
    [ 27, Moves.SEED_BOMB ],
    [ 30, Moves.ENERGY_BALL ],
    [ 34, Moves.LEECH_SEED ],
    [ 38, Moves.TERRAIN_PULSE ],
  ],
  [Species.DOLLIV]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SWEET_SCENT ],
    [ 5, Moves.ABSORB ],
    [ 7, Moves.GROWTH ],
    [ 10, Moves.RAZOR_LEAF ],
    [ 13, Moves.HELPING_HAND ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 23, Moves.GRASSY_TERRAIN ],
    [ 29, Moves.SEED_BOMB ],
    [ 34, Moves.ENERGY_BALL ],
    [ 37, Moves.LEECH_SEED ],
    [ 42, Moves.TERRAIN_PULSE ],
  ],
  [Species.ARBOLIVA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.MIRROR_COAT ],
    [ 5, Moves.ABSORB ],
    [ 7, Moves.GROWTH ],
    [ 10, Moves.RAZOR_LEAF ],
    [ 13, Moves.HELPING_HAND ],
    [ 16, Moves.FLAIL ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 23, Moves.GRASSY_TERRAIN ],
    [ 29, Moves.SEED_BOMB ],
    [ 34, Moves.ENERGY_BALL ],
    [ 39, Moves.LEECH_SEED ],
    [ 46, Moves.TERRAIN_PULSE ],
    [ 52, Moves.PETAL_BLIZZARD ],
    [ 58, Moves.PETAL_DANCE ],
  ],
  [Species.SQUAWKABILLY]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 1, Moves.MIMIC ],
    [ 6, Moves.QUICK_ATTACK ],
    [ 10, Moves.TORMENT ],
    [ 13, Moves.AERIAL_ACE ],
    [ 17, Moves.FURY_ATTACK ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.UPROAR ],
    [ 27, Moves.COPYCAT ],
    [ 30, Moves.FLY ],
    [ 34, Moves.FACADE ],
    [ 38, Moves.SWAGGER ],
    [ 42, Moves.BRAVE_BIRD ],
    [ 47, Moves.ROOST ],
    [ 52, Moves.REVERSAL ],
  ],
  [Species.NACLI]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.ROCK_THROW ],
    [ 7, Moves.MUD_SHOT ],
    [ 10, Moves.SMACK_DOWN ],
    [ 13, Moves.ROCK_POLISH ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.IRON_DEFENSE ],
    [ 25, Moves.RECOVER ],
    [ 30, Moves.ROCK_SLIDE ],
    [ 33, Moves.STEALTH_ROCK ],
    [ 35, Moves.HEAVY_SLAM ],
    [ 40, Moves.EARTHQUAKE ],
    [ 45, Moves.STONE_EDGE ],
  ],
  [Species.NACLSTACK]: [
    [ EVOLVE_MOVE, Moves.SALT_CURE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.ROCK_THROW ],
    [ 7, Moves.MUD_SHOT ],
    [ 10, Moves.SMACK_DOWN ],
    [ 13, Moves.ROCK_POLISH ],
    [ 16, Moves.HEADBUTT ],
    [ 20, Moves.IRON_DEFENSE ],
    [ 30, Moves.RECOVER ],
    [ 34, Moves.ROCK_SLIDE ],
    [ 38, Moves.STEALTH_ROCK ],
    [ 41, Moves.HEAVY_SLAM ],
    [ 45, Moves.EARTHQUAKE ],
    [ 51, Moves.STONE_EDGE ],
  ],
  [Species.GARGANACL]: [
    [ EVOLVE_MOVE, Moves.HAMMER_ARM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.BLOCK ],
    [ 1, Moves.ROCK_BLAST ],
    [ 1, Moves.WIDE_GUARD ],
    [ 5, Moves.ROCK_THROW ],
    [ 7, Moves.MUD_SHOT ],
    [ 10, Moves.ROCK_TOMB ],
    [ 13, Moves.ROCK_POLISH ],
    [ 16, Moves.HEADBUTT ],
    [ 24, Moves.SALT_CURE ],
    [ 30, Moves.RECOVER ],
    [ 34, Moves.ROCK_SLIDE ],
    [ 40, Moves.STEALTH_ROCK ],
    [ 44, Moves.HEAVY_SLAM ],
    [ 49, Moves.EARTHQUAKE ],
    [ 54, Moves.STONE_EDGE ],
    [ 60, Moves.EXPLOSION ],
  ],
  [Species.CHARCADET]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ASTONISH ],
    [ 8, Moves.CLEAR_SMOG ],
    [ 12, Moves.FIRE_SPIN ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 24, Moves.FLAME_CHARGE ],
    [ 28, Moves.INCINERATE ],
    [ 32, Moves.LAVA_PLUME ],
  ],
  [Species.ARMAROUGE]: [
    [ EVOLVE_MOVE, Moves.PSYSHOCK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.WIDE_GUARD ],
    [ 1, Moves.MYSTICAL_FIRE ],
    [ 8, Moves.CLEAR_SMOG ],
    [ 12, Moves.FIRE_SPIN ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 24, Moves.FLAME_CHARGE ],
    [ 28, Moves.INCINERATE ],
    [ 32, Moves.LAVA_PLUME ],
    [ 37, Moves.CALM_MIND ],
    [ 42, Moves.ALLY_SWITCH ],
    [ 48, Moves.FLAMETHROWER ],
    [ 56, Moves.EXPANDING_FORCE ],
    [ 62, Moves.ARMOR_CANNON ],
  ],
  [Species.CERULEDGE]: [
    [ EVOLVE_MOVE, Moves.SHADOW_CLAW ],
    [ RELEARN_MOVE, Moves.NIGHT_SLASH ],
    [ RELEARN_MOVE, Moves.SHADOW_SNEAK ],
    [ RELEARN_MOVE, Moves.QUICK_GUARD ],
    [ RELEARN_MOVE, Moves.SOLAR_BLADE ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ASTONISH ],
    [ 8, Moves.CLEAR_SMOG ],
    [ 12, Moves.FIRE_SPIN ],
    [ 16, Moves.WILL_O_WISP ],
    [ 20, Moves.NIGHT_SHADE ],
    [ 24, Moves.FLAME_CHARGE ],
    [ 28, Moves.INCINERATE ],
    [ 32, Moves.LAVA_PLUME ],
    [ 37, Moves.SWORDS_DANCE ],
    [ 42, Moves.ALLY_SWITCH ],
    [ 48, Moves.BITTER_BLADE ],
    [ 56, Moves.PSYCHO_CUT ],
    [ 62, Moves.FLARE_BLITZ ],
  ],
  [Species.TADBULB]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.MUD_SLAP ],
    [ 7, Moves.THUNDER_SHOCK ],
    [ 11, Moves.WATER_GUN ],
    [ 17, Moves.CHARGE ],
    [ 21, Moves.SPARK ],
    [ 24, Moves.MUD_SHOT ],
    [ 25, Moves.FLAIL ],
    [ 32, Moves.DISCHARGE ],
    [ 36, Moves.WEATHER_BALL ],
    [ 40, Moves.ELECTRIC_TERRAIN ],
    [ 45, Moves.SUCKER_PUNCH ],
    [ 50, Moves.ZAP_CANNON ],
  ],
  [Species.BELLIBOLT]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.SLACK_OFF ],
    [ 7, Moves.THUNDER_SHOCK ],
    [ 11, Moves.WATER_GUN ],
    [ 17, Moves.CHARGE ],
    [ 21, Moves.SPARK ],
    [ 24, Moves.MUD_SHOT ],
    [ 25, Moves.FLAIL ],
    [ 32, Moves.DISCHARGE ],
    [ 36, Moves.WEATHER_BALL ],
    [ 40, Moves.ELECTRIC_TERRAIN ],
    [ 45, Moves.SUCKER_PUNCH ],
    [ 50, Moves.ZAP_CANNON ],
  ],
  [Species.WATTREL]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 7, Moves.QUICK_ATTACK ],
    [ 11, Moves.PLUCK ],
    [ 15, Moves.SPARK ],
    [ 19, Moves.UPROAR ],
    [ 23, Moves.ROOST ],
    [ 27, Moves.DUAL_WINGBEAT ],
    [ 32, Moves.AGILITY ],
    [ 37, Moves.VOLT_SWITCH ],
    [ 43, Moves.DISCHARGE ],
  ],
  [Species.KILOWATTREL]: [
    [ EVOLVE_MOVE, Moves.ELECTRO_BALL ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 7, Moves.QUICK_ATTACK ],
    [ 11, Moves.PLUCK ],
    [ 15, Moves.SPARK ],
    [ 19, Moves.UPROAR ],
    [ 24, Moves.ROOST ],
    [ 30, Moves.DUAL_WINGBEAT ],
    [ 36, Moves.AGILITY ],
    [ 43, Moves.VOLT_SWITCH ],
    [ 48, Moves.DISCHARGE ],
    [ 55, Moves.HURRICANE ],
  ],
  [Species.MASCHIFF]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.SCARY_FACE ],
    [ 4, Moves.LICK ],
    [ 7, Moves.SNARL ],
    [ 10, Moves.HONE_CLAWS ],
    [ 14, Moves.BITE ],
    [ 18, Moves.ROAR ],
    [ 22, Moves.HEADBUTT ],
    [ 26, Moves.PAYBACK ],
    [ 31, Moves.CRUNCH ],
    [ 35, Moves.SWAGGER ],
    [ 39, Moves.REVERSAL ],
    [ 43, Moves.JAW_LOCK ],
    [ 49, Moves.DOUBLE_EDGE ],
  ],
  [Species.MABOSSTIFF]: [
    [ EVOLVE_MOVE, Moves.COMEUPPANCE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.SCARY_FACE ],
    [ 4, Moves.LICK ],
    [ 7, Moves.SNARL ],
    [ 10, Moves.HONE_CLAWS ],
    [ 14, Moves.BITE ],
    [ 18, Moves.ROAR ],
    [ 22, Moves.HEADBUTT ],
    [ 26, Moves.PAYBACK ],
    [ 34, Moves.CRUNCH ],
    [ 39, Moves.SWAGGER ],
    [ 43, Moves.REVERSAL ],
    [ 48, Moves.JAW_LOCK ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 60, Moves.OUTRAGE ],
  ],
  [Species.SHROODLE]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.ACID_SPRAY ],
    [ 8, Moves.BITE ],
    [ 8, Moves.FURY_SWIPES ],
    [ 11, Moves.SWITCHEROO ],
    [ 14, Moves.POISON_FANG ],
    [ 18, Moves.FLATTER ],
    [ 21, Moves.SLASH ],
    [ 25, Moves.U_TURN ],
    [ 29, Moves.POISON_JAB ],
    [ 33, Moves.TAUNT ],
    [ 36, Moves.SUBSTITUTE ],
    [ 40, Moves.KNOCK_OFF ],
    [ 45, Moves.GUNK_SHOT ],
  ],
  [Species.GRAFAIAI]: [
    [ EVOLVE_MOVE, Moves.DOODLE ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 5, Moves.ACID_SPRAY ],
    [ 8, Moves.FURY_SWIPES ],
    [ 11, Moves.SWITCHEROO ],
    [ 14, Moves.POISON_FANG ],
    [ 18, Moves.FLATTER ],
    [ 21, Moves.SLASH ],
    [ 25, Moves.U_TURN ],
    [ 33, Moves.POISON_JAB ],
    [ 37, Moves.TAUNT ],
    [ 40, Moves.SUBSTITUTE ],
    [ 45, Moves.KNOCK_OFF ],
    [ 51, Moves.GUNK_SHOT ],
  ],
  [Species.BRAMBLIN]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.ABSORB ],
    [ 9, Moves.RAPID_SPIN ],
    [ 13, Moves.BULLET_SEED ],
    [ 17, Moves.INFESTATION ],
    [ 21, Moves.HEX ],
    [ 25, Moves.MEGA_DRAIN ],
    [ 29, Moves.DISABLE ],
    [ 35, Moves.PHANTOM_FORCE ],
    [ 40, Moves.GIGA_DRAIN ],
    [ 45, Moves.CURSE ],
    [ 50, Moves.PAIN_SPLIT ],
    [ 55, Moves.POWER_WHIP ],
  ],
  [Species.BRAMBLEGHAST]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.ABSORB ],
    [ 9, Moves.RAPID_SPIN ],
    [ 13, Moves.BULLET_SEED ],
    [ 17, Moves.INFESTATION ],
    [ 21, Moves.HEX ],
    [ 25, Moves.MEGA_DRAIN ],
    [ 29, Moves.DISABLE ],
    [ 35, Moves.PHANTOM_FORCE ],
    [ 40, Moves.GIGA_DRAIN ],
    [ 45, Moves.CURSE ],
    [ 50, Moves.PAIN_SPLIT ],
    [ 55, Moves.POWER_WHIP ],
  ],
  [Species.TOEDSCOOL]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.MUD_SLAP ],
    [ 4, Moves.ABSORB ],
    [ 8, Moves.POISON_POWDER ],
    [ 8, Moves.STUN_SPORE ],
    [ 12, Moves.SUPERSONIC ],
    [ 15, Moves.TACKLE ],
    [ 16, Moves.MEGA_DRAIN ],
    [ 20, Moves.SCREECH ],
    [ 24, Moves.MUD_SHOT ],
    [ 28, Moves.HEX ],
    [ 32, Moves.SEED_BOMB ],
    [ 36, Moves.SPORE ],
    [ 40, Moves.GROWTH ],
    [ 44, Moves.GIGA_DRAIN ],
    [ 48, Moves.EARTH_POWER ],
    [ 52, Moves.POWER_WHIP ],
  ],
  [Species.TOEDSCRUEL]: [
    [ 1, Moves.WRAP ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.REFLECT_TYPE ],
    [ 4, Moves.ABSORB ],
    [ 8, Moves.POISON_POWDER ],
    [ 8, Moves.STUN_SPORE ],
    [ 12, Moves.SUPERSONIC ],
    [ 15, Moves.TACKLE ],
    [ 16, Moves.MEGA_DRAIN ],
    [ 20, Moves.SCREECH ],
    [ 24, Moves.MUD_SHOT ],
    [ 28, Moves.HEX ],
    [ 34, Moves.SEED_BOMB ],
    [ 40, Moves.SPORE ],
    [ 44, Moves.GROWTH ],
    [ 48, Moves.GIGA_DRAIN ],
    [ 54, Moves.EARTH_POWER ],
    [ 58, Moves.POWER_WHIP ],
  ],
  [Species.KLAWF]: [
    [ 1, Moves.ROCK_THROW ],
    [ 6, Moves.HARDEN ],
    [ 9, Moves.ROCK_SMASH ],
    [ 13, Moves.ROCK_TOMB ],
    [ 17, Moves.METAL_CLAW ],
    [ 21, Moves.PROTECT ],
    [ 24, Moves.ROCK_BLAST ],
    [ 29, Moves.X_SCISSOR ],
    [ 33, Moves.SWORDS_DANCE ],
    [ 37, Moves.FLAIL ],
    [ 42, Moves.ROCK_SLIDE ],
    [ 47, Moves.HIGH_HORSEPOWER ],
    [ 51, Moves.IRON_DEFENSE ],
    [ 56, Moves.GUILLOTINE ],
  ],
  [Species.CAPSAKID]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.LEAFAGE ],
    [ 4, Moves.BITE ],
    [ 10, Moves.GROWTH ],
    [ 13, Moves.RAZOR_LEAF ],
    [ 17, Moves.SUNNY_DAY ],
    [ 21, Moves.BULLET_SEED ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.ZEN_HEADBUTT ],
    [ 38, Moves.CRUNCH ],
    [ 44, Moves.SEED_BOMB ],
    [ 48, Moves.SOLAR_BEAM ],
  ],
  [Species.SCOVILLAIN]: [
    [ EVOLVE_MOVE, Moves.FLAMETHROWER ],
    [ EVOLVE_MOVE, Moves.SPICY_EXTRACT ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.LEAFAGE ],
    [ 4, Moves.BITE ],
    [ 10, Moves.GROWTH ],
    [ 13, Moves.RAZOR_LEAF ],
    [ 17, Moves.SUNNY_DAY ],
    [ 21, Moves.BULLET_SEED ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.ZEN_HEADBUTT ],
    [ 33, Moves.WORRY_SEED ],
    [ 38, Moves.CRUNCH ],
    [ 44, Moves.SEED_BOMB ],
    [ 48, Moves.SOLAR_BEAM ],
    [ 48, Moves.OVERHEAT ],
  ],
  [Species.RELLOR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 4, Moves.SAND_ATTACK ],
    [ 7, Moves.STRUGGLE_BUG ],
    [ 11, Moves.ROLLOUT ],
    [ 15, Moves.MUD_SHOT ],
    [ 20, Moves.BUG_BITE ],
    [ 24, Moves.TAKE_DOWN ],
    [ 29, Moves.DIG ],
    [ 35, Moves.LUNGE ],
  ],
  [Species.RABSCA]: [
    [ EVOLVE_MOVE, Moves.REVIVAL_BLESSING ],
    [ RELEARN_MOVE, Moves.SAFEGUARD ],
    [ RELEARN_MOVE, Moves.PSYCH_UP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 4, Moves.SAND_ATTACK ],
    [ 7, Moves.STRUGGLE_BUG ],
    [ 11, Moves.ROLLOUT ],
    [ 15, Moves.PSYBEAM ],
    [ 20, Moves.BUG_BITE ],
    [ 24, Moves.TAKE_DOWN ],
    [ 29, Moves.EXTRASENSORY ],
    [ 35, Moves.LUNGE ],
    [ 40, Moves.POWER_SWAP ],
    [ 40, Moves.GUARD_SWAP ],
    [ 40, Moves.SPEED_SWAP ],
    [ 45, Moves.BUG_BUZZ ],
    [ 50, Moves.PSYCHIC ],
  ],
  [Species.FLITTLE]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 5, Moves.CONFUSION ],
    [ 8, Moves.BABY_DOLL_EYES ],
    [ 11, Moves.DISARMING_VOICE ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 19, Moves.PSYBEAM ],
    [ 24, Moves.PLUCK ],
    [ 29, Moves.AGILITY ],
    [ 34, Moves.UPROAR ],
  ],
  [Species.ESPATHRA]: [
    [ EVOLVE_MOVE, Moves.LUMINA_CRASH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.PECK ],
    [ 1, Moves.DRILL_PECK ],
    [ 1, Moves.FEATHER_DANCE ],
    [ 5, Moves.CONFUSION ],
    [ 8, Moves.BABY_DOLL_EYES ],
    [ 11, Moves.DISARMING_VOICE ],
    [ 15, Moves.QUICK_ATTACK ],
    [ 19, Moves.PSYBEAM ],
    [ 24, Moves.PLUCK ],
    [ 29, Moves.AGILITY ],
    [ 34, Moves.UPROAR ],
    [ 43, Moves.DAZZLING_GLEAM ],
    [ 49, Moves.PSYCHIC ],
    [ 54, Moves.LAST_RESORT ],
  ],
  [Species.TINKATINK]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.FAIRY_WIND ],
    [ 5, Moves.BABY_DOLL_EYES ],
    [ 8, Moves.METAL_CLAW ],
    [ 11, Moves.COVET ],
    [ 14, Moves.ROCK_SMASH ],
    [ 17, Moves.DRAINING_KISS ],
    [ 21, Moves.SWEET_KISS ],
    [ 24, Moves.BRUTAL_SWING ],
    [ 27, Moves.SLAM ],
    [ 31, Moves.FLASH_CANNON ],
    [ 35, Moves.PLAY_ROUGH ],
    [ 39, Moves.FAKE_OUT ],
    [ 43, Moves.FLATTER ],
    [ 47, Moves.SKITTER_SMACK ],
    [ 52, Moves.KNOCK_OFF ],
  ],
  [Species.TINKATUFF]: [
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.FAIRY_WIND ],
    [ 5, Moves.BABY_DOLL_EYES ],
    [ 8, Moves.METAL_CLAW ],
    [ 11, Moves.COVET ],
    [ 14, Moves.ROCK_SMASH ],
    [ 17, Moves.DRAINING_KISS ],
    [ 21, Moves.SWEET_KISS ],
    [ 24, Moves.BRUTAL_SWING ],
    [ 27, Moves.SLAM ],
    [ 31, Moves.FLASH_CANNON ],
    [ 35, Moves.PLAY_ROUGH ],
    [ 39, Moves.FAKE_OUT ],
    [ 43, Moves.FLATTER ],
    [ 47, Moves.SKITTER_SMACK ],
    [ 52, Moves.KNOCK_OFF ],
  ],
  [Species.TINKATON]: [
    [ EVOLVE_MOVE, Moves.GIGATON_HAMMER ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.FAIRY_WIND ],
    [ 5, Moves.BABY_DOLL_EYES ],
    [ 8, Moves.METAL_CLAW ],
    [ 11, Moves.COVET ],
    [ 14, Moves.ROCK_SMASH ],
    [ 17, Moves.DRAINING_KISS ],
    [ 21, Moves.SWEET_KISS ],
    [ 24, Moves.BRUTAL_SWING ],
    [ 27, Moves.SLAM ],
    [ 31, Moves.FLASH_CANNON ],
    [ 35, Moves.PLAY_ROUGH ],
    [ 39, Moves.FAKE_OUT ],
    [ 43, Moves.FLATTER ],
    [ 47, Moves.SKITTER_SMACK ],
    [ 52, Moves.KNOCK_OFF ],
  ],
  [Species.WIGLETT]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.WATER_GUN ],
    [ 4, Moves.MUD_SLAP ],
    [ 8, Moves.WRAP ],
    [ 12, Moves.AQUA_JET ],
    [ 20, Moves.SLAM ],
    [ 20, Moves.WATER_PULSE ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.DIG ],
    [ 32, Moves.SUCKER_PUNCH ],
    [ 36, Moves.THROAT_CHOP ],
    [ 40, Moves.LIQUIDATION ],
  ],
  [Species.WUGTRIO]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.MUD_SLAP ],
    [ 12, Moves.AQUA_JET ],
    [ 16, Moves.SLAM ],
    [ 20, Moves.WATER_PULSE ],
    [ 24, Moves.HEADBUTT ],
    [ 30, Moves.TRIPLE_DIVE ],
    [ 36, Moves.DIG ],
    [ 42, Moves.SUCKER_PUNCH ],
    [ 48, Moves.THROAT_CHOP ],
    [ 54, Moves.LIQUIDATION ],
  ],
  [Species.BOMBIRDIER]: [
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.MEMENTO ],
    [ 1, Moves.HONE_CLAWS ],
    [ 7, Moves.THIEF ],
    [ 11, Moves.ROCK_THROW ],
    [ 16, Moves.WHIRLWIND ],
    [ 20, Moves.PLUCK ],
    [ 24, Moves.TORMENT ],
    [ 29, Moves.ROCK_TOMB ],
    [ 36, Moves.PAYBACK ],
    [ 42, Moves.DUAL_WINGBEAT ],
    [ 47, Moves.ROCK_SLIDE ],
    [ 53, Moves.KNOCK_OFF ],
    [ 60, Moves.PARTING_SHOT ],
  ],
  [Species.FINIZEN]: [
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 7, Moves.ASTONISH ],
    [ 10, Moves.FOCUS_ENERGY ],
    [ 13, Moves.AQUA_JET ],
    [ 17, Moves.DOUBLE_HIT ],
    [ 21, Moves.DIVE ],
    [ 25, Moves.CHARM ],
    [ 29, Moves.ACROBATICS ],
    [ 34, Moves.ENCORE ],
    [ 39, Moves.AQUA_TAIL ],
    [ 44, Moves.MIST ],
    [ 50, Moves.HYDRO_PUMP ],
  ],
  [Species.PALAFIN]: [
    [ EVOLVE_MOVE, Moves.FLIP_TURN ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.JET_PUNCH ],
    [ 7, Moves.ASTONISH ],
    [ 10, Moves.FOCUS_ENERGY ],
    [ 13, Moves.AQUA_JET ],
    [ 17, Moves.DOUBLE_HIT ],
    [ 21, Moves.DIVE ],
    [ 25, Moves.CHARM ],
    [ 29, Moves.ACROBATICS ],
    [ 34, Moves.ENCORE ],
    [ 39, Moves.AQUA_TAIL ],
    [ 44, Moves.MIST ],
    [ 50, Moves.HYDRO_PUMP ],
    [ 55, Moves.FOCUS_PUNCH ],
    [ 61, Moves.WAVE_CRASH ],
  ],
  [Species.VAROOM]: [
    [ 1, Moves.LICK ],
    [ 1, Moves.POISON_GAS ],
    [ 4, Moves.SMOG ],
    [ 7, Moves.TAUNT ],
    [ 10, Moves.ASSURANCE ],
    [ 13, Moves.SLUDGE ],
    [ 17, Moves.GYRO_BALL ],
    [ 21, Moves.HEADBUTT ],
    [ 25, Moves.SCREECH ],
    [ 28, Moves.IRON_HEAD ],
    [ 32, Moves.SWAGGER ],
    [ 36, Moves.POISON_JAB ],
    [ 41, Moves.UPROAR ],
    [ 46, Moves.SPIN_OUT ],
    [ 50, Moves.GUNK_SHOT ],
  ],
  [Species.REVAVROOM]: [
    [ EVOLVE_MOVE, Moves.SHIFT_GEAR ],
    [ 1, Moves.LICK ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.MAGNET_RISE ],
    [ 4, Moves.SMOG ],
    [ 7, Moves.TAUNT ],
    [ 10, Moves.ASSURANCE ],
    [ 13, Moves.SLUDGE ],
    [ 17, Moves.GYRO_BALL ],
    [ 21, Moves.HEADBUTT ],
    [ 25, Moves.SCREECH ],
    [ 28, Moves.IRON_HEAD ],
    [ 32, Moves.SWAGGER ],
    [ 36, Moves.POISON_JAB ],
    [ 46, Moves.UPROAR ],
    [ 52, Moves.SPIN_OUT ],
    [ 58, Moves.GUNK_SHOT ],
  ],
  [Species.CYCLIZAR]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 7, Moves.RAPID_SPIN ],
    [ 11, Moves.TAUNT ],
    [ 14, Moves.BREAKING_SWIPE ],
    [ 18, Moves.QUICK_ATTACK ],
    [ 23, Moves.BITE ],
    [ 27, Moves.U_TURN ],
    [ 31, Moves.SHED_TAIL ],
    [ 36, Moves.DRAGON_CLAW ],
    [ 40, Moves.SHIFT_GEAR ],
    [ 45, Moves.DRAGON_PULSE ],
    [ 51, Moves.DOUBLE_EDGE ],
    [ 57, Moves.DRAGON_RUSH ],
  ],
  [Species.ORTHWORM]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WRAP ],
    [ 1, Moves.HARDEN ],
    [ 7, Moves.MUD_SLAP ],
    [ 12, Moves.SMACK_DOWN ],
    [ 16, Moves.BULLDOZE ],
    [ 21, Moves.IRON_HEAD ],
    [ 26, Moves.TAKE_DOWN ],
    [ 30, Moves.DIG ],
    [ 34, Moves.SANDSTORM ],
    [ 38, Moves.IRON_DEFENSE ],
    [ 43, Moves.IRON_TAIL ],
    [ 47, Moves.EARTHQUAKE ],
    [ 52, Moves.SHED_TAIL ],
  ],
  [Species.GLIMMET]: [
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.SMACK_DOWN ],
    [ 7, Moves.ACID_SPRAY ],
    [ 11, Moves.ANCIENT_POWER ],
    [ 15, Moves.ROCK_POLISH ],
    [ 18, Moves.STEALTH_ROCK ],
    [ 22, Moves.VENOSHOCK ],
    [ 26, Moves.SANDSTORM ],
    [ 29, Moves.SELF_DESTRUCT ],
    [ 33, Moves.ROCK_SLIDE ],
    [ 37, Moves.POWER_GEM ],
    [ 41, Moves.ACID_ARMOR ],
    [ 46, Moves.SLUDGE_WAVE ],
  ],
  [Species.GLIMMORA]: [
    [ EVOLVE_MOVE, Moves.MORTAL_SPIN ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.TOXIC_SPIKES ],
    [ 1, Moves.SMACK_DOWN ],
    [ 1, Moves.SPIKY_SHIELD ],
    [ 7, Moves.ACID_SPRAY ],
    [ 11, Moves.ANCIENT_POWER ],
    [ 15, Moves.ROCK_POLISH ],
    [ 18, Moves.STEALTH_ROCK ],
    [ 22, Moves.VENOSHOCK ],
    [ 26, Moves.SANDSTORM ],
    [ 29, Moves.SELF_DESTRUCT ],
    [ 33, Moves.ROCK_SLIDE ],
    [ 39, Moves.POWER_GEM ],
    [ 44, Moves.ACID_ARMOR ],
    [ 50, Moves.SLUDGE_WAVE ],
  ],
  [Species.GREAVARD]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.LICK ],
    [ 6, Moves.TAIL_WHIP ],
    [ 6, Moves.BITE ],
    [ 9, Moves.ROAR ],
    [ 12, Moves.HEADBUTT ],
    [ 16, Moves.DIG ],
    [ 24, Moves.REST ],
    [ 28, Moves.CRUNCH ],
    [ 32, Moves.PLAY_ROUGH ],
    [ 37, Moves.HELPING_HAND ],
    [ 41, Moves.PHANTOM_FORCE ],
    [ 46, Moves.CHARM ],
    [ 52, Moves.DOUBLE_EDGE ],
  ],
  [Species.HOUNDSTONE]: [
    [ EVOLVE_MOVE, Moves.LAST_RESPECTS ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 3, Moves.LICK ],
    [ 6, Moves.TAIL_WHIP ],
    [ 6, Moves.BITE ],
    [ 9, Moves.ROAR ],
    [ 12, Moves.HEADBUTT ],
    [ 16, Moves.DIG ],
    [ 24, Moves.REST ],
    [ 28, Moves.CRUNCH ],
    [ 36, Moves.PLAY_ROUGH ],
    [ 41, Moves.HELPING_HAND ],
    [ 46, Moves.PHANTOM_FORCE ],
    [ 51, Moves.CHARM ],
    [ 58, Moves.DOUBLE_EDGE ],
  ],
  [Species.FLAMIGO]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.COPYCAT ],
    [ 5, Moves.DOUBLE_KICK ],
    [ 9, Moves.DETECT ],
    [ 12, Moves.WING_ATTACK ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 18, Moves.LOW_KICK ],
    [ 21, Moves.FEINT ],
    [ 27, Moves.PAYBACK ],
    [ 31, Moves.ROOST ],
    [ 35, Moves.AIR_SLASH ],
    [ 39, Moves.MEGA_KICK ],
    [ 44, Moves.WIDE_GUARD ],
    [ 48, Moves.THROAT_CHOP ],
    [ 54, Moves.BRAVE_BIRD ],
  ],
  [Species.CETODDLE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POWDER_SNOW ],
    [ 6, Moves.GROWL ],
    [ 9, Moves.ECHOED_VOICE ],
    [ 12, Moves.ICE_SHARD ],
    [ 15, Moves.REST ],
    [ 19, Moves.TAKE_DOWN ],
    [ 25, Moves.FLAIL ],
    [ 27, Moves.AVALANCHE ],
    [ 31, Moves.BOUNCE ],
    [ 36, Moves.BODY_SLAM ],
    [ 40, Moves.AMNESIA ],
    [ 44, Moves.ICE_SPINNER ],
    [ 49, Moves.DOUBLE_EDGE ],
    [ 53, Moves.BLIZZARD ],
  ],
  [Species.CETITAN]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POWDER_SNOW ],
    [ 6, Moves.GROWL ],
    [ 9, Moves.ECHOED_VOICE ],
    [ 12, Moves.ICE_SHARD ],
    [ 15, Moves.REST ],
    [ 19, Moves.TAKE_DOWN ],
    [ 25, Moves.FLAIL ],
    [ 27, Moves.AVALANCHE ],
    [ 31, Moves.BOUNCE ],
    [ 36, Moves.BODY_SLAM ],
    [ 40, Moves.AMNESIA ],
    [ 44, Moves.ICE_SPINNER ],
    [ 49, Moves.DOUBLE_EDGE ],
    [ 53, Moves.BLIZZARD ],
  ],
  [Species.VELUZA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.AQUA_JET ],
    [ 7, Moves.PLUCK ],
    [ 11, Moves.WATER_PULSE ],
    [ 15, Moves.FOCUS_ENERGY ],
    [ 20, Moves.SLASH ],
    [ 25, Moves.AQUA_CUTTER ],
    [ 30, Moves.FILLET_AWAY ],
    [ 35, Moves.NIGHT_SLASH ],
    [ 40, Moves.PSYCHO_CUT ],
    [ 45, Moves.LIQUIDATION ],
    [ 50, Moves.CRUNCH ],
    [ 55, Moves.FINAL_GAMBIT ],
  ],
  [Species.DONDOZO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.WATER_GUN ],
    [ 5, Moves.TICKLE ],
    [ 10, Moves.FLAIL ],
    [ 15, Moves.REST ],
    [ 15, Moves.SLEEP_TALK ],
    [ 20, Moves.DIVE ],
    [ 25, Moves.NOBLE_ROAR ],
    [ 30, Moves.SOAK ],
    [ 35, Moves.BODY_SLAM ],
    [ 40, Moves.AQUA_TAIL ],
    [ 45, Moves.RAIN_DANCE ],
    [ 50, Moves.ORDER_UP ],
    [ 55, Moves.HEAVY_SLAM ],
    [ 60, Moves.DOUBLE_EDGE ],
    [ 65, Moves.WAVE_CRASH ],
  ],
  [Species.TATSUGIRI]: [
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SPLASH ],
    [ 6, Moves.HARDEN ],
    [ 12, Moves.HELPING_HAND ],
    [ 17, Moves.WATER_PULSE ],
    [ 23, Moves.SOAK ],
    [ 28, Moves.TAUNT ],
    [ 34, Moves.MEMENTO ],
    [ 39, Moves.MUDDY_WATER ],
    [ 43, Moves.NASTY_PLOT ],
    [ 47, Moves.MIRROR_COAT ],
    [ 52, Moves.DRAGON_PULSE ],
  ],
  [Species.ANNIHILAPE]: [
    [ EVOLVE_MOVE, Moves.SHADOW_PUNCH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.COUNTER ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.FLING ],
    [ 5, Moves.FURY_SWIPES ],
    [ 8, Moves.LOW_KICK ],
    [ 12, Moves.SEISMIC_TOSS ],
    [ 17, Moves.SWAGGER ],
    [ 22, Moves.CROSS_CHOP ],
    [ 26, Moves.ASSURANCE ],
    [ 30, Moves.THRASH ],
    [ 35, Moves.RAGE_FIST ],
    [ 39, Moves.CLOSE_COMBAT ],
    [ 44, Moves.SCREECH ],
    [ 48, Moves.STOMPING_TANTRUM ],
    [ 53, Moves.OUTRAGE ],
    [ 57, Moves.FINAL_GAMBIT ],
  ],
  [Species.CLODSIRE]: [
    [ EVOLVE_MOVE, Moves.AMNESIA ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.POISON_STING ],
    [ 4, Moves.TOXIC_SPIKES ],
    [ 8, Moves.MUD_SHOT ],
    [ 12, Moves.POISON_TAIL ],
    [ 16, Moves.SLAM ],
    [ 21, Moves.YAWN ],
    [ 24, Moves.POISON_JAB ],
    [ 30, Moves.SLUDGE_WAVE ],
    [ 36, Moves.MEGAHORN ],
    [ 40, Moves.TOXIC ],
    [ 48, Moves.EARTHQUAKE ],
  ],
  [Species.FARIGIRAF]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.POWER_SWAP ],
    [ 1, Moves.GUARD_SWAP ],
    [ 5, Moves.CONFUSION ],
    [ 10, Moves.ASSURANCE ],
    [ 14, Moves.STOMP ],
    [ 19, Moves.PSYBEAM ],
    [ 23, Moves.AGILITY ],
    [ 28, Moves.DOUBLE_HIT ],
    [ 32, Moves.TWIN_BEAM ],
    [ 37, Moves.CRUNCH ],
    [ 41, Moves.BATON_PASS ],
    [ 46, Moves.NASTY_PLOT ],
    [ 50, Moves.PSYCHIC ],
  ],
  [Species.DUDUNSPARCE]: [
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.FLAIL ],
    [ 4, Moves.MUD_SLAP ],
    [ 8, Moves.ROLLOUT ],
    [ 12, Moves.GLARE ],
    [ 16, Moves.SCREECH ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 24, Moves.DRILL_RUN ],
    [ 28, Moves.YAWN ],
    [ 32, Moves.HYPER_DRILL ],
    [ 36, Moves.ROOST ],
    [ 40, Moves.DRAGON_RUSH ],
    [ 44, Moves.COIL ],
    [ 48, Moves.DOUBLE_EDGE ],
    [ 52, Moves.ENDEAVOR ],
    [ 56, Moves.HURRICANE ],
    [ 62, Moves.BOOMBURST ],
  ],
  [Species.KINGAMBIT]: [
    [ EVOLVE_MOVE, Moves.KOWTOW_CLEAVE ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.METAL_BURST ],
    [ 15, Moves.TORMENT ],
    [ 20, Moves.SCARY_FACE ],
    [ 25, Moves.ASSURANCE ],
    [ 30, Moves.METAL_SOUND ],
    [ 35, Moves.SLASH ],
    [ 40, Moves.NIGHT_SLASH ],
    [ 45, Moves.IRON_DEFENSE ],
    [ 50, Moves.RETALIATE ],
    [ 57, Moves.IRON_HEAD ],
    [ 64, Moves.SWORDS_DANCE ],
    [ 71, Moves.GUILLOTINE ],
  ],
  [Species.GREAT_TUSK]: [
    [ 1, Moves.HORN_ATTACK ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.SUNNY_DAY ],
    [ 7, Moves.BULLDOZE ],
    [ 14, Moves.TAUNT ],
    [ 21, Moves.RAPID_SPIN ],
    [ 28, Moves.BRICK_BREAK ],
    [ 35, Moves.STOMPING_TANTRUM ],
    [ 42, Moves.KNOCK_OFF ],
    [ 49, Moves.EARTHQUAKE ],
    [ 56, Moves.GIGA_IMPACT ],
    [ 63, Moves.CLOSE_COMBAT ],
    [ 70, Moves.ENDEAVOR ],
    [ 77, Moves.MEGAHORN ],
    [ 84, Moves.HEAD_SMASH ],
    [ 91, Moves.HEADLONG_RUSH ],
  ],
  [Species.SCREAM_TAIL]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ 1, Moves.POUND ],
    [ 1, Moves.SING ],
    [ 1, Moves.DISABLE ],
    [ 7, Moves.HOWL ],
    [ 14, Moves.NOBLE_ROAR ],
    [ 21, Moves.BITE ],
    [ 28, Moves.BODY_SLAM ],
    [ 35, Moves.REST ],
    [ 42, Moves.PLAY_ROUGH ],
    [ 49, Moves.HYPER_VOICE ],
    [ 56, Moves.PSYCHIC_FANGS ],
    [ 63, Moves.CRUNCH ],
    [ 70, Moves.WISH ],
    [ 77, Moves.GYRO_BALL ],
    [ 84, Moves.PERISH_SONG ],
    [ 91, Moves.BOOMBURST ],
  ],
  [Species.BRUTE_BONNET]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.ASTONISH ],
    [ 7, Moves.STUN_SPORE ],
    [ 14, Moves.MEGA_DRAIN ],
    [ 21, Moves.SYNTHESIS ],
    [ 28, Moves.CLEAR_SMOG ],
    [ 35, Moves.PAYBACK ],
    [ 42, Moves.THRASH ],
    [ 49, Moves.GIGA_DRAIN ],
    [ 56, Moves.SUCKER_PUNCH ],
    [ 63, Moves.SPORE ],
    [ 70, Moves.INGRAIN ],
    [ 77, Moves.RAGE_POWDER ],
    [ 91, Moves.SOLAR_BEAM ],
  ],
  [Species.FLUTTER_MANE]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ 1, Moves.CONFUSE_RAY ],
    [ 1, Moves.SPITE ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.PSYBEAM ], //Custom, moved from 7 to 1
    [ 14, Moves.MEAN_LOOK ],
    [ 21, Moves.MEMENTO ],
    [ 28, Moves.WISH ],
    [ 35, Moves.DAZZLING_GLEAM ],
    [ 42, Moves.SHADOW_BALL ],
    [ 49, Moves.MYSTICAL_FIRE ],
    [ 56, Moves.POWER_GEM ],
    [ 63, Moves.PSYSHOCK ],
    [ 70, Moves.PHANTOM_FORCE ],
    [ 77, Moves.PAIN_SPLIT ],
    [ 84, Moves.MOONBLAST ],
    [ 91, Moves.PERISH_SONG ],
  ],
  [Species.SLITHER_WING]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ 1, Moves.GUST ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.BUG_BITE ],
    [ 7, Moves.POISON_POWDER ],
    [ 7, Moves.STUN_SPORE ],
    [ 14, Moves.FLAME_CHARGE ],
    [ 21, Moves.STOMP ],
    [ 28, Moves.LOW_SWEEP ],
    [ 35, Moves.MORNING_SUN ],
    [ 42, Moves.LUNGE ],
    [ 49, Moves.SUPERPOWER ],
    [ 56, Moves.BULK_UP ],
    [ 63, Moves.DUAL_WINGBEAT ],
    [ 70, Moves.FIRST_IMPRESSION ],
    [ 77, Moves.WHIRLWIND ],
    [ 84, Moves.LEECH_LIFE ],
    [ 91, Moves.THRASH ],
  ],
  [Species.SANDY_SHOCKS]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ 1, Moves.SUPERSONIC ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 7, Moves.SPARK ],
    [ 14, Moves.BULLDOZE ],
    [ 21, Moves.CHARGE_BEAM ],
    [ 28, Moves.TRI_ATTACK ],
    [ 35, Moves.SCREECH ],
    [ 42, Moves.HEAVY_SLAM ],
    [ 49, Moves.METAL_SOUND ],
    [ 56, Moves.DISCHARGE ],
    [ 63, Moves.EARTH_POWER ],
    [ 70, Moves.MIRROR_COAT ],
    [ 77, Moves.GRAVITY ],
    [ 84, Moves.ZAP_CANNON ],
    [ 91, Moves.MAGNETIC_FLUX ],
  ],
  [Species.IRON_TREADS]: [
    [ 1, Moves.HORN_ATTACK ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.ROLLOUT ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 7, Moves.BULLDOZE ],
    [ 21, Moves.RAPID_SPIN ],
    [ 28, Moves.IRON_HEAD ],
    [ 35, Moves.STOMPING_TANTRUM ],
    [ 42, Moves.KNOCK_OFF ],
    [ 49, Moves.EARTHQUAKE ],
    [ 56, Moves.HEAVY_SLAM ],
    [ 63, Moves.WILD_CHARGE ],
    [ 70, Moves.ENDEAVOR ],
    [ 77, Moves.MEGAHORN ],
    [ 84, Moves.GIGA_IMPACT ],
    [ 91, Moves.STEEL_ROLLER ],
  ],
  [Species.IRON_BUNDLE]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.PRESENT ],
    [ 1, Moves.WATER_GUN ], //Custom
    [ 7, Moves.POWDER_SNOW ],
    [ 14, Moves.WHIRLPOOL ],
    [ 21, Moves.TAKE_DOWN ],
    [ 28, Moves.DRILL_PECK ],
    [ 35, Moves.HELPING_HAND ],
    [ 42, Moves.FREEZE_DRY ],
    [ 49, Moves.FLIP_TURN ],
    [ 56, Moves.ICE_BEAM ],
    [ 63, Moves.AGILITY ],
    [ 70, Moves.SNOWSCAPE ],
    [ 77, Moves.HYDRO_PUMP ],
    [ 84, Moves.AURORA_VEIL ],
    [ 91, Moves.BLIZZARD ],
  ],
  [Species.IRON_HANDS]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.ARM_THRUST ],
    [ 7, Moves.FAKE_OUT ],
    [ 14, Moves.WHIRLWIND ],
    [ 21, Moves.THUNDER_PUNCH ],
    [ 28, Moves.SLAM ],
    [ 35, Moves.FORCE_PALM ],
    [ 42, Moves.SEISMIC_TOSS ],
    [ 49, Moves.CHARGE ],
    [ 56, Moves.WILD_CHARGE ],
    [ 63, Moves.CLOSE_COMBAT ],
    [ 70, Moves.DETECT ],
    [ 77, Moves.HEAVY_SLAM ],
    [ 84, Moves.BELLY_DRUM ],
    [ 91, Moves.FOCUS_PUNCH ],
  ],
  [Species.IRON_JUGULIS]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.AIR_CUTTER ],
    [ 1, Moves.WORK_UP ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 7, Moves.ROAR ],
    [ 14, Moves.ASSURANCE ],
    [ 21, Moves.DRAGON_BREATH ],
    [ 28, Moves.SNARL ],
    [ 35, Moves.CRUNCH ],
    [ 42, Moves.HYPER_VOICE ],
    [ 56, Moves.AIR_SLASH ],
    [ 63, Moves.KNOCK_OFF ],
    [ 70, Moves.DARK_PULSE ],
    [ 77, Moves.OUTRAGE ],
    [ 84, Moves.DRAGON_PULSE ],
    [ 91, Moves.HYPER_BEAM ],
  ],
  [Species.IRON_MOTH]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.GUST ],
    [ 1, Moves.WHIRLWIND ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ACID_SPRAY ],
    [ 7, Moves.STRUGGLE_BUG ],
    [ 14, Moves.FIRE_SPIN ],
    [ 21, Moves.TAKE_DOWN ],
    [ 28, Moves.LUNGE ],
    [ 35, Moves.SCREECH ],
    [ 42, Moves.DISCHARGE ],
    [ 49, Moves.SLUDGE_WAVE ],
    [ 56, Moves.FIERY_DANCE ],
    [ 63, Moves.METAL_SOUND ],
    [ 70, Moves.MORNING_SUN ],
    [ 77, Moves.HURRICANE ],
    [ 84, Moves.BUG_BUZZ ],
    [ 91, Moves.OVERHEAT ],
  ],
  [Species.IRON_THORNS]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.IRON_DEFENSE ],
    [ 1, Moves.THUNDER_FANG ],
    [ 1, Moves.ICE_FANG ],
    [ 1, Moves.FIRE_FANG ],
    [ 7, Moves.SCREECH ],
    [ 21, Moves.ROCK_TOMB ],
    [ 28, Moves.BITE ],
    [ 35, Moves.CHARGE ],
    [ 42, Moves.ROCK_SLIDE ],
    [ 49, Moves.SANDSTORM ],
    [ 56, Moves.WILD_CHARGE ],
    [ 63, Moves.PIN_MISSILE ],
    [ 70, Moves.EARTHQUAKE ],
    [ 77, Moves.STEALTH_ROCK ],
    [ 84, Moves.STONE_EDGE ],
    [ 91, Moves.GIGA_IMPACT ],
  ],
  [Species.FRIGIBAX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 6, Moves.ICY_WIND ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 18, Moves.FOCUS_ENERGY ],
    [ 24, Moves.BITE ],
    [ 29, Moves.ICE_FANG ],
    [ 32, Moves.DRAGON_CLAW ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.ICE_BEAM ],
    [ 44, Moves.CRUNCH ],
    [ 48, Moves.ICICLE_CRASH ],
  ],
  [Species.ARCTIBAX]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 6, Moves.ICY_WIND ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 18, Moves.FOCUS_ENERGY ],
    [ 24, Moves.BITE ],
    [ 29, Moves.ICE_FANG ],
    [ 40, Moves.TAKE_DOWN ],
    [ 45, Moves.ICE_BEAM ],
    [ 50, Moves.CRUNCH ],
    [ 55, Moves.ICICLE_CRASH ],
  ],
  [Species.BAXCALIBUR]: [
    [ EVOLVE_MOVE, Moves.GLAIVE_RUSH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ICE_SHARD ],
    [ 1, Moves.DRAGON_TAIL ],
    [ 1, Moves.BREAKING_SWIPE ],
    [ 1, Moves.SNOWSCAPE ],
    [ 6, Moves.ICY_WIND ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 18, Moves.FOCUS_ENERGY ],
    [ 24, Moves.BITE ],
    [ 29, Moves.ICE_FANG ],
    [ 35, Moves.DRAGON_CLAW ],
    [ 42, Moves.TAKE_DOWN ],
    [ 48, Moves.ICE_BEAM ],
    [ 55, Moves.CRUNCH ],
    [ 62, Moves.ICICLE_CRASH ],
  ],
  [Species.GIMMIGHOUL]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ASTONISH ],
  ],
  [Species.GHOLDENGO]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.ASTONISH ],
    [ 7, Moves.NIGHT_SHADE ],
    [ 14, Moves.CONFUSE_RAY ],
    [ 21, Moves.SUBSTITUTE ],
    [ 28, Moves.METAL_SOUND ],
    [ 35, Moves.SHADOW_BALL ],
    [ 42, Moves.RECOVER ],
    [ 49, Moves.POWER_GEM ],
    [ 56, Moves.MAKE_IT_RAIN ],
    [ 63, Moves.NASTY_PLOT ],
    [ 70, Moves.MEMENTO ],
  ],
  [Species.WO_CHIEN]: [
    [ 1, Moves.ABSORB ],
    [ 1, Moves.SPITE ],
    [ 1, Moves.MEAN_LOOK ],
    [ 5, Moves.TICKLE ],
    [ 10, Moves.PAYBACK ],
    [ 15, Moves.POISON_POWDER ],
    [ 15, Moves.STUN_SPORE ],
    [ 20, Moves.MEGA_DRAIN ],
    [ 25, Moves.LEECH_SEED ],
    [ 30, Moves.GROWTH ],
    [ 35, Moves.INGRAIN ],
    [ 40, Moves.DARK_PULSE ],
    [ 45, Moves.GIGA_DRAIN ],
    [ 50, Moves.RUINATION ],
    [ 55, Moves.FOUL_PLAY ],
    [ 60, Moves.POWER_WHIP ],
    [ 65, Moves.GRASSY_TERRAIN ],
    [ 70, Moves.KNOCK_OFF ],
    [ 75, Moves.LEAF_STORM ],
  ],
  [Species.CHIEN_PAO]: [
    [ 1, Moves.SPITE ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.MEAN_LOOK ],
    [ 5, Moves.ICY_WIND ],
    [ 10, Moves.PAYBACK ],
    [ 15, Moves.MIST ],
    [ 15, Moves.HAZE ],
    [ 20, Moves.ICE_SHARD ],
    [ 25, Moves.SWORDS_DANCE ],
    [ 30, Moves.SNOWSCAPE ],
    [ 35, Moves.NIGHT_SLASH ],
    [ 40, Moves.DARK_PULSE ],
    [ 45, Moves.ICICLE_CRASH ],
    [ 50, Moves.RUINATION ],
    [ 55, Moves.SUCKER_PUNCH ],
    [ 60, Moves.SACRED_SWORD ],
    [ 65, Moves.RECOVER ],
    [ 70, Moves.THROAT_CHOP ],
    [ 75, Moves.SHEER_COLD ],
  ],
  [Species.TING_LU]: [
    [ 1, Moves.SPITE ],
    [ 1, Moves.MEAN_LOOK ],
    [ 1, Moves.SAND_TOMB ],
    [ 5, Moves.SPIKES ],
    [ 10, Moves.PAYBACK ],
    [ 15, Moves.STOMP ],
    [ 20, Moves.BULLDOZE ],
    [ 25, Moves.WHIRLWIND ],
    [ 30, Moves.TAUNT ],
    [ 35, Moves.THRASH ],
    [ 40, Moves.DARK_PULSE ],
    [ 45, Moves.STOMPING_TANTRUM ],
    [ 50, Moves.RUINATION ],
    [ 55, Moves.THROAT_CHOP ],
    [ 60, Moves.ROCK_SLIDE ],
    [ 65, Moves.MEMENTO ],
    [ 70, Moves.EARTHQUAKE ],
    [ 75, Moves.FISSURE ],
  ],
  [Species.CHI_YU]: [
    [ 1, Moves.EMBER ],
    [ 1, Moves.SPITE ],
    [ 1, Moves.MEAN_LOOK ],
    [ 5, Moves.FLAME_WHEEL ],
    [ 10, Moves.PAYBACK ],
    [ 15, Moves.WILL_O_WISP ],
    [ 20, Moves.FLAME_CHARGE ],
    [ 25, Moves.INCINERATE ],
    [ 30, Moves.CONFUSE_RAY ],
    [ 35, Moves.NASTY_PLOT ],
    [ 40, Moves.DARK_PULSE ],
    [ 45, Moves.LAVA_PLUME ],
    [ 50, Moves.RUINATION ],
    [ 55, Moves.BOUNCE ],
    [ 60, Moves.SWAGGER ],
    [ 65, Moves.INFERNO ],
    [ 70, Moves.MEMENTO ],
    [ 75, Moves.OVERHEAT ],
  ],
  [Species.ROARING_MOON]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ RELEARN_MOVE, Moves.JAW_LOCK ],
    [ RELEARN_MOVE, Moves.BREAKING_SWIPE ],
    [ RELEARN_MOVE, Moves.SCALE_SHOT ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 7, Moves.INCINERATE ],
    [ 14, Moves.HEADBUTT ],
    [ 21, Moves.SCARY_FACE ],
    [ 28, Moves.DRAGON_CLAW ],
    [ 35, Moves.ZEN_HEADBUTT ],
    [ 42, Moves.FLAMETHROWER ],
    [ 49, Moves.NIGHT_SLASH ],
    [ 56, Moves.DRAGON_DANCE ],
    [ 63, Moves.DRAGON_RUSH ],
    [ 70, Moves.FLY ],
    [ 77, Moves.THROAT_CHOP ],
    [ 84, Moves.ROOST ],
    [ 91, Moves.DOUBLE_EDGE ],
  ],
  [Species.IRON_VALIANT]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.DISABLE ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.FURY_CUTTER ],
    [ 1, Moves.SHADOW_SNEAK ],
    [ 7, Moves.HYPNOSIS ],
    [ 14, Moves.FEINT ],
    [ 21, Moves.FUTURE_SIGHT ],
    [ 28, Moves.DAZZLING_GLEAM ],
    [ 35, Moves.PSYCHO_CUT ],
    [ 42, Moves.NIGHT_SLASH ],
    [ 49, Moves.LEAF_BLADE ],
    [ 56, Moves.MOONBLAST ],
    [ 63, Moves.CLOSE_COMBAT ],
    [ 70, Moves.KNOCK_OFF ],
    [ 77, Moves.DESTINY_BOND ],
    [ 84, Moves.WIDE_GUARD ],
    [ 84, Moves.QUICK_GUARD ],
    [ 91, Moves.SPIRIT_BREAK ],
  ],
  [Species.KORAIDON]: [
    [ 1, Moves.SUNNY_DAY ],
    [ 1, Moves.BREAKING_SWIPE ],
    [ 7, Moves.ROCK_SMASH ],
    [ 14, Moves.ANCIENT_POWER ],
    [ 21, Moves.DRAIN_PUNCH ],
    [ 28, Moves.BRICK_BREAK ],
    [ 35, Moves.AGILITY ],
    [ 42, Moves.DRAGON_CLAW ],
    [ 49, Moves.FLAMETHROWER ],
    [ 56, Moves.COLLISION_COURSE ],
    [ 63, Moves.SCREECH ],
    [ 70, Moves.COUNTER ],
    [ 77, Moves.OUTRAGE ],
    [ 84, Moves.CLOSE_COMBAT ],
    [ 91, Moves.FLARE_BLITZ ],
    [ 98, Moves.GIGA_IMPACT ],
  ],
  [Species.MIRAIDON]: [
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 7, Moves.SHOCK_WAVE ],
    [ 14, Moves.CHARGE ],
    [ 21, Moves.PARABOLIC_CHARGE ],
    [ 28, Moves.DISCHARGE ],
    [ 35, Moves.AGILITY ],
    [ 42, Moves.DRAGON_PULSE ],
    [ 56, Moves.ELECTRO_DRIFT ],
    [ 63, Moves.METAL_SOUND ],
    [ 70, Moves.MIRROR_COAT ],
    [ 77, Moves.OUTRAGE ],
    [ 84, Moves.THUNDER ],
    [ 91, Moves.OVERHEAT ],
    [ 98, Moves.HYPER_BEAM ],
  ],
  [Species.WALKING_WAKE]: [
    [ RELEARN_MOVE, Moves.SUNNY_DAY ],
    [ RELEARN_MOVE, Moves.HONE_CLAWS ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROAR ],
    [ 1, Moves.TWISTER ],
    [ 1, Moves.AQUA_JET ],
    [ 7, Moves.BITE ],
    [ 14, Moves.WATER_PULSE ],
    [ 21, Moves.NOBLE_ROAR ],
    [ 28, Moves.DRAGON_BREATH ],
    [ 35, Moves.BREAKING_SWIPE ],
    [ 42, Moves.DRAGON_RUSH ],
    [ 56, Moves.HYDRO_STEAM ],
    [ 63, Moves.DRAGON_PULSE ],
    [ 70, Moves.OUTRAGE ],
    [ 77, Moves.FLAMETHROWER ],
    [ 84, Moves.HYDRO_PUMP ],
  ],
  [Species.IRON_LEAVES]: [
    [ RELEARN_MOVE, Moves.ELECTRIC_TERRAIN ],
    [ RELEARN_MOVE, Moves.QUASH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.WORK_UP ],
    [ 7, Moves.MAGICAL_LEAF ],
    [ 14, Moves.RETALIATE ],
    [ 21, Moves.QUICK_GUARD ],
    [ 28, Moves.NIGHT_SLASH ],
    [ 35, Moves.SWORDS_DANCE ],
    [ 42, Moves.SACRED_SWORD ],
    [ 49, Moves.LEAF_BLADE ],
    [ 56, Moves.PSYBLADE ],
    [ 63, Moves.CLOSE_COMBAT ],
    [ 70, Moves.IMPRISON ],
    [ 77, Moves.MEGAHORN ],
    [ 84, Moves.ALLY_SWITCH ],
    [ 91, Moves.SOLAR_BLADE ],
  ],
  [Species.DIPPLIN]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_HIT ],
    [ RELEARN_MOVE, Moves.DRAGON_CHEER ], // Custom
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.DRAGON_TAIL ],
    [ 8, Moves.GROWTH ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 16, Moves.PROTECT ],
    [ 20, Moves.BULLET_SEED ],
    [ 28, Moves.SYRUP_BOMB ],
    [ 32, Moves.DRAGON_PULSE ],
    [ 36, Moves.RECOVER ],
    [ 40, Moves.ENERGY_BALL ],
    [ 44, Moves.SUBSTITUTE ],
  ],
  [Species.POLTCHAGEIST]: [
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.ASTONISH ],
    [ 5, Moves.ABSORB ], //Custom, Moved from Level 6 to 5
    [ 12, Moves.LIFE_DEW ],
    [ 18, Moves.FOUL_PLAY ],
    [ 24, Moves.MEGA_DRAIN ],
    [ 30, Moves.HEX ],
    [ 36, Moves.RAGE_POWDER ],
    [ 42, Moves.GIGA_DRAIN ],
    [ 48, Moves.SHADOW_BALL ],
    [ 54, Moves.MEMENTO ],
    [ 60, Moves.LEAF_STORM ],
  ],
  [Species.SINISTCHA]: [
    [ EVOLVE_MOVE, Moves.MATCHA_GOTCHA ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.ASTONISH ],
    [ 6, Moves.ABSORB ],
    [ 12, Moves.LIFE_DEW ],
    [ 18, Moves.FOUL_PLAY ],
    [ 24, Moves.MEGA_DRAIN ],
    [ 30, Moves.HEX ],
    [ 36, Moves.RAGE_POWDER ],
    [ 42, Moves.STRENGTH_SAP ],
    [ 48, Moves.SHADOW_BALL ],
    [ 54, Moves.MEMENTO ],
    [ 60, Moves.LEAF_STORM ],
  ],
  [Species.OKIDOGI]: [
    [ 1, Moves.BITE ],
    [ 1, Moves.LOW_KICK ],
    [ 1, Moves.BULK_UP ],
    [ 8, Moves.HOWL ],
    [ 16, Moves.POISON_FANG ],
    [ 24, Moves.FORCE_PALM ],
    [ 32, Moves.COUNTER ],
    [ 40, Moves.POISON_JAB ],
    [ 48, Moves.BRUTAL_SWING ],
    [ 56, Moves.CRUNCH ],
    [ 64, Moves.SUPERPOWER ],
    [ 72, Moves.GIGA_IMPACT ],
  ],
  [Species.MUNKIDORI]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.FLATTER ],
    [ 8, Moves.HELPING_HAND ],
    [ 16, Moves.PSYBEAM ],
    [ 24, Moves.CLEAR_SMOG ],
    [ 32, Moves.POISON_JAB ],
    [ 40, Moves.PSYCHIC ],
    [ 48, Moves.SLUDGE_WAVE ],
    [ 56, Moves.NASTY_PLOT ],
    [ 64, Moves.FUTURE_SIGHT ],
    [ 72, Moves.PARTING_SHOT ],
  ],
  [Species.FEZANDIPITI]: [
    [ 1, Moves.DOUBLE_KICK ],
    [ 1, Moves.PECK ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.DISARMING_VOICE ],
    [ 8, Moves.QUICK_ATTACK ],
    [ 16, Moves.ATTRACT ],
    [ 24, Moves.WING_ATTACK ],
    [ 32, Moves.CROSS_POISON ],
    [ 40, Moves.TAIL_SLAP ],
    [ 48, Moves.BEAT_UP ],
    [ 56, Moves.SWAGGER ],
    [ 56, Moves.FLATTER ],
    [ 64, Moves.ROOST ],
    [ 72, Moves.MOONBLAST ],
  ],
  [Species.OGERPON]: [
    [ RELEARN_MOVE, Moves.DOUBLE_KICK ],
    [ RELEARN_MOVE, Moves.COUNTER ],
    [ RELEARN_MOVE, Moves.RETALIATE ],
    [ RELEARN_MOVE, Moves.HORN_LEECH ],
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FOLLOW_ME ],
    [ 6, Moves.FOCUS_ENERGY ],
    [ 12, Moves.GROWTH ],
    [ 18, Moves.SLAM ],
    [ 24, Moves.LOW_SWEEP ],
    [ 30, Moves.IVY_CUDGEL ],
    [ 36, Moves.THROAT_CHOP ],
    [ 42, Moves.SYNTHESIS ],
    [ 48, Moves.SPIKY_SHIELD ],
    [ 54, Moves.POWER_WHIP ],
    [ 60, Moves.SUPERPOWER ],
    [ 66, Moves.WOOD_HAMMER ],
  ],
  [Species.ARCHALUDON]: [
    [ EVOLVE_MOVE, Moves.ELECTRO_SHOT ],
    [ 1, Moves.LEER ],
    [ 1, Moves.METAL_CLAW ],
    [ 6, Moves.ROCK_SMASH ],
    [ 12, Moves.HONE_CLAWS ],
    [ 18, Moves.METAL_SOUND ],
    [ 24, Moves.BREAKING_SWIPE ],
    [ 30, Moves.DRAGON_TAIL ],
    [ 36, Moves.IRON_DEFENSE ],
    [ 42, Moves.FOCUS_ENERGY ],
    [ 48, Moves.DRAGON_CLAW ],
    [ 54, Moves.FLASH_CANNON ],
    [ 60, Moves.METAL_BURST ],
    [ 66, Moves.HYPER_BEAM ],
  ],
  [Species.HYDRAPPLE]: [
    [ EVOLVE_MOVE, Moves.FICKLE_BEAM ],
    [ RELEARN_MOVE, Moves.YAWN ],
    [ RELEARN_MOVE, Moves.DOUBLE_HIT ],
    [ RELEARN_MOVE, Moves.INFESTATION ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.SWEET_SCENT ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.DRAGON_TAIL ],
    [ 8, Moves.GROWTH ],
    [ 12, Moves.DRAGON_BREATH ],
    [ 16, Moves.PROTECT ],
    [ 20, Moves.BULLET_SEED ],
    [ 28, Moves.SYRUP_BOMB ],
    [ 32, Moves.DRAGON_PULSE ],
    [ 36, Moves.RECOVER ],
    [ 40, Moves.ENERGY_BALL ],
    [ 44, Moves.SUBSTITUTE ],
    [ 54, Moves.POWER_WHIP ],
  ],
  [Species.GOUGING_FIRE]: [
    [ RELEARN_MOVE, Moves.DOUBLE_KICK ],
    [ RELEARN_MOVE, Moves.ANCIENT_POWER ],
    [ RELEARN_MOVE, Moves.NOBLE_ROAR ],
    [ 1, Moves.STOMP ],
    [ 1, Moves.LEER ],
    [ 1, Moves.INCINERATE ],
    [ 1, Moves.SUNNY_DAY ],
    [ 7, Moves.FIRE_FANG ],
    [ 14, Moves.HOWL ],
    [ 21, Moves.BITE ],
    [ 28, Moves.DRAGON_CLAW ],
    [ 35, Moves.CRUSH_CLAW ],
    [ 42, Moves.MORNING_SUN ],
    [ 49, Moves.BURNING_BULWARK ],
    [ 56, Moves.DRAGON_RUSH ],
    [ 63, Moves.FIRE_BLAST ],
    [ 70, Moves.LAVA_PLUME ],
    [ 77, Moves.OUTRAGE ],
    [ 84, Moves.FLARE_BLITZ ],
    [ 91, Moves.RAGING_FURY ],
  ],
  [Species.RAGING_BOLT]: [
    [ RELEARN_MOVE, Moves.ANCIENT_POWER ],
    [ 1, Moves.TWISTER ],
    [ 1, Moves.SUNNY_DAY ],
    [ 1, Moves.SHOCK_WAVE ],
    [ 1, Moves.STOMP ],
    [ 7, Moves.CHARGE ],
    [ 14, Moves.DRAGON_BREATH ],
    [ 21, Moves.ELECTRIC_TERRAIN ],
    [ 28, Moves.DISCHARGE ],
    [ 35, Moves.DRAGON_TAIL ],
    [ 42, Moves.CALM_MIND ],
    [ 49, Moves.THUNDERCLAP ],
    [ 56, Moves.DRAGON_HAMMER ],
    [ 63, Moves.RISING_VOLTAGE ],
    [ 70, Moves.DRAGON_PULSE ],
    [ 77, Moves.ZAP_CANNON ],
    [ 84, Moves.BODY_PRESS ],
    [ 91, Moves.THUNDER ],
  ],
  [Species.IRON_BOULDER]: [
    [ 1, Moves.HORN_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 7, Moves.QUICK_ATTACK ],
    [ 14, Moves.SLASH ],
    [ 21, Moves.AGILITY ],
    [ 28, Moves.PSYCHO_CUT ],
    [ 35, Moves.COUNTER ],
    [ 42, Moves.ROCK_TOMB ],
    [ 49, Moves.SACRED_SWORD ],
    [ 56, Moves.MIGHTY_CLEAVE ],
    [ 63, Moves.SWORDS_DANCE ],
    [ 70, Moves.MEGAHORN ],
    [ 77, Moves.QUICK_GUARD ],
    [ 84, Moves.STONE_EDGE ],
    [ 91, Moves.GIGA_IMPACT ],
  ],
  [Species.IRON_CROWN]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.ELECTRIC_TERRAIN ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.METAL_CLAW ],
    [ 7, Moves.SMART_STRIKE ],
    [ 14, Moves.SLASH ],
    [ 21, Moves.IRON_DEFENSE ],
    [ 28, Moves.PSYSHOCK ],
    [ 35, Moves.PSYCHO_CUT ],
    [ 42, Moves.FLASH_CANNON ],
    [ 49, Moves.SACRED_SWORD ],
    [ 56, Moves.TACHYON_CUTTER ],
    [ 63, Moves.FUTURE_SIGHT ],
    [ 70, Moves.VOLT_SWITCH ],
    [ 77, Moves.QUICK_GUARD ],
    [ 84, Moves.METAL_BURST ],
    [ 91, Moves.HYPER_BEAM ],
  ],
  [Species.TERAPAGOS]: [
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.RAPID_SPIN ],
    [ 10, Moves.ANCIENT_POWER ],
    [ 20, Moves.HEADBUTT ],
    [ 30, Moves.PROTECT ],
    [ 40, Moves.EARTH_POWER ],
    [ 50, Moves.HEAVY_SLAM ],
    [ 60, Moves.TERA_STARSTORM ],
    [ 70, Moves.DOUBLE_EDGE ],
    [ 80, Moves.ROCK_POLISH ],
    [ 90, Moves.GYRO_BALL ],
  ],
  [Species.PECHARUNT]: [
    [ RELEARN_MOVE, Moves.DEFENSE_CURL ],
    [ RELEARN_MOVE, Moves.ROLLOUT ],
    [ RELEARN_MOVE, Moves.MEAN_LOOK ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.MEMENTO ],
    [ 1, Moves.ASTONISH ],
    [ 8, Moves.WITHDRAW ],
    [ 16, Moves.DESTINY_BOND ],
    [ 24, Moves.FAKE_TEARS ],
    [ 32, Moves.PARTING_SHOT ],
    [ 40, Moves.SHADOW_BALL ],
    [ 48, Moves.MALIGNANT_CHAIN ],
    [ 56, Moves.TOXIC ],
    [ 64, Moves.NASTY_PLOT ],
    [ 72, Moves.RECOVER ],
  ],
  [Species.ALOLA_RATTATA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 4, Moves.QUICK_ATTACK ],
    [ 7, Moves.FOCUS_ENERGY ],
    [ 10, Moves.BITE ],
    [ 13, Moves.PURSUIT ],
    [ 16, Moves.HYPER_FANG ],
    [ 19, Moves.ASSURANCE ],
    [ 22, Moves.CRUNCH ],
    [ 25, Moves.SUCKER_PUNCH ],
    [ 28, Moves.SUPER_FANG ],
    [ 31, Moves.DOUBLE_EDGE ],
    [ 34, Moves.ENDEAVOR ],
  ],
  [Species.ALOLA_RATICATE]: [
    [ EVOLVE_MOVE, Moves.SCARY_FACE ],
    [ 1, Moves.SWORDS_DANCE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 10, Moves.BITE ],
    [ 13, Moves.PURSUIT ],
    [ 16, Moves.HYPER_FANG ],
    [ 19, Moves.ASSURANCE ],
    [ 24, Moves.CRUNCH ],
    [ 29, Moves.SUCKER_PUNCH ],
    [ 34, Moves.SUPER_FANG ],
    [ 39, Moves.DOUBLE_EDGE ],
    [ 44, Moves.ENDEAVOR ],
  ],
  [Species.ALOLA_RAICHU]: [
    [ EVOLVE_MOVE, Moves.PSYCHIC ],
    [ EVOLVE_MOVE, Moves.ZIPPY_ZAP ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.THUNDER_SHOCK ],
    [ 1, Moves.THUNDER_WAVE ],
    [ 1, Moves.THUNDER ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.DOUBLE_TEAM ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.SWEET_KISS ],
    [ 1, Moves.CHARM ],
    [ 1, Moves.SPARK ],
    [ 1, Moves.IRON_TAIL ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.DISCHARGE ],
    [ 1, Moves.ELECTRO_BALL ],
    [ 1, Moves.PLAY_NICE ],
    [ 1, Moves.NUZZLE ],
    [ 5, Moves.THUNDERBOLT ],
    [ 50, Moves.PIKA_PAPOW ],
  ],
  [Species.ALOLA_SANDSHREW]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 3, Moves.MIST ],
    [ 6, Moves.POWDER_SNOW ],
    [ 9, Moves.ROLLOUT ],
    [ 12, Moves.FURY_CUTTER ],
    [ 15, Moves.RAPID_SPIN ],
    [ 18, Moves.METAL_CLAW ],
    [ 21, Moves.SWIFT ],
    [ 24, Moves.FURY_SWIPES ],
    [ 27, Moves.IRON_DEFENSE ],
    [ 30, Moves.SLASH ],
    [ 33, Moves.IRON_HEAD ],
    [ 36, Moves.GYRO_BALL ],
    [ 39, Moves.SWORDS_DANCE ],
    [ 42, Moves.SNOWSCAPE ],
    [ 45, Moves.BLIZZARD ],
  ],
  [Species.ALOLA_SANDSLASH]: [
    [ EVOLVE_MOVE, Moves.ICICLE_SPEAR ],
    [ RELEARN_MOVE, Moves.SCRATCH ],
    [ RELEARN_MOVE, Moves.MIST ],
    [ RELEARN_MOVE, Moves.BLIZZARD ],
    [ RELEARN_MOVE, Moves.DEFENSE_CURL ],
    [ RELEARN_MOVE, Moves.SWIFT ],
    [ RELEARN_MOVE, Moves.FURY_SWIPES ],
    [ RELEARN_MOVE, Moves.POWDER_SNOW ],
    [ RELEARN_MOVE, Moves.ROLLOUT ],
    [ RELEARN_MOVE, Moves.FURY_CUTTER ],
    [ RELEARN_MOVE, Moves.RAPID_SPIN ],
    [ RELEARN_MOVE, Moves.IRON_DEFENSE ],
    [ RELEARN_MOVE, Moves.GYRO_BALL ],
    [ RELEARN_MOVE, Moves.METAL_BURST ],
    [ RELEARN_MOVE, Moves.IRON_HEAD ],
    [ RELEARN_MOVE, Moves.SNOWSCAPE ],
    [ 1, Moves.ICICLE_CRASH ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.SLASH ],
    [ 1, Moves.SWORDS_DANCE ],
    [ 1, Moves.ICE_BALL ],
  ],
  [Species.ALOLA_VULPIX]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.POWDER_SNOW ],
    [ 4, Moves.DISABLE ],
    [ 8, Moves.ICE_SHARD ],
    [ 12, Moves.SPITE ],
    [ 16, Moves.ICY_WIND ],
    [ 20, Moves.CONFUSE_RAY ],
    [ 24, Moves.AURORA_BEAM ],
    [ 28, Moves.EXTRASENSORY ],
    [ 32, Moves.ICE_BEAM ],
    [ 36, Moves.IMPRISON ],
    [ 40, Moves.MIST ],
    [ 44, Moves.AURORA_VEIL ],
    [ 48, Moves.FREEZE_DRY ],
    [ 52, Moves.BLIZZARD ],
  ],
  [Species.ALOLA_NINETALES]: [
    [ EVOLVE_MOVE, Moves.DAZZLING_GLEAM ],
    [ RELEARN_MOVE, Moves.DISABLE ],
    [ RELEARN_MOVE, Moves.MIST ],
    [ RELEARN_MOVE, Moves.ICE_BEAM ],
    [ RELEARN_MOVE, Moves.AURORA_BEAM ],
    [ RELEARN_MOVE, Moves.CONFUSE_RAY ],
    [ RELEARN_MOVE, Moves.SPITE ],
    [ RELEARN_MOVE, Moves.POWDER_SNOW ],
    [ RELEARN_MOVE, Moves.EXTRASENSORY ],
    [ RELEARN_MOVE, Moves.NASTY_PLOT ],
    [ RELEARN_MOVE, Moves.ICE_SHARD ],
    [ RELEARN_MOVE, Moves.FREEZE_DRY ],
    [ RELEARN_MOVE, Moves.AURORA_VEIL ],
    [ 1, Moves.ICY_WIND ],
    [ 1, Moves.IMPRISON ],
    [ 1, Moves.BLIZZARD ],
    [ 1, Moves.TAIL_WHIP ],
  ],
  [Species.ALOLA_DIGLETT]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.METAL_CLAW ],
    [ 4, Moves.GROWL ],
    [ 8, Moves.ASTONISH ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.BULLDOZE ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 24, Moves.IRON_HEAD ],
    [ 28, Moves.SANDSTORM ],
    [ 32, Moves.DIG ],
    [ 36, Moves.EARTH_POWER ],
    [ 40, Moves.EARTHQUAKE ],
    [ 44, Moves.FISSURE ],
  ],
  [Species.ALOLA_DUGTRIO]: [
    [ EVOLVE_MOVE, Moves.SAND_TOMB ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.TRI_ATTACK ],
    [ 1, Moves.METAL_CLAW ],
    [ 1, Moves.ASTONISH ],
    [ 1, Moves.NIGHT_SLASH ],
    [ 1, Moves.ROTOTILLER ],
    [ 12, Moves.MUD_SLAP ],
    [ 16, Moves.BULLDOZE ],
    [ 20, Moves.SUCKER_PUNCH ],
    [ 24, Moves.IRON_HEAD ],
    [ 30, Moves.SANDSTORM ],
    [ 36, Moves.DIG ],
    [ 42, Moves.EARTH_POWER ],
    [ 48, Moves.EARTHQUAKE ],
    [ 54, Moves.FISSURE ],
  ],
  [Species.ALOLA_MEOWTH]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 4, Moves.FEINT ],
    [ 8, Moves.SCRATCH ],
    [ 12, Moves.PAY_DAY ],
    [ 16, Moves.BITE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.ASSURANCE ],
    [ 29, Moves.FURY_SWIPES ],
    [ 32, Moves.SCREECH ],
    [ 36, Moves.NIGHT_SLASH ],
    [ 40, Moves.NASTY_PLOT ],
    [ 44, Moves.PLAY_ROUGH ],
  ],
  [Species.ALOLA_PERSIAN]: [
    [ EVOLVE_MOVE, Moves.POWER_GEM ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.SWITCHEROO ],
    [ 1, Moves.QUASH ],
    [ 12, Moves.PAY_DAY ],
    [ 16, Moves.BITE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.ASSURANCE ],
    [ 31, Moves.FURY_SWIPES ],
    [ 36, Moves.SCREECH ],
    [ 42, Moves.NIGHT_SLASH ],
    [ 48, Moves.NASTY_PLOT ],
    [ 54, Moves.PLAY_ROUGH ],
  ],
  [Species.ALOLA_GEODUDE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 4, Moves.CHARGE ],
    [ 6, Moves.ROCK_POLISH ],
    [ 10, Moves.ROLLOUT ],
    [ 12, Moves.SPARK ],
    [ 16, Moves.ROCK_THROW ],
    [ 18, Moves.SMACK_DOWN ],
    [ 22, Moves.THUNDER_PUNCH ],
    [ 24, Moves.SELF_DESTRUCT ],
    [ 28, Moves.STEALTH_ROCK ],
    [ 30, Moves.ROCK_BLAST ],
    [ 34, Moves.DISCHARGE ],
    [ 36, Moves.EXPLOSION ],
    [ 40, Moves.DOUBLE_EDGE ],
    [ 42, Moves.STONE_EDGE ],
  ],
  [Species.ALOLA_GRAVELER]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.ROCK_POLISH ],
    [ 10, Moves.ROLLOUT ],
    [ 12, Moves.SPARK ],
    [ 16, Moves.ROCK_THROW ],
    [ 18, Moves.SMACK_DOWN ],
    [ 22, Moves.THUNDER_PUNCH ],
    [ 24, Moves.SELF_DESTRUCT ],
    [ 30, Moves.STEALTH_ROCK ],
    [ 34, Moves.ROCK_BLAST ],
    [ 40, Moves.DISCHARGE ],
    [ 44, Moves.EXPLOSION ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 54, Moves.STONE_EDGE ],
  ],
  [Species.ALOLA_GOLEM]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DEFENSE_CURL ],
    [ 1, Moves.CHARGE ],
    [ 1, Moves.ROCK_POLISH ],
    [ 1, Moves.HEAVY_SLAM ],
    [ 12, Moves.SPARK ],
    [ 16, Moves.ROCK_THROW ],
    [ 18, Moves.SMACK_DOWN ],
    [ 22, Moves.THUNDER_PUNCH ],
    [ 24, Moves.SELF_DESTRUCT ],
    [ 30, Moves.STEALTH_ROCK ],
    [ 34, Moves.ROCK_BLAST ],
    [ 40, Moves.DISCHARGE ],
    [ 44, Moves.EXPLOSION ],
    [ 50, Moves.DOUBLE_EDGE ],
    [ 54, Moves.STONE_EDGE ],
  ],
  [Species.ALOLA_GRIMER]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.POISON_GAS ],
    [ 4, Moves.HARDEN ],
    [ 7, Moves.BITE ],
    [ 12, Moves.DISABLE ],
    [ 15, Moves.ACID_SPRAY ],
    [ 18, Moves.POISON_FANG ],
    [ 21, Moves.MINIMIZE ],
    [ 26, Moves.TOXIC ],
    [ 29, Moves.KNOCK_OFF ],
    [ 32, Moves.CRUNCH ],
    [ 37, Moves.SCREECH ],
    [ 40, Moves.GUNK_SHOT ],
    [ 43, Moves.ACID_ARMOR ],
    [ 46, Moves.BELCH ],
    [ 48, Moves.MEMENTO ],
  ],
  [Species.ALOLA_MUK]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.BITE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POISON_GAS ],
    [ 12, Moves.DISABLE ],
    [ 15, Moves.ACID_SPRAY ],
    [ 18, Moves.POISON_FANG ],
    [ 21, Moves.MINIMIZE ],
    [ 26, Moves.TOXIC ],
    [ 29, Moves.KNOCK_OFF ],
    [ 32, Moves.CRUNCH ],
    [ 37, Moves.SCREECH ],
    [ 40, Moves.GUNK_SHOT ],
    [ 46, Moves.ACID_ARMOR ],
    [ 52, Moves.BELCH ],
    [ 57, Moves.MEMENTO ],
  ],
  [Species.ALOLA_EXEGGUTOR]: [
    [ EVOLVE_MOVE, Moves.DRAGON_HAMMER ],
    [ RELEARN_MOVE, Moves.GROWTH ],
    [ 1, Moves.BARRAGE ],
    [ 1, Moves.SEED_BOMB ],
    [ 1, Moves.PSYSHOCK ],
    [ 1, Moves.WOOD_HAMMER ],
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.SYNTHESIS ],
    [ 1, Moves.BULLET_SEED ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.EXTRASENSORY ],
    [ 1, Moves.UPROAR ],
    [ 1, Moves.WORRY_SEED ],
    [ 1, Moves.SOLAR_BEAM ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.HYPNOSIS ],
    [ 1, Moves.REFLECT ],
    [ 1, Moves.LEECH_SEED ],
  ],
  [Species.ALOLA_MAROWAK]: [
    [ EVOLVE_MOVE, Moves.SHADOW_BONE ],
    [ 1, Moves.BONE_CLUB ],
    [ 1, Moves.HEADBUTT ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.FIRE_SPIN ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.FALSE_SWIPE ],
    [ 1, Moves.RETALIATE ],
    [ 12, Moves.FLAME_WHEEL ],
    [ 16, Moves.HEX ],
    [ 20, Moves.FLING ],
    [ 24, Moves.STOMPING_TANTRUM ],
    [ 31, Moves.BONE_RUSH ],
    [ 36, Moves.WILL_O_WISP ],
    [ 42, Moves.ENDEAVOR ],
    [ 48, Moves.BONEMERANG ],
    [ 54, Moves.THRASH ],
    [ 60, Moves.FLARE_BLITZ ],
  ],
  [Species.ETERNAL_FLOETTE]: [
    [ 1, Moves.VINE_WHIP ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.FAIRY_WIND ],
    [ 10, Moves.LUCKY_CHANT ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.WISH ],
    [ 25, Moves.MAGICAL_LEAF ],
    [ 27, Moves.GRASSY_TERRAIN ],
    [ 33, Moves.PETAL_BLIZZARD ],
    [ 38, Moves.AROMATHERAPY ],
    [ 43, Moves.MISTY_TERRAIN ],
    [ 46, Moves.MOONBLAST ],
    [ 50, Moves.LIGHT_OF_RUIN ],
    [ 51, Moves.PETAL_DANCE ],
    [ 58, Moves.SOLAR_BEAM ],
  ],
  [Species.GALAR_MEOWTH]: [
    [ 1, Moves.GROWL ],
    [ 1, Moves.FAKE_OUT ],
    [ 4, Moves.HONE_CLAWS ],
    [ 8, Moves.SCRATCH ],
    [ 12, Moves.PAY_DAY ],
    [ 16, Moves.METAL_CLAW ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.SWAGGER ],
    [ 29, Moves.FURY_SWIPES ],
    [ 32, Moves.SCREECH ],
    [ 36, Moves.SLASH ],
    [ 40, Moves.METAL_SOUND ],
    [ 44, Moves.THRASH ],
  ],
  [Species.GALAR_PONYTA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 5, Moves.TAIL_WHIP ],
    [ 10, Moves.CONFUSION ],
    [ 15, Moves.FAIRY_WIND ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.PSYBEAM ],
    [ 30, Moves.STOMP ],
    [ 35, Moves.HEAL_PULSE ],
    [ 41, Moves.TAKE_DOWN ],
    [ 45, Moves.DAZZLING_GLEAM ],
    [ 50, Moves.PSYCHIC ],
    [ 55, Moves.HEALING_WISH ],
  ],
  [Species.GALAR_RAPIDASH]: [
    [ EVOLVE_MOVE, Moves.PSYCHO_CUT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.CONFUSION ],
    [ 1, Moves.QUICK_ATTACK ],
    [ 1, Moves.MEGAHORN ],
    [ 15, Moves.FAIRY_WIND ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.PSYBEAM ],
    [ 30, Moves.STOMP ],
    [ 35, Moves.HEAL_PULSE ],
    [ 43, Moves.TAKE_DOWN ],
    [ 49, Moves.DAZZLING_GLEAM ],
    [ 56, Moves.PSYCHIC ],
    [ 63, Moves.HEALING_WISH ],
  ],
  [Species.GALAR_SLOWPOKE]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CURSE ],
    [ 3, Moves.GROWL ],
    [ 6, Moves.ACID ],
    [ 9, Moves.YAWN ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SLACK_OFF ],
    [ 36, Moves.PSYCHIC ],
    [ 39, Moves.PSYCH_UP ],
    [ 42, Moves.RAIN_DANCE ],
    [ 45, Moves.HEAL_PULSE ],
  ],
  [Species.GALAR_SLOWBRO]: [
    [ EVOLVE_MOVE, Moves.SHELL_SIDE_ARM ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ACID ],
    [ 1, Moves.WITHDRAW ],
    [ 1, Moves.CURSE ],
    [ 9, Moves.YAWN ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SLACK_OFF ],
    [ 36, Moves.PSYCHIC ],
    [ 39, Moves.PSYCH_UP ],
    [ 42, Moves.RAIN_DANCE ],
    [ 45, Moves.HEAL_PULSE ],
  ],
  [Species.GALAR_FARFETCHD]: [
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.PECK ],
    [ 5, Moves.LEER ],
    [ 10, Moves.FURY_CUTTER ],
    [ 15, Moves.ROCK_SMASH ],
    [ 20, Moves.BRUTAL_SWING ],
    [ 25, Moves.DETECT ],
    [ 30, Moves.KNOCK_OFF ],
    [ 35, Moves.DEFOG ],
    [ 40, Moves.BRICK_BREAK ],
    [ 45, Moves.SWORDS_DANCE ],
    [ 50, Moves.SLAM ],
    [ 55, Moves.LEAF_BLADE ],
    [ 60, Moves.FINAL_GAMBIT ],
    [ 65, Moves.BRAVE_BIRD ],
  ],
  [Species.GALAR_WEEZING]: [
    [ EVOLVE_MOVE, Moves.DOUBLE_HIT ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.HAZE ],
    [ 1, Moves.SMOG ],
    [ 1, Moves.POISON_GAS ],
    [ 1, Moves.HEAT_WAVE ],
    [ 1, Moves.DEFOG ],
    [ 1, Moves.AROMATIC_MIST ],
    [ 1, Moves.STRANGE_STEAM ],
    [ 12, Moves.CLEAR_SMOG ],
    [ 16, Moves.ASSURANCE ],
    [ 20, Moves.SLUDGE ],
    [ 24, Moves.FAIRY_WIND ],
    [ 28, Moves.SELF_DESTRUCT ],
    [ 32, Moves.SLUDGE_BOMB ],
    [ 38, Moves.TOXIC ],
    [ 44, Moves.BELCH ],
    [ 50, Moves.EXPLOSION ],
    [ 56, Moves.MEMENTO ],
    [ 62, Moves.DESTINY_BOND ],
    [ 68, Moves.MISTY_TERRAIN ],
  ],
  [Species.GALAR_MR_MIME]: [
    [ 1, Moves.POUND ],
    [ 1, Moves.MIMIC ],
    [ 1, Moves.LIGHT_SCREEN ],
    [ 1, Moves.REFLECT ],
    [ 1, Moves.PROTECT ],
    [ 1, Moves.SAFEGUARD ],
    [ 1, Moves.BATON_PASS ],
    [ 1, Moves.ENCORE ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.ROLE_PLAY ],
    [ 1, Moves.RECYCLE ],
    [ 1, Moves.COPYCAT ],
    [ 1, Moves.ICE_SHARD ],
    [ 1, Moves.MISTY_TERRAIN ],
    [ 1, Moves.DAZZLING_GLEAM ],
    [ 12, Moves.CONFUSION ],
    [ 16, Moves.ALLY_SWITCH ],
    [ 20, Moves.ICY_WIND ],
    [ 24, Moves.DOUBLE_KICK ],
    [ 28, Moves.PSYBEAM ],
    [ 32, Moves.HYPNOSIS ],
    [ 36, Moves.MIRROR_COAT ],
    [ 40, Moves.SUCKER_PUNCH ],
    [ 44, Moves.FREEZE_DRY ],
    [ 48, Moves.PSYCHIC ],
    [ 52, Moves.TEETER_DANCE ],
  ],
  [Species.GALAR_ARTICUNO]: [
    [ 1, Moves.GUST ],
    [ 5, Moves.CONFUSION ],
    [ 10, Moves.REFLECT ],
    [ 15, Moves.HYPNOSIS ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.ANCIENT_POWER ],
    [ 30, Moves.TAILWIND ],
    [ 35, Moves.PSYCHO_CUT ],
    [ 40, Moves.RECOVER ],
    [ 45, Moves.FREEZING_GLARE ],
    [ 50, Moves.DREAM_EATER ],
    [ 55, Moves.HURRICANE ],
    [ 60, Moves.DOUBLE_TEAM ],
    [ 65, Moves.FUTURE_SIGHT ],
    [ 70, Moves.TRICK_ROOM ],
  ],
  [Species.GALAR_ZAPDOS]: [
    [ 1, Moves.PECK ],
    [ 1, Moves.FOCUS_ENERGY ],
    [ 5, Moves.ROCK_SMASH ],
    [ 10, Moves.LIGHT_SCREEN ],
    [ 15, Moves.PLUCK ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.ANCIENT_POWER ],
    [ 30, Moves.BRICK_BREAK ],
    [ 35, Moves.DRILL_PECK ],
    [ 40, Moves.QUICK_GUARD ],
    [ 45, Moves.THUNDEROUS_KICK ],
    [ 50, Moves.BULK_UP ],
    [ 55, Moves.COUNTER ],
    [ 60, Moves.DETECT ],
    [ 65, Moves.CLOSE_COMBAT ],
    [ 70, Moves.REVERSAL ],
  ],
  [Species.GALAR_MOLTRES]: [
    [ 1, Moves.GUST ],
    [ 1, Moves.LEER ],
    [ 5, Moves.PAYBACK ],
    [ 10, Moves.SAFEGUARD ],
    [ 15, Moves.WING_ATTACK ],
    [ 20, Moves.AGILITY ],
    [ 25, Moves.ANCIENT_POWER ],
    [ 30, Moves.SUCKER_PUNCH ],
    [ 35, Moves.AIR_SLASH ],
    [ 40, Moves.AFTER_YOU ],
    [ 45, Moves.FIERY_WRATH ],
    [ 50, Moves.NASTY_PLOT ],
    [ 55, Moves.HURRICANE ],
    [ 60, Moves.ENDURE ],
    [ 65, Moves.MEMENTO ],
    [ 70, Moves.SKY_ATTACK ],
  ],
  [Species.GALAR_SLOWKING]: [
    [ EVOLVE_MOVE, Moves.EERIE_SPELL ],
    [ RELEARN_MOVE, Moves.FUTURE_SIGHT ],
    [ RELEARN_MOVE, Moves.CHILLY_RECEPTION ],
    [ RELEARN_MOVE, Moves.TOXIC ],
    [ 1, Moves.POWER_GEM ],
    [ 1, Moves.NASTY_PLOT ],
    [ 1, Moves.SWAGGER ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CURSE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.ACID ],
    [ 9, Moves.YAWN ],
    [ 12, Moves.CONFUSION ],
    [ 15, Moves.DISABLE ],
    [ 18, Moves.WATER_PULSE ],
    [ 21, Moves.HEADBUTT ],
    [ 24, Moves.ZEN_HEADBUTT ],
    [ 27, Moves.AMNESIA ],
    [ 30, Moves.SURF ],
    [ 33, Moves.SLACK_OFF ],
    [ 36, Moves.PSYCHIC ],
    [ 39, Moves.PSYCH_UP ],
    [ 42, Moves.RAIN_DANCE ],
    [ 45, Moves.HEAL_PULSE ],
  ],
  [Species.GALAR_CORSOLA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 5, Moves.ASTONISH ],
    [ 10, Moves.DISABLE ],
    [ 15, Moves.SPITE ],
    [ 20, Moves.ANCIENT_POWER ],
    [ 25, Moves.HEX ],
    [ 30, Moves.CURSE ],
    [ 35, Moves.STRENGTH_SAP ],
    [ 40, Moves.POWER_GEM ],
    [ 45, Moves.NIGHT_SHADE ],
    [ 50, Moves.GRUDGE ],
    [ 55, Moves.MIRROR_COAT ],
  ],
  [Species.GALAR_ZIGZAGOON]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.LEER ],
    [ 3, Moves.SAND_ATTACK ],
    [ 6, Moves.LICK ],
    [ 9, Moves.SNARL ],
    [ 12, Moves.HEADBUTT ],
    [ 15, Moves.BABY_DOLL_EYES ],
    [ 18, Moves.PIN_MISSILE ],
    [ 21, Moves.REST ],
    [ 24, Moves.TAKE_DOWN ],
    [ 27, Moves.SCARY_FACE ],
    [ 30, Moves.COUNTER ],
    [ 33, Moves.TAUNT ],
    [ 36, Moves.DOUBLE_EDGE ],
  ],
  [Species.GALAR_LINOONE]: [
    [ EVOLVE_MOVE, Moves.NIGHT_SLASH ],
    [ 1, Moves.SAND_ATTACK ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.PIN_MISSILE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 1, Moves.SWITCHEROO ],
    [ 1, Moves.BABY_DOLL_EYES ],
    [ 9, Moves.SNARL ],
    [ 12, Moves.HEADBUTT ],
    [ 15, Moves.HONE_CLAWS ],
    [ 18, Moves.FURY_SWIPES ],
    [ 23, Moves.REST ],
    [ 28, Moves.TAKE_DOWN ],
    [ 33, Moves.SCARY_FACE ],
    [ 38, Moves.COUNTER ],
    [ 43, Moves.TAUNT ],
    [ 48, Moves.DOUBLE_EDGE ],
  ],
  [Species.GALAR_DARUMAKA]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POWDER_SNOW ],
    [ 4, Moves.TAUNT ],
    [ 8, Moves.BITE ],
    [ 12, Moves.AVALANCHE ],
    [ 16, Moves.WORK_UP ],
    [ 20, Moves.ICE_FANG ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.ICE_PUNCH ],
    [ 32, Moves.UPROAR ],
    [ 36, Moves.BELLY_DRUM ],
    [ 40, Moves.BLIZZARD ],
    [ 44, Moves.THRASH ],
    [ 48, Moves.SUPERPOWER ],
  ],
  [Species.GALAR_DARMANITAN]: [
    [ EVOLVE_MOVE, Moves.ICICLE_CRASH ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.BITE ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.TAUNT ],
    [ 12, Moves.AVALANCHE ],
    [ 16, Moves.WORK_UP ],
    [ 20, Moves.ICE_FANG ],
    [ 24, Moves.HEADBUTT ],
    [ 28, Moves.ICE_PUNCH ],
    [ 32, Moves.UPROAR ],
    [ 38, Moves.BELLY_DRUM ],
    [ 44, Moves.BLIZZARD ],
    [ 50, Moves.THRASH ],
    [ 56, Moves.SUPERPOWER ],
  ],
  [Species.GALAR_YAMASK]: [
    [ 1, Moves.PROTECT ],
    [ 1, Moves.ASTONISH ],
    [ 4, Moves.HAZE ],
    [ 8, Moves.NIGHT_SHADE ],
    [ 12, Moves.DISABLE ],
    [ 16, Moves.BRUTAL_SWING ],
    [ 20, Moves.CRAFTY_SHIELD ],
    [ 24, Moves.HEX ],
    [ 28, Moves.MEAN_LOOK ],
    [ 32, Moves.SLAM ],
    [ 36, Moves.CURSE ],
    [ 40, Moves.SHADOW_BALL ],
    [ 44, Moves.EARTHQUAKE ],
    [ 48, Moves.GUARD_SPLIT ],
    [ 48, Moves.POWER_SPLIT ],
    [ 52, Moves.DESTINY_BOND ],
  ],
  [Species.GALAR_STUNFISK]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.MUD_SLAP ],
    [ 1, Moves.METAL_CLAW ],
    [ 5, Moves.ENDURE ],
    [ 10, Moves.MUD_SHOT ],
    [ 15, Moves.REVENGE ],
    [ 20, Moves.METAL_SOUND ],
    [ 25, Moves.SUCKER_PUNCH ],
    [ 30, Moves.IRON_DEFENSE ],
    [ 35, Moves.BOUNCE ],
    [ 40, Moves.MUDDY_WATER ],
    [ 45, Moves.SNAP_TRAP ],
    [ 50, Moves.FLAIL ],
    [ 55, Moves.FISSURE ],
  ],
  [Species.HISUI_GROWLITHE]: [
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 4, Moves.HOWL ],
    [ 8, Moves.BITE ],
    [ 12, Moves.FLAME_WHEEL ],
    [ 16, Moves.HELPING_HAND ],
    [ 24, Moves.FIRE_FANG ],
    [ 28, Moves.RETALIATE ],
    [ 32, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 40, Moves.FLAMETHROWER ],
    [ 44, Moves.ROAR ],
    [ 48, Moves.ROCK_SLIDE ],
    [ 52, Moves.REVERSAL ],
    [ 56, Moves.FLARE_BLITZ ],
  ],
  [Species.HISUI_ARCANINE]: [
    [ EVOLVE_MOVE, Moves.EXTREME_SPEED ],
    [ 1, Moves.TAKE_DOWN ],
    [ 1, Moves.LEER ],
    [ 1, Moves.BITE ],
    [ 1, Moves.ROAR ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.ROCK_THROW ],
    [ 1, Moves.AGILITY ],
    [ 1, Moves.ROCK_SLIDE ],
    [ 1, Moves.FLAME_WHEEL ],
    [ 1, Moves.REVERSAL ],
    [ 1, Moves.CRUNCH ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.HOWL ],
    [ 1, Moves.FLARE_BLITZ ],
    [ 1, Moves.FIRE_FANG ],
    [ 1, Moves.RETALIATE ],
    [ 5, Moves.FLAMETHROWER ],
    [ 64, Moves.RAGING_FURY ],
  ],
  [Species.HISUI_VOLTORB]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CHARGE ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 6, Moves.STUN_SPORE ],
    [ 9, Moves.BULLET_SEED ],
    [ 11, Moves.ROLLOUT ],
    [ 13, Moves.SCREECH ],
    [ 16, Moves.CHARGE_BEAM ],
    [ 20, Moves.SWIFT ],
    [ 22, Moves.ELECTRO_BALL ],
    [ 26, Moves.SELF_DESTRUCT ],
    [ 29, Moves.ENERGY_BALL ],
    [ 34, Moves.SEED_BOMB ],
    [ 34, Moves.DISCHARGE ],
    [ 41, Moves.EXPLOSION ],
    [ 46, Moves.GYRO_BALL ],
    [ 50, Moves.GRASSY_TERRAIN ],
  ],
  [Species.HISUI_ELECTRODE]: [
    [ EVOLVE_MOVE, Moves.CHLOROBLAST ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.CHARGE ],
    [ 4, Moves.THUNDER_SHOCK ],
    [ 6, Moves.STUN_SPORE ],
    [ 9, Moves.BULLET_SEED ],
    [ 11, Moves.ROLLOUT ],
    [ 13, Moves.SCREECH ],
    [ 16, Moves.CHARGE_BEAM ],
    [ 20, Moves.SWIFT ],
    [ 22, Moves.ELECTRO_BALL ],
    [ 26, Moves.SELF_DESTRUCT ],
    [ 29, Moves.ENERGY_BALL ],
    [ 34, Moves.SEED_BOMB ],
    [ 34, Moves.DISCHARGE ],
    [ 41, Moves.EXPLOSION ],
    [ 46, Moves.GYRO_BALL ],
    [ 50, Moves.GRASSY_TERRAIN ],
  ],
  [Species.HISUI_TYPHLOSION]: [
    [ EVOLVE_MOVE, Moves.INFERNAL_PARADE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.DOUBLE_EDGE ],
    [ 1, Moves.LEER ],
    [ 1, Moves.EMBER ],
    [ 1, Moves.SMOKESCREEN ],
    [ 1, Moves.ERUPTION ],
    [ 1, Moves.GYRO_BALL ],
    [ 13, Moves.QUICK_ATTACK ],
    [ 20, Moves.FLAME_WHEEL ],
    [ 24, Moves.DEFENSE_CURL ],
    [ 31, Moves.SWIFT ],
    [ 35, Moves.FLAME_CHARGE ],
    [ 43, Moves.LAVA_PLUME ],
    [ 48, Moves.FLAMETHROWER ],
    [ 56, Moves.INFERNO ],
    [ 61, Moves.ROLLOUT ],
    [ 74, Moves.OVERHEAT ],
  ],
  [Species.HISUI_QWILFISH]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.POISON_STING ],
    [ 4, Moves.HARDEN ],
    [ 8, Moves.BITE ],
    [ 12, Moves.FELL_STINGER ],
    [ 16, Moves.MINIMIZE ],
    [ 20, Moves.SPIKES ],
    [ 24, Moves.BRINE ],
    [ 28, Moves.BARB_BARRAGE ],
    [ 32, Moves.PIN_MISSILE ],
    [ 36, Moves.TOXIC_SPIKES ],
    [ 40, Moves.STOCKPILE ],
    [ 40, Moves.SPIT_UP ],
    [ 44, Moves.TOXIC ],
    [ 48, Moves.CRUNCH ],
    [ 52, Moves.ACUPRESSURE ],
    [ 56, Moves.DESTINY_BOND ],
  ],
  [Species.HISUI_SNEASEL]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.ROCK_SMASH ],
    [ 6, Moves.TAUNT ],
    [ 12, Moves.QUICK_ATTACK ],
    [ 18, Moves.METAL_CLAW ],
    [ 24, Moves.POISON_JAB ],
    [ 30, Moves.BRICK_BREAK ],
    [ 36, Moves.HONE_CLAWS ],
    [ 42, Moves.SLASH ],
    [ 48, Moves.AGILITY ],
    [ 54, Moves.SCREECH ],
    [ 60, Moves.CLOSE_COMBAT ],
  ],
  [Species.HISUI_SAMUROTT]: [
    [ EVOLVE_MOVE, Moves.CEASELESS_EDGE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.SLASH ],
    [ 1, Moves.MEGAHORN ],
    [ 1, Moves.SUCKER_PUNCH ],
    [ 13, Moves.FOCUS_ENERGY ],
    [ 18, Moves.RAZOR_SHELL ],
    [ 21, Moves.FURY_CUTTER ],
    [ 25, Moves.WATER_PULSE ],
    [ 29, Moves.AERIAL_ACE ],
    [ 34, Moves.AQUA_JET ],
    [ 39, Moves.ENCORE ],
    [ 46, Moves.AQUA_TAIL ],
    [ 51, Moves.RETALIATE ],
    [ 58, Moves.SWORDS_DANCE ],
    [ 63, Moves.HYDRO_PUMP ],
  ],
  [Species.HISUI_LILLIGANT]: [
    [ EVOLVE_MOVE, Moves.VICTORY_DANCE ],
    [ 1, Moves.MEGA_KICK ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.MEGA_DRAIN ],
    [ 1, Moves.LEECH_SEED ],
    [ 1, Moves.GROWTH ],
    [ 1, Moves.STUN_SPORE ],
    [ 1, Moves.SLEEP_POWDER ],
    [ 1, Moves.GIGA_DRAIN ],
    [ 1, Moves.SYNTHESIS ],
    [ 1, Moves.SUNNY_DAY ],
    [ 1, Moves.HELPING_HAND ],
    [ 1, Moves.TEETER_DANCE ],
    [ 1, Moves.MAGICAL_LEAF ],
    [ 1, Moves.LEAF_BLADE ],
    [ 1, Moves.ENERGY_BALL ],
    [ 1, Moves.DEFOG ],
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.ENTRAINMENT ],
    [ 1, Moves.AFTER_YOU ],
    [ 1, Moves.PETAL_BLIZZARD ],
    [ 1, Moves.SOLAR_BLADE ],
    [ 5, Moves.AXE_KICK ],
  ],
  [Species.HISUI_ZORUA]: [
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 4, Moves.TORMENT ],
    [ 8, Moves.HONE_CLAWS ],
    [ 12, Moves.SHADOW_SNEAK ],
    [ 16, Moves.CURSE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.KNOCK_OFF ],
    [ 28, Moves.SPITE ],
    [ 32, Moves.AGILITY ],
    [ 36, Moves.SHADOW_BALL ],
    [ 40, Moves.BITTER_MALICE ],
    [ 44, Moves.NASTY_PLOT ],
    [ 48, Moves.FOUL_PLAY ],
  ],
  [Species.HISUI_ZOROARK]: [
    [ EVOLVE_MOVE, Moves.SHADOW_CLAW ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.TORMENT ],
    [ 1, Moves.U_TURN ],
    [ 1, Moves.HONE_CLAWS ],
    [ 12, Moves.SHADOW_SNEAK ],
    [ 16, Moves.CURSE ],
    [ 20, Moves.TAUNT ],
    [ 24, Moves.KNOCK_OFF ],
    [ 28, Moves.SPITE ],
    [ 34, Moves.AGILITY ],
    [ 40, Moves.SHADOW_BALL ],
    [ 46, Moves.BITTER_MALICE ],
    [ 52, Moves.NASTY_PLOT ],
    [ 58, Moves.FOUL_PLAY ],
  ],
  [Species.HISUI_BRAVIARY]: [
    [ EVOLVE_MOVE, Moves.ESPER_WING ],
    [ 1, Moves.WING_ATTACK ],
    [ 1, Moves.LEER ],
    [ 1, Moves.PECK ],
    [ 1, Moves.SKY_ATTACK ],
    [ 1, Moves.SUPERPOWER ],
    [ 1, Moves.HONE_CLAWS ],
    [ 18, Moves.TAILWIND ],
    [ 24, Moves.SCARY_FACE ],
    [ 30, Moves.AERIAL_ACE ],
    [ 36, Moves.SLASH ],
    [ 42, Moves.WHIRLWIND ],
    [ 48, Moves.CRUSH_CLAW ],
    [ 57, Moves.AIR_SLASH ],
    [ 64, Moves.DEFOG ],
    [ 72, Moves.THRASH ],
    [ 80, Moves.HURRICANE ],
  ],
  [Species.HISUI_SLIGGOO]: [
    [ EVOLVE_MOVE, Moves.SHELTER ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.ACID_ARMOR ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 15, Moves.PROTECT ],
    [ 20, Moves.FLAIL ],
    [ 25, Moves.WATER_PULSE ],
    [ 30, Moves.RAIN_DANCE ],
    [ 35, Moves.DRAGON_PULSE ],
    [ 43, Moves.CURSE ],
    [ 49, Moves.IRON_HEAD ],
    [ 56, Moves.MUDDY_WATER ],
  ],
  [Species.HISUI_GOODRA]: [
    [ EVOLVE_MOVE, Moves.IRON_TAIL ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.WATER_GUN ],
    [ 1, Moves.ABSORB ],
    [ 1, Moves.DRAGON_BREATH ],
    [ 1, Moves.FEINT ],
    [ 1, Moves.ACID_SPRAY ],
    [ 1, Moves.TEARFUL_LOOK ],
    [ 1, Moves.SHELTER ],
    [ 15, Moves.PROTECT ],
    [ 20, Moves.FLAIL ],
    [ 25, Moves.WATER_PULSE ],
    [ 30, Moves.RAIN_DANCE ],
    [ 35, Moves.DRAGON_PULSE ],
    [ 43, Moves.CURSE ],
    [ 49, Moves.BODY_SLAM ],
    [ 49, Moves.IRON_HEAD ],
    [ 58, Moves.MUDDY_WATER ],
    [ 67, Moves.HEAVY_SLAM ],
  ],
  [Species.HISUI_AVALUGG]: [
    [ EVOLVE_MOVE, Moves.ROCK_SLIDE ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.HARDEN ],
    [ 1, Moves.POWDER_SNOW ],
    [ 1, Moves.RAPID_SPIN ],
    [ 1, Moves.WIDE_GUARD ],
    [ 9, Moves.CURSE ],
    [ 12, Moves.ICY_WIND ],
    [ 15, Moves.PROTECT ],
    [ 18, Moves.AVALANCHE ],
    [ 21, Moves.BITE ],
    [ 24, Moves.ICE_FANG ],
    [ 27, Moves.IRON_DEFENSE ],
    [ 30, Moves.RECOVER ],
    [ 33, Moves.CRUNCH ],
    [ 36, Moves.TAKE_DOWN ],
    [ 41, Moves.BLIZZARD ],
    [ 46, Moves.DOUBLE_EDGE ],
    [ 51, Moves.STONE_EDGE ],
    [ 61, Moves.MOUNTAIN_GALE ],
  ],
  [Species.HISUI_DECIDUEYE]: [
    [ EVOLVE_MOVE, Moves.TRIPLE_ARROWS ],
    [ 1, Moves.TACKLE ],
    [ 1, Moves.GROWL ],
    [ 1, Moves.U_TURN ],
    [ 1, Moves.LEAF_STORM ],
    [ 1, Moves.LEAFAGE ],
    [ 9, Moves.PECK ],
    [ 12, Moves.SHADOW_SNEAK ],
    [ 15, Moves.RAZOR_LEAF ],
    [ 20, Moves.SYNTHESIS ],
    [ 25, Moves.PLUCK ],
    [ 30, Moves.BULK_UP ],
    [ 37, Moves.SUCKER_PUNCH ],
    [ 44, Moves.LEAF_BLADE ],
    [ 51, Moves.FEATHER_DANCE ],
    [ 58, Moves.BRAVE_BIRD ],
  ],
  [Species.PALDEA_TAUROS]: [
    [ 1, Moves.TACKLE ],
    [ 1, Moves.TAIL_WHIP ],
    [ 5, Moves.WORK_UP ],
    [ 10, Moves.DOUBLE_KICK ],
    [ 15, Moves.ASSURANCE ],
    [ 20, Moves.HEADBUTT ],
    [ 25, Moves.SCARY_FACE ],
    [ 30, Moves.ZEN_HEADBUTT ],
    [ 35, Moves.RAGING_BULL ],
    [ 40, Moves.REST ],
    [ 45, Moves.SWAGGER ],
    [ 50, Moves.THRASH ],
    [ 55, Moves.DOUBLE_EDGE ],
    [ 60, Moves.CLOSE_COMBAT ],
  ],
  [Species.PALDEA_WOOPER]: [
    [ 1, Moves.TAIL_WHIP ],
    [ 1, Moves.MUD_SHOT ],
    [ 4, Moves.TACKLE ],
    [ 8, Moves.POISON_TAIL ],
    [ 12, Moves.TOXIC_SPIKES ],
    [ 16, Moves.SLAM ],
    [ 21, Moves.YAWN ],
    [ 24, Moves.POISON_JAB ],
    [ 28, Moves.SLUDGE_WAVE ],
    [ 32, Moves.AMNESIA ],
    [ 36, Moves.TOXIC ],
    [ 40, Moves.EARTHQUAKE ],
  ],
  [Species.BLOODMOON_URSALUNA]: [
    [ RELEARN_MOVE, Moves.MOONLIGHT ],
    [ 1, Moves.HEADLONG_RUSH ],
    [ 1, Moves.SCRATCH ],
    [ 1, Moves.LEER ],
    [ 1, Moves.LICK ],
    [ 8, Moves.FURY_SWIPES ],
    [ 13, Moves.PAYBACK ],
    [ 17, Moves.HARDEN ],
    [ 22, Moves.SLASH ],
    [ 25, Moves.PLAY_NICE ],
    [ 35, Moves.SCARY_FACE ],
    [ 41, Moves.REST ],
    [ 41, Moves.SNORE ],
    [ 48, Moves.EARTH_POWER ],
    [ 56, Moves.MOONBLAST ],
    [ 64, Moves.HAMMER_ARM ],
    [ 70, Moves.BLOOD_MOON ],
  ]
};

export const pokemonFormLevelMoves: PokemonSpeciesFormLevelMoves = {
  [Species.PIKACHU]: { //Custom
    1: [
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.ZIPPY_ZAP ], //Custom
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 34, Moves.FLOATY_FALL ], //Custom
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 42, Moves.SPLISHY_SPLASH ], //Custom
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    2: [
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.SPARK ],
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    3: [
      [ 1, Moves.METEOR_MASH ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.SPARK ],
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    4: [
      [ 1, Moves.ICICLE_CRASH ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.SPARK ],
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    5: [
      [ 1, Moves.DRAINING_KISS ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.SPARK ],
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    6: [
      [ 1, Moves.ELECTRIC_TERRAIN ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.SPARK ],
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    7: [
      [ 1, Moves.FLYING_PRESS ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.SPARK ],
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
    8: [
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.SWEET_KISS ],
      [ 1, Moves.CHARM ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.PLAY_NICE ],
      [ 1, Moves.NUZZLE ],
      [ 4, Moves.THUNDER_WAVE ],
      [ 8, Moves.DOUBLE_TEAM ],
      [ 12, Moves.ELECTRO_BALL ],
      [ 16, Moves.FEINT ],
      [ 20, Moves.ZIPPY_ZAP ], //Custom
      [ 24, Moves.AGILITY ],
      [ 28, Moves.IRON_TAIL ],
      [ 32, Moves.DISCHARGE ],
      [ 34, Moves.FLOATY_FALL ], //Custom
      [ 36, Moves.THUNDERBOLT ],
      [ 40, Moves.LIGHT_SCREEN ],
      [ 42, Moves.SPLISHY_SPLASH ], //Custom
      [ 44, Moves.THUNDER ],
      [ 48, Moves.PIKA_PAPOW ],
    ],
  },
  [Species.EEVEE]: { //Custom
    1: [
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.HELPING_HAND ],
      [ 1, Moves.COVET ],
      [ 5, Moves.SAND_ATTACK ],
      [ 10, Moves.QUICK_ATTACK ],
      [ 15, Moves.BABY_DOLL_EYES ],
      [ 18, Moves.BOUNCY_BUBBLE ], //Custom
      [ 18, Moves.SIZZLY_SLIDE ], //Custom
      [ 18, Moves.BUZZY_BUZZ ], //Custom
      [ 20, Moves.SWIFT ],
      [ 25, Moves.BITE ],
      [ 30, Moves.COPYCAT ],
      [ 33, Moves.BADDY_BAD ], //Custom
      [ 33, Moves.GLITZY_GLOW ], //Custom
      [ 35, Moves.BATON_PASS ],
      [ 40, Moves.VEEVEE_VOLLEY ], //Custom, replaces Take Down
      [ 43, Moves.FREEZY_FROST ], //Custom
      [ 43, Moves.SAPPY_SEED ], //Custom
      [ 45, Moves.CHARM ],
      [ 50, Moves.DOUBLE_EDGE ],
      [ 53, Moves.SPARKLY_SWIRL ], //Custom
      [ 55, Moves.LAST_RESORT ],
    ],
    2: [
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.HELPING_HAND ],
      [ 1, Moves.COVET ],
      [ 5, Moves.SAND_ATTACK ],
      [ 10, Moves.QUICK_ATTACK ],
      [ 15, Moves.BABY_DOLL_EYES ],
      [ 18, Moves.BOUNCY_BUBBLE ], //Custom
      [ 18, Moves.SIZZLY_SLIDE ], //Custom
      [ 18, Moves.BUZZY_BUZZ ], //Custom
      [ 20, Moves.SWIFT ],
      [ 25, Moves.BITE ],
      [ 30, Moves.COPYCAT ],
      [ 33, Moves.BADDY_BAD ], //Custom
      [ 33, Moves.GLITZY_GLOW ], //Custom
      [ 35, Moves.BATON_PASS ],
      [ 40, Moves.VEEVEE_VOLLEY ], //Custom, replaces Take Down
      [ 43, Moves.FREEZY_FROST ], //Custom
      [ 43, Moves.SAPPY_SEED ], //Custom
      [ 45, Moves.CHARM ],
      [ 50, Moves.DOUBLE_EDGE ],
      [ 53, Moves.SPARKLY_SWIRL ], //Custom
      [ 55, Moves.LAST_RESORT ],
    ],
  },
  [Species.DEOXYS]: {
    1: [
      [ 1, Moves.CONFUSION ], //Custom
      [ 1, Moves.WRAP ],
      [ 1, Moves.LEER ],
      [ 7, Moves.NIGHT_SHADE ],
      [ 13, Moves.TELEPORT ],
      [ 19, Moves.TAUNT ],
      [ 25, Moves.PURSUIT ],
      [ 31, Moves.PSYCHIC ],
      [ 37, Moves.SUPERPOWER ],
      [ 43, Moves.PSYCHO_SHIFT ],
      [ 49, Moves.ZEN_HEADBUTT ],
      [ 55, Moves.COSMIC_POWER ],
      [ 61, Moves.ZAP_CANNON ],
      [ 67, Moves.PSYCHO_BOOST ],
      [ 73, Moves.HYPER_BEAM ],
    ],
    2: [
      [ 1, Moves.CONFUSION ], //Custom
      [ 1, Moves.WRAP ],
      [ 1, Moves.LEER ],
      [ 7, Moves.NIGHT_SHADE ],
      [ 13, Moves.TELEPORT ],
      [ 19, Moves.KNOCK_OFF ],
      [ 25, Moves.SPIKES ],
      [ 31, Moves.PSYCHIC ],
      [ 37, Moves.SNATCH ],
      [ 43, Moves.PSYCHO_SHIFT ],
      [ 49, Moves.ZEN_HEADBUTT ],
      [ 55, Moves.AMNESIA ],
      [ 55, Moves.IRON_DEFENSE ],
      [ 61, Moves.RECOVER ],
      [ 67, Moves.PSYCHO_BOOST ],
      [ 73, Moves.COUNTER ],
      [ 73, Moves.MIRROR_COAT ],
    ],
    3: [
      [ 1, Moves.CONFUSION ], //Custom
      [ 1, Moves.WRAP ],
      [ 1, Moves.LEER ],
      [ 7, Moves.NIGHT_SHADE ],
      [ 13, Moves.DOUBLE_TEAM ],
      [ 19, Moves.KNOCK_OFF ],
      [ 25, Moves.PURSUIT ],
      [ 31, Moves.PSYCHIC ],
      [ 37, Moves.SWIFT ],
      [ 43, Moves.PSYCHO_SHIFT ],
      [ 49, Moves.ZEN_HEADBUTT ],
      [ 55, Moves.AGILITY ],
      [ 61, Moves.RECOVER ],
      [ 67, Moves.PSYCHO_BOOST ],
      [ 73, Moves.EXTREME_SPEED ],
    ],
  },
  [Species.WORMADAM]: {
    1: [
      [ EVOLVE_MOVE, Moves.QUIVER_DANCE ],
      [ 1, Moves.TACKLE ],
      [ 1, Moves.PROTECT ],
      [ 1, Moves.SUCKER_PUNCH ],
      [ 1, Moves.BUG_BITE ],
      [ 1, Moves.PROTECT ],
      [ 10, Moves.TACKLE ],
      [ 20, Moves.STRING_SHOT ],
      [ 23, Moves.CONFUSION ],
      [ 26, Moves.ROCK_BLAST ],
      [ 29, Moves.HARDEN ],
      [ 32, Moves.PSYBEAM ],
      [ 35, Moves.INFESTATION ],
      [ 38, Moves.FLAIL ],
      [ 41, Moves.ATTRACT ],
      [ 44, Moves.PSYCHIC ],
      [ 47, Moves.FISSURE ],
      [ 50, Moves.BUG_BUZZ ],
    ],
    2: [
      [ EVOLVE_MOVE, Moves.QUIVER_DANCE ],
      [ 1, Moves.METAL_BURST ],
      [ 1, Moves.TACKLE ],
      [ 1, Moves.PROTECT ],
      [ 1, Moves.SUCKER_PUNCH ],
      [ 1, Moves.BUG_BITE ],
      [ 1, Moves.PROTECT ],
      [ 10, Moves.TACKLE ],
      [ 20, Moves.STRING_SHOT ],
      [ 23, Moves.CONFUSION ],
      [ 26, Moves.METAL_BURST ],
      [ 29, Moves.METAL_SOUND ],
      [ 32, Moves.PSYBEAM ],
      [ 35, Moves.INFESTATION ],
      [ 38, Moves.FLAIL ],
      [ 41, Moves.ATTRACT ],
      [ 44, Moves.PSYCHIC ],
      [ 47, Moves.IRON_HEAD ],
      [ 50, Moves.BUG_BUZZ ],
    ],
  },
  [Species.ROTOM]: {
    1: [
      [ 1, Moves.OVERHEAT ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.ASTONISH ],
      [ 5, Moves.THUNDER_SHOCK ],
      [ 10, Moves.CONFUSE_RAY ],
      [ 15, Moves.CHARGE ],
      [ 20, Moves.ELECTRO_BALL ],
      [ 25, Moves.THUNDER_WAVE ],
      [ 30, Moves.SHOCK_WAVE ],
      [ 35, Moves.HEX ],
      [ 40, Moves.SUBSTITUTE ],
      [ 45, Moves.TRICK ],
      [ 50, Moves.DISCHARGE ],
      [ 55, Moves.UPROAR ],
    ],
    2: [
      [ 1, Moves.HYDRO_PUMP ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.ASTONISH ],
      [ 5, Moves.THUNDER_SHOCK ],
      [ 10, Moves.CONFUSE_RAY ],
      [ 15, Moves.CHARGE ],
      [ 20, Moves.ELECTRO_BALL ],
      [ 25, Moves.THUNDER_WAVE ],
      [ 30, Moves.SHOCK_WAVE ],
      [ 35, Moves.HEX ],
      [ 40, Moves.SUBSTITUTE ],
      [ 45, Moves.TRICK ],
      [ 50, Moves.DISCHARGE ],
      [ 55, Moves.UPROAR ],
    ],
    3: [
      [ 1, Moves.BLIZZARD ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.ASTONISH ],
      [ 5, Moves.THUNDER_SHOCK ],
      [ 10, Moves.CONFUSE_RAY ],
      [ 15, Moves.CHARGE ],
      [ 20, Moves.ELECTRO_BALL ],
      [ 25, Moves.THUNDER_WAVE ],
      [ 30, Moves.SHOCK_WAVE ],
      [ 35, Moves.HEX ],
      [ 40, Moves.SUBSTITUTE ],
      [ 45, Moves.TRICK ],
      [ 50, Moves.DISCHARGE ],
      [ 55, Moves.UPROAR ],
    ],
    4: [
      [ 1, Moves.AIR_SLASH ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.ASTONISH ],
      [ 5, Moves.THUNDER_SHOCK ],
      [ 10, Moves.CONFUSE_RAY ],
      [ 15, Moves.CHARGE ],
      [ 20, Moves.ELECTRO_BALL ],
      [ 25, Moves.THUNDER_WAVE ],
      [ 30, Moves.SHOCK_WAVE ],
      [ 35, Moves.HEX ],
      [ 40, Moves.SUBSTITUTE ],
      [ 45, Moves.TRICK ],
      [ 50, Moves.DISCHARGE ],
      [ 55, Moves.UPROAR ],
    ],
    5: [
      [ 1, Moves.LEAF_STORM ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.ASTONISH ],
      [ 5, Moves.THUNDER_SHOCK ],
      [ 10, Moves.CONFUSE_RAY ],
      [ 15, Moves.CHARGE ],
      [ 20, Moves.ELECTRO_BALL ],
      [ 25, Moves.THUNDER_WAVE ],
      [ 30, Moves.SHOCK_WAVE ],
      [ 35, Moves.HEX ],
      [ 40, Moves.SUBSTITUTE ],
      [ 45, Moves.TRICK ],
      [ 50, Moves.DISCHARGE ],
      [ 55, Moves.UPROAR ],
    ],
  },
  [Species.SHAYMIN]: {
    1: [
      [ 1, Moves.LEAFAGE ], //Custom
      [ 1, Moves.GROWTH ],
      [ 10, Moves.MAGICAL_LEAF ],
      [ 19, Moves.LEECH_SEED ],
      [ 28, Moves.QUICK_ATTACK ],
      [ 37, Moves.SWEET_SCENT ],
      [ 46, Moves.NATURAL_GIFT ],
      [ 55, Moves.WORRY_SEED ],
      [ 64, Moves.AIR_SLASH ],
      [ 73, Moves.ENERGY_BALL ],
      [ 82, Moves.SWEET_KISS ],
      [ 91, Moves.LEAF_STORM ],
      [ 100, Moves.SEED_FLARE ],
    ]
  },
  [Species.KYUREM]: {
    1: [
      [ 1, Moves.DRAGON_BREATH ],
      [ 1, Moves.ANCIENT_POWER ],
      [ 1, Moves.NOBLE_ROAR ],
      [ 1, Moves.FREEZE_DRY ],
      [ 8, Moves.SLASH ],
      [ 16, Moves.ENDEAVOR ],
      [ 24, Moves.DRAGON_PULSE ],
      [ 32, Moves.ICE_BEAM ],
      [ 40, Moves.HYPER_VOICE ],
      [ 48, Moves.FUSION_BOLT ],
      [ 56, Moves.BLIZZARD ],
      [ 64, Moves.IMPRISON ],
      [ 72, Moves.OUTRAGE ],
      [ 80, Moves.FREEZE_SHOCK ],
      [ 88, Moves.SHEER_COLD ],
    ],
    2: [
      [ 1, Moves.DRAGON_BREATH ],
      [ 1, Moves.ANCIENT_POWER ],
      [ 1, Moves.NOBLE_ROAR ],
      [ 1, Moves.FREEZE_DRY ],
      [ 8, Moves.SLASH ],
      [ 16, Moves.ENDEAVOR ],
      [ 24, Moves.DRAGON_PULSE ],
      [ 32, Moves.ICE_BEAM ],
      [ 40, Moves.HYPER_VOICE ],
      [ 48, Moves.FUSION_FLARE ],
      [ 56, Moves.BLIZZARD ],
      [ 64, Moves.IMPRISON ],
      [ 72, Moves.OUTRAGE ],
      [ 80, Moves.ICE_BURN ],
      [ 88, Moves.SHEER_COLD ],
    ],
  },
  [Species.MEOWSTIC]: {
    1: [
      [ 1, Moves.SCRATCH ],
      [ 1, Moves.LEER ],
      [ 1, Moves.FAKE_OUT ],
      [ 1, Moves.MAGICAL_LEAF ],
      [ 1, Moves.DISARMING_VOICE ],
      [ 9, Moves.CONFUSION ],
      [ 12, Moves.STORED_POWER ],
      [ 15, Moves.CHARGE_BEAM ],
      [ 18, Moves.COVET ],
      [ 21, Moves.PSYBEAM ],
      [ 24, Moves.SUCKER_PUNCH ],
      [ 29, Moves.ROLE_PLAY ],
      [ 34, Moves.LIGHT_SCREEN ],
      [ 34, Moves.REFLECT ],
      [ 39, Moves.PSYSHOCK ],
      [ 44, Moves.EXTRASENSORY ],
      [ 49, Moves.SHADOW_BALL ],
      [ 54, Moves.PSYCHIC ],
      [ 59, Moves.FUTURE_SIGHT ],
    ],
  },
  [Species.HOOPA]: {
    1: [
      [ 1, Moves.CONFUSION ],
      [ 1, Moves.DESTINY_BOND ],
      [ 1, Moves.ALLY_SWITCH ],
      [ 6, Moves.ASTONISH ],
      [ 10, Moves.TRICK ],
      [ 15, Moves.LIGHT_SCREEN ],
      [ 19, Moves.PSYBEAM ],
      [ 25, Moves.SKILL_SWAP ],
      [ 29, Moves.GUARD_SPLIT ],
      [ 29, Moves.POWER_SPLIT ],
      [ 46, Moves.KNOCK_OFF ],
      [ 50, Moves.TRICK_ROOM ],
      [ 50, Moves.WONDER_ROOM ],
      [ 55, Moves.DARK_PULSE ],
      [ 75, Moves.PSYCHIC ],
      [ 85, Moves.HYPERSPACE_FURY ],
    ],
  },
  [Species.GRENINJA]: {
    1: [
      [ EVOLVE_MOVE, Moves.WATER_SHURIKEN ],
      [ EVOLVE_MOVE, Moves.WATER_SHURIKEN ],
      [ 1, Moves.POUND ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.WATER_GUN ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.HAZE ],
      [ 1, Moves.ROLE_PLAY ],
      [ 1, Moves.NIGHT_SLASH ],
      [ 10, Moves.LICK ],
      [ 14, Moves.WATER_PULSE ],
      [ 19, Moves.SMOKESCREEN ],
      [ 23, Moves.SHADOW_SNEAK ],
      [ 28, Moves.SPIKES ],
      [ 33, Moves.AERIAL_ACE ],
      [ 42, Moves.SUBSTITUTE ],
      [ 49, Moves.EXTRASENSORY ],
      [ 56, Moves.DOUBLE_TEAM ],
      [ 68, Moves.HYDRO_PUMP ],
    ],
  },
  [Species.LYCANROC]: {
    1: [
      [ EVOLVE_MOVE, Moves.COUNTER ],
      [ 1, Moves.SAND_ATTACK ],
      [ 1, Moves.TACKLE ],
      [ 1, Moves.LEER ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.REVERSAL ],
      [ 1, Moves.ENDURE ],
      [ 1, Moves.TAUNT ],
      [ 12, Moves.ROCK_THROW ],
      [ 16, Moves.HOWL ],
      [ 20, Moves.BITE ],
      [ 24, Moves.ROCK_TOMB ],
      [ 30, Moves.ROAR ],
      [ 36, Moves.ROCK_SLIDE ],
      [ 42, Moves.CRUNCH ],
      [ 48, Moves.SCARY_FACE ],
      [ 54, Moves.STEALTH_ROCK ],
      [ 60, Moves.STONE_EDGE ],
    ],
    2: [
      [ EVOLVE_MOVE, Moves.CRUSH_CLAW ],
      [ 1, Moves.SAND_ATTACK ],
      [ 1, Moves.TACKLE ],
      [ 1, Moves.LEER ],
      [ 1, Moves.COUNTER ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.DOUBLE_TEAM ],
      [ 1, Moves.REVERSAL ],
      [ 1, Moves.ENDURE ],
      [ 1, Moves.TAUNT ],
      [ 1, Moves.SUCKER_PUNCH ],
      [ 1, Moves.QUICK_GUARD ],
      [ 1, Moves.ACCELEROCK ],
      [ 12, Moves.ROCK_THROW ],
      [ 16, Moves.HOWL ],
      [ 20, Moves.BITE ],
      [ 24, Moves.ROCK_TOMB ],
      [ 30, Moves.ROAR ],
      [ 36, Moves.ROCK_SLIDE ],
      [ 42, Moves.CRUNCH ],
      [ 48, Moves.SCARY_FACE ],
      [ 54, Moves.STEALTH_ROCK ],
      [ 60, Moves.STONE_EDGE ],
    ],
  },
  [Species.NECROZMA]: {
    1: [
      [ EVOLVE_MOVE, Moves.SUNSTEEL_STRIKE ],
      [ 1, Moves.CONFUSION ],
      [ 1, Moves.METAL_CLAW ],
      [ 1, Moves.MORNING_SUN ],
      [ 1, Moves.MOONLIGHT ],
      [ 1, Moves.GRAVITY ],
      [ 1, Moves.CHARGE_BEAM ],
      [ 8, Moves.STEALTH_ROCK ],
      [ 16, Moves.SLASH ],
      [ 24, Moves.NIGHT_SLASH ],
      [ 32, Moves.PSYCHO_CUT ],
      [ 40, Moves.STORED_POWER ],
      [ 48, Moves.ROCK_BLAST ],
      [ 56, Moves.IRON_DEFENSE ],
      [ 64, Moves.POWER_GEM ],
      [ 72, Moves.PHOTON_GEYSER ],
      [ 80, Moves.AUTOTOMIZE ],
      [ 88, Moves.PRISMATIC_LASER ],
    ],
    2: [
      [ EVOLVE_MOVE, Moves.MOONGEIST_BEAM ],
      [ 1, Moves.CONFUSION ],
      [ 1, Moves.METAL_CLAW ],
      [ 1, Moves.MORNING_SUN ],
      [ 1, Moves.MOONLIGHT ],
      [ 1, Moves.GRAVITY ],
      [ 1, Moves.CHARGE_BEAM ],
      [ 8, Moves.STEALTH_ROCK ],
      [ 16, Moves.SLASH ],
      [ 24, Moves.NIGHT_SLASH ],
      [ 32, Moves.PSYCHO_CUT ],
      [ 40, Moves.STORED_POWER ],
      [ 48, Moves.ROCK_BLAST ],
      [ 56, Moves.IRON_DEFENSE ],
      [ 64, Moves.POWER_GEM ],
      [ 72, Moves.PHOTON_GEYSER ],
      [ 80, Moves.AUTOTOMIZE ],
      [ 88, Moves.PRISMATIC_LASER ],
    ],
    3: [
      [ EVOLVE_MOVE, Moves.SUNSTEEL_STRIKE ],
      [ EVOLVE_MOVE, Moves.MOONGEIST_BEAM ],
      [ 1, Moves.CONFUSION ],
      [ 1, Moves.METAL_CLAW ],
      [ 1, Moves.MORNING_SUN ],
      [ 1, Moves.MOONLIGHT ],
      [ 1, Moves.GRAVITY ],
      [ 1, Moves.CHARGE_BEAM ],
      [ 8, Moves.STEALTH_ROCK ],
      [ 16, Moves.SLASH ],
      [ 24, Moves.NIGHT_SLASH ],
      [ 32, Moves.PSYCHO_CUT ],
      [ 40, Moves.STORED_POWER ],
      [ 48, Moves.ROCK_BLAST ],
      [ 56, Moves.IRON_DEFENSE ],
      [ 64, Moves.POWER_GEM ],
      [ 72, Moves.PHOTON_GEYSER ],
      [ 80, Moves.AUTOTOMIZE ],
      [ 88, Moves.PRISMATIC_LASER ],
    ],
  },
  [Species.TOXTRICITY]: {
    1: [
      [ EVOLVE_MOVE, Moves.SPARK ],
      [ 1, Moves.LEER ],
      [ 1, Moves.GROWL ],
      [ 1, Moves.ACID ],
      [ 1, Moves.THUNDER_SHOCK ],
      [ 1, Moves.FLAIL ],
      [ 1, Moves.ACID_SPRAY ],
      [ 1, Moves.BELCH ],
      [ 1, Moves.NOBLE_ROAR ],
      [ 1, Moves.NUZZLE ],
      [ 1, Moves.TEARFUL_LOOK ],
      [ 4, Moves.CHARGE ],
      [ 8, Moves.SHOCK_WAVE ],
      [ 12, Moves.SCARY_FACE ],
      [ 16, Moves.TAUNT ],
      [ 24, Moves.SCREECH ],
      [ 28, Moves.SWAGGER ],
      [ 32, Moves.TOXIC ],
      [ 36, Moves.DISCHARGE ],
      [ 40, Moves.POISON_JAB ],
      [ 44, Moves.OVERDRIVE ],
      [ 48, Moves.BOOMBURST ],
      [ 52, Moves.MAGNETIC_FLUX ],
    ],
  },
  [Species.INDEEDEE]: {
    1: [
      [ 1, Moves.STORED_POWER ],
      [ 1, Moves.PLAY_NICE ],
      [ 5, Moves.BATON_PASS ],
      [ 10, Moves.DISARMING_VOICE ],
      [ 15, Moves.PSYBEAM ],
      [ 20, Moves.HELPING_HAND ],
      [ 25, Moves.FOLLOW_ME ],
      [ 30, Moves.HEALING_WISH ],
      [ 35, Moves.PSYCHIC ],
      [ 40, Moves.CALM_MIND ],
      [ 45, Moves.GUARD_SPLIT ],
      [ 50, Moves.PSYCHIC_TERRAIN ],
    ],
  },
  [Species.ZACIAN]: {
    1: [
      [ EVOLVE_MOVE, Moves.BEHEMOTH_BLADE ],
      [ 1, Moves.BITE ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.METAL_CLAW ],
      [ 1, Moves.HOWL ],
      [ 1, Moves.QUICK_GUARD ],
      [ 1, Moves.SACRED_SWORD ],
      [ 11, Moves.SLASH ],
      [ 22, Moves.SWORDS_DANCE ],
      [ 33, Moves.IRON_HEAD ],
      [ 44, Moves.NOBLE_ROAR ],
      [ 55, Moves.CRUNCH ],
      [ 66, Moves.MOONBLAST ],
      [ 77, Moves.CLOSE_COMBAT ],
      [ 88, Moves.GIGA_IMPACT ],
    ],
  },
  [Species.ZAMAZENTA]: {
    1: [
      [ EVOLVE_MOVE, Moves.BEHEMOTH_BASH ],
      [ 1, Moves.BITE ],
      [ 1, Moves.QUICK_ATTACK ],
      [ 1, Moves.METAL_CLAW ],
      [ 1, Moves.HOWL ],
      [ 1, Moves.WIDE_GUARD ],
      [ 11, Moves.SLASH ],
      [ 22, Moves.IRON_DEFENSE ],
      [ 33, Moves.IRON_HEAD ],
      [ 44, Moves.METAL_BURST ],
      [ 55, Moves.CRUNCH ],
      [ 66, Moves.MOONBLAST ],
      [ 77, Moves.CLOSE_COMBAT ],
      [ 88, Moves.GIGA_IMPACT ],
    ],
  },
  [Species.ETERNATUS]: {
    1: [
      [ 1, Moves.AGILITY ],
      [ 1, Moves.CONFUSE_RAY ],
      [ 1, Moves.POISON_TAIL ],
      [ 1, Moves.DRAGON_TAIL ],
      [ 8, Moves.TOXIC ],
      [ 16, Moves.VENOSHOCK ],
      [ 24, Moves.DRAGON_DANCE ],
      [ 32, Moves.CROSS_POISON ],
      [ 40, Moves.DRAGON_PULSE ],
      [ 48, Moves.FLAMETHROWER ],
      [ 56, Moves.DYNAMAX_CANNON ],
      [ 64, Moves.COSMIC_POWER ],
      [ 72, Moves.RECOVER ],
      [ 80, Moves.HYPER_BEAM ],
      [ 88, Moves.ETERNABEAM ],
    ],
  },
  [Species.URSHIFU]: {
    1: [
      [ EVOLVE_MOVE, Moves.SURGING_STRIKES ],
      [ 1, Moves.LEER ],
      [ 1, Moves.FOCUS_ENERGY ],
      [ 1, Moves.ENDURE ],
      [ 1, Moves.ROCK_SMASH ],
      [ 1, Moves.AQUA_JET ],
      [ 12, Moves.AERIAL_ACE ],
      [ 16, Moves.SCARY_FACE ],
      [ 20, Moves.HEADBUTT ],
      [ 24, Moves.BRICK_BREAK ],
      [ 28, Moves.DETECT ],
      [ 32, Moves.BULK_UP ],
      [ 36, Moves.IRON_HEAD ],
      [ 40, Moves.DYNAMIC_PUNCH ],
      [ 44, Moves.COUNTER ],
      [ 48, Moves.CLOSE_COMBAT ],
      [ 52, Moves.FOCUS_PUNCH ],
    ],
  },
  [Species.CALYREX]: {
    1: [
      [ 1, Moves.POUND ],
      [ 1, Moves.SWORDS_DANCE ],
      [ 1, Moves.STOMP ],
      [ 1, Moves.DOUBLE_KICK ],
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAKE_DOWN ],
      [ 1, Moves.THRASH ],
      [ 1, Moves.DOUBLE_EDGE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.MIST ],
      [ 1, Moves.MEGA_DRAIN ],
      [ 1, Moves.GROWTH ],
      [ 1, Moves.CONFUSION ],
      [ 1, Moves.TORMENT ],
      [ 1, Moves.TAUNT ],
      [ 1, Moves.IRON_DEFENSE ],
      [ 1, Moves.AVALANCHE ],
      [ 1, Moves.ICICLE_CRASH ],
      [ 1, Moves.GLACIAL_LANCE ],
      [ 8, Moves.LIFE_DEW ],
      [ 16, Moves.GIGA_DRAIN ],
      [ 24, Moves.PSYSHOCK ],
      [ 32, Moves.HELPING_HAND ],
      [ 40, Moves.AROMATHERAPY ],
      [ 40, Moves.GRASSY_TERRAIN ],
      [ 48, Moves.ENERGY_BALL ],
      [ 56, Moves.PSYCHIC ],
      [ 64, Moves.LEECH_SEED ],
      [ 72, Moves.HEAL_PULSE ],
      [ 80, Moves.SOLAR_BEAM ],
      [ 88, Moves.FUTURE_SIGHT ],
    ],
    2: [
      [ 1, Moves.POUND ],
      [ 1, Moves.STOMP ],
      [ 1, Moves.DOUBLE_KICK ],
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAKE_DOWN ],
      [ 1, Moves.THRASH ],
      [ 1, Moves.DOUBLE_EDGE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 1, Moves.DISABLE ],
      [ 1, Moves.MEGA_DRAIN ],
      [ 1, Moves.GROWTH ],
      [ 1, Moves.CONFUSION ],
      [ 1, Moves.AGILITY ],
      [ 1, Moves.CONFUSE_RAY ],
      [ 1, Moves.HAZE ],
      [ 1, Moves.SHADOW_BALL ],
      [ 1, Moves.NASTY_PLOT ],
      [ 1, Moves.HEX ],
      [ 1, Moves.ASTRAL_BARRAGE ],
      [ 8, Moves.LIFE_DEW ],
      [ 16, Moves.GIGA_DRAIN ],
      [ 24, Moves.PSYSHOCK ],
      [ 32, Moves.HELPING_HAND ],
      [ 40, Moves.GRASSY_TERRAIN ],
      [ 40, Moves.PSYCHIC_TERRAIN ],
      [ 48, Moves.ENERGY_BALL ],
      [ 56, Moves.PSYCHIC ],
      [ 64, Moves.LEECH_SEED ],
      [ 72, Moves.HEAL_PULSE ],
      [ 80, Moves.SOLAR_BEAM ],
      [ 88, Moves.FUTURE_SIGHT ],
    ],
  },
  [Species.OINKOLOGNE]: {
    1: [
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 3, Moves.DISARMING_VOICE ],
      [ 6, Moves.ECHOED_VOICE ],
      [ 9, Moves.MUD_SHOT ],
      [ 12, Moves.COVET ],
      [ 15, Moves.DIG ],
      [ 17, Moves.HEADBUTT ],
      [ 23, Moves.YAWN ],
      [ 28, Moves.TAKE_DOWN ],
      [ 30, Moves.WORK_UP ],
      [ 34, Moves.UPROAR ],
      [ 39, Moves.DOUBLE_EDGE ],
      [ 45, Moves.EARTH_POWER ],
      [ 51, Moves.BELCH ],
    ],
  },
  [Species.REVAVROOM]: {
    1: [
      [ EVOLVE_MOVE, Moves.WICKED_TORQUE ],
      [ EVOLVE_MOVE, Moves.SHIFT_GEAR ],
      [ 1, Moves.LICK ],
      [ 1, Moves.POISON_GAS ],
      [ 1, Moves.MAGNET_RISE ],
      [ 4, Moves.SMOG ],
      [ 7, Moves.TAUNT ],
      [ 10, Moves.ASSURANCE ],
      [ 13, Moves.SLUDGE ],
      [ 17, Moves.GYRO_BALL ],
      [ 21, Moves.HEADBUTT ],
      [ 25, Moves.SCREECH ],
      [ 28, Moves.IRON_HEAD ],
      [ 32, Moves.SWAGGER ],
      [ 36, Moves.POISON_JAB ],
      [ 46, Moves.UPROAR ],
      [ 52, Moves.SPIN_OUT ],
      [ 58, Moves.GUNK_SHOT ],
    ],
    2: [
      [ EVOLVE_MOVE, Moves.BLAZING_TORQUE ],
      [ EVOLVE_MOVE, Moves.SHIFT_GEAR ],
      [ 1, Moves.LICK ],
      [ 1, Moves.POISON_GAS ],
      [ 1, Moves.MAGNET_RISE ],
      [ 4, Moves.SMOG ],
      [ 7, Moves.TAUNT ],
      [ 10, Moves.ASSURANCE ],
      [ 13, Moves.SLUDGE ],
      [ 17, Moves.GYRO_BALL ],
      [ 21, Moves.HEADBUTT ],
      [ 25, Moves.SCREECH ],
      [ 28, Moves.IRON_HEAD ],
      [ 32, Moves.SWAGGER ],
      [ 36, Moves.POISON_JAB ],
      [ 46, Moves.UPROAR ],
      [ 52, Moves.SPIN_OUT ],
      [ 58, Moves.GUNK_SHOT ],
    ],
    3: [
      [ EVOLVE_MOVE, Moves.NOXIOUS_TORQUE ],
      [ EVOLVE_MOVE, Moves.SHIFT_GEAR ],
      [ 1, Moves.LICK ],
      [ 1, Moves.POISON_GAS ],
      [ 1, Moves.MAGNET_RISE ],
      [ 4, Moves.SMOG ],
      [ 7, Moves.TAUNT ],
      [ 10, Moves.ASSURANCE ],
      [ 13, Moves.SLUDGE ],
      [ 17, Moves.GYRO_BALL ],
      [ 21, Moves.HEADBUTT ],
      [ 25, Moves.SCREECH ],
      [ 28, Moves.IRON_HEAD ],
      [ 32, Moves.SWAGGER ],
      [ 36, Moves.POISON_JAB ],
      [ 46, Moves.UPROAR ],
      [ 52, Moves.SPIN_OUT ],
      [ 58, Moves.GUNK_SHOT ],
    ],
    4: [
      [ EVOLVE_MOVE, Moves.MAGICAL_TORQUE ],
      [ EVOLVE_MOVE, Moves.SHIFT_GEAR ],
      [ 1, Moves.LICK ],
      [ 1, Moves.POISON_GAS ],
      [ 1, Moves.MAGNET_RISE ],
      [ 4, Moves.SMOG ],
      [ 7, Moves.TAUNT ],
      [ 10, Moves.ASSURANCE ],
      [ 13, Moves.SLUDGE ],
      [ 17, Moves.GYRO_BALL ],
      [ 21, Moves.HEADBUTT ],
      [ 25, Moves.SCREECH ],
      [ 28, Moves.IRON_HEAD ],
      [ 32, Moves.SWAGGER ],
      [ 36, Moves.POISON_JAB ],
      [ 46, Moves.UPROAR ],
      [ 52, Moves.SPIN_OUT ],
      [ 58, Moves.GUNK_SHOT ],
    ],
    5: [
      [ EVOLVE_MOVE, Moves.COMBAT_TORQUE ],
      [ EVOLVE_MOVE, Moves.SHIFT_GEAR ],
      [ 1, Moves.LICK ],
      [ 1, Moves.POISON_GAS ],
      [ 1, Moves.MAGNET_RISE ],
      [ 4, Moves.SMOG ],
      [ 7, Moves.TAUNT ],
      [ 10, Moves.ASSURANCE ],
      [ 13, Moves.SLUDGE ],
      [ 17, Moves.GYRO_BALL ],
      [ 21, Moves.HEADBUTT ],
      [ 25, Moves.SCREECH ],
      [ 28, Moves.IRON_HEAD ],
      [ 32, Moves.SWAGGER ],
      [ 36, Moves.POISON_JAB ],
      [ 46, Moves.UPROAR ],
      [ 52, Moves.SPIN_OUT ],
      [ 58, Moves.GUNK_SHOT ],
    ],
  },
  [Species.PALDEA_TAUROS]: {
    1: [
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 5, Moves.WORK_UP ],
      [ 10, Moves.DOUBLE_KICK ],
      [ 15, Moves.FLAME_CHARGE ],
      [ 20, Moves.HEADBUTT ],
      [ 25, Moves.SCARY_FACE ],
      [ 30, Moves.ZEN_HEADBUTT ],
      [ 35, Moves.RAGING_BULL ],
      [ 40, Moves.REST ],
      [ 45, Moves.SWAGGER ],
      [ 50, Moves.THRASH ],
      [ 55, Moves.FLARE_BLITZ ],
      [ 60, Moves.CLOSE_COMBAT ],
    ],
    2: [
      [ 1, Moves.TACKLE ],
      [ 1, Moves.TAIL_WHIP ],
      [ 5, Moves.WORK_UP ],
      [ 10, Moves.DOUBLE_KICK ],
      [ 15, Moves.AQUA_JET ],
      [ 20, Moves.HEADBUTT ],
      [ 25, Moves.SCARY_FACE ],
      [ 30, Moves.ZEN_HEADBUTT ],
      [ 35, Moves.RAGING_BULL ],
      [ 40, Moves.REST ],
      [ 45, Moves.SWAGGER ],
      [ 50, Moves.THRASH ],
      [ 55, Moves.WAVE_CRASH ],
      [ 60, Moves.CLOSE_COMBAT ],
    ]
  }
};
